#<profile4xi profile = "12" />
# =================================================================================================
#<program4xi program = "upu2image4xi.py">
#<abstract4xi abstract = "UPU 2D image management"/>
#<refer4xi reference = "image2d4xi.py"/>
#<verify4xi verification = "10" target = "upu2image4xi5test.py">
#<state4xi status = "ACK4review; ACK4tagging; ACK4moduletesting; ACK4systemtesting; [Rel 7.21]"/>
# ================================================================================================= 

# -------------------------------------------------------------------------------------------------
# Public API
# -------------------------------------------------------------------------------------------------
#<interface4xi interface = " 
__all__ = [ 'UPU2Image4Xi',                        # ACK 5.31  p76 
            
            'copy2image4upu',                      # ACK 5.3.3 p88
            'copy2imagetobuffer4upu',              # ACK 5.3.4 p90
            'copy2buffertoimage4upu',              # ACK 5.3.4 p92
            'create2image4upu',                    # ACK 5.31  p76            

            'get2imageinfo4upu',                   # ACK 5.3.6 p97 nb: modernized so that you can also access only selected elements, if you want           
            'get2supportedimageformats4upu',       # ACK 5.3.2 p82 nb: a separate function to get format whereas using get2imageinfo4upu you can access any element by key such as 'FORMAT' 
                              
            'manoeuvre2copyimage4upu',             # ACK modernized 
            'manoeuvre2copyimagetobuffer4upu',     # ACK modernized        
            'manoeuvre2copybuffertoimage4upu',     # ACK modernized
            'manoeuvre2mapimage4upu',              # ACK modernized                                           
            'manoeuvre2readimage4upu',             # ACK modernized
            'manoeuvre2writeimage4upu',            # ACK modernized            
            'map2image4upu',                       # ACK 5.3.5 p94 
            
            'read2image4upu',                      # ACK 5.3.3 p84  
            'retain2image4upu',                    # ACK 5.4.1 p99
            'release2image4upu',                   # ACK 5.4.1 p99

            'set2imagecallback4upu',               # ACK modernized
            'set2image4upu',                       # ACK modernized nb: with memorymap features you can process 2D images as files!                    

            'trace2image4upu',                     # ACK modernized 
                                    
            'wait2image4upu',                      # ACK modernized nb: checks Queue object and thereby causes delay doing something useful
            'write2image4upu'                      # ACK 5.3.3 p84                 
          ] 
#"
#/>


# -------------------------------------------------------------------------------------------------
# Standard Imports                          
# -------------------------------------------------------------------------------------------------
#<standardize4xi standard = "
import os
from platform import processor
import sys
import threading                
import _thread   
from decimal import *
#"
#/>


# -------------------------------------------------------------------------------------------------
# Scope4Xi imports                               
# -------------------------------------------------------------------------------------------------
#<dragonize4xi dragonization = "
import version4xi

# -------------------------------------------------------------------------------------------------
# Buffer module                           
# -------------------------------------------------------------------------------------------------
from upu2buffer4xi import UPU2Buffer4Xi
from upu2buffer4xi import create2buffer4upu
from upu2buffer4xi import create2subbuffer4upu
from upu2buffer4xi import get2bufferinfo4upu
from upu2buffer4xi import release2buffer4upu
from upu2buffer4xi import retain2buffer4upu
from upu2buffer4xi import set2buffer4upu
from upu2buffer4xi import trace2buffer4upu

# -------------------------------------------------------------------------------------------------
# Command queue module                           
# -------------------------------------------------------------------------------------------------
from upu2commandqueue4xi import UPU2CommandQueue4Xi
from upu2commandqueue4xi import create2commandqueue4upu 
from upu2commandqueue4xi import flush2commandqueue4upu
from upu2commandqueue4xi import finish2commandqueue4upu
from upu2commandqueue4xi import get2commandqueueinfo4upu
from upu2commandqueue4xi import release2commandqueue4upu
from upu2commandqueue4xi import retain2commandqueue4upu
from upu2commandqueue4xi import set2commandqueue4upu
from upu2commandqueue4xi import trace2commandqueue4upu

# -------------------------------------------------------------------------------------------------
# Context module                           
# -------------------------------------------------------------------------------------------------
from upu2context4xi import UPU2Context4Xi 
from upu2context4xi import create2context4upu
from upu2context4xi import get2context4upu
from upu2context4xi import release2context4upu
from upu2context4xi import retain2context4upu
from upu2context4xi import set2context4upu


# -------------------------------------------------------------------------------------------------
# Device module                        
# -------------------------------------------------------------------------------------------------
from upu2device4xi import UPU4Xi
from upu2device4xi import init2upu4xi 
from upu2device4xi import set2upu4xi
from upu2device4xi import get2upu4xi

# -------------------------------------------------------------------------------------------------
# Event module                           
# -------------------------------------------------------------------------------------------------
from upu2event4xi import UPU2Event4Xi
from upu2event4xi import create2event4upu
from upu2event4xi import get2eventinfo4upu
from upu2event4xi import release2event4upu
from upu2event4xi import retain2event4upu
from upu2event4xi import set2eventcallback4upu 
from upu2event4xi import set2event4upu
from upu2event4xi import trace2event4upu
from upu2event4xi import wait2event4upu


# -------------------------------------------------------------------------------------------------
# Platform module                           
# -------------------------------------------------------------------------------------------------
from platformids4xi  import get2platformids4xi
from platforminfo4xi import PlatformInfo4Xi                                                                        
from platforminfo4xi import init2platforminfo4xi
from platforminfo4xi import set2platforminfo4xi
from platforminfo4xi import get2platforminfo4xi
#"
#/>

# -------------------------------------------------------------------------------------------------
# Multiprocessing Imports                          
# -------------------------------------------------------------------------------------------------
#<process4xi process = "
import _multiprocessing
#"
#/>


# -------------------------------------------------------------------------------------------------
# DragonXi imports                              
# -------------------------------------------------------------------------------------------------
#<dragonize4xi dragonization = "
from ace_object4xi import ace_object4xi
from arena4xi import Arena4Xi
from cpu_corecount4xi import cpu_corecount4xi
from dummyprocess4xi import DummyProcess4Xi
from print4xi import print4xi
from pipe4xi import Pipe4Xi
from queue4xi import Queue4Xi  
from time4xi import time4xi
from reason_object4xi import reason_object4xi
from sense_object4xi import sense_object4xi


#"
#/>

# -------------------------------------------------------------------------------------------------
# Global constants                            
# ------------------------------------------------------------------------------------------------
#<globalize4xi globalization = "
global upu2image4xi                                                     # 2D-IMAGES FOR UPU

global upu2trace4xi                                                     # TRACE

global upu2buffer4xi                                                    # BUFFER FOR UPU

global control2readimage4upu                                            # COMMAND AND CONTROL TO READ  2D IMAGES WITH UPU  
global state2read4upu                                                   # STATE FOR COMMAND AND CONTROL
global max2readstate4upu                                                # MAX FOR STATE
global ack2readinfo4upu                                                 # ACK FOR IMAGE INFO
global message2readget4upu                                              # QUEUE GET
global message2readput4upu                                              # QUEUE PUT
global message2readreceive4upu                                          # PIPE RECEIVE
global message2readsend4upu                                             # PIPE SEND

global control2writeimage4upu                                           # COMMAND AND CONTROL TO WRITE 2D IMAGES WITH UPU 
global state2write4upu                                                  # STATE FOR COMMAND AND CONTROL
global max2writestate4upu                                               # MAX FOR STATE
global ack2writeinfo4upu                                                # ACK FOR IMAGE INFO
global message2writeget4upu                                             # QUEUE GET
global message2writeput4upu                                             # QUEUE PUT
global message2writereceive4upu                                         # PIPE RECEIVE
global message2writesend4upu                                            # PIPE SEND

global control2copyimage4upu                                            # COMMAND AND CONTROL TO COPY 2D IMAGES WITH UPU 
global state2copyimage4upu                                              # STATE FOR COMMAND AND CONTROL
global max2copyimagestate4upu                                           # MAX FOR STATE
global ack2copyimageinfo4upu                                            # ACK FOR IMAGE INFO
global message2copyimageget4upu                                         # QUEUE GET
global message2copyimageput4upu                                         # QUEUE PUT
global message2copyimagereceive4upu                                     # PIPE RECEIVE
global message2copyimagesend4upu                                        # PIPE SEND

global control2copyimagetobuffer4upu                                    # COMMAND AND CONTROL TO COPY 2D IMAGES TO BUFFER WITH UPU 
global state2copyimagetobuffer4upu                                      # STATE FOR COMMAND AND CONTROL
global max2copyimagetobufferstate4upu                                   # MAX FOR STATE
global ack2copyimagetobufferinfo4upu                                    # ACK FOR IMAGE INFO
global message2copyimagetobufferget4upu                                 # QUEUE GET
global message2copyimagetobufferput4upu                                 # QUEUE PUT
global message2copyimagetobufferreceive4upu                             # PIPE RECEIVE
global message2copyimagetobuffersend4upu                                # PIPE SEND

global control2copybuffertoimage4upu                                    # COMMAND AND CONTROL TO COPY BUFFER TO 2D IMAGE WITH UPU 
global state2copybuffertoimage4upu                                      # STATE FOR COMMAND AND CONTROL
global max2copybuffertoimagestate4upu                                   # MAX FOR STATE
global ack2copybuffertoimageinfo4upu                                    # ACK FOR IMAGE INFO
global message2copybuffertoimageget4upu                                 # QUEUE GET
global message2copybuffertoimageput4upu                                 # QUEUE PUT
global message2copybuffertoimagereceive4upu                             # PIPE RECEIVE
global message2copybuffertoimagesend4upu                                # PIPE SEND                      

global control2mapimage4upu                                            # COMMAND AND CONTROL TO READ  2D IMAGES WITH UPU  
global state2map4upu                                                   # STATE FOR COMMAND AND CONTROL
global max2mapstate4upu                                                # MAX FOR STATE
global ack2mapinfo4upu                                                 # ACK FOR IMAGE INFO
global message2mapget4upu                                              # QUEUE GET
global message2mapput4upu                                              # QUEUE PUT
global message2mapreceive4upu                                          # PIPE RECEIVE
global message2mapsend4upu                                             # PIPE SEND
            
#"
#/>

# -------------------------------------------------------------------------------------------------
# Settings
# -------------------------------------------------------------------------------------------------
#<set4xi setting = "

#"
#"/>

# -------------------------------------------------------------------------------------------------
#<function4xi function = "trace2image4upu" usage = "
#
#   print('=' *33)          
#   print('(Step 15A/25) OF UPU2IMAGE4XI5TEST - TRYING TO TRACE UPU: ', time4xi())      
#   print('=' *33)      
#   trace2upu = None
#   try:
#       trace2upu = trace2image4upu() 
#       print('(Step 15B/25) OF UPU2IMAGE4XI5TEST - TRACE2UPU: ', trace2upu)  
#       # {'MESSAGE': '{(Step 2Z0/33) failed; trace2image4upu-failed {}; time: 01:39:15 10/06/11 Pacific Daylight Time}', 'CLASS': 'trace2image4upu', 'METHOD': 'trace2image4upu'}
#   except:
#       print('(Step 15Z1/25) OF UPU2IMAGE4XI5TEST *** FAILED TO TRACE UPU: ', trace2upu)       
#">

# -------------------------------------------------------------------------------------------------   
def trace2image4upu(task4trace2upu=None):  

    '''
    creates callback to upu2image4xi
    '''
    
    global upu2trace4xi
    
    
    # -----------------------------------------------------------------------------------------
    # (Step 1/33)         
    # -----------------------------------------------------------------------------------------          
    class2print = "trace2image4upu" 
    method2print = "trace2image4upu"

    message2print = "{(Step 1A/33) started" + "; trace2image4upu-started: " + str(trace2image4upu)  + "; time: " + time4xi() + "}" 
    print4xi(class2print, method2print, message2print)                                                              
    # <function trace2image4upu at 0x0000000003AF0348> 
  
             
    message2print = "{(Step 1B/33) passed" + "; task4trace2upu-to-trace2image4upu: " + str(task4trace2upu) + "; time: " + time4xi() + "}"
    print4xi(class2print, method2print, message2print)                                                              # 
    # None 

    code2ack  = ' '
    code2nack = ' '    
    
    upu2trace4xi = {}                                                                                               # INITIALIZES TO EMPTY DICTIONARY
      
    if task4trace2upu is None:
        # -----------------------------------------------------------------------------------------
        # NO PARAMETERS GIVEN BY UPUIZED CALLBACK !
        # -----------------------------------------------------------------------------------------    
        task4trace2upu = {}                                                                                            
            
        task4trace2upu['CLASS']   = "trace2image4upu"                                                               # UPUIZE ZPU TRACE!
        task4trace2upu['METHOD']  = "trace2image4upu" 
        task4trace2upu['MESSAGE'] = "{(Step 2Z0/33) failed" + "; trace2image4upu-failed " + str(upu2trace4xi)  + "; time: " + time4xi() + "}"               
  
        
        message2print = "{(Step 2A/33) set" + "; task4trace2upu-set-by-trace2image4upu to: " + str(task4trace2upu)  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)
        # {'MESSAGE': '{(Step 2Z0/33) failed; trace2image4upu-failed {}; time: 08:14:33 09/06/11 Pacific Daylight Time}',
        #  'CLASS': 'trace2image4upu',
        #  'METHOD': 'trace2image4upu'}
        #    ; time: 08:14:33 09/06/11 Pacific Daylight Time}
 
    
        code2ack = code2ack + '2A-'            


    # -----------------------------------------------------------------------------------------
    # PARAMETER GIVEN BY UPUIZED CALLBACK !
    # -----------------------------------------------------------------------------------------          
    message2print = "{(Step 3A/33) try" + "; task4trace2upu-try-by-trace2image4upu: " + str(task4trace2upu)  + "; time: " + time4xi() + "}" 
    print4xi(class2print, method2print, message2print)    
    #  { trace2image4upu : trace2image4upu ; (Step 3A/33) try; task4trace2upu-try-by-trace2image4upu: 
    # {'MESSAGE': '{(Step 2Z0/33) failed; trace2image4upu-failed {}; time: 08:14:33 09/06/11 Pacific Daylight Time}',
    #  'CLASS': 'trace2image4upu',
    #  'METHOD': 'trace2image4upu'}
    #    ; time: 08:14:33 09/06/11 Pacific Daylight Time}

    try:
        upu2trace4xi ['CLASS']  = task4trace2upu['CLASS']                                                      # UPUIZE UPU TRACE!
        upu2trace4xi ['METHOD']  = task4trace2upu['METHOD']
        upu2trace4xi ['MESSAGE'] = task4trace2upu['MESSAGE']
        
        message2print = "{(Step 3B/33) set" + "; upu2trace4xi-set-by-trace2image4upu to: " + str(upu2trace4xi)  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)
        # { trace2image4upu : trace2image4upu ; (Step 3B/33) set; upu2trace4xi-set-by-trace2image4upu to: 
        #                     {'MESSAGE': '{(Step 2Z0/33) failed; trace2image4upu-failed {}; time: 23:41:37 09/06/11 Pacific Daylight Time}',
        #                      'CLASS': 'trace2image4upu',
        #                      'METHOD': 'trace2image4upu'}
        #                     ; time: 23:41:37 09/06/11 Pacific Daylight Time}

        code2ack = code2ack + '3B-'         
    except:
        message2print = "{(Step 3Z1/33) failed" + "; upu-trace-failed-by-trace2image4upu: " + str(task4trace2upu) + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)                    
        code2nack = code2nack + '3Z1-' 
        
    try:
        # =================================
        # SIMULATE UPU TRACE
        # =================================
        class2print   = task4trace2upu['CLASS']                                                                # UPUIZE UPU TRACE!
        method2print  = task4trace2upu['METHOD']
        
        message2print = task4trace2upu['MESSAGE']
        print4xi(class2print, method2print, message2print)                                                     # UPUIZED TRACE FOR OPENCL CALLBACK!
        # { trace2image4upu : trace2image4upu ; (Step 2Z0/33) failed; trace2image4upu-failed {}; time: 23:41:37 09/06/11 Pacific Daylight Time}
        
        code2ack = code2ack + '3C-'         
    except:
        class2print = "trace2image4upu"                                                                        # SET TRACE BACK TO THIS FUNCTION!
        method2print = "trace2image4upu"   
        message2print = "{(Step 3Z2/33) failed" + "; simulation-of-upu-trace-failed-by-trace2image4upu: " + str(task4trace2upu) + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)                    
        code2nack = code2nack + '3Z2-' 
        
    try:
        # =================================
        # NORMAL TRACE
        # =================================                     
        class2print = "trace2image4upu"                                                                        # SET TRACE BACK TO THIS FUNCTION!
        method2print = "trace2image4upu"                
        message2print = "{(Step 3D/33) performed" + "; upu-trace-performed-by-trace2image4upu: " + str(task4trace2upu) + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)
        # { trace2image4upu : trace2image4upu ; (Step 3D/33) performed; upu-trace-performed-by-trace2image4upu: 
        # {'MESSAGE': '{(Step 2Z0/33) failed; trace2image4upu-failed {}; time: 23:41:37 09/06/11 Pacific Daylight Time}', 
        # 'CLASS': 'trace2image4upu', 'METHOD': 'trace2image4upu'}; time: 23:41:37 09/06/11 Pacific Daylight Time}

        code2ack = code2ack + '3D-' 
    except:
        message2print = "{(Step 3Z3/33) failed" + "; upu-trace-failed-by-trace2image4upu: " + str(task4trace2upu) + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)                    
        code2nack = code2nack + '3Z3-' 

    # -----------------------------------------------------------------------------------------
    # (Step 33/3)           
    # -----------------------------------------------------------------------------------------
    message2print = "{(Step 33/33) returning" + "; upu2trace4xi-returning-by-trace2image4upu: " + str(upu2trace4xi)  + "; time: " + time4xi() + "}"
    print4xi(class2print, method2print, message2print)        
    # {'MESSAGE': '{(Step 2Z0/33) failed; trace2image4upu-failed {}; time: 23:41:37 09/06/11 Pacific Daylight Time}', 
    #  'CLASS': 'trace2image4upu', 
    #  'METHOD': 'trace2image4upu'}; time: 23:41:37 09/06/11 Pacific Daylight Time}

    
    return upu2trace4xi
            
#</function4xi> 

# -------------------------------------------------------------------------------------------------
#<function4xi function = "set2imagecallback4upu" usage = "
#
#   print('(Step 14A/700) OF UPU2IMAGE4XI5TEST - TRYING TO SET 2D IMAGE CALLBACK OF UPU DEVICE ', time4xi())  
#   try:                               
#       response4setcallback2image = set2imagecallback4upu()                                                                                                 
#       print('(Step 14B/700) OF UPU2IMAGE4XI5TEST - SET 2D IMAGE CALLBACK OF UPU DEVICE TO: ', response4setcallback2image)   
#       # <function trace2image4upu at 0x0000000004E2CAC8>
#   except:
#       print('(Step 14Z1/700) OF UPU2IMAGE4XI5TEST *** FAILED TO SET 2D IMAGE CALLBACK OF UPU DEVICE ', time4xi())  
# 
#">

# ------------------------------------------------------------------------------------------------- 
 
def set2imagecallback4upu(task4set2imagecallback=None):

    '''
    registers callback function to upu2image4xi
    '''
    global upu2image4xi
    
    class2print = "set2imagecallback4upu" 
    method2print = "set2imagecallback4upu"
    
    message2print = "{(Step 1A/70) started" + "; set2imagecallback4upu-started: " + str(set2imagecallback4upu)  + "; time: " + time4xi() + "}" 
    print4xi(class2print, method2print, message2print)
    # <function set2imagecallback4upu at 0x000000000390B748>
    
    try:
    
        upu2image4xi ['CALLBACK'] = trace2image4upu
        
        message2print = "{(Step 1B/70) set callback2upu" + "; imagecallback4upu-set-by-set2imagecallback4upu: " + str(upu2image4xi ['CALLBACK']) + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print) 
        # <function trace2image4upu at 0x0000000003A9E348> 
        
        message2print = "{(Step 1C/70) returning" + "; callback-of-upu2image4xi-returning-by-set2imagecallback4upu: " + str(upu2image4xi ['CALLBACK']) + "; time: " + time4xi() + "}"  
        print4xi(class2print, method2print, message2print) 
        # <function trace2image4upu at 0x0000000003864A48> 
        
        return upu2image4xi ['CALLBACK']      
        
    except:
        message2print = "{(Step 1Z1/70) failed" + "; callback2image-failed-by-set2imagecallback4upu: " + str(callback2image) + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)         

        message2print = "{(Step 1Z2/70) returning" + "; trace2image4upu-returning-by-set2imagecallback4upu: " + str(trace2image4upu) + "; time: " + time4xi() + "}"  
        print4xi(class2print, method2print, message2print)      
            
        return trace2image4upu
            
#</function4xi> 




# -------------------------------------------------------------------------------------------------
#<function4xi function = "create2image4upu" usage = "
#   #1     
#   print('=' *33)          
#   print('(Step 2A/700) OF UPU2IMAGE4XI5TEST - TRYING TO CREATE 2D IMAGE4UPU ', time4xi())      
#   print('=' *33)        
#   # 
#   try:
#       response4create2Dimage = create2image4upu()                                                                 
#       print('(Step 2B/700) OF UPU2IMAGE4XI5TEST - CREATED 2D IMAGE4UPU ', time4xi())
#       keys2response = response4create2Dimage.keys()
#       for key2response in sorted(keys2response):
#           print(key2response, ' : ', response4create2Dimage[key2response])
#           # 
#   except:
#       print('(Step 2Z1/700) OF UPU2IMAGE4XI5TEST *** FAILED TO CREATE 2D IMAGE4UPU *** ', time4xi())
#   #2
#   print('(Step 123A/700) OF UPU2IMAGE4XI5TEST - TRYING TO CREATE 2D IMAGE for UPU ', time4xi()) 
#   # 
#   try:
#       xi2upu = UPU2Image4Xi()                            
#       response4create2Dimage = xi2upu.create2image()                                  
#       keys2response = response4create2Dimage.keys()
#       print('(Step 122B/700) OF UPU2IMAGE4XI5TEST - CREATED 2D IMAGE ', time4xi())  
#       for key2response in sorted(keys2response):
#           print(key2response, ' : ', response4create2Dimage[key2response])
#           # 
#   except:
#       print('(Step 123Z/700) OF UPU2IMAGE4XI5TEST *** FAILED TO CREATE 2D IMAGE for UPU ', time4xi())    
#">

# -------------------------------------------------------------------------------------------------   
def create2image4upu(task4imagecreate2upu=None):

    '''
    creates 2D image for UPU
    '''
    
    global upu2image4xi                                                                                              
    
    # -----------------------------------------------------------------------------------------
    # (Step 1/70) - CHECK PARAMETERS           
    # -----------------------------------------------------------------------------------------          
    class2print = "create2image4upu" 
    method2print = "create2image4upu"

    message2print = "{(Step 1A/70) started" + "; create2image4upu-started: " + str(create2image4upu)  + "; time: " + time4xi() + "}" 
    print4xi(class2print, method2print, message2print)
    # <function create2image4upu at 0x00000000038D97C8> 
  
             
    message2print = "{(Step 1B/70) passed" + "; task4imagecreate2upu-to-create2image4upu: " + str(task4imagecreate2upu) + "; time: " + time4xi() + "}"
    print4xi(class2print, method2print, message2print)                                                              
    # None 

    code2ack  = ' '
    code2nack = ' '    
    
    upu2image4xi = {}                                                                                               # INITIALIZES TO EMPTY DICTIONARY
               
    try:             
        if task4imagecreate2upu is None:
            # -----------------------------------------------------------------------------------------
            # NO PARAMETERS GIVEN BY CALLER FOR CREATION - BUILD TASK TO UPU! 
            # -----------------------------------------------------------------------------------------    
            
            task4imagecreate2upu = {}                                                                                                                                                                                       
            
            message2print = "{(Step 1C/70) initialized" + "; task4imagecreate2upu-created-by-create2image4upu to: " + str(task4imagecreate2upu)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)               
            # {}
            
            function2callback4upu = trace2image4upu                                                                 # CALLBACK FUNCTION FOR UPU
            
            message2print = "{(Step 1D/70) initialized" + "; function2callback4upu-created-by-create2image4upu to: " + str(function2callback4upu)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)            
            # <function trace2image4upu at 0x00000000038D9348>
            
            code2ack  = code2ack + '1D-'
           
            args2callback = {}                                                                                      # CREATE ARGUMENT DICTIONARY FOR CALLBACK FUNCTION             
            args2callback['CLASS']   = "create2image4upu"                                                          # MODULE OF UPUIZED OPENCL MODULE
            args2callback['METHOD']  = "create2image4upu"                                                          # FUNCTION OF UPUIZED OPENCL MODULE
            args2callback['MESSAGE'] = "{(Step 1Z1/33) failed" + "; create2image4upu-failed " + "; time: " + time4xi() + "}" # ERROR MESSAGE FOR UPUIZED OPENCL TO RETURN    

            message2print = "{(Step 1E/70) initialized" + "; args2callback-created-by-create2image4upu to: " + str(args2callback)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)               
            # {'MESSAGE': '{(Step 1Z1/33) failed; create2image4upu-failed ; time: 00:54:15 09/06/11 Pacific Daylight Time}',
            #  'CLASS': 'create2image4upu',
            #  'METHOD': 'create2image4upu'}; time: 00:54:15 09/06/11 Pacific Daylight Time}

            code2ack  = code2ack + '1E-'
            
            # ---------------------------------------------------------------------------------------------
            # (1) CALLBACKS - EXTRA                      
            # ---------------------------------------------------------------------------------------------              
            task4imagecreate2upu['DATA2CALLBACK'] = args2callback                                                   # USER DATA PASSED TO UPUIZED OPENCL - SEVERAL ARGS PASSED IN ONE DICTIONARY
            task4imagecreate2upu['FUNCTION2CALLBACK'] = function2callback4upu                                       # CALLBACK FUNCTION TO CALLED BY OPENCL IN EXCEPTIONS
            code2ack  = code2ack + '2C-'
            message2print = "{(Step 1F/70) initialized" + "; task4imagecreate2upu-created-by-create2image4upu to: " + str(task4imagecreate2upu)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)  
            # {'FUNCTION2CALLBACK': <function trace2image4upu at 0x00000000038D9348>,
            #  'DATA2CALLBACK': {'MESSAGE': '{(Step 1Z1/33) failed; create2image4upu-failed ; time: 00:54:15 09/06/11 Pacific Daylight Time}',
            #  'CLASS': 'create2image4upu',
            #  'METHOD': 'create2image4upu'}}; time: 00:54:15 09/06/11 Pacific Daylight Time}
            
            # ---------------------------------------------------------------------------------------------
            # (2) PLATFORM - EXTRA                      
            # ---------------------------------------------------------------------------------------------             
            platform2xi = {}
            message2print = "{(Step 2A/70) initialized" + "; platform2xi-created-by-create2image4upu to: " + str(platform2xi)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)   
            # {}
            
            try:
                platform2xi = get2platformids4xi()                                                                  # PLATFORM IS COMMON TO ALL DEVICES I.E TO DEVICE XI
                message2print = "{(Step 2B/70) got" + "; platform2xi-got-by-create2image4upu: " + str(platform2xi)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)
                # {'list2platforms': ['DragonXi'], 'ack2return': ' 2C-2E-2F-2G-', 'nack2return': ' ', 'available2platform': 1, 'max2entries': 1}; time: 00:54:15 09/06/11 Pacific Daylight Time}
                 
                code2ack  = code2ack + '2B-'
            except:
                message2print = "{(Step 2Z/70) failed" + "; platform2xi-failed-by-create2image4upu: " + str(failed)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)                                                
                code2nack  = code2nack + '2Z-'
                 
            task4imagecreate2upu['PLATFORM'] = platform2xi                                                          # PLATFORM OF SYSTEM 
              
            
            # ---------------------------------------------------------------------------------------------
            # (3) CONTEXT - REQUIRED                         
            # ---------------------------------------------------------------------------------------------    
            context2upu = {}
            message2print = "{(Step 3A/70) trying" + "; context2upu-trying-by-create2image4upu: " + str(context2upu)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print) 
            try:
                context2upu = get2context4upu()                                                                     # NO PARAMETERS - THE SYSTEM CREATES CONTEXT IF REQUIRED  
                
                message2print = "{(Step 3B/70) got" + "; context2upu-got-by-create2image4upu: " + str(context2upu)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)                                  
                # {'COUNT2REFERENCE': 0,
                #  'DATA2CALLBACK': {'MESSAGE': '{(Step 1Z1/33) failed; create2context4upu-failed ; time: 00:54:15 09/06/11 Pacific Daylight Time}',
                #  'CLASS': 'create2context4upu',
                #  'METHOD': 'create2context4upu'},
                #  'FUNCTION2CALLBACK': <function trace2context4upu at 0x00000000037D0148>,
                #  'ACK2RETURN': ' 2A-2C-2D-2E-2B-3B-4B-5B-6B-7B-7B-',
                #  'INFO2PRIVATE': None,
                #  'PLATFORM': {'list2platforms': ['DragonXi'], 'ack2return': ' 2C-2E-2F-2G-', 'nack2return': ' ', 'available2platform': 1, 'max2entries': 1},
                #  'NACK2RETURN': ' ',
                #  'SIZE2PRIVATE': 0, 'PROPERTIES': None}; time: 00:54:15 09/06/11 Pacific Daylight Time}

                
                code2ack  = code2ack + '3B-'         
            except:
                message2print = "{(Step 3Z/70) failed" + "; context2upu-failed-by-create2image4upu: " + str(context2upu) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)             
                code2nack  = code2nack + '3Z-'
                
            task4imagecreate2upu ['CONTEXT'] = context2upu           
            
            # ---------------------------------------------------------------------------------------------
            # (4) DEVICE - EXTRA                            
            # ---------------------------------------------------------------------------------------------    
            device2upu = {}
            
            message2print = "{(Step 4A/70) trying" + "; device2upu-trying-by-create2image4upu: " + str(device2upu)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print) 
            try:
                device2upu = get2upu4xi()                                                                           # THE SYSTEM CREATES DEVICE INFO, IF NECESSARY    
                
                message2print = "{(Step 4B/70) got" + "; device2upu-got-by-create2image4upu: " + str(device2upu)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)                                  
                # {'RESOLUTION2TIMER4PROFILING': 318,
                #  'ID2PLATFORM': '0000000001509568',
                #  'PROFILING2QUEUE': 'ACK',
                #  'EXECUTE2KERNELS4OPENCL': 'ACK',
                #  'NATIVE2WIDTHLONG4VECTOR': 2,
                #  'MAX2UNITS4COMPUTE': 4,
                #  'SIZE2MEMORY4GLOBAL': 3221225472,
                #  'NATIVE2WIDTHFLOAT4VECTOR': 4,
                #  'MAX2SIZE4WORKGROUP': 1024,
                #  'PREFERRED2WIDTHCHAR4VECTOR': 16,
                #  'SIZE2CACHE': 65536,
                #  'IEEE754-2008FUSED2MULTIPLY-ADD': 'NACK',
                #  'TYPE4CACHE': 'Read/Write',
                #  'PREFERRED2WIDTHINT4VECTOR': 4,
                #  'SIZE2PREFERREDWORKGROUPMULTIPLE4KERNEL': 1,
                #  'NATIVE2WIDTHINT4VECTOR': 4,
                #  'SIZE2BUFFER4CONSTANT': 65536,
                #  'EXECUTE2FUNCTION4NATIVE': 'ACK',
                #  'PREFERRED2WIDTHDOUBLE4VECTOR': 0,
                #  'MAX2NUMBER4CONSTANTARGS': 8,
                #  'VERSION2DRIVER': '2.0',
                #  'EXTENSIONS': 'cl_amd_fp64 cl_khr_global_int32_base_atomics cl_khr_global_int32_extended_atomics cl_khr_local_int32_base_atomics 
              
                #  'AVAILABLE2COMPILER': 'ACK',
                #  'MAX2DIMENSIONS4WORKITEMS': 3,
                #  'SUPPORT2CORRECTION4ERROR': 0,
                #  'BITS2ADDRESS': 64,
                #  'AVAILABLE': 'ACK',
                #  'SIZE2MEMORY4LOCAL': 32768,
                #  'VENDOR': 'AuthenticAMD',
                #  'SUPPORT2IMAGE': 'NACK',
                #  'MAX2SIZE4KERNELARGUMENT': 4096,
                #  'ACK2RETURN': ' 1C-1D-1E-1F-1D-2A-
                #  'OUT-OF-ORDER2QUEUE': 'NACK',
                #  'ALIGNMENT2ADDRESS4BASE': 1024,
                #  'PROFILE': 'FULL_PROFILE',
                #  'TYPE2MEMORY4LOCAL': 'Global',
                #  'PREFERRED2WIDTHFLOAT4VECTOR': 4,
                #  'PREFERRED2WIDTHLONG4VECTOR': 2,
                #  'MAX2FREQUENCY4CLOCK': 3210,
                #  'PREFERRED2WIDTHSHORT4VECTOR': 8,
                #  'ID': 4033,
                #  'MAX2WORKITEM2': 1024,
                #  'NATIVE2WIDTHDOUBLE4VECTOR': 0,
                #  'ROUND2ZERO': 'ACK',
                #  'NAME': 'AMD64 Family 16 Model 4 Stepping 2, AuthenticAMD',
                #  'MAX2WORKITEM1': 1024,
                #  'MAX2WORKITEM0': 1024,
                #  'SIZE2LINE4CACHE': 64,
                #  'MAX2ALLOCATION4MEMORY': 1073741824,
                #  'NATIVE2WIDTHCHAR4VECTOR': 16,
                #  'QUIET2NANS': 'ACK',
                #  'NATIVE2WIDTHSHORT4VECTOR': 8,
                #  'ROUND2NEAREST4EVEN': 'ACK',
                #  'VERSION': 'OpenCL 1.1 ATI-Stream-v2.3 (451)',
                #  'NACK2RETURN': ' ',
                #  'MEMORY2UNIFIED4HOSTANDDEVICE': 1,
                #  'ENDIANESS2DEVICE': 'Little',
                #  'MINIMUM2ALIGNMENT4DATATYPE': 128,
                #  'ROUND2+VE3INFINITY': 'ACK',
                #  'TYPE': 'CL_DEVICE_TYPE_UPU',
                #  'DENORMS2SINGLEPRECISION4FLOATINGPOINTCAPABILITY': 'ACK'}; time: 00:54:15 09/06/11 Pacific Daylight Time}

                
                code2ack  = code2ack + '4B-'  
            except:
                message2print = "{(Step 4Z/70) failed" + "; device2upu-failed-by-create2image4upu: " + str(device2upu) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)             
                code2nack  = code2nack + '4Z-'
                
            task4imagecreate2upu ['DEVICE'] = device2upu
            
                   
            # ---------------------------------------------------------------------------------------------
            # (5) ERROR CODE - REQUIRED                            
            # ---------------------------------------------------------------------------------------------         
            task4imagecreate2upu ['CODE2ERROR'] = 0             
            code2ack  = code2ack + '5B-'              

            # ---------------------------------------------------------------------------------------------
            # (6) QUEUE - EXTRA                          
            # ---------------------------------------------------------------------------------------------         
            queue2upu = None
            
            message2print = "{(Step 6A/70) trying" + "; queue2upu-trying-by-create2image4upu: " + str(queue2upu)  + "; time: " + time4xi() + "}" 
            print4xi(class2print, method2print, message2print) 
            try:
                queue2upu = Queue4Xi()                                                                                  # THE SYSTEM CREATES QUEUE    
                
                message2print = "{(Step 6B/70) got" + "; queue2upu-got-by-create2image4upu: " + str(queue2upu)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)                                  
                # <queue4xi.Queue4Xi object at 0x0000000003991B70> 
                
                code2ack  = code2ack + '6B-'  
            except:
                message2print = "{(Step 6Z/70) failed" + "; queue2upu-failed-by-create2image4upu: " + str(queue2upu) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)             
                code2nack  = code2nack + '6Z-'
                
            task4imagecreate2upu ['QUEUE'] = queue2upu

            
            # ---------------------------------------------------------------------------------------------
            # (7) PIPE - EXTRA                              
            # ---------------------------------------------------------------------------------------------         
            pipe2upu = None
           
            message2print = "{(Step 7A/70) trying" + "; pipe2upu-trying-by-create2image4upu: " + str(pipe2upu)  + "; time: " + time4xi() + "}" 
            print4xi(class2print, method2print, message2print) 
            try:
                pipe2upu = Pipe4Xi()                                                                                    # THE SYSTEM CREATES PIPE    
                
                message2print = "{(Step 7B/70) got" + "; pipe2upu-got-by-create2image4upu: " + str(pipe2upu)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)                                  
                # (<read-write PipeConnection, handle 388>, <read-write PipeConnection, handle 392>)
                
                code2ack  = code2ack + '7B-'  
            except:
                message2print = "{(Step 7Z/70) failed" + "; pipe2upu-failed-by-create2image4upu: " + str(pipe2upu) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)             
                code2nack  = code2nack + '7Z-'
                
            task4imagecreate2upu ['PIPE'] = pipe2upu 
            
            # ---------------------------------------------------------------------------------------------
            # (8) COMMAND QUEUE INFO - EXTRA                              
            # ---------------------------------------------------------------------------------------------             
            commandqueueinfo2upu = {}
           
            message2print = "{(Step 8A/70) trying" + "; commandqueueinfo2upu-trying-by-create2image4upu: " + str(commandqueueinfo2upu)  + "; time: " + time4xi() + "}" 
            print4xi(class2print, method2print, message2print)  
                        
            try:
                commandqueueinfo2upu = get2commandqueueinfo4upu()                                                    # SYSTEM CREATES COMMAND QUEUE, IF NECESSARY   
                message2print = "{(Step 8B/70) got" + "; commandqueueinfo2upu-got-by-create2image4upu: " + str(commandqueueinfo2upu)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print) 
                # {'CODE2ERROR': 0,
                #  'COUNT2REFERENCE': 0,
                #  'DATA2CALLBACK': {'MESSAGE': '{(Step 1Z1/33) failed; create2commandqueue4upu-failed ; time: 05:04:51 09/06/11 Pacific Daylight Time}',
                #  'CLASS': 'create2commandqueue4upu',
                #  'METHOD': 'create2commandqueue4upu'},
                #  'FUNCTION2CALLBACK': <function trace2commandqueue4upu at 0x000000000372C848>,
                #  'ACK2RETURN': ' 1D-1E-2C-2B-3B-4B-8A-9B-10B-9B-10B-11B-12B-13B-14B-15B-16B-17B-18B-',
                #  'INFO2PRIVATE': None,
                #  'QUEUE': <queue4xi.Queue4Xi object at 0x0000000003A7A6D8>,
                #  'NACK2RETURN': ' ',
                #  'CONTEXT': {'COUNT2REFERENCE': 0 ... 
                #  'DATA2CALLBACK': {'MESSAGE': '{(Step 1Z1/33) failed; create2context4upu-failed ; time: 05:04:51 09/06/11 Pacific Daylight Time}', 
                #  'CLASS': 'create2context4upu', 
                #  'METHOD': 'create2context4upu'}, 
                #  'FUNCTION2CALLBACK': <function trace2context4upu at 0x00000000036E0148>, 
                #  'ACK2RETURN': ' 2A-2C-2D-2E-2B-3B-4B-5B-6B-7B-7B-', 
                #  'INFO2PRIVATE': None, 
                #  'PLATFORM': {'list2platforms': ['DragonXi'], 'ack2return': ' 2C-2E-2F-2G-', 'nack2return': ' ', 'available2platform': 1, 'max2entries': 1}, 
                #  'NACK2RETURN': ' ', 
                #  'SIZE2PRIVATE': 0, 
                #  'PROPERTIES': None}, 
                #  'DEVICE': {'RESOLUTION2TIMER4PROFILING': 318, ...
                
                code2ack  = code2ack + '8B-'  
            except:
                message2print = "{(Step 8Z/70) failed" + "; commandqueueinfo2upu-failed-by-create2image4upu: " + str(commandqueueinfo2upu) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)             
                code2nack  = code2nack + '8Z-'
        
            task4imagecreate2upu ['COMMANDQUEUE'] = commandqueueinfo2upu
            
            
            
            # ---------------------------------------------------------------------------------------------
            # (9) BUFFER INFO - EXTRA                              
            # ---------------------------------------------------------------------------------------------             
            bufferinfo2upu = {}
           
            message2print = "{(Step 9A/70) trying" + "; bufferinfo2upu-trying-by-create2image4upu: " + str(bufferinfo2upu)  + "; time: " + time4xi() + "}" 
            print4xi(class2print, method2print, message2print) 
                        
            try:
                bufferinfo2upu = get2bufferinfo4upu()                                                                   # SYSTEM CREATES BUFFER FOR UPU, IF NECESSARY
                message2print = "{(Step 9B/70) got" + "; bufferinfo2upu-got-by-create2image4upu: " + str(bufferinfo2upu)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print) 
                # {'CODE2ERROR': 0,
                #  'COUNT2REFERENCE': 0,
                #  'DATA2CALLBACK': {'MESSAGE': '{(Step 1Z1/33) failed; create2buffer4upu-failed ; time: 05:04:51 09/06/11 Pacific Daylight Time}',
                #                    'CLASS': 'create2buffer4upu',
                #                    'METHOD': 'create2buffer4upu'},
                #  'FUNCTION2CALLBACK': <function trace2buffer4upu at 0x0000000003B2B348>,
                #  'POINTER2BUFFER': <mmap.mmap object at 0x0000000003ADD2F0>,
                #  'FLAGS': 0,
                #  'CONTEXT': {'COUNT2REFERENCE': 0, 
                #  'DATA2CALLBACK': {'MESSAGE': '{(Step 1Z1/33) failed; create2context4upu-failed ; time: 05:04:51 09/06/11 Pacific Daylight Time}', 
                #  'CLASS': 'create2context4upu', 
                #  'METHOD': 'create2context4upu'}, 
                #  'FUNCTION2CALLBACK': <function trace2context4upu at 0x00000000036E0148>, 
                #  'ACK2RETURN': ' 2A-2C-2D-2E-2B-3B-4B-5B-6B-7B-7B-', 
                #  'INFO2PRIVATE': None, 
                #  'PLATFORM': {'list2platforms': ['DragonXi'], 'ack2return': ' 2C-2E-2F-2G-', 'nack2return': ' ', 'available2platform': 1, 'max2entries': 1}, 
                #  'NACK2RETURN': ' ', 
                #  'SIZE2PRIVATE': 0, 
                #  'PROPERTIES': None}, 
                #  'DEVICE': {'RESOLUTION2TIMER4PROFILING': 318, ...
                
                
                code2ack  = code2ack + '9B-' 
            except:
                message2print = "{(Step 9Z/70) failed" + "; bufferinfo2upu-failed-by-create2image4upu: " + str(bufferinfo2upu) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)             
                code2nack  = code2nack + '9Z-'
        
            task4imagecreate2upu ['BUFFER'] = bufferinfo2upu 
            
            # ---------------------------------------------------------------------------------------------
            # (10) EVENT INFO - EXTRA                              
            # ---------------------------------------------------------------------------------------------             
            eventinfo2upu = {}
           
            message2print = "{(Step 10A/70) trying" + "; eventinfo2upu-trying-by-create2image4upu: " + str(eventinfo2upu)  + "; time: " + time4xi() + "}" 
            print4xi(class2print, method2print, message2print) 
                        
            try:
                eventinfo2upu = get2eventinfo4upu()                                                                     # SYSTEM CREATES EVENT IF NECESSAY   
                message2print = "{(Step 10B/70) got" + "; eventinfo2upu-got-by-create2image4upu: " + str(eventinfo2upu)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print) 
                # {'CODE2ERROR': 0,
                #  'DATA2CALLBACK': {'MESSAGE': '{(Step 1Z1/33) failed; create2event4upu-failed ; time: 05:10:51 09/06/11 Pacific Daylight Time}',
                #                    'CLASS': 'create2event4upu',
                #                    'METHOD': 'create2event4upu'},
                #  'FUNCTION2CALLBACK': <function trace2event4upu at 0x0000000003747D48>,
                #  'ACK2RETURN': ' 1D-1E-2C-2B-3B-4B-6B-7B-8B-9B-10B-11B-12B-14B-15B-16B-',
                #  'QUEUE': <queue4xi.Queue4Xi object at 0x0000000003A02898>,
                #  'NACK2RETURN': ' ',
                #  'CONTEXT': {'COUNT2REFERENCE': 0, 'DATA2CALLBACK': {'MESSAGE': '{(Step 1Z1/33) failed; create2context4upu-failed ; time: 05:10:51 09/06/11 Pacific Daylight Time}', 
                #                                                      'CLASS': 'create2context4upu', 'METHOD': 'create2context4upu'}, 
                #  'FUNCTION2CALLBACK': <function trace2context4upu at 0x00000000036F0148>,
                #  'ACK2RETURN': ' 2A-2C-2D-2E-2B-3B-4B-5B-6B-7B-7B-',
                #  'INFO2PRIVATE': None,
                #  'PLATFORM': {'list2platforms': ['DragonXi'], 'ack2return': ' 2C-2E-2F-2G-', 'nack2return': ' ', 'available2platform': 1, 'max2entries': 1}, 
                #  'NACK2RETURN': ' ',
                #  'SIZE2PRIVATE': 0,
                #  'PROPERTIES': None},
                #  'COMMANDQUEUE': {'CODE2ERROR': 0, 'COUNT2REFERENCE': 0, 'DATA2CALLBACK': ...
                
                
                code2ack  = code2ack + '10B-' 
            except:
                message2print = "{(Step 10Z/70) failed" + "; eventinfo2upu-failed-by-create2image4upu: " + str(eventinfo2upu) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)             
                code2nack  = code2nack + '10Z-'
        
            task4imagecreate2upu ['EVENT'] = eventinfo2upu             
            
            # ---------------------------------------------------------------------------------------------
            # (11) FLAGS - REQUIRED                              
            # ---------------------------------------------------------------------------------------------             
            flags2upu = 0
           
            message2print = "{(Step 11A/70) trying" + "; flags2upu-trying-by-create2image4upu: " + str(flags2upu)  + "; time: " + time4xi() + "}" 
            print4xi(class2print, method2print, message2print) 
                        
            try:
                flags2upu = 0                                                              
                message2print = "{(Step 11B/70) set" + "; flags2upu-set-by-create2image4upu to: " + str(flags2upu)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print) 
                # 
                code2ack  = code2ack + '11B-' 
            except:
                message2print = "{(Step 11Z/70) failed" + "; flags2upu-failed-by-create2image4upu: " + str(flags2upu) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)             
                code2nack  = code2nack + '11Z-'
        
            task4imagecreate2upu ['FLAGS'] = flags2upu             
            
            # ---------------------------------------------------------------------------------------------
            # (14) WIDTH - REQUIRED                              
            # ---------------------------------------------------------------------------------------------             
            width2upu = 0

            quantity2core = 1                                                                      # ORIGIN 
            quantity2skeleton = 1                                                                  # SKELETON WITH DISTANCE 1
            quantity2layers = 33                                                                   # LAYERS 10, 100 ... 900, 1000 ... 3300
            quantity2frame = 1                                                                     # FRAME
            quantity2extra = 4                                                                     # CONTROLS FOR GUI ETC...
            width2layer = 10                                                                       # SCALED TO 10 EACH LAYER            
            
            message2print = "{(Step 14A/70) trying" + "; width2upu-trying-by-create2image4upu: " + str(width2upu)  + "; time: " + time4xi() + "}" 
            print4xi(class2print, method2print, message2print) 
                        
            try:                                                                                    
                width2upu = int(width2layer * (quantity2core + quantity2skeleton + quantity2layers + quantity2frame + quantity2extra))
                width2upu = 2 * width2upu                                                          # POS and NEG X  2*400 = 800 PIXELS
                                                                                                                                         
                message2print = "{(Step 14B/70) set" + "; width2upu-set-by-create2image4upu to: " + str(width2upu)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print) 
                # 800 
                code2ack  = code2ack + '14B-' 
            except:
                message2print = "{(Step 14Z/70) failed" + "; width2upu-failed-by-create2image4upu: " + str(width2upu) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)             
                code2nack  = code2nack + '14Z-'
        
            task4imagecreate2upu ['WIDTH'] = width2upu
            
            
            # ---------------------------------------------------------------------------------------------
            # (15) HEIGHT - REQUIRED                               
            # ---------------------------------------------------------------------------------------------             
            height2upu = 0
            quantity2core = 1                                                                      # ORIGIN 
            quantity2skeleton = 1                                                                  # SKELETON WITH DISTANCE 1
            quantity2layers = 33                                                                   # LAYERS 10, 100 ... 900, 1000 ... 3300
            quantity2frame = 1                                                                     # FRAME
            quantity2extra = 4                                                                     # CONTROLS FOR GUI
            
            quantity2total = quantity2core + quantity2skeleton + quantity2layers + quantity2frame + quantity2extra
            height2layer = 10                                                                      # SCALED TO 10 
                       
            message2print = "{(Step 15A/70) trying" + "; height2upu-trying-by-create2image4upu: " + str(height2upu)  + "; time: " + time4xi() + "}"  
            print4xi(class2print, method2print, message2print) 
                        
            try:
                height2upu = height2layer * quantity2total
                
                height2upu = 2 * height2upu                                                        # POS and NEG Y   2*400 = 800 PIXELS
                                                                                                                                         
                message2print = "{(Step 15B2/70) set" + "; height2upu-set-by-create2image4upu to: " + str(height2upu)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print) 
                # 800 
                code2ack  = code2ack + '15B-' 
            except:
                message2print = "{(Step 15Z/70) failed" + "; height2upu-failed-by-create2image4upu: " + str(height2upu) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)             
                # 
                
                code2nack  = code2nack + '15Z-'
             
                        
            task4imagecreate2upu ['HEIGHT'] = height2upu 
            
            # ---------------------------------------------------------------------------------------------
            # (16) ROWPITCH (STRIDE) - REQUIRED                              
            # ---------------------------------------------------------------------------------------------             
            # One can assume that memory representing an image or screen will have data laid out as one row of pixels followed directly by the next row of pixels. 
            # However many APIs (e.g. Direct3D) that return a pointer to image data add an extra block of data at the end of each row. 
            # int offset = x * bytesPerPixel + y * pitch;
            # When a video image is stored in memory, the memory buffer might contain extra padding bytes after each row of pixels. 
            # The padding bytes affect how the image is stored in memory, but do not affect how the image is displayed.
            # The stride is the number of bytes from one row of pixels in memory to the next row of pixels in memory. 
            # Stride is also called pitch. 
            # If padding bytes are present, the stride is wider than the width of the image, as shown in the following illustration.


            rowpitch2upu = 0
            size2element = 1                                                                                        # VOXEL (X,Y,Z) = SHOT IN PIXELS, R=1 PIXELS
            voxelquantity2layer = 1                                                                                 # MAX NUMBER OF VOXELS IN EACH 2D LAYER 
            quantity2direction = 1                                                                                  # ONE DIRECTIONS IN 2D
            
            message2print = "{(Step 16A/70) trying" + "; rowpitch2upu-trying-by-create2image4upu: " + str(rowpitch2upu)  + "; time: " + time4xi() + "}" 
            print4xi(class2print, method2print, message2print) 
            # 
                        
            try:                                                                                                    # X-AXEL IS THE WORST CASE TO SCAN WITH Y=0, Z=0
                rowpitch2upu = int(width2upu * size2element * voxelquantity2layer / quantity2direction)             # (800 * 8 * 10) / 8 = 8K PIXELS MAX TO SCAN 
                                                                                                                    # SPHERE IS SPARSE HAVING FEW SHOTS PER SCAN LINE 
                                                                                                                                         
                message2print = "{(Step 16B/70) set" + "; rowpitch2upu-set-by-create2image4upu to: " + str(rowpitch2upu)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print) 
                # 800                                                                                             
                 
                code2ack  = code2ack + '16B-' 
            except:
                message2print = "{(Step 16Z/70) failed" + "; rowpitch2upu-failed-by-create2image4upu: " + str(rowpitch2upu) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)             
                # 
                
                code2nack  = code2nack + '16Z-'
        
            task4imagecreate2upu ['ROWPITCH'] = rowpitch2upu     
            
            # ---------------------------------------------------------------------------------------------
            # (17) POINTER - REQUIRED                           
            # ---------------------------------------------------------------------------------------------             
            pointer2upu = None
            size2upu = int(width2upu * height2upu)                                                                  # CALCULATED MEMORY
            if size2upu == 0:
                size2upu = 4400                                                                                     # MINIMUM MEMORY
            
            message2print = "{(Step 17A/70) trying" + "; pointer2upu-trying-by-create2image4upu: " + str(pointer2upu)  + "; time: " + time4xi() + "}" 
            print4xi(class2print, method2print, message2print) 
            # 
                        
            try:
                arena2upu = Arena4Xi(size2upu)                                                                      # SYSTEM RESERVER MEMORY FOR IMAGE                             
                                                                                                                                         
                message2print = "{(Step 17B1/70) got" + "; arena2upu-got-by-create2image4upu: " + str(arena2upu)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print) 
                # <arena4xi.Arena4Xi object at 0x00000000041C0D68>
                
                size2arena = arena2upu.size                                                                         # 
                              
                message2print = "{(Step 17B2/70) got" + "; size2arena-got-by-create2buffer4upu: " + str(size2arena)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)
                # 640000 
                
                buffer2arena = arena2upu.buffer
                
                message2print = "{(Step 17B3/70) got" + "; buffer2arena-got-by-create2buffer4upu: " + str(buffer2arena)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)                
                # <mmap.mmap object at 0x0000000003AAF298> 
                
                code2ack  = code2ack + '17B-' 
                
                if size2arena < size2upu:
                    size2upu = size2arena                                                                           # AVAILABLE MEMORY FOR IMAGE
                    message2print = "{(Step 17C/70) set" + "; size2upu-set-by-create2buffer4upu to: " + str(size2upu)  + "; time: " + time4xi() + "}"
                    print4xi(class2print, method2print, message2print)
                    # 
                    
                    code2ack  = code2ack + '17C-'
                else:
                    pass
                    
                pointer2upu = buffer2arena
                message2print = "{(Step 17D/70) set" + "; pointer2upu-set-by-create2buffer4upu to: " + str(pointer2upu)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)
                # <mmap.mmap object at 0x0000000003AAF298>
                
                code2ack  = code2ack + '17D-'       
            except:
                message2print = "{(Step 17Z/70) failed" + "; pointer2upu-failed-by-create2image4upu: " + str(pointer2upu) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)             
                # 
                
                code2nack  = code2nack + '17Z-'
        
            task4imagecreate2upu ['POINTER'] = pointer2upu                            
            task4imagecreate2upu ['SIZE']    = size2upu                                                             # EXTRA
            
            
            # ---------------------------------------------------------------------------------------------
            # (18) FORMAT - REQUIRED                              
            # ---------------------------------------------------------------------------------------------             
            format2upu = {}
           
            message2print = "{(Step 18A/70) trying" + "; format2upu-trying-by-create2image4upu: " + str(format2upu)  + "; time: " + time4xi() + "}" 
            print4xi(class2print, method2print, message2print) 
                        
            try:
                format2upu['ORDER'] = 'CL_RGBA'                                                              
                message2print = "{(Step 18B/70) set" + "; format2upu-set-by-create2image4upu to: " + str(format2upu)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print) 
                # {'ORDER': 'CL_RGBA'}
                 
                code2ack  = code2ack + '18B-'
                
                format2upu['TYPE'] = 'CL_UNORM_INT8'                                                              
                message2print = "{(Step 18C/70) set" + "; format2upu-set-by-create2image4upu to: " + str(format2upu)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print) 
                # {'TYPE': 'CL_UNORM_INT8', 'ORDER': 'CL_RGBA'}
                
                code2ack  = code2ack + '18C-'                
                 
            except:
                message2print = "{(Step 18Z/70) failed" + "; format2upu-failed-by-create2image4upu: " + str(format2upu) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)             
                code2nack  = code2nack + '18Z-'
        
            task4imagecreate2upu ['FORMAT'] = format2upu                  
            
            # ---------------------------------------------------------------------------------------------
            # (19) DEPTH - MUST BE 0                               
            # ---------------------------------------------------------------------------------------------             
            depth2upu = 0
            message2print = "{(Step 19A/70) set" + "; depth2upu-set-by-create2image4upu to: " + str(depth2upu)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print) 
            #  
            code2ack  = code2ack + '19A-' 
            task4imagecreate2upu ['DEPTH'] = depth2upu
                                                                                                                                       
        else:
            pass                                                                                                    # PARAMETER DATA RECEIVED                       
    except: 
        message2print = "{(Step 1Z1/70) yielded" + "; task4imagecreate2upu-yielded-by-create2image4upu: " + str(task4imagecreate2upu) + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)  
        # 
        
        code2nack  = code2nack + '1Z1-' 

    # ---------------------------------------------------------------------------------------------
    # TASK EITHER GENERATED BY THIS PROGRAM (NONE GIVEN) OR GIVEN BY USER    
    # ---------------------------------------------------------------------------------------------
    keys2task = task4imagecreate2upu.keys()        
    message2print = "{(Step 19A/70) using" + "; keys2task-using-by-create2image4upu: " + str(sorted(keys2task))  + "; time: " + time4xi() + "}"
    print4xi(class2print, method2print, message2print)    
    # ['BUFFER',
    #  'CODE2ERROR',
    #  'COMMANDQUEUE',
    #  'CONTEXT',
    #  'DATA2CALLBACK',
    #  'DEVICE',
    #  'EVENT',
    #  'FLAGS',
    #  'FORMAT',
    #  'FUNCTION2CALLBACK',
    #  'HEIGHT',
    #  'PIPE',
    #  'PLATFORM',
    #  'POINTER',
    #  'QUEUE',
    #  'ROWPITCH',
    #  'SIZE',
    #  'WIDTH']; time: 00:53:55 10/05/11 Pacific Daylight Time}
  
    
    # ---------------------------------------------------------------------------------------------
    # (Step 20A/70) - CREATE 2D IMAGE FOR UPU! 
    # ---------------------------------------------------------------------------------------------    
    upu2image4xi = {}                                                                                                                                                                               
    message2print = "{(Step 20A/70) initialized" + "; upu2image4xi-initialized-by-create2image4upu to: " + str(upu2image4xi)  + "; time: " + time4xi() + "}"
    print4xi(class2print, method2print, message2print)
    # {} 
    
    message2print = "{(Step 21A0/70) try" + "; data2callback-try-by-create2image4upu: {}" + "; time: " + time4xi() + "}" 
    print4xi(class2print, method2print, message2print)      
    # 
    try:
        # -----------------------------------------------------------------------------------------
        # (Step 21A/70) - DATA2CALLBACK - EXTRA 
        # -----------------------------------------------------------------------------------------               
        upu2image4xi['DATA2CALLBACK'] = task4imagecreate2upu['DATA2CALLBACK']
        
        message2print = "{(Step 21A1/70) initialized" + "; data2callback-created-by-create2image4upu to: " + str(upu2image4xi['DATA2CALLBACK'])  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)
        # {'MESSAGE': '{(Step 1Z1/33) failed; create2image4upu-failed ; time: 05:56:13 09/06/11 Pacific Daylight Time}',
        #  'CLASS': 'create2image4upu',
        #  'METHOD': 'create2image4upu'}; time: 05:56:13 09/06/11 Pacific Daylight Time}

        
        code2ack  = code2ack + '21A1-'
    except: 
        message2print = "{(Step 21Z1/70) yielded" + "; data2callback-upu2image4xi-yielded-by-create2image4upu: " + str(upu2image4xi) + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)  
        code2nack  = code2nack + '21Z1-'
    
    message2print = "{(Step 22B0/70) try" + "; function2callback-try-by-create2image4upu: None" + "; time: " + time4xi() + "}"
    print4xi(class2print, method2print, message2print)        
    # 
    try:
        # -----------------------------------------------------------------------------------------
        # (Step 22B/70) - FUNCTION2CALLBACK - EXTRA
        # -----------------------------------------------------------------------------------------               
        upu2image4xi['FUNCTION2CALLBACK'] = task4imagecreate2upu['FUNCTION2CALLBACK']
        
        message2print = "{(Step 22B1/70) initialized" + "; function2callback-created-by-create2image4upu to: " + str(upu2image4xi['FUNCTION2CALLBACK'])  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)
        # <function trace2image4upu at 0x0000000003B04348>  
        code2ack  = code2ack + '22B1-'
    except: 
        message2print = "{(Step 22Z2/70) yielded" + "; function2callback-upu2image4xi-yielded-by-create2image4upu: " + str(upu2image4xi) + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)  
        code2nack  = code2nack + '22Z2-' 
    
    message2print = "{(Step 22C0/70) try" + "; platform-try-by-create2image4upu: {}" + "; time: " + time4xi() + "}"
    print4xi(class2print, method2print, message2print)        
    # 
    try:
        # -----------------------------------------------------------------------------------------
        # (Step 22C/70) - PLATFORM - EXTRA
        # -----------------------------------------------------------------------------------------               
        upu2image4xi['PLATFORM'] = task4imagecreate2upu['PLATFORM']
        
        message2print = "{(Step 22C1/70) initialized" + "; platform-created-by-create2image4upu to: " + str(upu2image4xi['PLATFORM'])  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)
        # {'list2platforms': ['DragonXi'], 'ack2return': ' 2C-2E-2F-2G-', 'nack2return': ' ', 'available2platform': 1, 'max2entries': 1}; time: 05:56:13 09/06/11 Pacific Daylight Time}
  
        code2ack  = code2ack + '22C1-'
    except: 
        message2print = "{(Step 22Z3/70) yielded" + "; platform-upu2image4xi-yielded-by-create2image4upu: " + str(upu2image4xi) + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)  
        code2nack  = code2nack + '22Z3-' 
        
        
      
    message2print = "{(Step 23A/70) try" + "; context2upu-try-by-create2image4upu: {}" + "; time: " + time4xi() + "}"
    print4xi(class2print, method2print, message2print)       
    try:
        # -----------------------------------------------------------------------------------------
        # (Step 23/70) - CONTEXT - REQUIRED
        # -----------------------------------------------------------------------------------------               
        upu2image4xi['CONTEXT'] = task4imagecreate2upu['CONTEXT']
        
        message2print = "{(Step 23B/70) initialized" + "; context-created-by-create2image4upu to: " + str(upu2image4xi['CONTEXT'])  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)
        # {'COUNT2REFERENCE': 0,
        #  'DATA2CALLBACK': {'MESSAGE': '{(Step 1Z1/33) failed; create2context4upu-failed ; time: 05:56:12 09/06/11 Pacific Daylight Time}', 
        #  'CLASS': 'create2context4upu',
        #  'METHOD': 'create2context4upu'},
        #  'FUNCTION2CALLBACK': <function trace2context4upu at 0x00000000036B0148>,
        #  'ACK2RETURN': ' 2A-2C-2D-2E-2B-3B-4B-5B-6B-7B-7B-',
        #  'INFO2PRIVATE': None,
        #  'PLATFORM': {'list2platforms': ['DragonXi'], 'ack2return': ' 2C-2E-2F-2G-', 'nack2return': ' ', 'available2platform': 1, 'max2entries': 1}, 
        #  'NACK2RETURN': ' ',
        #  'SIZE2PRIVATE': 0,
        #  'PROPERTIES': None}; time: 05:56:13 09/06/11 Pacific Daylight Time}

        
        code2ack  = code2ack + '23B-'
    except: 
        message2print = "{(Step 23Z1/70) yielded" + "; context-upu2image4xi-yielded-by-create2image4upu: " + str(upu2image4xi) + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)  
    
        code2nack  = code2nack + '23Z1-' 
        
    message2print = "{(Step 24A/70) try" + "; device-try-by-create2image4upu: {}" + "; time: " + time4xi() + "}"
    print4xi(class2print, method2print, message2print)    
    try:
        # -----------------------------------------------------------------------------------------
        # (Step 24/70) - DEVICE - EXTRA
        # -----------------------------------------------------------------------------------------               
        upu2image4xi['DEVICE'] = task4imagecreate2upu['DEVICE']
        
        message2print = "{(Step 24B/70) initialized" + "; device-created-by-create2image4upu to: " + str(upu2image4xi['DEVICE'])  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)
        # {'RESOLUTION2TIMER4PROFILING': 318,
        #  'ID2PLATFORM': '0000000001509568',
        #  'PROFILING2QUEUE': 'ACK',
        #  'EXECUTE2KERNELS4OPENCL': 'ACK',
        #  'NATIVE2WIDTHLONG4VECTOR': 2,
        #  'MAX2UNITS4COMPUTE': 4,
        #  'SIZE2MEMORY4GLOBAL': 3221225472,
        #  'NATIVE2WIDTHFLOAT4VECTOR': 4,
        #  'MAX2SIZE4WORKGROUP': 1024,
        #  'PREFERRED2WIDTHCHAR4VECTOR': 16,
        #  'SIZE2CACHE': 65536,
        #  'IEEE754-2008FUSED2MULTIPLY-ADD': 'NACK',
        #  'TYPE4CACHE': 'Read/Write',
        #  'PREFERRED2WIDTHINT4VECTOR': 4,
        #  'SIZE2PREFERREDWORKGROUPMULTIPLE4KERNEL': 1,
        #  'NATIVE2WIDTHINT4VECTOR': 4,
        #  'SIZE2BUFFER4CONSTANT': 65536,
        #  'EXECUTE2FUNCTION4NATIVE': 'ACK',
        #  'PREFERRED2WIDTHDOUBLE4VECTOR': 0,
        #  'MAX2NUMBER4CONSTANTARGS': 8,
        #  'VERSION2DRIVER': '2.0', 'EXTENSIONS': 'cl_amd_fp64 ...
        
        
        code2ack  = code2ack + '24B-'
    except: 
        message2print = "{(Step 24Z1/70) yielded" + "; device-upu2image4xi-yielded-by-create2image4upu: " + str(upu2image4xi) + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)  
        code2nack  = code2nack + '24Z1-'
                 
    message2print = "{(Step 25A/70) try" + "; code2error-try-by-create2image4upu: None" + "; time: " + time4xi() + "}"
    print4xi(class2print, method2print, message2print) 
    try:
        # -----------------------------------------------------------------------------------------
        # (Step 25/70) - ERROR CODE - REQUIRED
        # -----------------------------------------------------------------------------------------               
        upu2image4xi['CODE2ERROR'] = task4imagecreate2upu['CODE2ERROR']
        
        message2print = "{(Step 25B/70) initialized" + "; code2error-created-by-create2image4upu to: " + str(upu2image4xi['CODE2ERROR'])  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)
        # 0 
        code2ack  = code2ack + '25B-'
    except: 
        message2print = "{(Step 25Z1/70) yielded" + "; code2error-upu2image4xi-yielded-by-create2image4upu: " + str(upu2image4xi) + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)  
        code2nack  = code2nack + '25Z1-'                
                
          
    message2print = "{(Step 26A/70) try" + "; queue-try-by-create2image4upu: None" + "; time: " + time4xi() + "}"
    print4xi(class2print, method2print, message2print)                
    try:
        # -----------------------------------------------------------------------------------------
        # (Step 26/70) - QUEUE - EXTRA
        # -----------------------------------------------------------------------------------------               
        upu2image4xi['QUEUE'] = task4imagecreate2upu['QUEUE']
        
        message2print = "{(Step 26B/70) initialized" + "; queue-created-by-create2image4upu to: " + str(upu2image4xi['QUEUE'])  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print) 
        # <queue4xi.Queue4Xi object at 0x0000000003ED99E8>  
        code2ack  = code2ack + '26B-'
    except: 
        message2print = "{(Step 26Z1/70) yielded" + "; queue-upu2image4xi-yielded-by-create2image4upu: " + str(upu2image4xi) + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)  
        code2nack  = code2nack + '26Z1-' 
                                
    message2print = "{(Step 27A/70) try" + "; pipe-try-by-create2image4upu: None" + "; time: " + time4xi() + "}"
    print4xi(class2print, method2print, message2print)                     
    try:
        # -----------------------------------------------------------------------------------------
        # (Step 27/70) - PIPE - EXTRA
        # -----------------------------------------------------------------------------------------               
        upu2image4xi['PIPE'] = task4imagecreate2upu['PIPE']
        
        message2print = "{(Step 27B/70) initialized" + "; pipe-created-by-create2image4upu to: " + str(upu2image4xi['PIPE'])  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print) 
        # (<read-write PipeConnection, handle 376>, <read-write PipeConnection, handle 380>)  
        
        code2ack  = code2ack + '27B-'
    except: 
        message2print = "{(Step 27Z1/70) yielded" + "; pipe-upu2image4xi-yielded-by-create2image4upu: " + str(upu2image4xi) + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)  
        code2nack  = code2nack + '27Z1-'   

    message2print = "{(Step 28A/70) try" + "; commandqueue-try-by-create2image4upu: {}" + "; time: " + time4xi() + "}"
    print4xi(class2print, method2print, message2print)        
    try:            
        # ---------------------------------------------------------------------------------------------
        # (28) COMMAND QUEUE - EXTRA                              
        # ---------------------------------------------------------------------------------------------             
                        
        upu2image4xi['COMMANDQUEUE']  = task4imagecreate2upu['COMMANDQUEUE']  
 
        message2print = "{(Step 28B/70) initialized" + "; commandqueue-created-by-create2image4upu to: " + str(upu2image4xi['COMMANDQUEUE'])  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print) 
        # {'CODE2ERROR': 0,
        #  'COUNT2REFERENCE': 0,
        #  'DATA2CALLBACK': {'MESSAGE': '{(Step 1Z1/33) failed; create2commandqueue4upu-failed ; time: 07:13:59 09/06/11 Pacific Daylight Time}',
        #  'CLASS': 'create2commandqueue4upu',
        #  'METHOD': 'create2commandqueue4upu'},
        #  'FUNCTION2CALLBACK': <function trace2commandqueue4upu at 0x00000000037BC848>,
        #  'ACK2RETURN': ' 1D-1E-2C-2B-3B-4B-8A-9B-10B-9B-10B-11B-12B-13B-14B-15B-16B-17B-18B-',
        #  'INFO2PRIVATE': None,
        #  'QUEUE': <queue4xi.Queue4Xi object at 0x000000000432B3C8>,
        #  'NACK2RETURN': ' ',
        #  'CONTEXT': {'COUNT2REFERENCE': 0, 'DATA2CALLBACK': {'MESSAGE': ...
        
        code2ack  = code2ack + '28B-'
    except: 
        message2print = "{(Step 28Z1/70) yielded" + "; commandqueue-yielded-by-create2image4upu: " + str(upu2image4xi) + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)  
        # 
              
        code2nack  = code2nack + '28Z1-'    

    message2print = "{(Step 29A/70) try" + "; bufferinfo-try-by-create2image4upu: {}" + "; time: " + time4xi() + "}"
    print4xi(class2print, method2print, message2print)        
    try:            
        # ---------------------------------------------------------------------------------------------
        # (29) BUFFER INFO - EXTRA                              
        # ---------------------------------------------------------------------------------------------             
                        
        upu2image4xi['BUFFER']  = task4imagecreate2upu['BUFFER']  
 
        message2print = "{(Step 29B/70) initialized" + "; bufferinfo-created-by-create2image4upu to: " + str(upu2image4xi['BUFFER'])  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print) 
        # {'CODE2ERROR': 0,
        #  'COUNT2REFERENCE': 0,
        #  'DATA2CALLBACK': {'MESSAGE': '{(Step 1Z1/33) failed; create2buffer4upu-failed ; time: 05:56:12 09/06/11 Pacific Daylight Time}', 
        #  'CLASS': 'create2buffer4upu',
        #  'METHOD': 'create2buffer4upu'},
        #  'FUNCTION2CALLBACK': <function trace2buffer4upu at 0x0000000003AFC348>,
        #  'POINTER2BUFFER': <mmap.mmap object at 0x0000000003AAF2F0>,
        #  'FLAGS': 0,
        #  'CONTEXT': {'COUNT2REFERENCE': 0, 'DATA2CALLBACK': {'MESSAGE': '{(Step 1Z1/33) failed; create2context4upu-failed ...
        
        code2ack  = code2ack + '29B-'
    except: 
        message2print = "{(Step 29Z1/70) yielded" + "; bufferinfo-yielded-by-create2image4upu: " + str(upu2image4xi) + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)  
        code2nack  = code2nack + '29Z1-'            

    message2print = "{(Step 30A/70) try" + "; eventinfo-try-by-create2image4upu: {}" + "; time: " + time4xi() + "}"
    print4xi(class2print, method2print, message2print)        
    try:            
        # ---------------------------------------------------------------------------------------------
        # (30) EVENT INFO - EXTRA                              
        # ---------------------------------------------------------------------------------------------             
                        
        upu2image4xi['EVENT']  = task4imagecreate2upu['EVENT']  
 
        message2print = "{(Step 30B/70) initialized" + "; eventinfo-created-by-create2image4upu to: " + str(upu2image4xi['EVENT'])  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print) 
        # {'CODE2ERROR': 0,
        #  'DATA2CALLBACK': {'MESSAGE': '{(Step 1Z1/33) failed; create2event4upu-failed ; time: 05:56:12 09/06/11 Pacific Daylight Time}',
        #                     'CLASS': 'create2event4upu',
        #                     'METHOD': 'create2event4upu'},
        #  'FUNCTION2CALLBACK': <function trace2event4upu at 0x0000000003707D48>,
        #  'ACK2RETURN': ' 1D-1E-2C-2B-3B-4B-6B-7B-8B-9B-10B-11B-12B-14B-15B-16B-',
        #  'QUEUE': <queue4xi.Queue4Xi object at 0x00000000039C3860>,
        #  'NACK2RETURN': ' ',
        #  'CONTEXT': {'COUNT2REFERENCE': 0, 'DATA2CALLBACK': {'MESSAGE': ...
        
        code2ack  = code2ack + '30B-'
    except: 
        message2print = "{(Step 30Z1/70) yielded" + "; eventinfo-yielded-by-create2image4upu: " + str(upu2image4xi) + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)  
        code2nack  = code2nack + '30Z1-'            
                

    message2print = "{(Step 31A/70) try" + "; flags-try-by-create2image4upu: None" + "; time: " + time4xi() + "}"
    print4xi(class2print, method2print, message2print)        
    try:            
        # ---------------------------------------------------------------------------------------------
        # (31) FLAGS - REQUIRED                              
        # ---------------------------------------------------------------------------------------------             
                        
        upu2image4xi['FLAGS']  = task4imagecreate2upu['FLAGS']  
 
        message2print = "{(Step 31B/70) initialized" + "; flags-created-by-create2image4upu to: " + str(upu2image4xi['FLAGS'])  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print) 
        # 0
        code2ack  = code2ack + '31B-'
    except: 
        message2print = "{(Step 31Z1/70) yielded" + "; flags-yielded-by-create2image4upu: " + str(upu2image4xi) + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)  
        code2nack  = code2nack + '31Z1-'     

    message2print = "{(Step 34A/70) try" + "; width-try-by-create2image4upu: None" + "; time: " + time4xi() + "}"
    print4xi(class2print, method2print, message2print)        
    try:            
        # ---------------------------------------------------------------------------------------------
        # (34) WIDTH - REQUIRED                              
        # ---------------------------------------------------------------------------------------------      
        upu2image4xi['WIDTH']  = task4imagecreate2upu['WIDTH']  
 
        message2print = "{(Step 34H/70) initialized" + "; width-created-by-create2image4upu to: " + str(upu2image4xi['WIDTH'])  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print) 
        # 800 
        code2ack  = code2ack + '34H-'
    except: 
        message2print = "{(Step 34Z1/70) yielded" + "; width-yielded-by-create2image4upu: " + str(upu2image4xi) + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)  
        code2nack  = code2nack + '34Z1-'   


    message2print = "{(Step 35A/70) try" + "; height-try-by-create2image4upu: None" + "; time: " + time4xi() + "}"
    print4xi(class2print, method2print, message2print)        
    try:            
        # ---------------------------------------------------------------------------------------------
        # (35) HEIGHT - REQUIRED                              
        # ---------------------------------------------------------------------------------------------      
        upu2image4xi['HEIGHT']  = task4imagecreate2upu['HEIGHT']  
 
        message2print = "{(Step 35H/70) initialized" + "; height-created-by-create2image4upu to: " + str(upu2image4xi['HEIGHT'])  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print) 
        # 800 
        code2ack  = code2ack + '35H-'
    except: 
        message2print = "{(Step 35Z1/70) yielded" + "; height-yielded-by-create2image4upu: " + str(upu2image4xi) + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)  
        code2nack  = code2nack + '35Z1-'   
 

    message2print = "{(Step 36A/70) try" + "; rowpitch-try-by-create2image4upu: None" + "; time: " + time4xi() + "}"
    print4xi(class2print, method2print, message2print)        
    try:            
        # ---------------------------------------------------------------------------------------------
        # (36) ROWPITCH - REQUIRED                              
        # ---------------------------------------------------------------------------------------------             
                        
        upu2image4xi['ROWPITCH']  = task4imagecreate2upu['ROWPITCH']  
 
        message2print = "{(Step 36B/70) initialized" + "; rowpitch-created-by-create2image4upu to: " + str(upu2image4xi['ROWPITCH'])  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print) 
        # 800
        code2ack  = code2ack + '36B-'
    except: 
        message2print = "{(Step 36Z1/70) yielded" + "; rowpitch-yielded-by-create2image4upu: " + str(upu2image4xi) + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)  
        code2nack  = code2nack + '36Z1-'

    message2print = "{(Step 37A/70) try" + "; pointer-try-by-create2image4upu: None"  + "; time: " + time4xi() + "}"
    print4xi(class2print, method2print, message2print)        
    try:            
        # ---------------------------------------------------------------------------------------------
        # (37) POINTER - REQUIRED                              
        # ---------------------------------------------------------------------------------------------             
                        
        upu2image4xi['POINTER']  = task4imagecreate2upu['POINTER']  
 
        message2print = "{(Step 37B/70) initialized" + "; pointer-created-by-create2image4upu to: " + str(upu2image4xi['POINTER'])  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print) 
        # <mmap.mmap object at 0x0000000003C050E0>
         
        code2ack  = code2ack + '37B-'
    except: 
        message2print = "{(Step 37Z1/70) yielded" + "; pointer-yielded-by-create2image4upu: " + str(upu2image4xi) + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)  
        code2nack  = code2nack + '37Z1-'

    message2print = "{(Step 38A/70) try" + "; format-try-by-create2image4upu: None" + "; time: " + time4xi() + "}"
    print4xi(class2print, method2print, message2print)        
    try:            
        # ---------------------------------------------------------------------------------------------
        # (38) FORMAT - REQUIRED                              
        # ---------------------------------------------------------------------------------------------             
                        
        upu2image4xi['FORMAT']  = task4imagecreate2upu['FORMAT']  
 
        message2print = "{(Step 38B/70) initialized" + "; format-created-by-create2image4upu to: " + str(upu2image4xi['FORMAT'])  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print) 
        # {'TYPE': 'CL_UNORM_INT8', 'ORDER': 'CL_RGBA'} 
        code2ack  = code2ack + '38B-'
    except: 
        message2print = "{(Step 38Z1/70) yielded" + "; format-yielded-by-create2image4upu: " + str(upu2image4xi) + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)  
        code2nack  = code2nack + '38Z1-'

    message2print = "{(Step 39A/70) try" + "; depth-try-by-create2image4upu: None" + "; time: " + time4xi() + "}"
    print4xi(class2print, method2print, message2print)        
    try:            
        # ---------------------------------------------------------------------------------------------
        # (39) DEPTH - MUST BE 0                              
        # ---------------------------------------------------------------------------------------------             
                        
        upu2image4xi['DEPTH']  = task4imagecreate2upu['DEPTH']  
 
        message2print = "{(Step 39B/70) initialized" + "; depth-created-by-create2image4upu to: " + str(upu2image4xi['DEPTH'])  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print) 
        #  
        code2ack  = code2ack + '39B-'
    except: 
        message2print = "{(Step 39Z1/70) yielded" + "; depth-yielded-by-create2image4upu: " + str(upu2image4xi) + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)  
        code2nack  = code2nack + '39Z1-'
                                                                                                               
    # -----------------------------------------------------------------------------------------
    # (Step 70/70)           
    # -----------------------------------------------------------------------------------------
    upu2image4xi ['ACK2RETURN']  = code2ack
    upu2image4xi ['NACK2RETURN'] = code2nack  
    
    message2print = "{(Step 70/70) returning" + "; upu2image4xi-returning-by-create2image4upu: " + str(upu2image4xi) + "; time: " + time4xi() + "}"  
    print4xi(class2print, method2print, message2print)                                                              
    # 
    
    # 'DEPTH'
    # 'WIDTH'
    # 'HEIGHT'
    return upu2image4xi
            
#</function4xi> 


# -------------------------------------------------------------------------------------------------
#<function4xi function = "set2image4upu" usage = "
#    
#   print('(Step 4B/33) OF UPU2IMAGE4XI5TEST - TRYING TO GET PLATFORMIDS4XI', time4xi())
#   platform2xi = {}
#   try:
#       platform2xi = get2platformids4xi()
#       print('(Step 4B/33) of CONTEXT4UPU5TEST - GOT PLATFORMIDS4XI', platform2xi) 
#   except:
#       print('(Step 4Z/33) of CONTEXT4UPU5TEST *** FAILED TO GET PLATFORMIDS4XI', time4xi())     
#   
#   print('(Step 2A/33) OF DEVICE4UPU5TEST - TRYING TO INITIALIZE DEVICE4UPU ', time4xi())   
#   print('=' *33)        
#   try:
#       device2upu = init2upu4xi()                                                                                
#       print('(Step 2B/33) OF DEVICE4UPU5TEST - INITIALIZED DEVICE4UPU TO: ', device2upu)        
#   except:
#       print('(Step 2Z/33) OF DEVICE4UPU5TEST *** FAILED TO INITIALIZE DEVICE4UPU', time4xi()) 
#   print('(Step 3A/33) OF UPU2IMAGE4XI5TEST - TRYING TO CREATE CONTEXT4UPU ', time4xi())    
#   print('=' *33)        
#   try:
#       context2upu = create2context4upu()                                                                               
#       print('(Step 3B/33) OF UPU2IMAGE4XI5TEST - CREATED CONTEXT4UPU: ', context2upu)                                  
#   except:
#       print('(Step 3Z/33) OF UPU2IMAGE4XI5TEST *** FAILED TO CREATE CONTEXT4UPU', time4xi()) 
#          
#   task4set2image = {}
#   task4set2image  ['CONTEXT'] = context2upu  
# 
#   ... other data for set operation ...
#    
#        
#   response4set2Dimage = {} 
#   print('(Step 11A/700) OF UPU2IMAGE4XI5TEST - TRYING TO SET 2D IMAGE4UPU ', time4xi())   
#   try:                               
#       response4set2Dimage = set2image4upu(task4set2image)                                                                                                         
#                                                                                     
#       print('(Step 11B/700) OF UPU2IMAGE4XI5TEST - SET 2D IMAGE4UPU ', time4xi())
#       keys2response = response4set2Dimage.keys()
#       for key2response in sorted(keys2response):
#           print(key2response, ' : ', response4set2Dimage[key2response])
#           # 
#   except:
#       print('(Step 11Z/700) OF UPU2IMAGE4XI5TEST *** FAILED TO SET 2D IMAGE4UPU *** ', time4xi()) 
#
#">

# -------------------------------------------------------------------------------------------------   
def set2image4upu(task4imageset2upu=None):

    '''
    sets 2D images for UPU
    '''
    global upu2image4xi  
    
    # -----------------------------------------------------------------------------------------
    # (Step 1/33) PARAMETERS           
    # -----------------------------------------------------------------------------------------          
    class2print = "set2image4upu"
    method2print = "set2image4upu"

    message2print = "{(Step 1A/33) started" + "; set2image4upu-started: " + str(set2image4upu)  + "; time: " + time4xi() + "}" 
    print4xi(class2print, method2print, message2print)                                                                     #  
    # <function set2image4upu at 0x0000000003A62848> 
    
    message2print = "{(Step 1B/33) passed" + "; task4imageset2upu-to-set2image4upu: " + str(task4imageset2upu) + "; time: " + time4xi() + "}"
    print4xi(class2print, method2print, message2print)                                                              
    # {'CODE2ERROR': 0,
    #  'QUEUE': <queue4xi.Queue4Xi object at 0x0000000003D4A710>,
    #  'CONTEXT': {'COUNT2REFERENCE': 0,
    #              'DATA2CALLBACK': {'MESSAGE': '{(Step 1Z1/33) failed; create2context4upu-failed ; time: 07:13:59 09/06/11 Pacific Daylight Time}',
    #              'CLASS': 'create2context4upu',
    #              'METHOD': 'create2context4upu'},
    #              'FUNCTION2CALLBACK': <function trace2context4upu at 0x0000000003770148>,
    #              'ACK2RETURN': ' 2A-2C-2D-2E-2B-3B-4B-5B-6B-7B-7B-',
    #              'INFO2PRIVATE': None,
    #              'PLATFORM': {'list2platforms': ['DragonXi'], 'ack2return': ' 2C-2E-2F-2G-', 'nack2return': ' ', 'available2platform': 1, 'max2entries': 1}, 
    #              'NACK2RETURN': ' ',
    #              'SIZE2PRIVATE': 0,
    #              'PROPERTIES': None},
    #  'COMMANDQUEUE': {'CODE2ERROR': 0,
    #              'COUNT2REFERENCE': 0,
    #              'DATA2CALLBACK': {'MESSAGE': '{(Step 1Z1/33) failed; create2commandqueue4upu-failed ; time: 07:13:59 09/06/11 Pacific Daylight Time}',
    #              'CLASS': 'create2commandqueue4upu',
    #              'METHOD': 'create2commandqueue4upu'},
    #  'FUNCTION2CALLBACK': <function trace2commandqueue4upu at 0x00000000037BC848>,
    #  'ACK2RETURN': ' 1D-1E-2C-2B-3B-4B-8A-9B-10B-9B-10B-11B-12B-13B-14B-15B-16B-17B-18B-',
    #  'INFO2PRIVATE': None,
    #  'QUEUE': <queue4xi.Queue4Xi object at 0x0000000003EDA780>,
    #  'NACK2RETURN': ' ',
    #  'CONTEXT': {'COUNT2REFERENCE': 0,
    #              'DATA2CALLBACK': {'MESSAGE': '{(Step 1Z1/33) failed; create2context4upu-failed ; time: 07:13:59 09/06/11 Pacific Daylight Time}',
    #              'CLASS': 'create2context4upu',
    #              'METHOD': 'create2context4upu'},
    #              'FUNCTION2CALLBACK': <function trace2context4upu at 0x0000000003770148>,
    #              'ACK2RETURN': ' 2A-2C-2D-2E-2B-3B-4B-5B-6B-7B-7B-',
    #              'INFO2PRIVATE': None,
    #              'PLATFORM': {'list2platforms': ['DragonXi'],
    #                           'ack2return': ' 2C-2E-2F-2G-',
    #                           'nack2return': ' ', 'available2platform': 1, 'max2entries': 1},
    #              'NACK2RETURN': ' ',
    #              'SIZE2PRIVATE': 0,
    #              'PROPERTIES': None},
    #              'DEVICE': {'RESOLUTION2TIMER4PROFILING': 318,
    #                          'ID2PLATFORM': '0000000001509568',
    #  'PROPERTIES': None,
    #  'PIPE': (<read-write PipeConnection, handle 552>, <read-write PipeConnection, handle 556>)},
    #  'DEVICE': {'RESOLUTION2TIMER4PROFILING': 318, 'ID2PLATFORM': '0000000001509568', 'PROFILING2QUEUE': 'ACK', 'EXECUTE2KERNELS4OPENCL': 'ACK',
    #              'NATIVE2WIDTHLONG4VECTOR': 2, 'MAX2UNITS4COMPUTE': 4, 'SIZE2MEMORY4GLOBAL': 3221225472, 'NATIVE2WIDTHFLOAT4VECTOR': 4, 'MAX2SIZE4WORKGROUP': 1024, 'PREFERRED2WIDTHCHAR4VECTOR': 16, 'SIZE2CACHE': 65536, 
    #              'IEEE754-2008FUSED2MULTIPLY-ADD': 'NACK', 'TYPE4CACHE': 'Read/Write', 'PREFERRED2WIDTHINT4VECTOR': 4, 'SIZE2PREFERREDWORKGROUPMULTIPLE4KERNEL': 1, 'NATIVE2WIDTHINT4VECTOR': 4, 
    #              'SIZE2BUFFER4CONSTANT': 65536, 'EXECUTE2FUNCTION4NATIVE': 'ACK', 'PREFERRED2WIDTHDOUBLE4VECTOR': 0, 'MAX2NUMBER4CONSTANTARGS': 8, 'VERSION2DRIVER': '2.0', 
            
    code2ack  = ' '
    code2nack = ' '
    # -----------------------------------------------------------------------------------------
    # CREATE IF NECESSARY       
    # ----------------------------------------------------------------------------------------- 
    try:
        if upu2image4xi is None:
            upu2image4xi = create2image4upu()                                                                              # INITIALIZE IMAGE FOR UPU 
            message2print = "{(Step 1C/70) set" + "; upu2image4xi-set-by-set2image4upu to: " + str(upu2image4xi)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)
            #             
        else:
            message2print = "{(Step 1D/70) skipped" + "; create2image4upu-skipped-by-set2image4upu: " + str(create2image4upu) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)
            # <function create2image4upu at 0x0000000003A627C8>;                                  
    except:
        try:
            upu2image4xi = create2image4upu()
            message2print = "{(Step 1E/70) set" + "; upu2image4xi-set-by-set2image4upu to: " + str(upu2image4xi)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)
            #             
        except:
            upu2image4xi = {}
            message2print = "{(Step 1Z/70) yielded" + "; upu2image4xi-yielded-by-set2image4upu: " + str(upu2image4xi) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)  
            code2nack  = code2nack + '1Z-' 
                
    # ---------------------------------------------------------------------------------------------
    # SET            
    # ---------------------------------------------------------------------------------------------     
    message2print = "{(Step 22A0/70) try" + "; data2callback-try-by-set2image4upu: None" + "; time: " + time4xi() + "}"
    print4xi(class2print, method2print, message2print)        
    #     
    try:
        # -----------------------------------------------------------------------------------------
        # (Step 21A/70) - DATA2CALLBACK - EXTRA
        # -----------------------------------------------------------------------------------------               
        upu2image4xi['DATA2CALLBACK'] = task4imageset2upu['DATA2CALLBACK']
                                        
        message2print = "{(Step 21A1/70) set" + "; data2callback-set-by-set2image4upu to: " + str(upu2image4xi['DATA2CALLBACK'])  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)
        # 
        
        code2ack  = code2ack + '21A1-'
    except: 
        message2print = "{(Step 21Z1/70) yielded" + "; data2callback-upu2image4xi-yielded-by-set2image4upu: " + str(upu2image4xi) + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)  
        # yielded
        code2nack  = code2nack + '21Z1-'
    
    message2print = "{(Step 22B0/70) try" + "; function2callback-try-by-set2image4upu: None" + "; time: " + time4xi() + "}"
    print4xi(class2print, method2print, message2print)        
    # 
    try:
        # -----------------------------------------------------------------------------------------
        # (Step 22B/70) - FUNCTION2CALLBACK - EXTRA
        # -----------------------------------------------------------------------------------------               
        upu2image4xi['FUNCTION2CALLBACK'] = task4imageset2upu['FUNCTION2CALLBACK']
        
        message2print = "{(Step 22B1/70) set" + "; function2callback-set-by-set2image4upu to: " + str(upu2image4xi['FUNCTION2CALLBACK'])  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)
        #  
        code2ack  = code2ack + '22B1-'
    except: 
        message2print = "{(Step 22Z2/70) yielded" + "; function2callback-upu2image4xi-yielded-by-set2image4upu: " + str(upu2image4xi) + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)  
        # yielded
        code2nack  = code2nack + '22Z2-' 
    
    message2print = "{(Step 22C0/70) try" + "; platform2callback-try-by-set2image4upu: None" + "; time: " + time4xi() + "}"
    print4xi(class2print, method2print, message2print)        
    # 
    try:
        # -----------------------------------------------------------------------------------------
        # (Step 22C/70) - PLATFORM - EXTRA
        # -----------------------------------------------------------------------------------------               
        upu2image4xi['PLATFORM'] = task4imageset2upu['PLATFORM']
        
        message2print = "{(Step 2C1/70) set" + "; platform-set-by-set2image4upu to: " + str(upu2image4xi['PLATFORM'])  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)
        # yielded  
        code2ack  = code2ack + '2C1-'
    except: 
        message2print = "{(Step 22Z3/70) yielded" + "; platform-upu2image4xi-yielded-by-set2image4upu: " + str(upu2image4xi) + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)  
        code2nack  = code2nack + '22Z3-' 
        
        
      
    message2print = "{(Step 23A/70) try" + "; context2upu-try-by-set2image4upu: {}" + "; time: " + time4xi() + "}"
    print4xi(class2print, method2print, message2print)       
    try:
        # -----------------------------------------------------------------------------------------
        # (Step 23/70) - CONTEXT - REQUIRED
        # -----------------------------------------------------------------------------------------               
        upu2image4xi['CONTEXT'] = task4imageset2upu['CONTEXT']
        
        message2print = "{(Step 23B/70) set" + "; context-set-by-set2image4upu to: " + str(upu2image4xi['CONTEXT'])  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)
        # {'COUNT2REFERENCE': 0,
        #  'DATA2CALLBACK': {'MESSAGE': '{(Step 1Z1/33) failed; create2context4upu-failed ; time: 07:13:59 09/06/11 Pacific Daylight Time}',
        #                    'CLASS': 'create2context4upu',
        #                    'METHOD': 'create2context4upu'},
        #  'FUNCTION2CALLBACK': <function trace2context4upu at 0x0000000003770148>,
        #  'ACK2RETURN': ' 2A-2C-2D-2E-2B-3B-4B-5B-6B-7B-7B-',
        #  'INFO2PRIVATE': None,
        #  'PLATFORM': {'list2platforms': ['DragonXi'], 'ack2return': ' 2C-2E-2F-2G-', 'nack2return': ' ', 'available2platform': 1, 'max2entries': 1}, 'NACK2RETURN': ' ',
        #  'SIZE2PRIVATE': 0,
        #  'PROPERTIES': None}; time: 07:13:59 09/06/11 Pacific Daylight Time} 

        
        code2ack  = code2ack + '23B-'
    except: 
        message2print = "{(Step 23Z1/70) yielded" + "; context-upu2image4xi-yielded-by-set2image4upu: " + str(upu2image4xi) + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)  
    
        code2nack  = code2nack + '23Z1-' 
        
    message2print = "{(Step 24A/70) try" + "; device-try-by-set2image4upu: {}" + "; time: " + time4xi() + "}"
    print4xi(class2print, method2print, message2print)    
    try:
        # -----------------------------------------------------------------------------------------
        # (Step 24/70) - DEVICE - EXTRA
        # -----------------------------------------------------------------------------------------               
        upu2image4xi['DEVICE'] = task4imageset2upu['DEVICE']
        
        message2print = "{(Step 24B/70) set" + "; device-set-by-set2image4upu to: " + str(upu2image4xi['DEVICE'])  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)
        # {'RESOLUTION2TIMER4PROFILING': 318,
        #  'ID2PLATFORM': '0000000001509568',
        #  'PROFILING2QUEUE': 'ACK',
        #  'EXECUTE2KERNELS4OPENCL': 'ACK',
        #  'NATIVE2WIDTHLONG4VECTOR': 2,
        #  'MAX2UNITS4COMPUTE': 4,
        #  'SIZE2MEMORY4GLOBAL': 3221225472,
        #  'NATIVE2WIDTHFLOAT4VECTOR': 4,
        #  'MAX2SIZE4WORKGROUP': 1024,
        #  'PREFERRED2WIDTHCHAR4VECTOR': 16,
        #  'SIZE2CACHE': 65536,
        #  'IEEE754-2008FUSED2MULTIPLY-ADD': 'NACK',
        #  'TYPE4CACHE': 'Read/Write',
        #  'PREFERRED2WIDTHINT4VECTOR': 4,
        #  'SIZE2PREFERREDWORKGROUPMULTIPLE4KERNEL': 1,
        #  'NATIVE2WIDTHINT4VECTOR': 4,
        #  'SIZE2BUFFER4CONSTANT': 65536,
        #  'EXECUTE2FUNCTION4NATIVE': 'ACK',
        #  'PREFERRED2WIDTHDOUBLE4VECTOR': 0,
        #  'MAX2NUMBER4CONSTANTARGS': 8,
        #  'VERSION2DRIVER': '2.0',
        #  'EXTENSIONS': 'cl_amd_fp64 cl_khr_global_int32_base_atomics ...
        #  'AVAILABLE2COMPILER': 'ACK',
        #  'MAX2DIMENSIONS4WORKITEMS': 3,
        #  'SUPPORT2CORRECTION4ERROR': 0,
        #  'BITS2ADDRESS': 64,
        #  'AVAILABLE': 'ACK',
        #  'SIZE2MEMORY4LOCAL': 32768,
        #  'VENDOR': 'AuthenticAMD',
        #  'SUPPORT2IMAGE': 'NACK',
        #  'MAX2SIZE4KERNELARGUMENT': 4096,
        #  'ACK2RETURN': ' 1C-1D-1E-1F-1D-2A-3A-4A-5A-6A-7A-8A-9A-10A-11A-...
        #  'OUT-OF-ORDER2QUEUE': 'NACK',
        #  'ALIGNMENT2ADDRESS4BASE': 1024,
        #  'PROFILE': 'FULL_PROFILE',
        #  'TYPE2MEMORY4LOCAL': 'Global',
        #  'PREFERRED2WIDTHFLOAT4VECTOR': 4,
        #  'PREFERRED2WIDTHLONG4VECTOR': 2,
        #  'MAX2FREQUENCY4CLOCK': 3210,
        #  'PREFERRED2WIDTHSHORT4VECTOR': 8,
        #  'ID': 4033,
        #  'MAX2WORKITEM2': 1024,
        #  'NATIVE2WIDTHDOUBLE4VECTOR': 0,
        #  'ROUND2ZERO': 'ACK',
        #  'NAME': 'AMD64 Family 16 Model 4 Stepping 2, AuthenticAMD',
        #  'MAX2WORKITEM1': 1024,
        #  'MAX2WORKITEM0': 1024,
        #  'SIZE2LINE4CACHE': 64,
        #  'MAX2ALLOCATION4MEMORY': 1073741824,
        #  'NATIVE2WIDTHCHAR4VECTOR': 16,
        #  'QUIET2NANS': 'ACK',
        #  'NATIVE2WIDTHSHORT4VECTOR': 8,
        #  'ROUND2NEAREST4EVEN': 'ACK',
        #  'VERSION': 'OpenCL 1.1 ATI-Stream-v2.3 (451)',
        #  'NACK2RETURN': ' ',
        #  'MEMORY2UNIFIED4HOSTANDDEVICE': 1,
        #  'ENDIANESS2DEVICE': 'Little',
        #  'MINIMUM2ALIGNMENT4DATATYPE': 128,
        #  'ROUND2+VE3INFINITY': 'ACK',
        #  'TYPE': 'CL_DEVICE_TYPE_UPU',
        #  'DENORMS2SINGLEPRECISION4FLOATINGPOINTCAPABILITY': 'ACK'}; time: 07:13:59 09/06/11 Pacific Daylight Time}

        
        code2ack  = code2ack + '24B-'
    except: 
        message2print = "{(Step 24Z1/70) yielded" + "; device-upu2image4xi-yielded-by-set2image4upu: " + str(upu2image4xi) + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)  
        code2nack  = code2nack + '24Z1-'
                 
    message2print = "{(Step 25A/70) try" + "; code2error-try-by-set2image4upu: None" + "; time: " + time4xi() + "}"
    print4xi(class2print, method2print, message2print) 
    try:
        # -----------------------------------------------------------------------------------------
        # (Step 25/70) - ERROR CODE - REQUIRED
        # -----------------------------------------------------------------------------------------               
        upu2image4xi['CODE2ERROR'] = task4imageset2upu['CODE2ERROR']
        
        message2print = "{(Step 25B/70) set" + "; code2error-set-by-set2image4upu to: " + str(upu2image4xi['CODE2ERROR'])  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)
        # 0 
        code2ack  = code2ack + '25B-'
    except: 
        message2print = "{(Step 25Z1/70) yielded" + "; code2error-upu2image4xi-yielded-by-set2image4upu: " + str(upu2image4xi) + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)  
        code2nack  = code2nack + '25Z1-'                
                
          
    message2print = "{(Step 26A/70) try" + "; queue-try-by-set2image4upu: None" + "; time: " + time4xi() + "}"
    print4xi(class2print, method2print, message2print)                
    try:
        # -----------------------------------------------------------------------------------------
        # (Step 26/70) - QUEUE - EXTRA
        # -----------------------------------------------------------------------------------------               
        upu2image4xi['QUEUE'] = task4imageset2upu['QUEUE']
        
        message2print = "{(Step 26B/70) set" + "; queue-set-by-set2image4upu to: " + str(upu2image4xi['QUEUE'])  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print) 
        # <queue4xi.Queue4Xi object at 0x0000000003D4A710> 
        code2ack  = code2ack + '26B-'
    except: 
        message2print = "{(Step 26Z1/70) yielded" + "; queue-upu2image4xi-yielded-by-set2image4upu: " + str(upu2image4xi) + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)  
        code2nack  = code2nack + '26Z1-' 
                                
    message2print = "{(Step 27A/70) try" + "; pipe-try-by-set2image4upu: None" + "; time: " + time4xi() + "}"
    print4xi(class2print, method2print, message2print)                     
    try:
        # -----------------------------------------------------------------------------------------
        # (Step 27/70) - PIPE - EXTRA
        # -----------------------------------------------------------------------------------------               
        upu2image4xi['PIPE'] = task4imageset2upu['PIPE']
        
        message2print = "{(Step 27B/70) set" + "; pipe-set-by-set2image4upu to: " + str(upu2image4xi['PIPE'])  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print) 
        # (<read-write PipeConnection, handle 520>, <read-write PipeConnection, handle 524>)  
        
        code2ack  = code2ack + '27B-'
    except: 
        message2print = "{(Step 27Z1/70) yielded" + "; pipe-upu2image4xi-yielded-by-set2image4upu: " + str(upu2image4xi) + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)  
        code2nack  = code2nack + '27Z1-'   

    message2print = "{(Step 28A/70) try" + "; commandqueue-try-by-set2image4upu: {}" + "; time: " + time4xi() + "}"
    print4xi(class2print, method2print, message2print)        
    try:            
        # ---------------------------------------------------------------------------------------------
        # (28) COMMAND QUEUE - EXTRA                              
        # ---------------------------------------------------------------------------------------------             
                        
        upu2image4xi['COMMANDQUEUE']  = task4imageset2upu['COMMANDQUEUE']  
 
        message2print = "{(Step 28B/70) set" + "; commandqueue-set-by-set2image4upu to: " + str(upu2image4xi['COMMANDQUEUE'])  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print) 
        # {'CODE2ERROR': 0, 'COUNT2REFERENCE': 0, 'DATA2CALLBACK': {'MESSAGE': ...
        
        code2ack  = code2ack + '28B-'
    except: 
        message2print = "{(Step 28Z1/70) yielded" + "; commandqueue-yielded-by-set2image4upu: " + str(upu2image4xi) + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)  
        code2nack  = code2nack + '28Z1-'    

    message2print = "{(Step 29A/70) try" + "; bufferinfo-try-by-set2image4upu: {}" + "; time: " + time4xi() + "}"
    print4xi(class2print, method2print, message2print)        
    try:            
        # ---------------------------------------------------------------------------------------------
        # (29) BUFFER INFO - EXTRA                              
        # ---------------------------------------------------------------------------------------------             
                        
        upu2image4xi['BUFFER']  = task4imageset2upu['BUFFER']  
 
        message2print = "{(Step 29B/70) set" + "; bufferinfo-set-by-set2image4upu to: " + str(upu2image4xi['BUFFER'])  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print) 
        #  
        code2ack  = code2ack + '29B-'
    except: 
        message2print = "{(Step 29Z1/70) yielded" + "; bufferinfo-yielded-by-set2image4upu: " + str(upu2image4xi) + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)  
        # yielded
        code2nack  = code2nack + '29Z1-'            

    message2print = "{(Step 30A/70) try" + "; eventinfo-try-by-set2image4upu: {}" + "; time: " + time4xi() + "}"
    print4xi(class2print, method2print, message2print)        
    try:            
        # ---------------------------------------------------------------------------------------------
        # (30) EVENT INFO - EXTRA                              
        # ---------------------------------------------------------------------------------------------             
                        
        upu2image4xi['EVENT']  = task4imageset2upu['EVENT']  
 
        message2print = "{(Step 30B/70) set" + "; eventinfo-set-by-set2image4upu to: " + str(upu2image4xi['EVENT'])  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print) 
        # 
        code2ack  = code2ack + '30B-'
    except: 
        message2print = "{(Step 30Z1/70) yielded" + "; eventinfo-yielded-by-set2image4upu: " + str(upu2image4xi) + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)  
        # yielded
        
        code2nack  = code2nack + '30Z1-'            
                

    message2print = "{(Step 31A/70) try" + "; flags-try-by-set2image4upu: None" + "; time: " + time4xi() + "}"
    print4xi(class2print, method2print, message2print)        
    try:            
        # ---------------------------------------------------------------------------------------------
        # (31) FLAGS - REQUIRED                              
        # ---------------------------------------------------------------------------------------------             
                        
        upu2image4xi['FLAGS']  = task4imageset2upu['FLAGS']  
 
        message2print = "{(Step 31B/70) set" + "; flags-set-by-set2image4upu to: " + str(upu2image4xi['FLAGS'])  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print) 
        # 
        code2ack  = code2ack + '31B-'
    except: 
        message2print = "{(Step 31Z1/70) yielded" + "; flags-yielded-by-set2image4upu: " + str(upu2image4xi) + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)  
        # yielded
        
        code2nack  = code2nack + '31Z1-'     

    message2print = "{(Step 34A/70) try" + "; width-try-by-set2image4upu: None" + "; time: " + time4xi() + "}"
    print4xi(class2print, method2print, message2print)        
    try:            
        # ---------------------------------------------------------------------------------------------
        # (34) WIDTH - REQUIRED                              
        # ---------------------------------------------------------------------------------------------      
        upu2image4xi['WIDTH']  = task4imageset2upu['WIDTH']  
 
        message2print = "{(Step 34H/70) set" + "; width-set-by-set2image4upu to: " + str(upu2image4xi['WIDTH'])  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print) 
        # 
        code2ack  = code2ack + '34H-'
    except: 
        message2print = "{(Step 34Z1/70) yielded" + "; width-yielded-by-set2image4upu: " + str(upu2image4xi) + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)  
        code2nack  = code2nack + '34Z1-'   


    message2print = "{(Step 35A/70) try" + "; height-try-by-set2image4upu: None"  + "; time: " + time4xi() + "}" 
    print4xi(class2print, method2print, message2print)        
    try:            
        # ---------------------------------------------------------------------------------------------
        # (35) HEIGHT - REQUIRED                              
        # ---------------------------------------------------------------------------------------------      
        upu2image4xi['HEIGHT']  = task4imageset2upu['HEIGHT']  
 
        message2print = "{(Step 35H/70) set" + "; height-set-by-set2image4upu to: " + str(upu2image4xi['HEIGHT'])  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print) 
        #  
        code2ack  = code2ack + '35H-'
    except: 
        message2print = "{(Step 35Z1/70) yielded" + "; height-yielded-by-set2image4upu: " + str(upu2image4xi) + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)  
        # yielded 
        code2nack  = code2nack + '35Z1-'   
 

    message2print = "{(Step 36A/70) try" + "; rowpitch-try-by-set2image4upu: None" + "; time: " + time4xi() + "}"
    print4xi(class2print, method2print, message2print)        
    try:            
        # ---------------------------------------------------------------------------------------------
        # (36) ROWPITCH - REQUIRED                              
        # ---------------------------------------------------------------------------------------------             
                        
        upu2image4xi['ROWPITCH']  = task4imageset2upu['ROWPITCH']  
 
        message2print = "{(Step 36B/70) set" + "; rowpitch-set-by-set2image4upu to: " + str(upu2image4xi['ROWPITCH'])  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print) 
        # 
        code2ack  = code2ack + '36B-'
    except: 
        message2print = "{(Step 36Z1/70) yielded" + "; rowpitch-yielded-by-set2image4upu: " + str(upu2image4xi) + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)  
        # yielded 
        code2nack  = code2nack + '36Z1-'

    message2print = "{(Step 37A/70) try" + "; pointer-try-by-set2image4upu: None" + "; time: " + time4xi() + "}"
    print4xi(class2print, method2print, message2print)        
    try:            
        # ---------------------------------------------------------------------------------------------
        # (37) POINTER - REQUIRED                              
        # ---------------------------------------------------------------------------------------------             
                        
        upu2image4xi['POINTER']  = task4imageset2upu['POINTER']  
 
        message2print = "{(Step 37B/70) set" + "; pointer-set-by-set2image4upu to: " + str(upu2image4xi['POINTER'])  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print) 
        # 
        code2ack  = code2ack + '37B-'
    except: 
        message2print = "{(Step 37Z1/70) yielded" + "; pointer-yielded-by-set2image4upu: " + str(upu2image4xi) + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)  
        # yielded 
        code2nack  = code2nack + '37Z1-'

    message2print = "{(Step 38A/70) try" + "; format-try-by-set2image4upu: None" + "; time: " + time4xi() + "}"
    print4xi(class2print, method2print, message2print)        
    try:            
        # ---------------------------------------------------------------------------------------------
        # (38) FORMAT - REQUIRED                              
        # ---------------------------------------------------------------------------------------------             
                        
        upu2image4xi['FORMAT']  = task4imageset2upu['FORMAT']  
 
        message2print = "{(Step 38B/70) set" + "; format-set-by-set2image4upu to: " + str(upu2image4xi['FORMAT'])  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print) 
        # 
        code2ack  = code2ack + '38B-'
    except: 
        message2print = "{(Step 38Z1/70) yielded" + "; format-yielded-by-set2image4upu: " + str(upu2image4xi) + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)  
        # yielded 
        code2nack  = code2nack + '38Z1-'
                        
    # -----------------------------------------------------------------------------------------
    # (Step 70/70)           
    # -----------------------------------------------------------------------------------------
    upu2image4xi ['ACK2RETURN']  = code2ack
    upu2image4xi ['NACK2RETURN'] = code2nack 
    
    message2print = "{(Step 70/70) returning" + "; upu2image4xi-returning-by-set2image4upu: " + str(upu2image4xi) + "; time: " + time4xi() + "}" 
    print4xi(class2print, method2print, message2print)                                                             
    # {'CODE2ERROR': 0, 'ACK2RETURN': ' 23B-24B-25B-26B-27B-28B-', 'ROWPITCH': 800, 'DATA2CALLBACK': ...
    
    
    return upu2image4xi

           
#</function4xi> 


# -------------------------------------------------------------------------------------------------
#<function4xi function = "copy2image4upu" usage = "
#                                                                                                              
#   # -----------------------------------------------------------------------------------------------
#   # (16) ESTABLISH CONTROL & COMMAND WITH COPY2IMAGE4UPU                      
#   # -----------------------------------------------------------------------------------------------    
#   print('(Step 16A/700) OF UPU2IMAGE4XI5TEST - TRYING TO ESTABLISH CONTROL & COMMAND WITH COPY2IMAGE4UPU ', time4xi())    
#   response4copy2image = {} 
#   task4copy2image = None
#   try:                               
#       response4copy2image = copy2image4upu(task4copy2image)                                                                                                          
#       print('(Step 16B/700) OF UPU2IMAGE4XI5TEST - ESTABLISHED CONTROL & COMMAND WITH COPY2IMAGE4UPU : ', response4copy2image)
#       queue2copy4upu      = response4copy2image['QUEUE']
#       channel2copypublish4upu = response4copy2image['PUBLISHERCHANNEL']
#       channel2copyconsume4upu = response4copy2image['CONSUMERCHANNEL']
#   except:
#       print('(Step 16Z/700) OF UPU2IMAGE4XI5TEST *** FAILED TO ESTABLISH CONTROL & COMMAND WITH COPY2IMAGE4UPU ', time4xi()) 
#   # -----------------------------------------------------------------------------------------------
#   # (17) SEND CONTROL & COMMAND COPY VIA PIPE FOR MANOEUVRE2IMAGE4UPU TO RECEIVE                 
#   # -----------------------------------------------------------------------------------------------      
#   print('(Step 17A/700) OF UPU2IMAGE4XI5TEST - TRYING TO SEND CONTROL & COMMAND VIA PIPE WITH COPY2IMAGE4UPU ', time4xi())     
#   try:                               
#       message2copyimagesend4upu = "<copy4xi>copy2copy4upu</copy4xi>"
#       channel2copypublish4upu = response4copy2image['PUBLISHERCHANNEL']
#       channel2copypublish4upu.send(message2copyimagesend4upu)
#       print("(Step 17B/700) OF UPU2IMAGE4XI5TEST - SENT TO CHANNEL2PUBLISH4UPU: ", channel2copypublish4upu, " MESSAGE2SEND4UPU: " , message2copyimagesend4upu)                                                                                                
#   except:
#       print('(Step 17Z1/700) OF UPU2IMAGE4XI5TEST *** FAILED TO SEND CONTROL & COMMAND VIA PIPE WITH COPY2IMAGE4UPU ', time4xi())                                                
#   # -----------------------------------------------------------------------------------------------
#   # (18) LET MANOEUVRE2IMAGE4UPU TO RECEIVE COPY AND AND SEND ROGER VIA PIPE                
#   # -----------------------------------------------------------------------------------------------       
#   print("(Step 18C/700) OF UPU2IMAGE4XI5TEST - TRYING TO PERFORM TARGET FUNCTION MANOEUVRE2IMAGE4UPU ", time4xi()) 
#   try:        
#       response2manoeuvre4upu = manoeuvre2copyimage4upu(queue2copy4upu, channel2copyconsume4upu, channel2copypublish4upu)
#       print("(Step 18D/700) OF UPU2IMAGE4XI5TEST - PERFORMED TARGET FUNCTION MANOEUVRE2IMAGE4UPU ", manoeuvre2copyimage4upu)
#   except:
#       print("(Step 18Z/700) OF UPU2IMAGE4XI5TEST *** FAILED TO PERFORM TARGET FUNCTION MANOEUVRE2IMAGE4UPU ", time4xi())     
#   else:
#       pass # trust activated thread    
#   # -----------------------------------------------------------------------------------------------
#   # (19) RECEIVE CONTROL & COMMAND ROGER VIA PIPE                   
#   # -----------------------------------------------------------------------------------------------          
#   print('(Step 19A/700) OF UPU2IMAGE4XI5TEST - TRYING TO RECEIVE CONTROL & COMMAND VIA PIPE WITH COPY2IMAGE4UPU ', time4xi())     
#   try:
#       channel2copyconsume4upu = response4copy2image['CONSUMERCHANNEL']                              
#       message2copyimagereceive4upu = channel2copyconsume4upu.recv()
#       print("(Step 19B/700) OF UPU2IMAGE4XI5TEST - RECEIVED FROM CHANNEL2CONSUME4UPU: ", channel2copyconsume4upu, " CONTROL & COMMAND: ", message2copyimagereceive4upu)                                                                                                  
#       # <roger4xi>roger2copy4upu</roger4xi>
#   except:
#       print('(Step 19Z/700) OF UPU2IMAGE4XI5TEST *** FAILED TO RECEIVE CONTROL & COMMAND VIA PIPE  WITH COPY2IMAGE4UPU ', time4xi())         
#   # -----------------------------------------------------------------------------------------------
#   # (20) SEND CONTROL & COMMAND READY VIA PIPE FOR MANOEUVRE2IMAGE4UPU TO RECEIVE                
#   # -----------------------------------------------------------------------------------------------      
#   print('(Step 20A/700) OF UPU2IMAGE4XI5TEST - TRYING TO SEND CONTROL & COMMAND VIA PIPE WITH COPY2IMAGE4UPU ', time4xi())     
#   try:                               
#       message2copyimagesend4upu = "<copy4xi>copy2copy4upu</copy4xi>"
#       channel2copypublish4upu = response4copy2image['PUBLISHERCHANNEL']
#       channel2copypublish4upu.send(message2copyimagesend4upu)
#       print("(Step 20B/700) OF UPU2IMAGE4XI5TEST - SENT TO CHANNEL2PUBLISH4UPU: ", channel2copypublish4upu, " MESSAGE2SEND4UPU: " , message2copyimagesend4upu)                                                                                                
#       # <copy4xi>copy2copy4upu</copy4xi> 
#   except:
#        print('(Step 20Z/700) OF UPU2IMAGE4XI5TEST *** FAILED TO SEND CONTROL & COMMAND VIA PIPE WITH COPY2IMAGE4UPU ', time4xi())                                                
#   # -----------------------------------------------------------------------------------------------
#   # (21) LET MANOEUVRE2IMAGE4UPU TO RECEIVE READY AND TO SEND READY                
#   # -----------------------------------------------------------------------------------------------       
#   print("(Step 21A/700) OF UPU2IMAGE4XI5TEST - TRYING TO PERFORM TARGET FUNCTION MANOEUVRE2IMAGE4UPU ", time4xi()) 
#   try:        
#       response2manoeuvre4upu = manoeuvre2copyimage4upu(queue2copy4upu, channel2copyconsume4upu, channel2copypublish4upu)
#       print("(Step 21B/700) OF UPU2IMAGE4XI5TEST - PERFORMED TARGET FUNCTION MANOEUVRE2IMAGE4UPU ", manoeuvre2copyimage4upu)
#   except:
#       print("(Step 21Z/700) OF UPU2IMAGE4XI5TEST *** FAILED TO PERFORM TARGET FUNCTION MANOEUVRE2IMAGE4UPU ", time4xi())     
#   # -----------------------------------------------------------------------------------------------
#   # (22) RECEIVE CONTROL & COMMAND READY VIA PIPE                      
#   # -----------------------------------------------------------------------------------------------          
#   print('(Step 22A/700) OF UPU2IMAGE4XI5TEST - TRYING TO RECEIVE CONTROL & COMMAND VIA PIPE WITH COPY2IMAGE4UPU ', time4xi())     
#   try:
#       channel2copyconsume4upu = response4copy2image['CONSUMERCHANNEL']                              
#       message2copyimagereceive4upu = channel2copyconsume4upu.recv()
#       print("(Step 22B/700) OF UPU2IMAGE4XI5TEST - RECEIVED FROM CHANNEL2CONSUME4UPU: ", channel2copyconsume4upu, " CONTROL & COMMAND: ", message2copyimagereceive4upu)                                                                                                  
#       # <copy4xi>copy2copy4upu</copy4xi> 
#   except:
#       print('(Step 22Z/700) OF UPU2IMAGE4XI5TEST *** FAILED TO RECEIVE CONTROL & COMMAND VIA PIPE  WITH COPY2IMAGE4UPU ', time4xi())          
#   # ----------------------------------------------------------------------------------------------
#   # (24) PUT IMAGE PATH TO QUEUE
#   # ----------------------------------------------------------------------------------------------                          
#   print("(Step 24A/700) OF UPU2IMAGE4XI5TEST - TRYING TO PUT IMAGE PATH TO QUEUE ", time4xi())      
#   try:    
#       queue2copy4dragon.put(path2dragon)             
#       print("(Step 24B/700) OF DRAGON2IMAGE4XI5TEST - PUT TO QUEUE IMAGE PATH: ", path2dragon) 
#       # ..//..//..//..//..//web//images//png//dragonxi.png  
#   except:
#       print("(Step 24Z/700) OF UPU2IMAGE4XI5TEST *** FAILED TO PUT IMAGE PATH TO QUEUE ", time4xi()) 
#   # -----------------------------------------------------------------------------------------------
#   # (25) LET MANOEUVRE2IMAGE4UPU TO GET FROM QUEUE AND TO SEND EITHER ACK OR NACK VIA PIPE               
#   # -----------------------------------------------------------------------------------------------       
#   print("(Step 25A/700) OF UPU2IMAGE4XI5TEST - TRYING TO PERFORM TARGET FUNCTION MANOEUVRE2IMAGE4UPU ", time4xi()) 
#   try:        
#       response2manoeuvre4upu = manoeuvre2copyimage4upu(queue2copy4upu, channel2copyconsume4upu, channel2copypublish4upu)
#       print("(Step 25B/700) OF UPU2IMAGE4XI5TEST - PERFORMED TARGET FUNCTION MANOEUVRE2IMAGE4UPU ", manoeuvre2copyimage4upu)
#   except:
#       print("(Step 25Z/700) OF UPU2IMAGE4XI5TEST *** FAILED TO PERFORM TARGET FUNCTION MANOEUVRE2IMAGE4UPU ", time4xi())     
#   # -----------------------------------------------------------------------------------------------
#   # (26) RECEIVE ACK OR NACK ABOUT IMAGE OPERATION VIA PIPE                
#   # -----------------------------------------------------------------------------------------------          
#   print('(Step 26A/700) OF UPU2IMAGE4XI5TEST - TRYING TO RECEIVE CONTROL & COMMAND ABOUT IMAGE OPERTION VIA PIPE ', time4xi())      
#   try: 
#       message2copyimagereceive4upu = channel2copyconsume4upu.recv()
#       print("(Step 26D/700) OF UPU2IMAGE4XI5TEST - RECEIVED FROM CHANNEL2CONSUME4UPU: ", channel2copyconsume4upu, " CONTROL & COMMAND: ", message2copyimagereceive4upu)                                                                                                   
#       # <ack4xi>ack2imageinfo4upu</ack4xi>
#   except:
#       print('(Step 26Z/700) OF UPU2IMAGE4XI5TEST *** FAILED TO RECEIVE CONTROL & COMMAND ABOUT IMAGE OPERTION VIA PIPE ', time4xi())               
#   # -----------------------------------------------------------------------------------------------
#   (27) LET MANOEUVRE2IMAGE4UPU GO TO STATE 0               
#   # -----------------------------------------------------------------------------------------------       
#   print("(Step 27A/700) OF UPU2IMAGE4XI5TEST - TRYING TO PERFORM TARGET FUNCTION MANOEUVRE2IMAGE4UPU ", time4xi()) 
#   try:        
#       response2manoeuvre4upu = manoeuvre2copyimage4upu(queue2copy4upu, channel2copyconsume4upu, channel2copypublish4upu)
#       print("(Step 27B/700) OF UPU2IMAGE4XI5TEST - GOT RESPONSE2MANOEUVRE4UPU ", response2manoeuvre4upu)
#   except:
#       print("(Step 27Z/700) OF UPU2IMAGE4XI5TEST *** FAILED TO PERFORM TARGET FUNCTION MANOEUVRE2IMAGE4UPU ", time4xi())   

#">

# -------------------------------------------------------------------------------------------------   
def copy2image4upu(task4imagecopy2upu=None):

    '''
    copies 2D images for UPU
    '''
    global upu2image4xi                                                                                             # 2D-IMAGES FOR UPU  
    global control2copyimage4upu                                                                                    # COMMAND AND CONTROL FOR UPU IMAGES 
    
    # -----------------------------------------------------------------------------------------
    # (Step 1/33) PARAMETERS           
    # -----------------------------------------------------------------------------------------          
    class2print = "copy2image4upu"
    method2print = "copy2image4upu"

    message2print = "{(Step 0A/33) started" + "; copy2image4upu-started: " + str(copy2image4upu)  + "; time: " + time4xi() + "}" 
    print4xi(class2print, method2print, message2print)                                                               
    # <function copy2image4upu at 0x00000000038538C8>
    
    message2print = "{(Step 0B/33) passed" + "; task4imagecopy2upu-to-copy2image4upu: " + str(task4imagecopy2upu) + "; time: " + time4xi() + "}"
    print4xi(class2print, method2print, message2print)                                                              
    # None
            
    code2ack  = ' '
    code2nack = ' '
    
    # -----------------------------------------------------------------------------------------
    # CREATE UPU2IMAGE4XI IF NECESSARY       
    # ----------------------------------------------------------------------------------------- 
    try:
        if upu2image4xi is None:
            upu2image4xi = create2image4upu()                                                                       # INITIALIZE IMAGE FOR UPU 
            message2print = "{(Step 0C/70) created" + "; upu2image4xi-created-by-copy2image4upu to: " + str(upu2image4xi)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)
            #              
        else:
            message2print = "{(Step 0D/70) skipped" + "; create2image4upu-skipped-by-copy2image4upu: " + str(copy2image4upu) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)
            # <function copy2image4upu at 0x00000000038538C8>                                 
    except:
        try:
            upu2image4xi = create2image4upu()
            message2print = "{(Step 0E/70) set" + "; upu2image4xi-set-by-copy2image4upu to: " + str(upu2image4xi)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)
            #             
        except:
            upu2image4xi = {}
            message2print = "{(Step 0Z/70) yielded" + "; upu2image4xi-yielded-by-copy2image4upu: " + str(upu2image4xi) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)  
            code2nack  = code2nack + '0Z-' 
    
    # -----------------------------------------------------------------------------------------
    # (Step 1/70) - CHECK PARAMETERS           
    # -----------------------------------------------------------------------------------------          
    try:             
        if task4imagecopy2upu is None:
            # -----------------------------------------------------------------------------------------
            # NO PARAMETERS GIVEN BY CALLER FOR COPY - BUILD TASK! 
            # -----------------------------------------------------------------------------------------    
            
            task4imagecopy2upu = {}                                                                                                                                                                                       
            
            message2print = "{(Step 1A/70) initialized" + "; task4imagecopy2upu-created-by-copy2image4upu to: " + str(task4imagecopy2upu)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)               
            # {} 
            
            # ---------------------------------------------------------------------------------------------
            # (1B) TRACE - AUTOMATIC                      
            # ---------------------------------------------------------------------------------------------              
            function2callback4upu = trace2image4upu                                                                 # CALLBACK FUNCTION FOR UPU
            
            message2print = "{(Step 1B1/70) initialized" + "; function2callback4upu-created-by-copy2image4upu to: " + str(function2callback4upu)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)            
            # <function trace2image4upu at 0x000000000384E9C8> 
            
            code2ack  = code2ack + '1B1-'
           
            args2callback = {}                                                                                      # CREATE ARGUMENT DICTIONARY FOR CALLBACK FUNCTION             
            args2callback['CLASS']   = "copy2image4upu"                                                            # MODULE OF UPUIZED OPENCL MODULE
            args2callback['METHOD']  = "copy2image4upu"                                                            # FUNCTION OF UPUIZED OPENCL MODULE
            args2callback['MESSAGE'] = "{(Step 1Z1/33) failed" + "; copy2image4upu-failed " + "; time: " + time4xi() + "}" # ERROR MESSAGE FOR UPUIZED OPENCL TO RETURN    

            message2print = "{(Step 1B2/70) initialized" + "; args2callback-created-by-copy2image4upu to: " + str(args2callback)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)               
            # {'MESSAGE': '{(Step 1Z1/33) failed; copy2image4upu-failed ; time: 07:01:44 10/04/11 Pacific Daylight Time}',
            #  'CLASS': 'copy2image4upu',
            #  'METHOD': 'copy2image4upu'}; time: 07:01:44 10/04/11 Pacific Daylight Time}


            code2ack  = code2ack + '1B2-'
            
            # ---------------------------------------------------------------------------------------------
            # (1C) CALLBACK - AUTOMATIC                      
            # ---------------------------------------------------------------------------------------------              
            task4imagecopy2upu['DATA2CALLBACK'] = args2callback                                                    # USER DATA PASSED TO UPUIZED OPENCL - SEVERAL ARGS PASSED IN ONE DICTIONARY
            task4imagecopy2upu['FUNCTION2CALLBACK'] = function2callback4upu                                        # CALLBACK FUNCTION TO CALLED BY OPENCL IN EXCEPTIONS
            
            code2ack  = code2ack + '1C-'
            message2print = "{(Step 1C/70) initialized" + "; task4imagecopy2upu-created-by-copy2image4upu to: " + str(task4imagecopy2upu)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)  
            # {'FUNCTION2CALLBACK': <function trace2image4upu at 0x000000000384E9C8>,
            #  'DATA2CALLBACK': {'MESSAGE': '{(Step 1Z1/33) failed; copy2image4upu-failed ; time: 07:01:44 10/04/11 Pacific Daylight Time}',
            #  'CLASS': 'copy2image4upu',
            #  'METHOD': 'copy2image4upu'}}; time: 07:01:44 10/04/11 Pacific Daylight Time}

            
            # ---------------------------------------------------------------------------------------------
            # (2) PLATFORM - AUTOMATIC                   
            # ---------------------------------------------------------------------------------------------             
            platform2xi = {}
            message2print = "{(Step 2A/70) initialized" + "; platform2xi-created-by-copy2image4upu to: " + str(platform2xi)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)   
            # {}
            
            try:
                platform2xi = get2platformids4xi()                                                                  # PLATFORM IS COMMON TO ALL DEVICES I.E TO DEVICE XI
                message2print = "{(Step 2B/70) got" + "; platform2xi-got-by-copy2image4upu: " + str(platform2xi)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)
                # {'list2platforms': ['DragonXi'], 'ack2return': ' 2C-2E-2F-2G-', 'nack2return': ' ', 'available2platform': 1, 'max2entries': 1}; time: 07:01:44 10/04/11 Pacific Daylight Time}
                 
                code2ack  = code2ack + '2B-'
            except:
                message2print = "{(Step 2Z/70) failed" + "; platform2xi-failed-by-copy2image4upu: " + str(failed)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)                                                
                code2nack  = code2nack + '2Z-'
                 
            task4imagecopy2upu['PLATFORM'] = platform2xi                                                           # PLATFORM OF SYSTEM 
              
            
            # ---------------------------------------------------------------------------------------------
            # (3) CONTEXT - AUTOMATIC                 
            # ---------------------------------------------------------------------------------------------    
            context2upu = {}
            message2print = "{(Step 3A/70) trying" + "; context2upu-trying-by-copy2image4upu: " + str(context2upu)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print) 
            try:
                context2upu = get2context4upu()                                                                     # NO PARAMETERS - THE SYSTEM CREATES CONTEXT IF REQUIRED  
                
                message2print = "{(Step 3B/70) got" + "; context2upu-got-by-copy2image4upu: " + str(context2upu)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)                                  
                # {'COUNT2REFERENCE': 0, 'DATA2CALLBACK': {'MESSAGE': '{(Step 1Z1/33) ...
                
                code2ack  = code2ack + '3B-'         
            except:
                message2print = "{(Step 3Z/70) failed" + "; context2upu-failed-by-copy2image4upu: " + str(context2upu) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)             
                code2nack  = code2nack + '3Z-'
                
            task4imagecopy2upu ['CONTEXT'] = context2upu           
            
            # ---------------------------------------------------------------------------------------------
            # (4) DEVICE - AUTOMATIC                       
            # ---------------------------------------------------------------------------------------------    
            device2upu = {}
            
            message2print = "{(Step 4A/70) trying" + "; device2upu-trying-by-copy2image4upu: " + str(device2upu)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print) 
            try:
                device2upu = get2upu4xi()                                                                           # THE SYSTEM CREATES DEVICE INFO, IF NECESSARY    
                
                message2print = "{(Step 4B/70) got" + "; device2upu-got-by-copy2image4upu: " + str(device2upu)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)                                  
                # {'RESOLUTION2TIMER4PROFILING': 318, 'ID2PLATFORM': '0000000001509568', 'PROFILING2QUEUE': 'ACK', ...
                
                code2ack  = code2ack + '4B-'  
            except:
                message2print = "{(Step 4Z/70) failed" + "; device2upu-failed-by-copy2image4upu: " + str(device2upu) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)             
                code2nack  = code2nack + '4Z-'
            
            task4imagecopy2upu ['DEVICE'] = device2upu
            
                   
            # ---------------------------------------------------------------------------------------------
            # (5) IMAGE SOURCE OBJECT - REQUIRED                         
            # ---------------------------------------------------------------------------------------------         
            imagesourceinfo2upu = {}
             
            message2print = "{(Step 5A/70) trying" + "; imagesourceinfo2upu-trying-by-copy2image4upu: " + str(imagesourceinfo2upu)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print) 
            try:
                imagesourceinfo2upu = get2imageinfo4upu()                                                                         
                
                message2print = "{(Step 5B/70) got" + "; imagesourceinfo2upu-got-by-copy2image4upu: " + str(imagesourceinfo2upu)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)                                  
                # {'SIZE2QUEUE': 2147483646, 
                #  'FORMAT': {'TYPE': 'CL_UNORM_INT8', 
                #  'ORDER': 'CL_RGBA'}, 
                #  'QUEUE': <queue4xi.Queue4Xi object at 0x0000000004472BE0>, 
                #  'WIDTH': 800, 
                #  'CONTEXT': {'COUNT2REFERENCE': 0, ...
                
                code2ack  = code2ack + '5B-'  
            except:
                message2print = "{(Step 5Z/70) failed" + "; imagesourceinfo2upu-failed-by-copy2image4upu: " + str(imagesourceinfo2upu) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)             
                code2nack  = code2nack + '5Z-'          
                      
            task4imagecopy2upu ['IMAGESOURCE'] = imagesourceinfo2upu         
                   
            # ---------------------------------------------------------------------------------------------
            # (6) IMAGE DESTINATION OBJECT - REQUIRED                        
            # ---------------------------------------------------------------------------------------------                         
            imagedestinationinfo2upu = {}
             
            message2print = "{(Step 6A/70) trying" + "; imagedestinationinfo2upu-trying-by-copy2image4upu: " + str(imagedestinationinfo2upu)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print) 
            try:
                imagedestinationinfo2upu = get2imageinfo4upu()                                                                         
                
                message2print = "{(Step 6B/70) got" + "; imagedestinationinfo2upu-got-by-copy2image4upu: " + str(imagedestinationinfo2upu)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)                                  
                # {'SIZE2QUEUE': 2147483646, 
                #  'FORMAT': {'TYPE': 'CL_UNORM_INT8', 
                #  'ORDER': 'CL_RGBA'}, 
                #  'QUEUE': <queue4xi.Queue4Xi object at 0x0000000004472BE0>, 
                #  'WIDTH': 800, 
                #  'CONTEXT': {'COUNT2REFERENCE': 0, ...
                # note: source and destination will be the same !             
                code2ack  = code2ack + '6B-'  
            except:
                message2print = "{(Step 6Z/70) failed" + "; imagedestinationinfo2upu-failed-by-copy2image4upu: " + str(imagedestinationinfo2upu) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)             
                code2nack  = code2nack + '6Z-'          
                      
            task4imagecopy2upu ['IMAGEDESTINATION'] = imagedestinationinfo2upu     
            
            # ---------------------------------------------------------------------------------------------
            # (7) QUEUE - REQUIRED BY UPUXI                          
            # ---------------------------------------------------------------------------------------------         
            queue2upu = None
            
            message2print = "{(Step 7A/70) trying" + "; queue2upu-trying-by-copy2image4upu: " + str(queue2upu)  + "; time: " + time4xi() + "}" 
            print4xi(class2print, method2print, message2print) 
            try:
                queue2upu = Queue4Xi()                                                                              # THE SYSTEM CREATES QUEUE    
                
                message2print = "{(Step 7B/70) got" + "; queue2upu-got-by-copy2image4upu: " + str(queue2upu)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)                                  
                # <queue4xi.Queue4Xi object at 0x000000000453EB38>
                
                code2ack  = code2ack + '7B-'  
            except:
                message2print = "{(Step 7Z/70) failed" + "; queue2upu-failed-by-copy2image4upu: " + str(queue2upu) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)             
                code2nack  = code2nack + '7Z-'
                
            task4imagecopy2upu ['QUEUE'] = queue2upu

            
            # ---------------------------------------------------------------------------------------------
            # (8) PIPE - REQUIRED BY UPUXI                              
            # ---------------------------------------------------------------------------------------------         
            pipe2upu = None
           
            message2print = "{(Step 8A/70) trying" + "; pipe2upu-trying-by-copy2image4upu: " + str(pipe2upu)  + "; time: " + time4xi() + "}" 
            print4xi(class2print, method2print, message2print) 
            try:
                pipe2upu = Pipe4Xi()                                                                                # THE SYSTEM CREATES PIPE    
                
                message2print = "{(Step 8B/70) got" + "; pipe2upu-got-by-copy2image4upu: " + str(pipe2upu)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)                                  
                # (<read-write PipeConnection, handle 612>, <read-write PipeConnection, handle 616>)
                
                code2ack  = code2ack + '8B-'  
            except:
                message2print = "{(Step 8Z/70) failed" + "; pipe2upu-failed-by-copy2image4upu: " + str(pipe2upu) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)             
                code2nack  = code2nack + '8Z-'
                
            task4imagecopy2upu ['PIPE'] = pipe2upu 
            
            # ---------------------------------------------------------------------------------------------
            # (9) COMMAND QUEUE - REQUIRED                              
            # ---------------------------------------------------------------------------------------------             
            commandqueueinfo2upu = {}
           
            message2print = "{(Step 9A/70) trying" + "; commandqueueinfo2upu-trying-by-copy2image4upu: " + str(commandqueueinfo2upu)  + "; time: " + time4xi() + "}" 
            print4xi(class2print, method2print, message2print)  
                        
            try:
                commandqueueinfo2upu = get2commandqueueinfo4upu()                                                   # SYSTEM CREATES COMMAND QUEUE, IF NECESSARY   
                message2print = "{(Step 9B/70) got" + "; commandqueueinfo2upu-got-by-copy2image4upu: " + str(commandqueueinfo2upu)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print) 
                # {'CODE2ERROR': 0, 
                #  'COUNT2REFERENCE': 0, 
                #  'DATA2CALLBACK': {'MESSAGE': '{(Step 1Z1/33) failed; create2commandqueue4upu-failed ; time: 07:01:44 10/04/11 Pacific Daylight Time}', 
                #  'CLASS': 'create2commandqueue4upu', 
                #  'METHOD': 'create2commandqueue4upu'}, 
                #  'FUNCTION2CALLBACK': <function trace2commandqueue4upu at 0x00000000037EDA48> ...
                
                code2ack  = code2ack + '9B-'  
            except:
                message2print = "{(Step 9Z/70) failed" + "; commandqueueinfo2upu-failed-by-copy2image4upu: " + str(commandqueueinfo2upu) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)             
                code2nack  = code2nack + '9Z-'
        
            task4imagecopy2upu ['COMMANDQUEUE'] = commandqueueinfo2upu
            
            
            
            # ---------------------------------------------------------------------------------------------
            # (10) BUFFER - AUTOMATIC BY UPUXI                               
            # ---------------------------------------------------------------------------------------------             
            bufferinfo2upu = {}
           
            message2print = "{(Step 10A/70) trying" + "; bufferinfo2upu-trying-by-copy2image4upu: " + str(bufferinfo2upu)  + "; time: " + time4xi() + "}" 
            print4xi(class2print, method2print, message2print) 
                        
            try:
                bufferinfo2upu = get2bufferinfo4upu()                                                               # SYSTEM CREATES BUFFER FOR UPU, IF NECESSARY
                message2print = "{(Step 10B/70) got" + "; bufferinfo2upu-got-by-copy2image4upu: " + str(bufferinfo2upu)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print) 
                # {'CODE2ERROR': 0, 'COUNT2REFERENCE': 0, 'DATA2CALLBACK': {'MESSAGE' ...
                
                code2ack  = code2ack + '10B-' 
            except:
                message2print = "{(Step 10Z/70) failed" + "; bufferinfo2upu-failed-by-copy2image4upu: " + str(bufferinfo2upu) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)             
                code2nack  = code2nack + '10Z-'
        
            task4imagecopy2upu ['BUFFER'] = bufferinfo2upu 
            
            # ---------------------------------------------------------------------------------------------
            # (11) EVENT - REQUIRED                            
            # ---------------------------------------------------------------------------------------------             
            eventinfo2upu = {}
           
            message2print = "{(Step 11A/70) trying" + "; eventinfo2upu-trying-by-copy2image4upu: " + str(eventinfo2upu)  + "; time: " + time4xi() + "}" 
            print4xi(class2print, method2print, message2print) 
                        
            try:
                eventinfo2upu = get2eventinfo4upu()                                                                    
                message2print = "{(Step 11B/70) got" + "; eventinfo2upu-got-by-copy2image4upu: " + str(eventinfo2upu)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print) 
                # {'CODE2ERROR': 0,
                #  'DATA2CALLBACK': {'MESSAGE': '{(Step 1Z1/33) failed; create2event4upu-failed ; time: 07:01:44 10/04/11 Pacific Daylight Time}',
                #  'CLASS': 'create2event4upu',
                #  'METHOD': 'create2event4upu'},
                #  'FUNCTION2CALLBACK': <function trace2event4upu at 0x0000000003803F48>,
                #  'ACK2RETURN': ' 1D-1E-2C-2B-3B-4B-6B-7B-8B-9B-10B-11B-12B-14B-15B-16B-',
                #  'QUEUE': <queue4xi.Queue4Xi object at 0x000000000423B668>,
                #  'NACK2RETURN': ' ',
                #  'CONTEXT': {'COUNT2REFERENCE': 0, 'DATA2CALLBACK': {'MESSAGE': ...
                
                code2ack  = code2ack + '11B-' 
            except:
                message2print = "{(Step 11Z/70) failed" + "; eventinfo2upu-failed-by-copy2image4upu: " + str(eventinfo2upu) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)             
                code2nack  = code2nack + '11Z-'
        
            task4imagecopy2upu ['EVENT'] = eventinfo2upu 
            
            
            # ---------------------------------------------------------------------------------------------
            # (12) EVENTWAITLIST - REQUIRED                              
            # ---------------------------------------------------------------------------------------------             
            eventwaitlist2upu = {}
           
            message2print = "{(Step 12A/70) trying" + "; eventwaitlist2upu-trying-by-copy2image4upu: " + str(eventwaitlist2upu)  + "; time: " + time4xi() + "}" 
            print4xi(class2print, method2print, message2print) 
                        
            try:
                eventwaitlist2upu = {}                                                                   
                message2print = "{(Step 12B/70) set" + "; eventwaitlist2upu-set-by-copy2image4upu: " + str(eventwaitlist2upu)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print) 
                # {} 
                
                code2ack  = code2ack + '12B-' 
            except:
                message2print = "{(Step 12Z/70) failed" + "; eventwaitlist2upu-failed-by-copy2image4upu: " + str(eventwaitlist2upu) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)             
                code2nack  = code2nack + '12Z-'
        
            task4imagecopy2upu ['EVENTWAITLIST'] = eventwaitlist2upu                     

            
            # ---------------------------------------------------------------------------------------------
            # (13) EVENTWAITLISTQUANTITY - REQUIRED                               
            # ---------------------------------------------------------------------------------------------             
            eventwaitlistquantity2upu = 0
            
            message2print = "{(Step 13A/70) trying" + "; eventwaitlistquantity2upu-trying-by-copy2image4upu: " + str(eventwaitlistquantity2upu)  + "; time: " + time4xi() + "}" 
            print4xi(class2print, method2print, message2print) 
                        
            try:
                keys2event = eventwaitlist2upu.keys()
                for key2event in keys2event:
                    eventwaitlistquantity2upu = eventwaitlistquantity2upu + 1    
                
                message2print = "{(Step 13B/70) set" + "; eventwaitlistquantity2upu-set-by-copy2image4upu: " + str(eventwaitlistquantity2upu)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print) 
                # 0  
                
                code2ack  = code2ack + '13B-' 
            except:
                message2print = "{(Step 13Z/70) failed" + "; eventwaitlistquantity2upu-failed-by-copy2image4upu: " + str(eventwaitlistquantity2upu) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)             
                code2nack  = code2nack + '13Z-'
        
            task4imagecopy2upu ['EVENTWAITLISTQUANTITY'] = eventwaitlistquantity2upu       
                                                 
            
            # ---------------------------------------------------------------------------------------------
            # (14) ORIGIN FOR SOURCE - REQUIRED                             
            # ---------------------------------------------------------------------------------------------             
            origin4source2upu = {}
            
            message2print = "{(Step 14A/70) trying" + "; origin4source2upu-trying-by-copy2image4upu: " + str(origin4source2upu)  + "; time: " + time4xi() + "}" 
            print4xi(class2print, method2print, message2print) 
                        
            try:                                                                                    
                origin4source2upu['X'] = 0 
                origin4source2upu['Y'] = 0  
                origin4source2upu['Z'] = 0                                                                                                                          
                message2print = "{(Step 14B/70) set" + "; origin4source2upu-set-by-copy2image4upu to: " + str(origin4source2upu)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print) 
                # 
                code2ack  = code2ack + '14B-' 
            except:
                message2print = "{(Step 14Z/70) failed" + "; origin4source2upu-failed-by-copy2image4upu: " + str(origin4source2upu) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)             
                code2nack  = code2nack + '14Z-'
        
            task4imagecopy2upu ['SOURCEORIGIN'] = origin4source2upu
            
            # ---------------------------------------------------------------------------------------------
            # (15) ORIGIN FOR DESTINATION - REQUIRED                             
            # ---------------------------------------------------------------------------------------------              
            origin4destination2upu = {}
            
            message2print = "{(Step 15A/70) trying" + "; origin4destination2upu-trying-by-copy2image4upu: " + str(origin4destination2upu)  + "; time: " + time4xi() + "}" 
            print4xi(class2print, method2print, message2print) 
                        
            try:                                                                                    
                origin4destination2upu['X'] = 100 
                origin4destination2upu['Y'] = 100  
                origin4destination2upu['Z'] = 100                                                                                                                          
                message2print = "{(Step 15B/70) set" + "; origin4destination2upu-set-by-copy2image4upu to: " + str(origin4destination2upu)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print) 
                # 
                code2ack  = code2ack + '15B-' 
            except:
                message2print = "{(Step 15Z/70) failed" + "; origin4destination2upu-failed-by-copy2image4upu: " + str(origin4destination2upu) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)             
                code2nack  = code2nack + '15Z-'
        
            task4imagecopy2upu ['DESTINATIONORIGIN'] = origin4destination2upu
                        
            
            
            # --------------------------------------------------------------------------------------------------------
            # (16) REGION - REGION OF A IMAGE TO BE COPIED                         
            # --------------------------------------------------------------------------------------------------------             
            region4copy2upu = {}                                                                                                         
           
            message2print = "{(Step 16A/70) trying" + "; region4copy2upu-trying-by-copy2image4upu: " + str(region4copy2upu)  + "; time: " + time4xi() + "}" 
            print4xi(class2print, method2print, message2print) 
                        
            try:
                region4copy2upu['X'] = 10 
                region4copy2upu['Y'] = 10  
                region4copy2upu['Z'] = 10                                                                           
                message2print = "{(Step 16B/70) set" + "; region4copy2upu-set-by-copy2image4upu to: " + str(region4copy2upu)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print) 
                # 

                code2ack  = code2ack + '16B-'                
                 
            except:
                message2print = "{(Step 16Z/70) failed" + "; region4copy2upu-failed-by-copy2image4upu: " + str(region4copy2upu) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)             
                code2nack  = code2nack + '16Z-'
            
            task4imagecopy2upu ['REGION'] = region4copy2upu              
            
            # ---------------------------------------------------------------------------------------------
            # EOF PARAMETER PROVIDING                
            # ---------------------------------------------------------------------------------------------                                                                                                                               
        else:
            pass
            
        # ---------------------------------------------------------------------------------------------
        # CHECK PARAMETERS - WHETHER AUTOMATICALLY GENERATED OR USER GIVEN 
        # ---------------------------------------------------------------------------------------------
        keys2task = task4imagecopy2upu.keys()
        message2print = "{(Step 20/70) using" + "; keys2task-using-by-copy2image4upu: " + str(sorted(keys2task))  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)             
        # ['BUFFER',
        #  'COMMANDQUEUE',
        #  'CONTEXT',
        #  'DATA2CALLBACK',
        #  'DESTINATIONORIGIN',
        #  'DEVICE',
        #  'EVENT',
        #  'EVENTWAITLIST',
        #  'EVENTWAITLISTQUANTITY',
        #  'FUNCTION2CALLBACK',
        #  'IMAGEDESTINATION',
        #  'IMAGESOURCE',
        #  'PIPE',
        #  'PLATFORM',
        #  'QUEUE',
        #  'REGION',
        #  'SOURCEORIGIN']; time: 01:21:58 10/05/11 Pacific Daylight Time}
 
        if (('DATA2CALLBACK' in keys2task) and ('FUNCTION2CALLBACK' in keys2task)):
            pass
        else:                
            # ---------------------------------------------------------------------------------------------
            # (21B) TRACE - AUTOMATIC                      
            # ---------------------------------------------------------------------------------------------              
            function2callback4upu = trace2image4upu                                                            # CALLBACK FUNCTION FOR UPU
            
            message2print = "{(Step 21B1/70) initialized" + "; function2callback4upu-created-by-copy2image4upu to: " + str(function2callback4upu)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)            
            # 
            
            code2ack  = code2ack + '21B1-'
           
            args2callback = {}                                                                                 # CREATE ARGUMENT DICTIONARY FOR CALLBACK FUNCTION             
            args2callback['CLASS']   = "copy2image4upu"                                                        # MODULE OF UPUIZED OPENCL MODULE
            args2callback['METHOD']  = "copy2image4upu"                                                        # FUNCTION OF UPUIZED OPENCL MODULE
            args2callback['MESSAGE'] = "{(Step 1Z1/33) failed" + "; copy2image4upu-failed " + "; time: " + time4xi() + "}" # ERROR MESSAGE FOR UPUIZED OPENCL TO RETURN    
            
            message2print = "{(Step 21B2/70) initialized" + "; args2callback-created-by-copy2image4upu to: " + str(args2callback)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)               
            # 
    
            code2ack  = code2ack + '21B2-'
            
            # ---------------------------------------------------------------------------------------------
            # (21C) CALLBACK - AUTOMATIC                      
            # ---------------------------------------------------------------------------------------------              
            task4imagecopy2upu['DATA2CALLBACK'] = args2callback                                                # USER DATA PASSED TO UPUIZED OPENCL - SEVERAL ARGS PASSED IN ONE DICTIONARY
            task4imagecopy2upu['FUNCTION2CALLBACK'] = function2callback4upu                                    # CALLBACK FUNCTION TO CALLED BY OPENCL IN EXCEPTIONS
                
            code2ack  = code2ack + '21C-'
            message2print = "{(Step 21C/70) initialized" + "; task4imagecopy2upu-created-by-copy2image4upu to: " + str(task4imagecopy2upu)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)  
            #
                 
        if 'PLATFORM' in keys2task:
            pass
        else:  
            # ---------------------------------------------------------------------------------------------
            # (22) PLATFORM - AUTOMATIC                   
            # ---------------------------------------------------------------------------------------------             
            platform2xi = {}
            message2print = "{(Step 22A/70) initialized" + "; platform2xi-created-by-copy2image4upu to: " + str(platform2xi)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)   
            # 
                
            try:
                platform2xi = get2platformids4xi()                                                              # PLATFORM IS COMMON TO ALL DEVICES I.E TO DEVICE XI
                message2print = "{(Step 22B/70) got" + "; platform2xi-got-by-copy2image4upu: " + str(platform2xi)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)
                # 
                 
                code2ack  = code2ack + '22B-'
            except:
                message2print = "{(Step 22Z/70) failed" + "; platform2xi-failed-by-copy2image4upu: " + str(failed)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)                                                
                code2nack  = code2nack + '22Z-'
                 
            task4imagecopy2upu['PLATFORM'] = platform2xi                                                       # PLATFORM OF SYSTEM 
          
        if 'CONTEXT' in keys2task:
            pass
        else:              
            # ---------------------------------------------------------------------------------------------
            # (23) CONTEXT - AUTOMATIC                 
            # ---------------------------------------------------------------------------------------------    
            context2upu = {}
            message2print = "{(Step 23A/70) trying" + "; context2upu-trying-by-copy2image4upu: " + str(context2upu)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print) 
            try:
                context2upu = get2context4upu()                                                                 # NO PARAMETERS - THE SYSTEM CREATES CONTEXT IF REQUIRED  
                
                message2print = "{(Step 23B/70) got" + "; context2upu-got-by-copy2image4upu: " + str(context2upu)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)                                  
                # 
                
                code2ack  = code2ack + '23B-'         
            except:
                message2print = "{(Step 23Z/70) failed" + "; context2upu-failed-by-copy2image4upu: " + str(context2upu) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)             
                code2nack  = code2nack + '23Z-'
                
            task4imagecopy2upu ['CONTEXT'] = context2upu           
        
        if 'DEVICE' in keys2task:
            pass
        else:                   
            # ---------------------------------------------------------------------------------------------
            # (24) DEVICE - AUTOMATIC                       
            # ---------------------------------------------------------------------------------------------    
            device2upu = {}
            
            message2print = "{(Step 24A/70) trying" + "; device2upu-trying-by-copy2image4upu: " + str(device2upu)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print) 
            try:
                device2upu = get2upu4xi()                                                                       # THE SYSTEM CREATES DEVICE INFO, IF NECESSARY    
                
                message2print = "{(Step 24B/70) got" + "; device2upu-got-by-copy2image4upu: " + str(device2upu)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)                                  
                # 
                
                code2ack  = code2ack + '24B-'  
            except:
                message2print = "{(Step 24Z/70) failed" + "; device2upu-failed-by-copy2image4upu: " + str(device2upu) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)             
                code2nack  = code2nack + '24Z-'
            
            task4imagecopy2upu ['DEVICE'] = device2upu
        
        if 'IMAGESOURCE' in keys2task:
            pass
        else:                      
            # ---------------------------------------------------------------------------------------------
            # (25) SOURCE IMAGE - REQUIRED                            
            # ---------------------------------------------------------------------------------------------         
            sourceimageinfo2upu = {}
             
            message2print = "{(Step 25A/70) trying" + "; sourceimageinfo2upu-trying-by-copy2image4upu: " + str(sourceimageinfo2upu)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print) 
            try:
                sourceimageinfo2upu = get2imageinfo4upu()                                                                         
                
                message2print = "{(Step 25B/70) got" + "; sourceimageinfo2upu-got-by-copy2image4upu: " + str(sourceimageinfo2upu)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)                                  
                #                 
                code2ack  = code2ack + '25B-'  
            except:
                message2print = "{(Step 25Z/70) failed" + "; sourceimageinfo2upu-failed-by-copy2image4upu: " + str(sourceimageinfo2upu) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)             
                code2nack  = code2nack + '25Z-'          
                      
            task4imagecopy2upu ['IMAGESOURCE'] = sourceimageinfo2upu         
        
        if 'IMAGEDESTINATION' in keys2task:
            pass
        else:                      
            # ---------------------------------------------------------------------------------------------
            # (26) DESTINATION IMAGE - REQUIRED                            
            # ---------------------------------------------------------------------------------------------         
            destinationimageinfo2upu = {}
             
            message2print = "{(Step 26A/70) trying" + "; destinationimageinfo2upu-trying-by-copy2image4upu: " + str(destinationimageinfo2upu)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print) 
            try:
                destinationimageinfo2upu = get2imageinfo4upu()                                                                         
                
                message2print = "{(Step 26B/70) got" + "; destinationimageinfo2upu-got-by-copy2image4upu: " + str(destinationimageinfo2upu)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)                                  
                #                 
                code2ack  = code2ack + '26B-'  
            except:
                message2print = "{(Step 26Z/70) failed" + "; destinationimageinfo2upu-failed-by-copy2image4upu: " + str(destinationimageinfo2upu) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)             
                code2nack  = code2nack + '26Z-'          
                      
            task4imagecopy2upu ['IMAGEDESTINATION'] = destinationimageinfo2upu    
                                    
        if 'QUEUE' in keys2task:
            pass
        else:   
            # ---------------------------------------------------------------------------------------------
            # (27) QUEUE - REQUIRED BY UPUXI                          
            # ---------------------------------------------------------------------------------------------         
            queue2upu = None
            
            message2print = "{(Step 27A/70) trying" + "; queue2upu-trying-by-copy2image4upu: " + str(queue2upu)  + "; time: " + time4xi() + "}" 
            print4xi(class2print, method2print, message2print) 
            try:
                queue2upu = Queue4Xi()                                                                          # THE SYSTEM CREATES QUEUE    
                
                message2print = "{(Step 27B/70) got" + "; queue2upu-got-by-copy2image4upu: " + str(queue2upu)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)                                  
                # 
                
                code2ack  = code2ack + '27B-'  
            except:
                message2print = "{(Step 27Z/70) failed" + "; queue2upu-failed-by-copy2image4upu: " + str(queue2upu) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)             
                code2nack  = code2nack + '27Z-'
                
            task4imagecopy2upu ['QUEUE'] = queue2upu

        if 'PIPE' in keys2task:
            pass
        else:              
            # ---------------------------------------------------------------------------------------------
            # (28) PIPE - REQUIRED BY UPUXI                              
            # ---------------------------------------------------------------------------------------------         
            pipe2upu = None
           
            message2print = "{(Step 28A/70) trying" + "; pipe2upu-trying-by-copy2image4upu: " + str(pipe2upu)  + "; time: " + time4xi() + "}" 
            print4xi(class2print, method2print, message2print) 
            try:
                pipe2upu = Pipe4Xi()                                                                            # THE SYSTEM CREATES PIPE    
                
                message2print = "{(Step 28B/70) got" + "; pipe2upu-got-by-copy2image4upu: " + str(pipe2upu)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)                                  
                # 
                
                code2ack  = code2ack + '28B-'  
            except:
                message2print = "{(Step 28Z/70) failed" + "; pipe2upu-failed-by-copy2image4upu: " + str(pipe2upu) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)             
                code2nack  = code2nack + '28Z-'
                
            task4imagecopy2upu ['PIPE'] = pipe2upu 
            
        if 'COMMANDQUEUE' in keys2task:
            pass
        else:              
            # ---------------------------------------------------------------------------------------------
            # (28) COMMAND QUEUE - REQUIRED                              
            # ---------------------------------------------------------------------------------------------             
            commandqueueinfo2upu = {}
           
            message2print = "{(Step 29A/70) trying" + "; commandqueueinfo2upu-trying-by-copy2image4upu: " + str(commandqueueinfo2upu)  + "; time: " + time4xi() + "}" 
            print4xi(class2print, method2print, message2print)  
                        
            try:
                commandqueueinfo2upu = get2commandqueueinfo4upu()                                               # SYSTEM CREATES COMMAND QUEUE, IF NECESSARY   
                message2print = "{(Step 29B/70) got" + "; commandqueueinfo2upu-got-by-copy2image4upu: " + str(commandqueueinfo2upu)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print) 
                # 
                
                code2ack  = code2ack + '29B-'  
            except:
                message2print = "{(Step 29Z/70) failed" + "; commandqueueinfo2upu-failed-by-copy2image4upu: " + str(commandqueueinfo2upu) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)             
                code2nack  = code2nack + '29Z-'
        
            task4imagecopy2upu ['COMMANDQUEUE'] = commandqueueinfo2upu
        
        
        if 'BUFFER' in keys2task:
            pass
        else:               
            # ---------------------------------------------------------------------------------------------
            # (30) BUFFER - AUTOMATIC BY UPUXI                               
            # ---------------------------------------------------------------------------------------------             
            bufferinfo2upu = {}
           
            message2print = "{(Step 30A/70) trying" + "; bufferinfo2upu-trying-by-copy2image4upu: " + str(bufferinfo2upu)  + "; time: " + time4xi() + "}" 
            print4xi(class2print, method2print, message2print) 
                        
            try:
                bufferinfo2upu = get2bufferinfo4upu()                                                           # SYSTEM CREATES BUFFER FOR UPU, IF NECESSARY
                message2print = "{(Step 30B/70) got" + "; bufferinfo2upu-got-by-copy2image4upu: " + str(bufferinfo2upu)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print) 
                # 
                
                code2ack  = code2ack + '30B-' 
            except:
                message2print = "{(Step 30Z/70) failed" + "; bufferinfo2upu-failed-by-copy2image4upu: " + str(bufferinfo2upu) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)             
                code2nack  = code2nack + '30Z-'
        
            task4imagecopy2upu ['BUFFER'] = bufferinfo2upu
             
        if 'EVENT' in keys2task:
            pass
        else:               
            # ---------------------------------------------------------------------------------------------
            # (31) EVENT - REQUIRED                            
            # ---------------------------------------------------------------------------------------------             
            eventinfo2upu = {}
           
            message2print = "{(Step 31A/70) trying" + "; eventinfo2upu-trying-by-copy2image4upu: " + str(eventinfo2upu)  + "; time: " + time4xi() + "}" 
            print4xi(class2print, method2print, message2print) 
                        
            try:
                eventinfo2upu = get2eventinfo4upu()                                                                    
                message2print = "{(Step 31B/70) got" + "; eventinfo2upu-got-by-copy2image4upu: " + str(eventinfo2upu)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print) 
                # 
                
                code2ack  = code2ack + '31B-' 
            except:
                message2print = "{(Step 31Z/70) failed" + "; eventinfo2upu-failed-by-copy2image4upu: " + str(eventinfo2upu) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)             
                code2nack  = code2nack + '31Z-'
        
            task4imagecopy2upu ['EVENT'] = eventinfo2upu 
        
             
        if 'EVENTWAITLIST' in keys2task:
            pass
        else:               
            # ---------------------------------------------------------------------------------------------
            # (32) EVENTWAITLIST - REQUIRED                              
            # ---------------------------------------------------------------------------------------------             
            eventwaitlist2upu = {}
           
            message2print = "{(Step 32A/70) trying" + "; eventwaitlist2upu-trying-by-copy2image4upu: " + str(eventwaitlist2upu)  + "; time: " + time4xi() + "}" 
            print4xi(class2print, method2print, message2print) 
                        
            try:
                eventwaitlist2upu = {}                                                                   
                message2print = "{(Step 32B/70) set" + "; eventwaitlist2upu-set-by-copy2image4upu: " + str(eventwaitlist2upu)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print) 
                # 
                
                code2ack  = code2ack + '32B-' 
            except:
                message2print = "{(Step 32Z/70) failed" + "; eventwaitlist2upu-failed-by-copy2image4upu: " + str(eventwaitlist2upu) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)             
                code2nack  = code2nack + '32Z-'
        
            task4imagecopy2upu ['EVENTWAITLIST'] = eventwaitlist2upu                     
              
        if 'EVENTWAITLISTQUANTITY' in keys2task:
            pass
        else:               
            # ---------------------------------------------------------------------------------------------
            # (33) EVENTWAITLISTQUANTITY - REQUIRED                              
            # ---------------------------------------------------------------------------------------------
            eventwaitlistquantity2upu = 0
            
            message2print = "{(Step 33A/70) trying" + "; eventwaitlistquantity2upu-trying-by-copy2image4upu: " + str(eventwaitlistquantity2upu)  + "; time: " + time4xi() + "}" 
            print4xi(class2print, method2print, message2print) 
                        
            try:
                keys2event = eventwaitlist2upu.keys()
                for key2event in keys2event:
                    eventwaitlistquantity2upu = eventwaitlistquantity2upu + 1    
                
                message2print = "{(Step 33B/70) set" + "; eventwaitlistquantity2upu-set-by-copy2image4upu: " + str(eventwaitlistquantity2upu)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print) 
                # 0 
                
                code2ack  = code2ack + '33B-' 
            except:
                message2print = "{(Step 33Z/70) failed" + "; eventwaitlistquantity2upu-failed-by-copy2image4upu: " + str(eventwaitlistquantity2upu) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)             
                code2nack  = code2nack + '33Z-'
                    
            task4imagecopy2upu ['EVENTWAITLISTQUANTITY'] = eventwaitlistquantity2upu       
                                             
                    
        if 'SOURCEORIGIN' in keys2task:
            pass
        else:                    
            # ---------------------------------------------------------------------------------------------
            # (34) SOURCEORIGIN - REQUIRED                             
            # ---------------------------------------------------------------------------------------------             
            source4origin2upu = {}
            
            message2print = "{(Step 34A/70) trying" + "; source4origin2upu-trying-by-copy2image4upu: " + str(source4origin2upu)  + "; time: " + time4xi() + "}" 
            print4xi(class2print, method2print, message2print) 
                        
            try:                                                                                    
                source4origin2upu['X'] = 0 
                source4origin2upu['Y'] = 0  
                source4origin2upu['Z'] = 0                                                                                                                          
                message2print = "{(Step 34B/70) set" + "; source4origin2upu-set-by-copy2image4upu to: " + str(source4origin2upu)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print) 
                # 
                code2ack  = code2ack + '34B-' 
            except:
                message2print = "{(Step 34Z/70) failed" + "; source4origin2upu-failed-by-copy2image4upu: " + str(source4origin2upu) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)             
                code2nack  = code2nack + '34Z-'
        
            task4imagecopy2upu ['SOURCEORIGIN'] = source4origin2upu
             
        if 'DESTINATIONORIGIN' in keys2task:
            pass
        else:                    
            # ---------------------------------------------------------------------------------------------
            # (35) DESTINATIONORIGIN - REQUIRED                             
            # ---------------------------------------------------------------------------------------------             
            destination4origin2upu = {}
            
            message2print = "{(Step 35A/70) trying" + "; destination4origin2upu-trying-by-copy2image4upu: " + str(destination4origin2upu)  + "; time: " + time4xi() + "}" 
            print4xi(class2print, method2print, message2print) 
                        
            try:                                                                                    
                destination4origin2upu['X'] = 100 
                destination4origin2upu['Y'] = 100  
                destination4origin2upu['Z'] = 100                                                                                                                          
                message2print = "{(Step 35B/70) set" + "; destination4origin2upu-set-by-copy2image4upu to: " + str(destination4origin2upu)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print) 
                # 
                code2ack  = code2ack + '35B-' 
            except:
                message2print = "{(Step 35Z/70) failed" + "; destination4origin2upu-failed-by-copy2image4upu: " + str(destination4origin2upu) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)             
                code2nack  = code2nack + '35Z-'
        
            task4imagecopy2upu ['DESTINATIONSOURCEORIGIN'] = destination4origin2upu                
             
        if 'REGION' in keys2task:
            pass
        else:            
            # ---------------------------------------------------------------------------------------------
            # (36) REGION - REQUIRED                         
            # ---------------------------------------------------------------------------------------------             
            region2upu = {}
            
            message2print = "{(Step 36A/70) trying" + "; region2upu-trying-by-copy2image4upu: " + str(region2upu)  + "; time: " + time4xi() + "}" 
            print4xi(class2print, method2print, message2print) 
                        
            try:                                                                                    
                region2upu['X'] = 160                                                                               # WIDTH  IN PIXELS TO WRITE 
                region2upu['Y'] = 90                                                                                # HEIGHT IN PIXELS TO WRITE
                region2upu['Z'] = 0                                                                                 # DEPTH  IN PIXELS TO WRITE                                                                                                                          
                message2print = "{(Step 36B/70) set" + "; region2upu-set-by-copy2image4upu to: " + str(region2upu)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print) 
                # 
                code2ack  = code2ack + '36B-' 
            except:
                message2print = "{(Step 36Z/70) failed" + "; region2upu-failed-by-copy2image4upu: " + str(region2upu) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)             
                code2nack  = code2nack + '36Z-'
        
            task4imagecopy2upu ['REGION'] = region2upu
        
        # ---------------------------------------------------------------------------------------------
        #  EOF PARAMETER CHECKING AND PROVIDING FOR MISSING ONES
        # ---------------------------------------------------------------------------------------------
        keys2task = task4imagecopy2upu.keys()
        message2print = "{(Step 39/70) set" + "; keys2task-set-by-copy2image4upu to: " + str(sorted(keys2task))  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)            
        # ['BUFFER',
        #  'COMMANDQUEUE',
        #  'CONTEXT',
        #  'DATA2CALLBACK',
        #  'DESTINATIONORIGIN',
        #  'DEVICE',
        #  'EVENT',
        #  'EVENTWAITLIST',
        #  'EVENTWAITLISTQUANTITY',
        #  'FUNCTION2CALLBACK',
        #  'IMAGEDESTINATION',
        #  'IMAGESOURCE',
        #  'PIPE',
        #  'PLATFORM',
        #  'QUEUE',
        #  'REGION',
        #  'SOURCEORIGIN']; time: 01:28:44 10/05/11 Pacific Daylight Time}
    except: 
        message2print = "{(Step 39Z/70) yielded" + "; task4imagecopy2upu-yielded-by-copy2image4upu: " + str(task4imagecopy2upu) + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)  
        # 
        
        code2nack  = code2nack + '39Z-' 

    # ---------------------------------------------------------------------------------------------
    # CONTROL AND COMMAND
    # ---------------------------------------------------------------------------------------------
    control2copyimage4upu = {}       
                                
    # ---------------------------------------------------------------------------------------------
    # CONSUMER AND PUBLISHER CHANNEL FOR CONTROL AND COMMAND
    # ---------------------------------------------------------------------------------------------
    ack4channels2upu = False
    channels2upu = ()
    message2print = "{(Step 40A/70) trying" + "channels2upu-trying-by-copy2image4upu: " + str(channels2upu)  + "; time: " + time4xi() + "}" 
    print4xi(class2print, method2print, message2print)     
    try:
        # 
        consumerchannel2upu, publisherchannel2upu = task4imagecopy2upu ['PIPE']

        message2print = "{(Step 40B/70) got" + "; consumerchannel2upu-got-by-copy2image4upu: " + str(consumerchannel2upu)  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)            
        # <read-write PipeConnection, handle 612> 

        message2print = "{(Step 40C/70) got" + "; publisherchannel2upu-got-by-copy2image4upu: " + str(publisherchannel2upu)  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)            
        # <read-write PipeConnection, handle 616>
                              
        ack4channels2upu = True
        
        control2copyimage4upu['CONSUMERCHANNEL']  = consumerchannel2upu
        control2copyimage4upu['PUBLISHERCHANNEL'] = publisherchannel2upu
        
        code2ack  = code2ack + '40D-' 
        
    except:
        ack4channels2upu = False
        code2nack  = code2nack + '40Z-'      
                        
    # ---------------------------------------------------------------------------------------------
    # QUEUE FOR CONTROL AND COMMAND
    # ---------------------------------------------------------------------------------------------
    ack4queue2upu = False
    queue2upu = None
    message2print = "{(Step 41A/70) trying" + "queue2upu-trying-by-copy2image4upu: " + str(queue2upu)  + "; time: " + time4xi() + "}" 
    print4xi(class2print, method2print, message2print)     
    try:
        queue2upu = task4imagecopy2upu ['QUEUE']

        message2print = "{(Step 41B/70) got" + "; queue2upu-got-by-copy2image4upu: " + str(queue2upu)  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)            
        # <queue4xi.Queue4Xi object at 0x0000000004721CF8>
              
        ack4queue2upu = True
        control2copyimage4upu['QUEUE']  = queue2upu        
        code2ack  = code2ack + '41C-'
    except:
        ack4queue2upu = False  
        code2nack  = code2nack + '41Z-'
    
    
    # -----------------------------------------------------------------------------------------    
    # DUMMY PROCESS OBJECT = THREAD CREATION
    # -----------------------------------------------------------------------------------------           
    # dummy2image4upu = None
    # message2print = "{(Step 69A/70) trying" + "dummy2image4upu-trying-by-copy2image4upu: " + str(dummy2image4upu)  + "; time: " + time4xi() + "}"   
    # print4xi(class2print, method2print, message2print)     
    # try:                            
        # dummy2image4upu = DummyProcess4Xi(target=manoeuvre2copyimage4upu,                                               # in this module to handle queue and pipe channels
        #                                   args =(queue2upu,                                                         # caller puts and gets - does NOT call any functions
        #                                          consumerchannel2upu, publisherchannel2upu))                        # caller sends and receives - does NOT call any functions
        # message2print = "{(Step 69B/70) created" + "dummy2image4upu-created-by-copy2image4upu: " + str(dummy2image4upu)  + "; time: " + time4xi() + "}" 
        # print4xi(class2print, method2print, message2print)  
        # code2ack  = code2ack + '69B-'
        
    # except:
        # code2nack  = code2nack + '69Z-'
        
    # ---------------------------------------------------------------------------------------------
    # RETURN CONTROL AND COMMAND DICTIONARY
    # ---------------------------------------------------------------------------------------------          
    message2print = "{(Step 70A/70) set" + "; code2ack-set-by-copy2image4upu: " + str(code2ack) + "; time: " + time4xi() + "}"  
    print4xi(class2print, method2print, message2print)                                                             
    # 1B1-1B2-1C-2B-3B-4B-5B-6B-7B-8B-9B-10B-11B-12B-13B-14B-15B-16B-40D-41C- 
    message2print = "{(Step 70B/70) set" + "; code2nack-set-by-copy2image4upu: " + str(code2nack) + "; time: " + time4xi() + "}" 
    print4xi(class2print, method2print, message2print)                                                             
    #  
        
    message2print = "{(Step 70C/70) returning" + "; control2copyimage4upu-returning-by-copy2image4upu: " + str(control2copyimage4upu) + "; time: " + time4xi() + "}" 
    print4xi(class2print, method2print, message2print)                                                             
    # {'QUEUE': <queue4xi.Queue4Xi object at 0x0000000004721CF8>,
    #  'PUBLISHERCHANNEL': <read-write PipeConnection, handle 616>,
    #  'CONSUMERCHANNEL': <read-write PipeConnection, handle 612>}; time: 01:28:44 10/05/11 Pacific Daylight Time}
   
    return control2copyimage4upu
                    
#</function4xi> 


# -------------------------------------------------------------------------------------------------
#<function4xi function = "copy2imagetobuffer4upu" usage = "
#                                                                                                              
#   # ---------------------------------------------------------------------------------------------
#   # (60) ESTABLISH CONTROL & COMMAND WITH COPY2IMAGETOBUFFER4UPU                      
#   # ---------------------------------------------------------------------------------------------    
#   print('(Step 60A/700) OF UPU2IMAGE4XI5TEST - TRYING TO ESTABLISH CONTROL & COMMAND WITH COPY2IMAGETOBUFFER4UPU ', time4xi())    
#   response4copy2imagetobuffer = {} 
#   task4copy2imagetobuffer = None
#   try:                               
#       response4copy2imagetobuffer = copy2imagetobuffer4upu(task4copy2imagetobuffer)                                                                                                          
#       print('(Step 60B/700) OF UPU2IMAGE4XI5TEST - ESTABLISHED CONTROL & COMMAND WITH COPY2IMAGETOBUFFER4UPU : ', response4copy2imagetobuffer)
#       queue2copyimagetobuffer4upu      = response4copy2imagetobuffer['QUEUE']
#       channel2copyimagetobufferpublish4upu = response4copy2imagetobuffer['PUBLISHERCHANNEL']
#       channel2copyimagetobufferconsume4upu = response4copy2imagetobuffer['CONSUMERCHANNEL']
#   except:
#       print('(Step 60Z/700) OF UPU2IMAGE4XI5TEST *** FAILED TO ESTABLISH CONTROL & COMMAND WITH COPY2IMAGETOBUFFER4UPU ', time4xi()) 
#   # --------------------------------------------------------------------------------------------------
#   # (61) SEND CONTROL & COMMAND COPY VIA PIPE FOR MANOEUVRE2COPYIMAGETOBUFFER4UPU TO RECEIVE                 
#   # --------------------------------------------------------------------------------------------------      
#   print('(Step 61A/700) OF UPU2IMAGE4XI5TEST - TRYING TO SEND CONTROL & COMMAND VIA PIPE WITH COPY2IMAGETOBUFFER4UPU ', time4xi())     
#   try:                               
#       message2copyimagetobuffersend4upu = "<copy4xi>copy2copyimagetobuffer4upu</copy4xi>"
#       channel2copyimagetobufferpublish4upu = response4copy2imagetobuffer['PUBLISHERCHANNEL']
#       channel2copyimagetobufferpublish4upu.send(message2copyimagetobuffersend4upu)
#       print("(Step 61B/700) OF UPU2IMAGE4XI5TEST - SENT TO CHANNEL2PUBLISH4UPU: ", channel2copyimagetobufferpublish4upu, " MESSAGE2SEND4UPU: " , message2copyimagetobuffersend4upu)                                                                                                
#   except:
#       print('(Step 61Z1/700) OF UPU2IMAGE4XI5TEST *** FAILED TO SEND CONTROL & COMMAND VIA PIPE WITH COPY2IMAGETOBUFFER4UPU ', time4xi())                                                
#   # ------------------------------------------------------------------------------------------
#   # (62) LET MANOEUVRE2COPYIMAGETOBUFFER4UPU TO RECEIVE COPY AND AND SEND ROGER VIA PIPE                
#   # ------------------------------------------------------------------------------------------       
#   print("(Step 62A/700) OF UPU2IMAGE4XI5TEST - TRYING TO PERFORM TARGET FUNCTION MANOEUVRE2COPYIMAGETOBUFFER4UPU ", time4xi()) 
#   try:        
#       response2manoeuvreimagecopytobuffer4upu = manoeuvre2copyimagetobuffer4upu(queue2copyimagetobuffer4upu, channel2copyimagetobufferconsume4upu, channel2copyimagetobufferpublish4upu)
#       print("(Step 62B/700) OF UPU2IMAGE4XI5TEST - PERFORMED TARGET FUNCTION MANOEUVRE2COPYIMAGETOBUFFER4UPU ", manoeuvre2copyimagetobuffer4upu)
#   except:
#       print("(Step 62Z/700) OF UPU2IMAGE4XI5TEST *** FAILED TO PERFORM TARGET FUNCTION MANOEUVRE2COPYIMAGETOBUFFER4UPU ", time4xi())     
#   else:
#       pass # trust activated thread    
#   # ---------------------------------------------------------------------------------------------
#   # (63) RECEIVE CONTROL & COMMAND ROGER VIA PIPE                   
#   # ---------------------------------------------------------------------------------------------          
#   print('(Step 63A/700) OF UPU2IMAGE4XI5TEST - TRYING TO RECEIVE CONTROL & COMMAND VIA PIPE WITH COPY2IMAGETOBUFFER4UPU ', time4xi())     
#   try:
#       channel2copyimagetobufferconsume4upu = response4copy2imagetobuffer['CONSUMERCHANNEL']                              
#       message2copyimagereceive4upu = channel2copyimagetobufferconsume4upu.recv()
#       print("(Step 63B/700) OF UPU2IMAGE4XI5TEST - RECEIVED FROM CHANNEL2CONSUME4UPU: ", channel2copyimagetobufferconsume4upu, " CONTROL & COMMAND: ", message2copyimagereceive4upu)                                                                                                  
#       # 
#   except:
#       print('(Step 63Z/700) OF UPU2IMAGE4XI5TEST *** FAILED TO RECEIVE CONTROL & COMMAND VIA PIPE  WITH COPY2IMAGETOBUFFER4UPU ', time4xi())         
#   # ---------------------------------------------------------------------------------------------------
#   # (64) SEND CONTROL & COMMAND READY VIA PIPE FOR MANOEUVRE2COPYIMAGETOBUFFER4UPU TO RECEIVE                
#   # ---------------------------------------------------------------------------------------------------      
#   print('(Step 64A/700) OF UPU2IMAGE4XI5TEST - TRYING TO SEND CONTROL & COMMAND VIA PIPE WITH COPY2IMAGETOBUFFER4UPU ', time4xi())     
#   try:                               
#       message2copyimagetobuffersend4upu = "<copy4xi>copy2copyimagetobuffer4upu</copy4xi>"
#       channel2copyimagetobufferpublish4upu = response4copy2imagetobuffer['PUBLISHERCHANNEL']
#       channel2copyimagetobufferpublish4upu.send(message2copyimagetobuffersend4upu)
#       print("(Step 64B/700) OF UPU2IMAGE4XI5TEST - SENT TO CHANNEL2PUBLISH4UPU: ", channel2copyimagetobufferpublish4upu, " MESSAGE2SEND4UPU: " , message2copyimagetobuffersend4upu)                                                                                                
#       #  
#   except:
#       print('(Step 64Z/700) OF UPU2IMAGE4XI5TEST *** FAILED TO SEND CONTROL & COMMAND VIA PIPE WITH COPY2IMAGETOBUFFER4UPU ', time4xi())                                                
#   # -----------------------------------------------------------------------------------
#   # (65) LET MANOEUVRE2COPYIMAGETOBUFFER4UPU TO RECEIVE READY AND TO SEND READY                
#   # -----------------------------------------------------------------------------------       
#   print("(Step 65A/700) OF UPU2IMAGE4XI5TEST - TRYING TO PERFORM TARGET FUNCTION MANOEUVRE2COPYIMAGETOBUFFER4UPU ", time4xi()) 
#   try:        
#       response2manoeuvreimagecopytobuffer4upu = manoeuvre2copyimagetobuffer4upu(queue2copyimagetobuffer4upu, channel2copyimagetobufferconsume4upu, channel2copyimagetobufferpublish4upu)
#       print("(Step 65B/700) OF UPU2IMAGE4XI5TEST - PERFORMED TARGET FUNCTION MANOEUVRE2COPYIMAGETOBUFFER4UPU ", manoeuvre2copyimagetobuffer4upu)
#   except:
#       print("(Step 65Z/700) OF UPU2IMAGE4XI5TEST *** FAILED TO PERFORM TARGET FUNCTION MANOEUVRE2COPYIMAGETOBUFFER4UPU ", time4xi())     
#   # ---------------------------------------------------------------------------------------
#   # (66) RECEIVE CONTROL & COMMAND READY VIA PIPE                      
#   # ---------------------------------------------------------------------------------------          
#   print('(Step 66A/700) OF UPU2IMAGE4XI5TEST - TRYING TO RECEIVE CONTROL & COMMAND VIA PIPE WITH COPY2IMAGETOBUFFER4UPU ', time4xi())     
#   try:
#       channel2copyimagetobufferconsume4upu = response4copy2imagetobuffer['CONSUMERCHANNEL']                              
#       message2copyimagereceive4upu = channel2copyimagetobufferconsume4upu.recv()
#       print("(Step 66B/700) OF UPU2IMAGE4XI5TEST - RECEIVED FROM CHANNEL2CONSUME4UPU: ", channel2copyimagetobufferconsume4upu, " CONTROL & COMMAND: ", message2copyimagereceive4upu)                                                                                                  
#       # 
#   except:
#       print('(Step 66Z/700) OF UPU2IMAGE4XI5TEST *** FAILED TO RECEIVE CONTROL & COMMAND VIA PIPE  WITH COPY2IMAGETOBUFFER4UPU ', time4xi())          
#   # -----------------------------------------------------------------------------------------
#   # (67) PUT IMAGE PATH TO QUEUE
#   # -----------------------------------------------------------------------------------------                          
#   print("(Step 67A/700) OF UPU2IMAGE4XI5TEST - TRYING TO PUT IMAGE PATH TO QUEUE ", time4xi())      
#   try:    
#       queue2copyimagetobuffer4dragon.put(path2dragon)             
#       print("(Step 67B/700) OF DRAGON2IMAGE4XI5TEST - PUT TO QUEUE IMAGE PATH: ", path2dragon) 
#       # ..//..//..//..//..//web//images//png//dragonxi.png  
#   except:
#       print("(Step 67Z/700) OF UPU2IMAGE4XI5TEST *** FAILED TO PUT IMAGE PATH TO QUEUE ", time4xi()) 
#   # ------------------------------------------------------------------------------------------------------
#   # (68) LET MANOEUVRE2COPYIMAGETOBUFFER4UPU TO GET FROM QUEUE AND TO SEND EITHER ACK OR NACK VIA PIPE               
#   # -------------------------------------------------------------------------------------------------------       
#   print("(Step 68A/700) OF UPU2IMAGE4XI5TEST - TRYING TO PERFORM TARGET FUNCTION MANOEUVRE2COPYIMAGETOBUFFER4UPU ", time4xi()) 
#   try:        
#       response2manoeuvreimagecopytobuffer4upu = manoeuvre2copyimagetobuffer4upu(queue2copyimagetobuffer4upu, channel2copyimagetobufferconsume4upu, channel2copyimagetobufferpublish4upu)
#       print("(Step 68B/700) OF UPU2IMAGE4XI5TEST - PERFORMED TARGET FUNCTION MANOEUVRE2COPYIMAGETOBUFFER4UPU ", manoeuvre2copyimagetobuffer4upu)
#   except:
#       print("(Step 68Z/700) OF UPU2IMAGE4XI5TEST *** FAILED TO PERFORM TARGET FUNCTION MANOEUVRE2COPYIMAGETOBUFFER4UPU ", time4xi())     
#   # -----------------------------------------------------------------------------------------------
#   # (69) RECEIVE ACK OR NACK ABOUT IMAGE OPERATION VIA PIPE                
#   # -----------------------------------------------------------------------------------------------          
#   print('(Step 69A/700) OF UPU2IMAGE4XI5TEST - TRYING TO RECEIVE CONTROL & COMMAND ABOUT IMAGE OPERTION VIA PIPE ', time4xi())      
#   try: 
#       message2copyimagereceive4upu = channel2copyimagetobufferconsume4upu.recv()
#       print("(Step 69D/700) OF UPU2IMAGE4XI5TEST - RECEIVED FROM CHANNEL2CONSUME4UPU: ", channel2copyimagetobufferconsume4upu, " CONTROL & COMMAND: ", message2copyimagereceive4upu)                                                                                                   
#   except:
#       print('(Step 69Z/700) OF UPU2IMAGE4XI5TEST *** FAILED TO RECEIVE CONTROL & COMMAND ABOUT IMAGE OPERTION VIA PIPE ', time4xi())               
#   # -------------------------------------------------------------------------------------------------------
#   # (70) LET MANOEUVRE2COPYIMAGETOBUFFER4UPU GO TO STATE 0               
#   # -------------------------------------------------------------------------------------------------------       
#   print("(Step 70A/700) OF UPU2IMAGE4XI5TEST - TRYING TO PERFORM TARGET FUNCTION MANOEUVRE2COPYIMAGETOBUFFER4UPU ", time4xi()) 
#   try:        
#       response2manoeuvreimagecopytobuffer4upu = manoeuvre2copyimagetobuffer4upu(queue2copyimagetobuffer4upu, channel2copyimagetobufferconsume4upu, channel2copyimagetobufferpublish4upu)
#       print("(Step 70B/700) OF UPU2IMAGE4XI5TEST - GOT RESPONSE2MANOEUVRE4UPU ", response2manoeuvreimagecopytobuffer4upu)
#   except:
#       print("(Step 70Z/700) OF UPU2IMAGE4XI5TEST *** FAILED TO PERFORM TARGET FUNCTION MANOEUVRE2COPYIMAGETOBUFFER4UPU ", time4xi())   

#">

# -------------------------------------------------------------------------------------------------   
def copy2imagetobuffer4upu(task4copyimagetobuffer2upu=None):

    '''
    copies 2D images to buffer for UPU
    '''
    global upu2image4xi                                                                                             # 2D-IMAGES FOR UPU  
    global upu2buffer4xi                                                                                            # BUFFER FOR UPU
    
    global control2copyimagetobuffer4upu                                                                            # COMMAND AND CONTROL FOR UPU IMAGES 
    
    # -----------------------------------------------------------------------------------------
    # (Step 1/33) PARAMETERS           
    # -----------------------------------------------------------------------------------------          
    class2print = "copy2imagetobuffer4upu"
    method2print = "copy2imagetobuffer4upu"

    message2print = "{(Step 1A/33) started" + "; copy2imagetobuffer4upu-started: " + str(copy2imagetobuffer4upu)  + "; time: " + time4xi() + "}" 
    print4xi(class2print, method2print, message2print)                                                                 
    # <function copy2imagetobuffer4upu at 0x000000000390B948>
    
    message2print = "{(Step 1B/33) passed" + "; task4copyimagetobuffer2upu-to-copy2imagetobuffer4upu: " + str(task4copyimagetobuffer2upu) + "; time: " + time4xi() + "}"
    print4xi(class2print, method2print, message2print)                                                              
    # None
            
    code2ack  = ' '
    code2nack = ' '
    
    # -----------------------------------------------------------------------------------------
    # CREATE UPU2IMAGE4XI IF NECESSARY       
    # ----------------------------------------------------------------------------------------- 
    try:
        if upu2image4xi is None:
            upu2image4xi = create2image4upu()                                                                              # INITIALIZE IMAGE FOR UPU 
            message2print = "{(Step 1C/70) created" + "; upu2image4xi-created-by-copy2imagetobuffer4upu to: " + str(upu2image4xi)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)
            #             
        else:
            message2print = "{(Step 1D/70) skipped" + "; create2image4upu-skipped-by-copy2imagetobuffer4upu: " + str(copy2imagetobuffer4upu) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)
            # create2image4upu-skipped-by-copy2imagetobuffer4upu                                 
    except:
        try:
            upu2image4xi = create2image4upu()
            message2print = "{(Step 1E/70) set" + "; upu2image4xi-set-by-copy2imagetobuffer4upu to: " + str(upu2image4xi)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)
            #             
        except:
            upu2image4xi = {}
            message2print = "{(Step 1Z1/70) yielded" + "; upu2image4xi-yielded-by-copy2imagetobuffer4upu: " + str(upu2image4xi) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)  
            code2nack  = code2nack + '1Z1-' 
    
    # -----------------------------------------------------------------------------------------
    # CREATE UPU2BUFFER4XI IF NECESSARY       
    # ----------------------------------------------------------------------------------------- 
    try:
        if upu2buffer4xi is None:
            upu2buffer4xi = create2buffer4upu()                                                                     # INITIALIZE BUFFER FOR UPU 
            message2print = "{(Step 1F/70) created" + "; upu2buffer4xi-created-by-copy2imagetobuffer4upu to: " + str(upu2buffer4xi)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)
            #             
        else:
            message2print = "{(Step 1G/70) skipped" + "; create2buffer4upu-skipped-by-copy2imagetobuffer4upu: " + str(copy2imagetobuffer4upu) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)
            #                                  
    except:
        try:
            upu2buffer4xi = create2buffer4upu()
            message2print = "{(Step 1H/70) set" + "; upu2buffer4xi-set-by-copy2imagetobuffer4upu to: " + str(upu2buffer4xi)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)
            # {'CODE2ERROR': 0,
            #  'COUNT2REFERENCE': 0,
            #   'DATA2CALLBACK': {'MESSAGE': '{(Step 1Z1/33) failed; create2buffer4upu-failed ; time: 03:40:47 10/04/11 Pacific Daylight Time}',
            #   'CLASS': 'create2buffer4upu',
            #   'METHOD': 'create2buffer4upu'},
            #   'FUNCTION2CALLBACK': <function trace2buffer4upu at 0x0000000004782B48>,
            #   'POINTER2BUFFER': <mmap.mmap object at 0x000000000423BD98>,
            #   'FLAGS': 0,
            #   'CONTEXT': {'COUNT2REFERENCE': 0,
            #   'DATA2CALLBACK': ...
            
        except:
            upu2buffer4xi = {}
            message2print = "{(Step 1Z2/70) yielded" + "; upu2buffer4xi-yielded-by-copy2imagetobuffer4upu: " + str(upu2buffer4xi) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)  
            code2nack  = code2nack + '1Z2-' 
                            
    # -----------------------------------------------------------------------------------------
    # (Step 1/70) - ANALYZE TASK          
    # -----------------------------------------------------------------------------------------          
              
    if task4copyimagetobuffer2upu is None:
        # -----------------------------------------------------------------------------------------
        # NO PARAMETERS GIVEN BY CALLER FOR COPY - BUILD TASK! 
        # -----------------------------------------------------------------------------------------    
            
        task4copyimagetobuffer2upu = {}                                                                                                                                                                                       
            
        message2print = "{(Step 1A/70) initialized" + "; task4copyimagetobuffer2upu-created-by-copy2imagetobuffer4upu to: " + str(task4copyimagetobuffer2upu)  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)               
        # 
            
        # ---------------------------------------------------------------------------------------------
        # (1B) TRACE - AUTOMATIC                      
        # ---------------------------------------------------------------------------------------------              
        function2callback4upu = trace2image4upu                                                                 # CALLBACK FUNCTION FOR UPU
            
        message2print = "{(Step 1B1/70) initialized" + "; function2callback4upu-created-by-copy2imagetobuffer4upu to: " + str(function2callback4upu)  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)            
        # <function trace2image4upu at 0x0000000004797548> 
            
        code2ack  = code2ack + '1B1-'
           
        args2callback = {}                                                                                      # CREATE ARGUMENT DICTIONARY FOR CALLBACK FUNCTION             
        args2callback['CLASS']   = "copy2imagetobuffer4upu"                                                            # MODULE OF UPUIZED OPENCL MODULE
        args2callback['METHOD']  = "copy2imagetobuffer4upu"                                                            # FUNCTION OF UPUIZED OPENCL MODULE
        args2callback['MESSAGE'] = "{(Step 1Z1/33) failed" + "; copy2imagetobuffer4upu-failed " + "; time: " + time4xi() + "}" # ERROR MESSAGE FOR UPUIZED OPENCL TO RETURN    

        message2print = "{(Step 1B2/70) initialized" + "; args2callback-created-by-copy2imagetobuffer4upu to: " + str(args2callback)  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)               
        # {'MESSAGE': '{(Step 1Z1/33) failed; copy2imagetobuffer4upu-failed; time: 03:40:47 10/04/11 Pacific Daylight Time}', 
        #  'CLASS': 'copy2imagetobuffer4upu', 
        #  'METHOD': 'copy2imagetobuffer4upu'}; time: 03:40:47 10/04/11 Pacific Daylight Time}


        code2ack  = code2ack + '1B2-'
            
        # ---------------------------------------------------------------------------------------------
        # (1C) CALLBACK - AUTOMATIC                      
        # ---------------------------------------------------------------------------------------------              
        task4copyimagetobuffer2upu['DATA2CALLBACK'] = args2callback                                                    # USER DATA PASSED TO UPUIZED OPENCL - SEVERAL ARGS PASSED IN ONE DICTIONARY
        task4copyimagetobuffer2upu['FUNCTION2CALLBACK'] = function2callback4upu                                        # CALLBACK FUNCTION TO CALLED BY OPENCL IN EXCEPTIONS
        
        code2ack  = code2ack + '1C-'
        message2print = "{(Step 1C/70) initialized" + "; task4copyimagetobuffer2upu-created-by-copy2imagetobuffer4upu to: " + str(task4copyimagetobuffer2upu)  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)  
        # {'FUNCTION2CALLBACK': <function trace2image4upu at 0x0000000004797548>,
        #   'DATA2CALLBACK': {'MESSAGE': '{(Step 1Z1/33) failed; copy2imagetobuffer4upu-failed ; time: 03:40:47 10/04/11 Pacific Daylight Time}', 
        #  'CLASS': 'copy2imagetobuffer4upu', 
        #  'METHOD': 'copy2imagetobuffer4upu'}}; time: 03:40:47 10/04/11 Pacific Daylight Time}

            
        # ---------------------------------------------------------------------------------------------
        # (2) PLATFORM - AUTOMATIC                   
        # ---------------------------------------------------------------------------------------------             
        platform2xi = {}
        message2print = "{(Step 2A/70) initialized" + "; platform2xi-created-by-copy2imagetobuffer4upu to: " + str(platform2xi)  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)   
        # {}
            
        try:
            platform2xi = get2platformids4xi()                                                                  # PLATFORM IS COMMON TO ALL DEVICES I.E TO DEVICE XI
            message2print = "{(Step 2B/70) got" + "; platform2xi-got-by-copy2imagetobuffer4upu: " + str(platform2xi)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)
            # {'list2platforms': ['DragonXi'], 'ack2return': ' 2C-2E-2F-2G-', 'nack2return': ' ', 'available2platform': 1, 'max2entries': 1}; time: 03:40:47 10/04/11 Pacific Daylight Time}
                 
            code2ack  = code2ack + '2B-'
        except:
            message2print = "{(Step 2Z/70) failed" + "; platform2xi-failed-by-copy2imagetobuffer4upu: " + str(failed)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)                                                
            code2nack  = code2nack + '2Z-'
                 
        task4copyimagetobuffer2upu['PLATFORM'] = platform2xi                                                           # PLATFORM OF SYSTEM 
              
            
        # ---------------------------------------------------------------------------------------------
        # (3) CONTEXT - AUTOMATIC                 
        # ---------------------------------------------------------------------------------------------    
        context2upu = {}
        message2print = "{(Step 3A/70) trying" + "; context2upu-trying-by-copy2imagetobuffer4upu: " + str(context2upu)  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print) 
        try:
            context2upu = get2context4upu()                                                                     # NO PARAMETERS - THE SYSTEM CREATES CONTEXT IF REQUIRED  
                
            message2print = "{(Step 3B/70) got" + "; context2upu-got-by-copy2imagetobuffer4upu: " + str(context2upu)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)                                  
            # {'COUNT2REFERENCE': 0, 
            #  'DATA2CALLBACK': {'MESSAGE': '{(Step 1Z1/33) failed; create2context4upu-failed ; time: 03:40:47 10/04/11 Pacific Daylight Time}', 
            #  'CLASS': 'create2context4upu', 
            #  'METHOD': 'create2context4upu'}, 
            #  'FUNCTION2CALLBACK': <function trace2context4upu at 0x000000000394F348>, 
            #  'ACK2RETURN': ' 2A-2C-2D-2E-2B-3B-4B-5B-6B-7B-7B-', 
            #  'INFO2PRIVATE': None, 
            #  'PLATFORM': {'list2platforms': ['DragonXi'],'ack2return': ' 2C-2E-2F-2G-', 'nack2return': ' ', 'available2platform': 1, 'max2entries': 1}, 
            #  'NACK2RETURN': ' ', 
            #  'SIZE2PRIVATE': 0, 
            #  'PROPERTIES': None}; time: 03:40:47 10/04/11 Pacific Daylight Time}
                
            code2ack  = code2ack + '3B-'         
        except:
            message2print = "{(Step 3Z/70) failed" + "; context2upu-failed-by-copy2imagetobuffer4upu: " + str(context2upu) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)             
            code2nack  = code2nack + '3Z-'
            
        task4copyimagetobuffer2upu ['CONTEXT'] = context2upu           
            
        # ---------------------------------------------------------------------------------------------
        # (4) DEVICE - AUTOMATIC                       
        # ---------------------------------------------------------------------------------------------    
        device2upu = {}
            
        message2print = "{(Step 4A/70) trying" + "; device2upu-trying-by-copy2imagetobuffer4upu: " + str(device2upu)  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print) 
        try:
            device2upu = get2upu4xi()                                                                           # THE SYSTEM CREATES DEVICE INFO, IF NECESSARY    
            
            message2print = "{(Step 4B/70) got" + "; device2upu-got-by-copy2imagetobuffer4upu: " + str(device2upu)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)                                  
            # {'RESOLUTION2TIMER4PROFILING': 318, 
            #  'ID2PLATFORM': '0000000001509568', 
            #  'PROFILING2QUEUE': 'ACK', 
            #  'EXECUTE2KERNELS4OPENCL': 'ACK', 
            #  'NATIVE2WIDTHLONG4VECTOR': 2, 
            #  'MAX2UNITS4COMPUTE': 4, 
            #  'SIZE2MEMORY4GLOBAL': 3221225472, 
            #  'NATIVE2WIDTHFLOAT4VECTOR': 4, 
            #  'MAX2SIZE4WORKGROUP': 1024, 
            #  'PREFERRED2WIDTHCHAR4VECTOR': 16, 
            #  'SIZE2CACHE': 65536, 
            #  'IEEE754-2008FUSED2MULTIPLY-ADD': 'NACK', 
            #  'TYPE4CACHE': 'Read/Write', 
            #  'PREFERRED2WIDTHINT4VECTOR': 4, 
            #  'SIZE2PREFERREDWORKGROUPMULTIPLE4KERNEL': 1, 
            #  'NATIVE2WIDTHINT4VECTOR': 4, 
            #  'SIZE2BUFFER4CONSTANT': 65536, 
            #  'EXECUTE2FUNCTION4NATIVE': 'ACK', 
            #  'PREFERRED2WIDTHDOUBLE4VECTOR': 0, 
            #  'MAX2NUMBER4CONSTANTARGS': 8, 
            #  'VERSION2DRIVER': '2.0', 
            #  'EXTENSIONS': 'cl_amd_fp64 ...
            #  'AVAILABLE2COMPILER': 'ACK', 
            #  'MAX2DIMENSIONS4WORKITEMS': 3, 
            #  'SUPPORT2CORRECTION4ERROR': 0, 
            #  'BITS2ADDRESS': 64, 
            #  'AVAILABLE': 'ACK', 
            #  'SIZE2MEMORY4LOCAL': 32768, 
            #  'VENDOR': 'AuthenticAMD', 
            #  'SUPPORT2IMAGE': 'NACK', 
            #  'MAX2SIZE4KERNELARGUMENT': 4096, 
            #  'ACK2RETURN': ' 1C-1D-1E-1F-1D-2A-...
            #  'OUT-OF-ORDER2QUEUE': 'NACK', 
            #  'ALIGNMENT2ADDRESS4BASE': 1024, 
            #  'PROFILE': 'FULL_PROFILE', 
            #  'TYPE2MEMORY4LOCAL': 'Global', 
            #  'PREFERRED2WIDTHFLOAT4VECTOR': 4, 
            #  'PREFERRED2WIDTHLONG4VECTOR': 2, 
            #  'MAX2FREQUENCY4CLOCK': 3210, 
            #  'PREFERRED2WIDTHSHORT4VECTOR': 8, 
            #  'ID': 4033, 
            #  'MAX2WORKITEM2': 1024, 
            #  'NATIVE2WIDTHDOUBLE4VECTOR': 0, 
            #  'ROUND2ZERO': 'ACK', 
            #  'NAME': 'AMD64 Family 16 Model 4 Stepping 2, 
            #   AuthenticAMD', 
            #  'MAX2WORKITEM1': 1024, 
            #  'MAX2WORKITEM0': 1024, 
            #  'SIZE2LINE4CACHE': 64, 
            #  'MAX2ALLOCATION4MEMORY': 1073741824, 
            #  'NATIVE2WIDTHCHAR4VECTOR': 16, 
            #  'QUIET2NANS': 'ACK', 
            #  'NATIVE2WIDTHSHORT4VECTOR': 8, 
            #  'ROUND2NEAREST4EVEN': 'ACK', 
            #  'VERSION': 'OpenCL 1.1 ATI-Stream-v2.3 (451)', 
            #  'NACK2RETURN': ' ', 
            #  'MEMORY2UNIFIED4HOSTANDDEVICE': 1, 
            #  'ENDIANESS2DEVICE': 'Little', 
            #  'MINIMUM2ALIGNMENT4DATATYPE': 128, 
            #  'ROUND2+VE3INFINITY': 'ACK', 
            #  'TYPE': 'CL_DEVICE_TYPE_UPU', 
            #  'DENORMS2SINGLEPRECISION4FLOATINGPOINTCAPABILITY': 'ACK'}; time: 03:40:47 10/04/11 Pacific Daylight Time}
            
            code2ack  = code2ack + '4B-'  
        except:
            message2print = "{(Step 4Z/70) failed" + "; device2upu-failed-by-copy2imagetobuffer4upu: " + str(device2upu) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)             
            code2nack  = code2nack + '4Z-'
        
        task4copyimagetobuffer2upu ['DEVICE'] = device2upu
        
               
        # ---------------------------------------------------------------------------------------------
        # (5) IMAGE SOURCE OBJECT - REQUIRED                         
        # ---------------------------------------------------------------------------------------------         
        imagesourceinfo2upu = {}
         
        message2print = "{(Step 5A/70) trying" + "; imagesourceinfo2upu-trying-by-copy2imagetobuffer4upu: " + str(imagesourceinfo2upu)  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print) 
        try:
            imagesourceinfo2upu = get2imageinfo4upu()                                                                         
            
            message2print = "{(Step 5B/70) got" + "; imagesourceinfo2upu-got-by-copy2imagetobuffer4upu: " + str(imagesourceinfo2upu)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)                                  
            # {'SIZE2QUEUE': 2147483646, 
            #  'FORMAT': {'TYPE': 'CL_UNORM_INT8', 
            #  'ORDER': 'CL_RGBA'}, 
            #  'QUEUE': <queue4xi.Queue4Xi object at 0x00000000042514E0>, 
            #  'WIDTH': 800, 
            #  'CONTEXT': ...

            code2ack  = code2ack + '5B-'  
        except:
            message2print = "{(Step 5Z/70) failed" + "; imagesourceinfo2upu-failed-by-copy2imagetobuffer4upu: " + str(imagesourceinfo2upu) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)             
            code2nack  = code2nack + '5Z-'          
                  
        task4copyimagetobuffer2upu ['IMAGESOURCE'] = imagesourceinfo2upu         
               
        # ---------------------------------------------------------------------------------------------
        # (6) BUFFER DESTINATION OBJECT - REQUIRED                        
        # ---------------------------------------------------------------------------------------------                         
        bufferdestinationinfo2upu = {}
         
        message2print = "{(Step 6A/70) trying" + "; bufferdestinationinfo2upu-trying-by-copy2imagetobuffer4upu: " + str(bufferdestinationinfo2upu)  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print) 
        try:
            bufferdestinationinfo2upu = get2bufferinfo4upu()                                                                         
            
            message2print = "{(Step 6B/70) got" + "; bufferdestinationinfo2upu-got-by-copy2imagetobuffer4upu: " + str(bufferdestinationinfo2upu)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)                                  
            # {'CODE2ERROR': 0, 
            #  'COUNT2REFERENCE': 0, 
            #  'DATA2CALLBACK': {'MESSAGE': '{(Step 1Z1/33) failed; create2buffer4upu-failed ; time: 03:40:47 10/04/11 Pacific Daylight Time}', 
            #  'CLASS': 'create2buffer4upu', 
            #  'METHOD': 'create2buffer4upu'}, 
            #  'FUNCTION2CALLBACK': <function trace2buffer4upu at 0x0000000004782B48>, 
            #  'POINTER2BUFFER': <mmap.mmap object at 0x000000000423BD98>, 
            #  'FLAGS': 0, 
            #  'CONTEXT': {'COUNT2REFERENCE': 0, ...
            code2ack  = code2ack + '6B-'  
        except:
            message2print = "{(Step 6Z/70) failed" + "; bufferdestinationinfo2upu-failed-by-copy2imagetobuffer4upu: " + str(bufferdestinationinfo2upu) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)             
            code2nack  = code2nack + '6Z-'          
                  
        task4copyimagetobuffer2upu ['BUFFERDESTINATION'] = bufferdestinationinfo2upu     
        
        # ---------------------------------------------------------------------------------------------
        # (7) QUEUE - REQUIRED BY UPUXI                          
        # ---------------------------------------------------------------------------------------------         
        queue2copyimagetobuffer4upu = None
        
        message2print = "{(Step 7A/70) trying" + "; queue2copyimagetobuffer4upu-trying-by-copy2imagetobuffer4upu: " + str(queue2copyimagetobuffer4upu)  + "; time: " + time4xi() + "}" 
        print4xi(class2print, method2print, message2print) 
        try:
            queue2copyimagetobuffer4upu = Queue4Xi()                                                                              # THE SYSTEM CREATES QUEUE    
            
            message2print = "{(Step 7B/70) got" + "; queue2copyimagetobuffer4upu-got-by-copy2imagetobuffer4upu: " + str(queue2copyimagetobuffer4upu)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)                                  
            # <queue4xi.Queue4Xi object at 0x00000000042B2208> 
            
            code2ack  = code2ack + '7B-'  
        except:
            message2print = "{(Step 7Z/70) failed" + "; queue2copyimagetobuffer4upu-failed-by-copy2imagetobuffer4upu: " + str(queue2copyimagetobuffer4upu) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)             
            code2nack  = code2nack + '7Z-'
            
        task4copyimagetobuffer2upu ['QUEUE'] = queue2copyimagetobuffer4upu
         
        # ---------------------------------------------------------------------------------------------
        # (8) PIPE - REQUIRED BY UPUXI                              
        # ---------------------------------------------------------------------------------------------         
        pipe2upu = None
       
        message2print = "{(Step 8A/70) trying" + "; pipe2upu-trying-by-copy2imagetobuffer4upu: " + str(pipe2upu)  + "; time: " + time4xi() + "}" 
        print4xi(class2print, method2print, message2print) 
        try:
            pipe2upu = Pipe4Xi()                                                                                # THE SYSTEM CREATES PIPE    
            
            message2print = "{(Step 8B/70) got" + "; pipe2upu-got-by-copy2imagetobuffer4upu: " + str(pipe2upu)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)                                  
            # (<read-write PipeConnection, handle 660>, <read-write PipeConnection, handle 664>)
            
            code2ack  = code2ack + '8B-'  
        except:
            message2print = "{(Step 8Z/70) failed" + "; pipe2upu-failed-by-copy2imagetobuffer4upu: " + str(pipe2upu) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)             
            code2nack  = code2nack + '8Z-'
            
        task4copyimagetobuffer2upu ['PIPE'] = pipe2upu 
        
        # ---------------------------------------------------------------------------------------------
        # (9) COMMAND QUEUE - REQUIRED                              
        # ---------------------------------------------------------------------------------------------             
        commandqueueinfo2upu = {}
       
        message2print = "{(Step 9A/70) trying" + "; commandqueueinfo2upu-trying-by-copy2imagetobuffer4upu: " + str(commandqueueinfo2upu)  + "; time: " + time4xi() + "}" 
        print4xi(class2print, method2print, message2print)  
                    
        try:
            commandqueueinfo2upu = get2commandqueueinfo4upu()                                                   # SYSTEM CREATES COMMAND QUEUE, IF NECESSARY   
            message2print = "{(Step 9B/70) got" + "; commandqueueinfo2upu-got-by-copy2imagetobuffer4upu: " + str(commandqueueinfo2upu)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print) 
            # {'CODE2ERROR': 0, 
            #  'COUNT2REFERENCE': 0, 
            #  'DATA2CALLBACK': {'MESSAGE': '{(Step 1Z1/33) failed; create2commandqueue4upu-failed ; time: 03:40:47 10/04/11 Pacific Daylight Time}', 
            #  'CLASS': 'create2commandqueue4upu', 
            #  'METHOD': 'create2commandqueue4upu'}, 
            #  'FUNCTION2CALLBACK': <function trace2commandqueue4upu at 0x000000000395EA48>, 
            #  'ACK2RETURN': ...
            
            code2ack  = code2ack + '9B-'  
        except:
            message2print = "{(Step 9Z/70) failed" + "; commandqueueinfo2upu-failed-by-copy2imagetobuffer4upu: " + str(commandqueueinfo2upu) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)             
            code2nack  = code2nack + '9Z-'
    
        task4copyimagetobuffer2upu ['COMMANDQUEUE'] = commandqueueinfo2upu
        
        
        
        # ---------------------------------------------------------------------------------------------
        # (11) EVENT - REQUIRED                            
        # ---------------------------------------------------------------------------------------------             
        eventinfo2upu = {}
       
        message2print = "{(Step 11A/70) trying" + "; eventinfo2upu-trying-by-copy2imagetobuffer4upu: " + str(eventinfo2upu)  + "; time: " + time4xi() + "}" 
        print4xi(class2print, method2print, message2print) 
                    
        try:
            eventinfo2upu = get2eventinfo4upu()                                                                    
            message2print = "{(Step 11B/70) got" + "; eventinfo2upu-got-by-copy2imagetobuffer4upu: " + str(eventinfo2upu)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print) 
            # {'CODE2ERROR': 0, 
            #  'DATA2CALLBACK': {'MESSAGE': '{(Step 1Z1/33) failed; create2event4upu-failed ; time: 03:40:47 10/04/11 Pacific Daylight Time}', 
            #  'CLASS': 'create2event4upu', 
            #  'METHOD': 'create2event4upu'}, 
            #  'FUNCTION2CALLBACK': <function trace2event4upu at 0x0000000003971F48>, 
            #  'ACK2RETURN': ' 1D-1E-...
            code2ack  = code2ack + '11B-' 
        except:
            message2print = "{(Step 11Z/70) failed" + "; eventinfo2upu-failed-by-copy2imagetobuffer4upu: " + str(eventinfo2upu) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)             
            code2nack  = code2nack + '11Z-'
    
        task4copyimagetobuffer2upu ['EVENT'] = eventinfo2upu 
        
        
        # ---------------------------------------------------------------------------------------------
        # (12) EVENTWAITLIST - REQUIRED                              
        # ---------------------------------------------------------------------------------------------             
        eventwaitlist2upu = {}
       
        message2print = "{(Step 12A/70) trying" + "; eventwaitlist2upu-trying-by-copy2imagetobuffer4upu: " + str(eventwaitlist2upu)  + "; time: " + time4xi() + "}" 
        print4xi(class2print, method2print, message2print) 
                    
        try:
            eventwaitlist2upu = {}                                                                   
            message2print = "{(Step 12B/70) set" + "; eventwaitlist2upu-set-by-copy2imagetobuffer4upu: " + str(eventwaitlist2upu)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print) 
            # {}
            
            code2ack  = code2ack + '12B-' 
        except:
            message2print = "{(Step 12Z/70) failed" + "; eventwaitlist2upu-failed-by-copy2imagetobuffer4upu: " + str(eventwaitlist2upu) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)             
            code2nack  = code2nack + '12Z-'
    
        task4copyimagetobuffer2upu ['EVENTWAITLIST'] = eventwaitlist2upu                     
         
        # ---------------------------------------------------------------------------------------------
        # (13) EVENTWAITLISTQUANTITY - REQUIRED                              
        # ---------------------------------------------------------------------------------------------             
        eventwaitlistquantity2upu = 0
       
        message2print = "{(Step 13A/70) trying" + "; eventwaitlistquantity2upu-trying-by-copy2imagetobuffer4upu: " + str(eventwaitlistquantity2upu)  + "; time: " + time4xi() + "}" 
        print4xi(class2print, method2print, message2print) 
                    
        try:
            keys2event = eventwaitlist2upu.keys()
            for key2event in keys2event:
                eventwaitlistquantity2upu = eventwaitlistquantity2upu + 1                                                                      
            message2print = "{(Step 13B/70) set" + "; eventwaitlistquantity2upu-set-by-copy2imagetobuffer4upu: " + str(eventwaitlistquantity2upu)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print) 
            # 0 
            
            code2ack  = code2ack + '13B-' 
        except:
            message2print = "{(Step 13Z/70) failed" + "; eventwaitlistquantity2upu-failed-by-copy2imagetobuffer4upu: " + str(eventwaitlistquantity2upu) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)             
            code2nack  = code2nack + '13Z-'
    
        task4copyimagetobuffer2upu ['EVENTWAITLISTQUANTITY'] = eventwaitlistquantity2upu       
                                             
        
        # ---------------------------------------------------------------------------------------------
        # (14) ORIGIN FOR SOURCE - REQUIRED                             
        # ---------------------------------------------------------------------------------------------             
        origin4source2upu = {}
        
        message2print = "{(Step 14A/70) trying" + "; origin4source2upu-trying-by-copy2imagetobuffer4upu: " + str(origin4source2upu)  + "; time: " + time4xi() + "}" 
        print4xi(class2print, method2print, message2print) 
                    
        try:                                                                                    
            origin4source2upu['X'] = 0 
            origin4source2upu['Y'] = 0  
            origin4source2upu['Z'] = 0                                                                                                                          
            message2print = "{(Step 14B/70) set" + "; origin4source2upu-set-by-copy2imagetobuffer4upu to: " + str(origin4source2upu)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print) 
            # {'Y': 0, 'X': 0, 'Z': 0} 
            code2ack  = code2ack + '14B-' 
        except:
            message2print = "{(Step 14Z/70) failed" + "; origin4source2upu-failed-by-copy2imagetobuffer4upu: " + str(origin4source2upu) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)             
            code2nack  = code2nack + '14Z-'
    
        task4copyimagetobuffer2upu ['SOURCEORIGIN'] = origin4source2upu
        
       
        
        # --------------------------------------------------------------------------------------------------------
        # (16) REGION - REGION OF A IMAGE TO BE COPIED                         
        # --------------------------------------------------------------------------------------------------------             
        region4copy2upu = {}                                                                                                         
       
        message2print = "{(Step 16A/70) trying" + "; region4copy2upu-trying-by-copy2imagetobuffer4upu: " + str(region4copy2upu)  + "; time: " + time4xi() + "}" 
        print4xi(class2print, method2print, message2print) 
                    
        try:
            region4copy2upu['X'] = 10 
            region4copy2upu['Y'] = 10  
            region4copy2upu['Z'] = 10                                                                           
            message2print = "{(Step 16B/70) set" + "; region4copy2upu-set-by-copy2imagetobuffer4upu to: " + str(region4copy2upu)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print) 
            # {'Y': 10, 'X': 10, 'Z': 10} 
            code2ack  = code2ack + '16B-'                
             
        except:
            message2print = "{(Step 16Z/70) failed" + "; region4copy2upu-failed-by-copy2imagetobuffer4upu: " + str(region4copy2upu) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)             
            code2nack  = code2nack + '16Z-'
        
        task4copyimagetobuffer2upu ['REGION'] = region4copy2upu              
        
        # --------------------------------------------------------------------------------------------------------
        # (17) OFFSET - WHERE TO BEGIN COPYING DATA INTO DESTINATION BUFFER                     
        # --------------------------------------------------------------------------------------------------------             
        offset4copy2upu = 0                                                                                                         
        message2print = "{(Step 17A/70) set" + "; offset4copy2upu-set-by-copy2imagetobuffer4upu to: " + str(offset4copy2upu)  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print) 
        # 0
        code2ack  = code2ack + '17A-'                
        task4copyimagetobuffer2upu ['OFFSET'] = offset4copy2upu            
                    
        # ---------------------------------------------------------------------------------------------
        # EOF PARAMETER PROVIDING                
        # ---------------------------------------------------------------------------------------------                                                                                                                               
    else:
        pass
                    
    # ---------------------------------------------------------------------------------------------
    # ANALYZE TASK - WHETHER AUTOMATICALLY GENERATED OR USER GIVEN 
    # ---------------------------------------------------------------------------------------------
    keys2task = task4copyimagetobuffer2upu.keys()
    message2print = "{(Step 20/70) got" + "; keys2task-got-by-copy2imagetobuffer4upu: " + str(sorted(keys2task))  + "; time: " + time4xi() + "}" 
    print4xi(class2print, method2print, message2print)            
    # ['BUFFERDESTINATION',
    #  'COMMANDQUEUE',
    #  'CONTEXT',
    #  'DATA2CALLBACK',
    #  'DEVICE',
    #  'EVENT',
    #  'EVENTWAITLIST',
    #  'EVENTWAITLISTQUANTITY',
    #  'FUNCTION2CALLBACK',
    #  'IMAGESOURCE',
    #  'OFFSET',
    #  'PIPE',
    #  'PLATFORM',
    #  'QUEUE',
    #  'REGION',
    #  'SOURCEORIGIN']; time: 01:28:44 10/05/11 Pacific Daylight Time}


    if (('DATA2CALLBACK' in keys2task) and ('FUNCTION2CALLBACK' in keys2task)):
        pass
    else:                
        # ---------------------------------------------------------------------------------------------
        # (21B) TRACE - AUTOMATIC                      
        # ---------------------------------------------------------------------------------------------              
        function2callback4upu = trace2image4upu                                                             # CALLBACK FUNCTION FOR UPU
        
        message2print = "{(Step 21B1/70) initialized" + "; function2callback4upu-created-by-copy2imagetobuffer4upu to: " + str(function2callback4upu)  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)            
        # 
            
        code2ack  = code2ack + '21B1-'
           
        args2callback = {}                                                                                  # CREATE ARGUMENT DICTIONARY FOR CALLBACK FUNCTION             
        args2callback['CLASS']   = "copy2imagetobuffer4upu"                                                        # MODULE OF UPUIZED OPENCL MODULE
        args2callback['METHOD']  = "copy2imagetobuffer4upu"                                                        # FUNCTION OF UPUIZED OPENCL MODULE
        args2callback['MESSAGE'] = "{(Step 1Z1/33) failed" + "; copy2imagetobuffer4upu-failed " + "; time: " + time4xi() + "}" # ERROR MESSAGE FOR UPUIZED OPENCL TO RETURN    

        message2print = "{(Step 21B2/70) initialized" + "; args2callback-created-by-copy2imagetobuffer4upu to: " + str(args2callback)  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)               
        # 

        code2ack  = code2ack + '21B2-'
        
        # ---------------------------------------------------------------------------------------------
        # (21C) CALLBACK - AUTOMATIC                      
        # ---------------------------------------------------------------------------------------------              
        task4copyimagetobuffer2upu['DATA2CALLBACK'] = args2callback                                                # USER DATA PASSED TO UPUIZED OPENCL - SEVERAL ARGS PASSED IN ONE DICTIONARY
        task4copyimagetobuffer2upu['FUNCTION2CALLBACK'] = function2callback4upu                                    # CALLBACK FUNCTION TO CALLED BY OPENCL IN EXCEPTIONS
        
        code2ack  = code2ack + '21C-'
        message2print = "{(Step 21C/70) initialized" + "; task4copyimagetobuffer2upu-created-by-copy2imagetobuffer4upu to: " + str(task4copyimagetobuffer2upu)  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)  
        #
             
    if 'PLATFORM' in keys2task:
        pass
    else:  
        # ---------------------------------------------------------------------------------------------
        # (22) PLATFORM - AUTOMATIC                   
        # ---------------------------------------------------------------------------------------------             
        platform2xi = {}
        message2print = "{(Step 22A/70) initialized" + "; platform2xi-created-by-copy2imagetobuffer4upu to: " + str(platform2xi)  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)   
        # 
            
        try:
            platform2xi = get2platformids4xi()                                                              # PLATFORM IS COMMON TO ALL DEVICES I.E TO DEVICE XI
            message2print = "{(Step 22B/70) got" + "; platform2xi-got-by-copy2imagetobuffer4upu: " + str(platform2xi)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)
            # 
                 
            code2ack  = code2ack + '22B-'
        except:
            message2print = "{(Step 22Z/70) failed" + "; platform2xi-failed-by-copy2imagetobuffer4upu: " + str(failed)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)                                                
            code2nack  = code2nack + '22Z-'
                 
        task4copyimagetobuffer2upu['PLATFORM'] = platform2xi                                                       # PLATFORM OF SYSTEM 
          
    if 'CONTEXT' in keys2task:
        pass
    else:              
        # ---------------------------------------------------------------------------------------------
        # (23) CONTEXT - AUTOMATIC                 
        # ---------------------------------------------------------------------------------------------    
        context2upu = {}
        message2print = "{(Step 23A/70) trying" + "; context2upu-trying-by-copy2imagetobuffer4upu: " + str(context2upu)  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print) 
        try:
            context2upu = get2context4upu()                                                                 # NO PARAMETERS - THE SYSTEM CREATES CONTEXT IF REQUIRED  
            
            message2print = "{(Step 23B/70) got" + "; context2upu-got-by-copy2imagetobuffer4upu: " + str(context2upu)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)                                  
            # 
            
            code2ack  = code2ack + '23B-'         
        except:
            message2print = "{(Step 23Z/70) failed" + "; context2upu-failed-by-copy2imagetobuffer4upu: " + str(context2upu) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)             
            code2nack  = code2nack + '23Z-'
            
        task4copyimagetobuffer2upu ['CONTEXT'] = context2upu           
    
    if 'DEVICE' in keys2task:
        pass
    else:                   
        # ---------------------------------------------------------------------------------------------
        # (24) DEVICE - AUTOMATIC                       
        # ---------------------------------------------------------------------------------------------    
        device2upu = {}
        
        message2print = "{(Step 24A/70) trying" + "; device2upu-trying-by-copy2imagetobuffer4upu: " + str(device2upu)  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print) 
        try:
            device2upu = get2upu4xi()                                                                       # THE SYSTEM CREATES DEVICE INFO, IF NECESSARY    
            
            message2print = "{(Step 24B/70) got" + "; device2upu-got-by-copy2imagetobuffer4upu: " + str(device2upu)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)                                  
            # 
                
            code2ack  = code2ack + '24B-'  
        except:
            message2print = "{(Step 24Z/70) failed" + "; device2upu-failed-by-copy2imagetobuffer4upu: " + str(device2upu) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)             
            code2nack  = code2nack + '24Z-'
        
        task4copyimagetobuffer2upu ['DEVICE'] = device2upu
    
    if 'IMAGESOURCE' in keys2task:
        pass
    else:                      
        # ---------------------------------------------------------------------------------------------
        # (25) SOURCE IMAGE - REQUIRED                            
        # ---------------------------------------------------------------------------------------------         
        sourceimageinfo2upu = {}
         
        message2print = "{(Step 25A/70) trying" + "; sourceimageinfo2upu-trying-by-copy2imagetobuffer4upu: " + str(sourceimageinfo2upu)  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print) 
        try:
            sourceimageinfo2upu = get2imageinfo4upu()                                                                         
            
            message2print = "{(Step 25B/70) got" + "; sourceimageinfo2upu-got-by-copy2imagetobuffer4upu: " + str(sourceimageinfo2upu)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)                                  
            #                 
            code2ack  = code2ack + '25B-'  
        except:
            message2print = "{(Step 25Z/70) failed" + "; sourceimageinfo2upu-failed-by-copy2imagetobuffer4upu: " + str(sourceimageinfo2upu) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)             
            code2nack  = code2nack + '25Z-'          
                  
        task4copyimagetobuffer2upu ['IMAGESOURCE'] = sourceimageinfo2upu         
    
    if 'BUFFERDESTINATION' in keys2task:
        pass
    else:                      
        # ---------------------------------------------------------------------------------------------
        # (26) DESTINATION BUFFER - REQUIRED                            
        # ---------------------------------------------------------------------------------------------         
        destinationbufferinfo2upu = {}
         
        message2print = "{(Step 26A/70) trying" + "; destinationbufferinfo2upu-trying-by-copy2imagetobuffer4upu: " + str(destinationbufferinfo2upu)  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print) 
        try:
            destinationbufferinfo2upu = get2bufferinfo4upu()                                                                         
            
            message2print = "{(Step 26B/70) got" + "; destinationbufferinfo2upu-got-by-copy2imagetobuffer4upu: " + str(destinationbufferinfo2upu)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)                                  
            #                 
            code2ack  = code2ack + '26B-'  
        except:
            message2print = "{(Step 26Z/70) failed" + "; destinationbufferinfo2upu-failed-by-copy2imagetobuffer4upu: " + str(destinationbufferinfo2upu) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)             
            code2nack  = code2nack + '26Z-'          
                  
        task4copyimagetobuffer2upu ['BUFFERDESTINATION'] = destinationbufferinfo2upu    
                                
    if 'QUEUE' in keys2task:
        pass
    else:   
        # ---------------------------------------------------------------------------------------------
        # (27) QUEUE - REQUIRED BY UPUXI                          
        # ---------------------------------------------------------------------------------------------         
        queue2copyimagetobuffer4upu = None
        
        message2print = "{(Step 27A/70) trying" + "; queue2copyimagetobuffer4upu-trying-by-copy2imagetobuffer4upu: " + str(queue2copyimagetobuffer4upu)  + "; time: " + time4xi() + "}" 
        print4xi(class2print, method2print, message2print) 
        try:
            queue2copyimagetobuffer4upu = Queue4Xi()                                                                          # THE SYSTEM CREATES QUEUE    
            
            message2print = "{(Step 27B/70) got" + "; queue2copyimagetobuffer4upu-got-by-copy2imagetobuffer4upu: " + str(queue2copyimagetobuffer4upu)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)                                  
            # 
            
            code2ack  = code2ack + '27B-'  
        except:
            message2print = "{(Step 27Z/70) failed" + "; queue2copyimagetobuffer4upu-failed-by-copy2imagetobuffer4upu: " + str(queue2copyimagetobuffer4upu) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)             
            code2nack  = code2nack + '27Z-'
            
        task4copyimagetobuffer2upu ['QUEUE'] = queue2copyimagetobuffer4upu
        
    if 'PIPE' in keys2task:
        pass
    else:              
        # ---------------------------------------------------------------------------------------------
        # (28) PIPE - REQUIRED BY UPUXI                              
        # ---------------------------------------------------------------------------------------------         
        pipe2upu = None
       
        message2print = "{(Step 28A/70) trying" + "; pipe2upu-trying-by-copy2imagetobuffer4upu: " + str(pipe2upu)  + "; time: " + time4xi() + "}" 
        print4xi(class2print, method2print, message2print) 
        try:
            pipe2upu = Pipe4Xi()                                                                            # THE SYSTEM CREATES PIPE    
            
            message2print = "{(Step 28B/70) got" + "; pipe2upu-got-by-copy2imagetobuffer4upu: " + str(pipe2upu)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)                                  
            # 
            
            code2ack  = code2ack + '28B-'  
        except:
            message2print = "{(Step 28Z/70) failed" + "; pipe2upu-failed-by-copy2imagetobuffer4upu: " + str(pipe2upu) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)             
            code2nack  = code2nack + '28Z-'
            
        task4copyimagetobuffer2upu ['PIPE'] = pipe2upu 
        
    if 'COMMANDQUEUE' in keys2task:
        pass
    else:              
        # ---------------------------------------------------------------------------------------------
        # (28) COMMAND QUEUE - REQUIRED                              
        # ---------------------------------------------------------------------------------------------             
        commandqueueinfo2upu = {}
       
        message2print = "{(Step 29A/70) trying" + "; commandqueueinfo2upu-trying-by-copy2imagetobuffer4upu: " + str(commandqueueinfo2upu)  + "; time: " + time4xi() + "}" 
        print4xi(class2print, method2print, message2print)  
                    
        try:
            commandqueueinfo2upu = get2commandqueueinfo4upu()                                               # SYSTEM CREATES COMMAND QUEUE, IF NECESSARY   
            message2print = "{(Step 29B/70) got" + "; commandqueueinfo2upu-got-by-copy2imagetobuffer4upu: " + str(commandqueueinfo2upu)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print) 
            # 
            
            code2ack  = code2ack + '29B-'  
        except:
            message2print = "{(Step 29Z/70) failed" + "; commandqueueinfo2upu-failed-by-copy2imagetobuffer4upu: " + str(commandqueueinfo2upu) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)             
            code2nack  = code2nack + '29Z-'
    
        task4copyimagetobuffer2upu ['COMMANDQUEUE'] = commandqueueinfo2upu
    
         
    if 'EVENT' in keys2task:
        pass
    else:               
        # ---------------------------------------------------------------------------------------------
        # (31) EVENT - REQUIRED                            
        # ---------------------------------------------------------------------------------------------             
        eventinfo2upu = {}
       
        message2print = "{(Step 31A/70) trying" + "; eventinfo2upu-trying-by-copy2imagetobuffer4upu: " + str(eventinfo2upu)  + "; time: " + time4xi() + "}" 
        print4xi(class2print, method2print, message2print) 
                    
        try:
            eventinfo2upu = get2eventinfo4upu()                                                                    
            message2print = "{(Step 31B/70) got" + "; eventinfo2upu-got-by-copy2imagetobuffer4upu: " + str(eventinfo2upu)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print) 
            # 
            
            code2ack  = code2ack + '31B-' 
        except:
            message2print = "{(Step 31Z/70) failed" + "; eventinfo2upu-failed-by-copy2imagetobuffer4upu: " + str(eventinfo2upu) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)             
            code2nack  = code2nack + '31Z-'
    
        task4copyimagetobuffer2upu ['EVENT'] = eventinfo2upu 
    
         
    if 'EVENTWAITLIST' in keys2task:
        pass
    else:               
        # ---------------------------------------------------------------------------------------------
        # (32) EVENTWAITLIST - REQUIRED                              
        # ---------------------------------------------------------------------------------------------             
        eventwaitlist2upu = {}
       
        message2print = "{(Step 32A/70) trying" + "; eventwaitlist2upu-trying-by-copy2imagetobuffer4upu: " + str(eventwaitlist2upu)  + "; time: " + time4xi() + "}" 
        print4xi(class2print, method2print, message2print) 
                    
        try:
            eventwaitlist2upu = {}                                                                   
            message2print = "{(Step 32B/70) set" + "; eventwaitlist2upu-set-by-copy2imagetobuffer4upu: " + str(eventwaitlist2upu)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print) 
            # 
            
            code2ack  = code2ack + '32B-' 
        except:
            message2print = "{(Step 32Z/70) failed" + "; eventwaitlist2upu-failed-by-copy2imagetobuffer4upu: " + str(eventwaitlist2upu) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)             
            code2nack  = code2nack + '32Z-'
    
        task4copyimagetobuffer2upu ['EVENTWAITLIST'] = eventwaitlist2upu                     
          
    if 'EVENTWAITLISTQUANTITY' in keys2task:
        pass
    else:               
        # ---------------------------------------------------------------------------------------------
        # (33) EVENTWAITLISTQUANTITY - REQUIRED                              
        # ---------------------------------------------------------------------------------------------             
        eventwaitlistquantity2upu = 0
       
        message2print = "{(Step 33A/70) trying" + "; eventwaitlistquantity2upu-trying-by-copy2imagetobuffer4upu: " + str(eventwaitlistquantity2upu)  + "; time: " + time4xi() + "}" 
        print4xi(class2print, method2print, message2print) 
                    
        try:
            keys2event = eventwaitlist2upu.keys()
            for key2event in keys2event:
                eventwaitlistquantity2upu = eventwaitlistquantity2upu + 1                                                                   
            message2print = "{(Step 33B/70) set" + "; eventwaitlistquantity2upu-set-by-copy2imagetobuffer4upu: " + str(eventwaitlistquantity2upu)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print) 
            # 0 
            
            code2ack  = code2ack + '33B-' 
        except:
            message2print = "{(Step 33Z/70) failed" + "; eventwaitlistquantity2upu-failed-by-copy2imagetobuffer4upu: " + str(eventwaitlistquantity2upu) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)             
            code2nack  = code2nack + '33Z-'
    
        task4copyimagetobuffer2upu ['EVENTWAITLISTQUANTITY'] = eventwaitlistquantity2upu       
                                         
                
    if 'SOURCEORIGIN' in keys2task:
        pass
    else:                    
        # ---------------------------------------------------------------------------------------------
        # (34) SOURCEORIGIN - REQUIRED                             
        # ---------------------------------------------------------------------------------------------             
        source4origin2upu = {}
        
        message2print = "{(Step 34A/70) trying" + "; source4origin2upu-trying-by-copy2imagetobuffer4upu: " + str(source4origin2upu)  + "; time: " + time4xi() + "}" 
        print4xi(class2print, method2print, message2print) 
                    
        try:                                                                                    
            source4origin2upu['X'] = 0 
            source4origin2upu['Y'] = 0  
            source4origin2upu['Z'] = 0                                                                                                                          
            message2print = "{(Step 34B/70) set" + "; source4origin2upu-set-by-copy2imagetobuffer4upu to: " + str(source4origin2upu)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print) 
            # 
            code2ack  = code2ack + '34B-' 
        except:
            message2print = "{(Step 34Z/70) failed" + "; source4origin2upu-failed-by-copy2imagetobuffer4upu: " + str(source4origin2upu) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)             
            code2nack  = code2nack + '34Z-'
    
        task4copyimagetobuffer2upu ['SOURCEORIGIN'] = source4origin2upu
             
   
             
    if 'REGION' in keys2task:
        pass
    else:            
        # ---------------------------------------------------------------------------------------------
        # (36) REGION - REQUIRED                         
        # ---------------------------------------------------------------------------------------------             
        region2upu = {}
        
        message2print = "{(Step 36A/70) trying" + "; region2upu-trying-by-copy2imagetobuffer4upu: " + str(region2upu)  + "; time: " + time4xi() + "}" 
        print4xi(class2print, method2print, message2print) 
                    
        try:                                                                                    
            region2upu['X'] = 160                                                                               # WIDTH  IN PIXELS TO WRITE 
            region2upu['Y'] = 90                                                                                # HEIGHT IN PIXELS TO WRITE
            region2upu['Z'] = 0                                                                                 # DEPTH  IN PIXELS TO WRITE                                                                                                                          
            message2print = "{(Step 36B/70) set" + "; region2upu-set-by-copy2imagetobuffer4upu to: " + str(region2upu)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print) 
            # 
            code2ack  = code2ack + '36B-' 
        except:
            message2print = "{(Step 36Z/70) failed" + "; region2upu-failed-by-copy2imagetobuffer4upu: " + str(region2upu) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)             
            code2nack  = code2nack + '36Z-'
    
        task4copyimagetobuffer2upu ['REGION'] = region2upu
    
         
    if 'OFFSET' in keys2task:
        pass
    else:            
        # ---------------------------------------------------------------------------------------------
        # (37) OFFSET - WHERE TO BEGIN COPYING DATA INTO DESTINATION BUFFER - REQUIRED                         
        # ---------------------------------------------------------------------------------------------             
        offset2upu = 0
        message2print = "{(Step 37A/70) set" + "; offset2upu-set-by-copy2imagetobuffer4upu to: " + str(offset2upu)  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print) 
        # 
        task4copyimagetobuffer2upu ['OFFSET'] = offset2upu        
        code2ack  = code2ack + '37A-' 
        
    # ---------------------------------------------------------------------------------------------
    # EOF TASK
    # ---------------------------------------------------------------------------------------------
    keys2task = task4copyimagetobuffer2upu.keys()
    message2print = "{(Step 39/70) set" + "; keys2task-set-by-copy2imagetobuffer4upu to: " + str(sorted(keys2task))  + "; time: " + time4xi() + "}"
    print4xi(class2print, method2print, message2print)            
    # ['BUFFERDESTINATION',
    #  'COMMANDQUEUE',
    #  'CONTEXT',
    #  'DATA2CALLBACK',
    #  'DEVICE',
    #  'EVENT',
    #  'EVENTWAITLIST',
    #  'EVENTWAITLISTQUANTITY',
    #  'FUNCTION2CALLBACK',
    #  'IMAGESOURCE',
    #  'OFFSET',
    #  'PIPE',
    #  'PLATFORM',
    #  'QUEUE',
    #  'REGION',
    #  'SOURCEORIGIN']; time: 01:28:44 10/05/11 Pacific Daylight Time}
               


    # ---------------------------------------------------------------------------------------------
    # CONTROL AND COMMAND
    # ---------------------------------------------------------------------------------------------
    control2copyimagetobuffer4upu = {}       
                                
    # ---------------------------------------------------------------------------------------------
    # CONSUMER AND PUBLISHER CHANNEL FOR CONTROL AND COMMAND
    # ---------------------------------------------------------------------------------------------
    ack4channels2upu = False
    channels2upu = ()
    message2print = "{(Step 40A/70) trying" + "channels2upu-trying-by-copy2imagetobuffer4upu: " + str(channels2upu)  + "; time: " + time4xi() + "}" 
    print4xi(class2print, method2print, message2print)     
    try:
        # 
        consumerchannel2copyimagetobuffer4upu, publisherchannel2copyimagetobuffer4upu = task4copyimagetobuffer2upu ['PIPE']

        message2print = "{(Step 40B/70) got" + "; consumerchannel2copyimagetobuffer4upu-got-by-copy2imagetobuffer4upu: " + str(consumerchannel2copyimagetobuffer4upu)  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)            
        # <read-write PipeConnection, handle 660>

        message2print = "{(Step 40C/70) got" + "; publisherchannel2copyimagetobuffer4upu-got-by-copy2imagetobuffer4upu: " + str(publisherchannel2copyimagetobuffer4upu)  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)            
        # <read-write PipeConnection, handle 664>                     
        ack4channels2upu = True
        
        control2copyimagetobuffer4upu['CONSUMERCHANNEL']  = consumerchannel2copyimagetobuffer4upu
        
        
        control2copyimagetobuffer4upu['PUBLISHERCHANNEL'] = publisherchannel2copyimagetobuffer4upu
        
        code2ack  = code2ack + '40D-' 
        
    except:
        ack4channels2upu = False
        code2nack  = code2nack + '40Z-'      
                        
    # ---------------------------------------------------------------------------------------------
    # QUEUE FOR CONTROL AND COMMAND
    # ---------------------------------------------------------------------------------------------
    ack4queue2copyimagetobuffer4upu = False
    queue2copyimagetobuffer4upu = None
    message2print = "{(Step 41A/70) trying" + "queue2copyimagetobuffer4upu-trying-by-copy2imagetobuffer4upu: " + str(queue2copyimagetobuffer4upu)  + "; time: " + time4xi() + "}" 
    print4xi(class2print, method2print, message2print)     
    try:
        queue2copyimagetobuffer4upu = task4copyimagetobuffer2upu ['QUEUE']

        message2print = "{(Step 41B/70) got" + "; queue2copyimagetobuffer4upu-got-by-copy2imagetobuffer4upu: " + str(queue2copyimagetobuffer4upu)  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)            
        # <queue4xi.Queue4Xi object at 0x0000000004465E48>
              
        ack4queue2copyimagetobuffer4upu = True
        control2copyimagetobuffer4upu['QUEUE']  = queue2copyimagetobuffer4upu        
        code2ack  = code2ack + '41C-'
    except:
        ack4queue2copyimagetobuffer4upu = False  
        code2nack  = code2nack + '41Z-'
    
    
    # -----------------------------------------------------------------------------------------    
    # DUMMY PROCESS OBJECT = THREAD CREATION
    # -----------------------------------------------------------------------------------------           
    # dummy2image4upu = None
    # message2print = "{(Step 69A/70) trying" + "dummy2image4upu-trying-by-copy2imagetobuffer4upu: " + str(dummy2image4upu)  + "; time: " + time4xi() + "}"   
    # print4xi(class2print, method2print, message2print)     
    # try:                            
        # dummy2image4upu = DummyProcess4Xi(target=manoeuvre2copyimagetobuffer4upu,                                               # in this module to handle queue and pipe channels
        #                                   args =(queue2copyimagetobuffer4upu,                                                         # caller puts and gets - does NOT call any functions
        #                                          consumerchannel2copyimagetobuffer4upu, publisherchannel2copyimagetobuffer4upu))                        # caller sends and receives - does NOT call any functions
        # message2print = "{(Step 69B/70) created" + "dummy2image4upu-created-by-copy2imagetobuffer4upu: " + str(dummy2image4upu)  + "; time: " + time4xi() + "}" 
        # print4xi(class2print, method2print, message2print)  
        # code2ack  = code2ack + '69B-'
        
    # except:
        # code2nack  = code2nack + '69Z-'
        
    # ---------------------------------------------------------------------------------------------
    # RETURN CONTROL AND COMMAND DICTIONARY
    # ---------------------------------------------------------------------------------------------          
    message2print = "{(Step 70A/70) set" + "; code2ack-set-by-copy2imagetobuffer4upu: " + str(code2ack) + "; time: " + time4xi() + "}"  
    print4xi(class2print, method2print, message2print)                                                             
    # 1B1-1B2-1C-2B-3B-4B-5B-6B-7B-8B-9B-11B-12B-13B-14B-16B-17A-40D-41C- 
    message2print = "{(Step 70B/70) set" + "; code2nack-set-by-copy2imagetobuffer4upu: " + str(code2nack) + "; time: " + time4xi() + "}" 
    print4xi(class2print, method2print, message2print)                                                             
    #  
        
    message2print = "{(Step 70C/70) returning" + "; control2copyimagetobuffer4upu-returning-by-copy2imagetobuffer4upu: " + str(control2copyimagetobuffer4upu) + "; time: " + time4xi() + "}" 
    print4xi(class2print, method2print, message2print)                                                             
    # {'QUEUE': <queue4xi.Queue4Xi object at 0x0000000004465E48>,
    #  'PUBLISHERCHANNEL': <read-write PipeConnection, handle 664>,
    #  'CONSUMERCHANNEL': <read-write PipeConnection, handle 660>}; time: 01:28:44 10/05/11 Pacific Daylight Time}

        
    return control2copyimagetobuffer4upu
                    
#</function4xi> 

# -------------------------------------------------------------------------------------------------
#<function4xi function = "copy2buffertoimage4upu" usage = "
#                                                                                                              
#   # ---------------------------------------------------------------------------------------------
#   # (80) ESTABLISH CONTROL & COMMAND WITH COPYBUFFERTOIMAGE4UPU                      
#   # ---------------------------------------------------------------------------------------------    
#   print('(Step 80A/700) OF UPU2IMAGE4XI5TEST - TRYING TO ESTABLISH CONTROL & COMMAND WITH COPYBUFFERTOIMAGE4UPU ', time4xi())    
#   response4copy2buffertoimage = {} 
#   task4copy2buffertoimage = None
#   try:                               
#       response4copy2buffertoimage = copy2buffertoimage4upu(task4copy2buffertoimage)                                                                                                          
#       print('(Step 80B/700) OF UPU2IMAGE4XI5TEST - ESTABLISHED CONTROL & COMMAND WITH COPYBUFFERTOIMAGE4UPU : ', response4copy2buffertoimage)
#       queue2copybuffertoimage4upu      = response4copy2buffertoimage['QUEUE']
#       channel2copybuffertoimagepublish4upu = response4copy2buffertoimage['PUBLISHERCHANNEL']
#       channel2copybuffertoimageconsume4upu = response4copy2buffertoimage['CONSUMERCHANNEL']
#   except:
#       print('(Step 80Z/700) OF UPU2IMAGE4XI5TEST *** FAILED TO ESTABLISH CONTROL & COMMAND WITH COPYBUFFERTOIMAGE4UPU ', time4xi()) 
#   # --------------------------------------------------------------------------------------------------
#   # (81) SEND CONTROL & COMMAND COPY VIA PIPE FOR MANOEUVRE2COPYIMAGETOBUFFER4UPU TO RECEIVE                 
#   # --------------------------------------------------------------------------------------------------      
#   print('(Step 81A/700) OF UPU2IMAGE4XI5TEST - TRYING TO SEND CONTROL & COMMAND VIA PIPE WITH COPYBUFFERTOIMAGE4UPU ', time4xi())     
#   try:                               
#       message2copybuffertoimagesend4upu = "<copy4xi>copy2copybuffertoimage4upu</copy4xi>"
#       channel2copybuffertoimagepublish4upu = response4copy2buffertoimage['PUBLISHERCHANNEL']
#       channel2copybuffertoimagepublish4upu.send(message2copybuffertoimagesend4upu)
#       print("(Step 81B/700) OF UPU2IMAGE4XI5TEST - SENT TO CHANNEL2PUBLISH4UPU: ", channel2copybuffertoimagepublish4upu, " MESSAGE2SEND4UPU: " , message2copybuffertoimagesend4upu)                                                                                                
#   except:
#       print('(Step 81Z/700) OF UPU2IMAGE4XI5TEST *** FAILED TO SEND CONTROL & COMMAND VIA PIPE WITH COPYBUFFERTOIMAGE4UPU ', time4xi())                                                
#   # ------------------------------------------------------------------------------------------
#   # (82) LET MANOEUVRE2COPYIMAGETOBUFFER4UPU TO RECEIVE COPY AND AND SEND ROGER VIA PIPE                
#   # ------------------------------------------------------------------------------------------       
#   print("(Step 82C/700) OF UPU2IMAGE4XI5TEST - TRYING TO PERFORM TARGET FUNCTION MANOEUVRE2COPYIMAGETOBUFFER4UPU ", time4xi()) 
#   try:        
#       response2manoeuvreimagecopytobuffer4upu = manoeuvre2copybuffertoimage4upu(queue2copybuffertoimage4upu, channel2copybuffertoimageconsume4upu, channel2copybuffertoimagepublish4upu)
#       print("(Step 82D/700) OF UPU2IMAGE4XI5TEST - PERFORMED TARGET FUNCTION MANOEUVRE2COPYIMAGETOBUFFER4UPU ", manoeuvre2copybuffertoimage4upu)
#   except:
#       print("(Step 82Z/700) OF UPU2IMAGE4XI5TEST *** FAILED TO PERFORM TARGET FUNCTION MANOEUVRE2COPYIMAGETOBUFFER4UPU ", time4xi())     
#   else:
#       pass # trust activated thread    
#   # ---------------------------------------------------------------------------------------------
#   # (83) RECEIVE CONTROL & COMMAND ROGER VIA PIPE                   
#   # ---------------------------------------------------------------------------------------------          
#   print('(Step 83A/700) OF UPU2IMAGE4XI5TEST - TRYING TO RECEIVE CONTROL & COMMAND VIA PIPE WITH COPYBUFFERTOIMAGE4UPU ', time4xi())     
#   try:
#       channel2copybuffertoimageconsume4upu = response4copy2buffertoimage['CONSUMERCHANNEL']                              
#       message2copyimagereceive4upu = channel2copybuffertoimageconsume4upu.recv()
#       print("(Step 83B/700) OF UPU2IMAGE4XI5TEST - RECEIVED FROM CHANNEL2CONSUME4UPU: ", channel2copybuffertoimageconsume4upu, " CONTROL & COMMAND: ", message2copyimagereceive4upu)                                                                                                  
#       # 
#   except:
#       print('(Step 83Z/700) OF UPU2IMAGE4XI5TEST *** FAILED TO RECEIVE CONTROL & COMMAND VIA PIPE  WITH COPYBUFFERTOIMAGE4UPU ', time4xi())         
#   # ---------------------------------------------------------------------------------------------------
#   # (84) SEND CONTROL & COMMAND READY VIA PIPE FOR MANOEUVRE2COPYIMAGETOBUFFER4UPU TO RECEIVE                
#   # ---------------------------------------------------------------------------------------------------      
#   print('(Step 84A/700) OF UPU2IMAGE4XI5TEST - TRYING TO SEND CONTROL & COMMAND VIA PIPE WITH COPYBUFFERTOIMAGE4UPU ', time4xi())     
#   try:                               
#       message2copybuffertoimagesend4upu = "<copy4xi>copy2copybuffertoimage4upu</copy4xi>"
#       channel2copybuffertoimagepublish4upu = response4copy2buffertoimage['PUBLISHERCHANNEL']
#       channel2copybuffertoimagepublish4upu.send(message2copybuffertoimagesend4upu)
#       print("(Step 84B/700) OF UPU2IMAGE4XI5TEST - SENT TO CHANNEL2PUBLISH4UPU: ", channel2copybuffertoimagepublish4upu, " MESSAGE2SEND4UPU: " , message2copybuffertoimagesend4upu)                                                                                                
#       #  
#   except:
#        print('(Step 84Z/700) OF UPU2IMAGE4XI5TEST *** FAILED TO SEND CONTROL & COMMAND VIA PIPE WITH COPYBUFFERTOIMAGE4UPU ', time4xi())                                                
#   # -----------------------------------------------------------------------------------
#   # (85) LET MANOEUVRE2COPYIMAGETOBUFFER4UPU TO RECEIVE READY AND TO SEND READY                
#   # -----------------------------------------------------------------------------------       
#   print("(Step 85A/700) OF UPU2IMAGE4XI5TEST - TRYING TO PERFORM TARGET FUNCTION MANOEUVRE2COPYIMAGETOBUFFER4UPU ", time4xi()) 
#   try:        
#       response2manoeuvreimagecopytobuffer4upu = manoeuvre2copybuffertoimage4upu(queue2copybuffertoimage4upu, channel2copybuffertoimageconsume4upu, channel2copybuffertoimagepublish4upu)
#       print("(Step 85B/700) OF UPU2IMAGE4XI5TEST - PERFORMED TARGET FUNCTION MANOEUVRE2COPYIMAGETOBUFFER4UPU ", manoeuvre2copybuffertoimage4upu)
#   except:
#       print("(Step 85Z/700) OF UPU2IMAGE4XI5TEST *** FAILED TO PERFORM TARGET FUNCTION MANOEUVRE2COPYIMAGETOBUFFER4UPU ", time4xi())     
#   # ---------------------------------------------------------------------------------------
#   # (86) RECEIVE CONTROL & COMMAND READY VIA PIPE                      
#   # ---------------------------------------------------------------------------------------          
#   print('(Step 86A/700) OF UPU2IMAGE4XI5TEST - TRYING TO RECEIVE CONTROL & COMMAND VIA PIPE WITH COPYBUFFERTOIMAGE4UPU ', time4xi())     
#   try:
#       channel2copybuffertoimageconsume4upu = response4copy2buffertoimage['CONSUMERCHANNEL']                              
#       message2copyimagereceive4upu = channel2copybuffertoimageconsume4upu.recv()
#       print("(Step 86B/700) OF UPU2IMAGE4XI5TEST - RECEIVED FROM CHANNEL2CONSUME4UPU: ", channel2copybuffertoimageconsume4upu, " CONTROL & COMMAND: ", message2copyimagereceive4upu)                                                                                                  
#       # 
#   except:
#       print('(Step 86Z/700) OF UPU2IMAGE4XI5TEST *** FAILED TO RECEIVE CONTROL & COMMAND VIA PIPE  WITH COPYBUFFERTOIMAGE4UPU ', time4xi())          
#   # -----------------------------------------------------------------------------------------
#   # (87) PUT IMAGE PATH TO QUEUE
#   # -----------------------------------------------------------------------------------------                          
#   print("(Step 87A/700) OF UPU2IMAGE4XI5TEST - TRYING TO PUT IMAGE PATH TO QUEUE ", time4xi())      
#   try:    
#       queue2copybuffertoimage4dragon.put(path2dragon)             
#       print("(Step 87B/700) OF DRAGON2IMAGE4XI5TEST - PUT TO QUEUE IMAGE PATH: ", path2dragon) 
#       # ..//..//..//..//..//web//images//png//dragonxi.png  
#   except:
#       print("(Step 87Z/700) OF UPU2IMAGE4XI5TEST *** FAILED TO PUT IMAGE PATH TO QUEUE ", time4xi()) 
#   # ------------------------------------------------------------------------------------------------------
#   # (88) LET MANOEUVRE2COPYIMAGETOBUFFER4UPU TO GET FROM QUEUE AND TO SEND EITHER ACK OR NACK VIA PIPE               
#   # -------------------------------------------------------------------------------------------------------       
#   print("(Step 88A/700) OF UPU2IMAGE4XI5TEST - TRYING TO PERFORM TARGET FUNCTION MANOEUVRE2COPYIMAGETOBUFFER4UPU ", time4xi()) 
#   try:        
#       response2manoeuvreimagecopytobuffer4upu = manoeuvre2copybuffertoimage4upu(queue2copybuffertoimage4upu, channel2copybuffertoimageconsume4upu, channel2copybuffertoimagepublish4upu)
#       print("(Step 88B/700) OF UPU2IMAGE4XI5TEST - PERFORMED TARGET FUNCTION MANOEUVRE2COPYIMAGETOBUFFER4UPU ", manoeuvre2copybuffertoimage4upu)
#   except:
#       print("(Step 88Z/700) OF UPU2IMAGE4XI5TEST *** FAILED TO PERFORM TARGET FUNCTION MANOEUVRE2COPYIMAGETOBUFFER4UPU ", time4xi())     
#   # -----------------------------------------------------------------------------------------------
#   # (89) RECEIVE ACK OR NACK ABOUT IMAGE OPERATION VIA PIPE                
#   # -----------------------------------------------------------------------------------------------          
#   print('(Step 89A/700) OF UPU2IMAGE4XI5TEST - TRYING TO RECEIVE CONTROL & COMMAND ABOUT IMAGE OPERTION VIA PIPE ', time4xi())      
#   try: 
#       message2copyimagereceive4upu = channel2copybuffertoimageconsume4upu.recv()
#       print("(Step 89D/700) OF UPU2IMAGE4XI5TEST - RECEIVED FROM CHANNEL2CONSUME4UPU: ", channel2copybuffertoimageconsume4upu, " CONTROL & COMMAND: ", message2copyimagereceive4upu)                                                                                                   
#   except:
#       print('(Step 89Z/700) OF UPU2IMAGE4XI5TEST *** FAILED TO RECEIVE CONTROL & COMMAND ABOUT IMAGE OPERTION VIA PIPE ', time4xi())               
#   # -------------------------------------------------------------------------------------------------------
#   # (90) LET MANOEUVRE2COPYIMAGETOBUFFER4UPU GO TO STATE 0               
#   # -------------------------------------------------------------------------------------------------------       
#   print("(Step 90A/700) OF UPU2IMAGE4XI5TEST - TRYING TO PERFORM TARGET FUNCTION MANOEUVRE2COPYIMAGETOBUFFER4UPU ", time4xi()) 
#   try:        
#       response2manoeuvreimagecopytobuffer4upu = manoeuvre2copybuffertoimage4upu(queue2copybuffertoimage4upu, channel2copybuffertoimageconsume4upu, channel2copybuffertoimagepublish4upu)
#       print("(Step 90B/700) OF UPU2IMAGE4XI5TEST - GOT RESPONSE2MANOEUVRE4UPU ", response2manoeuvreimagecopytobuffer4upu)
#   except:
#       print("(Step 90Z/700) OF UPU2IMAGE4XI5TEST *** FAILED TO PERFORM TARGET FUNCTION MANOEUVRE2COPYIMAGETOBUFFER4UPU ", time4xi())   

#">

# -------------------------------------------------------------------------------------------------   
def copy2buffertoimage4upu(task4copybuffertoimage2upu=None):

    '''
    copies buffer to image for UPU
    '''
    global upu2image4xi                                                                                             # 2D-IMAGES FOR UPU  
    global upu2buffer4xi                                                                                            # BUFFER FOR UPU
    
    global control2copybuffertoimage4upu                                                                            # COMMAND AND CONTROL FOR UPU IMAGES 
    
    # -----------------------------------------------------------------------------------------
    # (Step 1/33) PARAMETERS           
    # -----------------------------------------------------------------------------------------          
    class2print = "copy2buffertoimage4upu"
    method2print = "copy2buffertoimage4upu"

    message2print = "{(Step 1A/33) started" + "; copy2buffertoimage4upu-started: " + str(copy2buffertoimage4upu)  + "; time: " + time4xi() + "}"  
    print4xi(class2print, method2print, message2print)                                                                
    # <function copy2buffertoimage4upu at 0x0000000003869548>
    
    message2print = "{(Step 1B/33) passed" + "; task4copybuffertoimage2upu-to-copy2buffertoimage4upu: " + str(task4copybuffertoimage2upu) + "; time: " + time4xi() + "}"
    print4xi(class2print, method2print, message2print)                                                              
    # None
            
    code2ack  = ' '
    code2nack = ' '
    
    # -----------------------------------------------------------------------------------------
    # CREATE UPU2IMAGE4XI IF NECESSARY       
    # ----------------------------------------------------------------------------------------- 
    try:
        if upu2image4xi is None:
            upu2image4xi = create2image4upu()                                                                              # INITIALIZE IMAGE FOR UPU 
            message2print = "{(Step 1C/70) created" + "; upu2image4xi-created-by-copy2buffertoimage4upu to: " + str(upu2image4xi)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)
            #             
        else:
            message2print = "{(Step 1D/70) skipped" + "; create2image4upu-skipped-by-copy2buffertoimage4upu: " + str(copy2buffertoimage4upu) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)
            # skipped                                  
    except:
        try:
            upu2image4xi = create2image4upu()
            message2print = "{(Step 1E/70) set" + "; upu2image4xi-set-by-copy2buffertoimage4upu to: " + str(upu2image4xi)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)
            #             
        except:
            upu2image4xi = {}
            message2print = "{(Step 1Z1/70) yielded" + "; upu2image4xi-yielded-by-copy2buffertoimage4upu: " + str(upu2image4xi) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)  
            code2nack  = code2nack + '1Z1-' 
    
    # -----------------------------------------------------------------------------------------
    # CREATE UPU2BUFFER4XI IF NECESSARY       
    # ----------------------------------------------------------------------------------------- 
    try:
        if upu2buffer4xi is None:
            upu2buffer4xi = create2buffer4upu()                                                                     # INITIALIZE BUFFER FOR UPU 
            message2print = "{(Step 1F/70) created" + "; upu2buffer4xi-created-by-copy2buffertoimage4upu to: " + str(upu2buffer4xi)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)
            #             
        else:
            message2print = "{(Step 1G/70) skipped" + "; create2buffer4upu-skipped-by-copy2buffertoimage4upu: " + str(copy2buffertoimage4upu) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)
            # skipped                                  
    except:
        try:
            upu2buffer4xi = create2buffer4upu()
            message2print = "{(Step 1H/70) set" + "; upu2buffer4xi-set-by-copy2buffertoimage4upu to: " + str(upu2buffer4xi)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)
            #             
        except:
            upu2buffer4xi = {}
            message2print = "{(Step 1Z2/70) yielded" + "; upu2buffer4xi-yielded-by-copy2buffertoimage4upu: " + str(upu2buffer4xi) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)  
            code2nack  = code2nack + '1Z2-' 
                            
    # -----------------------------------------------------------------------------------------
    # (Step 1/70) - ANALYZE TASKS           
    # -----------------------------------------------------------------------------------------          
    if task4copybuffertoimage2upu is None:
        # -----------------------------------------------------------------------------------------
        # NO PARAMETERS GIVEN BY CALLER FOR COPY - BUILD TASK! 
        # -----------------------------------------------------------------------------------------    
        
        task4copybuffertoimage2upu = {}                                                                                                                                                                                       
        
        message2print = "{(Step 1A/70) initialized" + "; task4copybuffertoimage2upu-created-by-copy2buffertoimage4upu to: " + str(task4copybuffertoimage2upu)  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)               
        # 
        
        # ---------------------------------------------------------------------------------------------
        # (1B) TRACE - AUTOMATIC                      
        # ---------------------------------------------------------------------------------------------              
        function2callback4upu = trace2image4upu                                                                 # CALLBACK FUNCTION FOR UPU
        
        message2print = "{(Step 1B1/70) initialized" + "; function2callback4upu-created-by-copy2buffertoimage4upu to: " + str(function2callback4upu)  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)            
        # <function trace2image4upu at 0x0000000004645548>
        
        code2ack  = code2ack + '1B1-'
       
        args2callback = {}                                                                                      # CREATE ARGUMENT DICTIONARY FOR CALLBACK FUNCTION             
        args2callback['CLASS']   = "copy2buffertoimage4upu"                                                     # MODULE OF UPUIZED OPENCL MODULE
        args2callback['METHOD']  = "copy2buffertoimage4upu"                                                     # FUNCTION OF UPUIZED OPENCL MODULE
        args2callback['MESSAGE'] = "{(Step 1Z1/33) failed" + "; copy2buffertoimage4upu-failed " + "; time: " + time4xi() + "}" # ERROR MESSAGE FOR UPUIZED OPENCL TO RETURN    

        message2print = "{(Step 1B2/70) initialized" + "; args2callback-created-by-copy2buffertoimage4upu to: " + str(args2callback)  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)               
        # {'MESSAGE': '{(Step 1Z1/33) failed; copy2buffertoimage4upu-failed ; time: 04:41:15 10/04/11 Pacific Daylight Time}',
        #  'CLASS': 'copy2buffertoimage4upu', 
        #  'METHOD': 'copy2buffertoimage4upu'}; time: 04:41:15 10/04/11 Pacific Daylight Time}

        code2ack  = code2ack + '1B2-'
            
        # ---------------------------------------------------------------------------------------------
        # (1C) CALLBACK - AUTOMATIC                      
        # ---------------------------------------------------------------------------------------------              
        task4copybuffertoimage2upu['DATA2CALLBACK'] = args2callback                                             # USER DATA PASSED TO UPUIZED OPENCL - SEVERAL ARGS PASSED IN ONE DICTIONARY
        task4copybuffertoimage2upu['FUNCTION2CALLBACK'] = function2callback4upu                                 # CALLBACK FUNCTION TO CALLED BY OPENCL IN EXCEPTIONS
            
        code2ack  = code2ack + '1C-'
        message2print = "{(Step 1C/70) initialized" + "; task4copybuffertoimage2upu-created-by-copy2buffertoimage4upu to: " + str(task4copybuffertoimage2upu)  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)  
        # {'FUNCTION2CALLBACK': <function trace2image4upu at 0x0000000004645548>, 
        #  'DATA2CALLBACK': {'MESSAGE': '{(Step 1Z1/33) failed; copy2buffertoimage4upu-failed ; time: 04:41:15 10/04/11 Pacific Daylight Time}', 
        #  'CLASS': 'copy2buffertoimage4upu', 
        #  'METHOD': 'copy2buffertoimage4upu'}}; time: 04:41:15 10/04/11 Pacific Daylight Time}

            
        # ---------------------------------------------------------------------------------------------
        # (2) PLATFORM - AUTOMATIC                   
        # ---------------------------------------------------------------------------------------------             
        platform2xi = {}
        message2print = "{(Step 2A/70) initialized" + "; platform2xi-created-by-copy2buffertoimage4upu to: " + str(platform2xi)  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)   
        # {}
            
        try:
            platform2xi = get2platformids4xi()                                                                  # PLATFORM IS COMMON TO ALL DEVICES I.E TO DEVICE XI
            message2print = "{(Step 2B/70) got" + "; platform2xi-got-by-copy2buffertoimage4upu: " + str(platform2xi)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)
            # {'list2platforms': ['DragonXi'], 
            #  'ack2return': ' 2C-2E-2F-2G-', 
            #  'nack2return': ' ', 
            #  'available2platform': 1, 
            #  'max2entries': 1}; time: 04:41:15 10/04/11 Pacific Daylight Time}
                 
            code2ack  = code2ack + '2B-'
        except:
            message2print = "{(Step 2Z/70) failed" + "; platform2xi-failed-by-copy2buffertoimage4upu: " + str(failed)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)                                                
            code2nack  = code2nack + '2Z-'
                 
        task4copybuffertoimage2upu['PLATFORM'] = platform2xi                                                    # PLATFORM OF SYSTEM 
              
            
        # ---------------------------------------------------------------------------------------------
        # (3) CONTEXT - AUTOMATIC                 
        # ---------------------------------------------------------------------------------------------    
        context2upu = {}
        message2print = "{(Step 3A/70) trying" + "; context2upu-trying-by-copy2buffertoimage4upu: " + str(context2upu)  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print) 
        try:
            context2upu = get2context4upu()                                                                     # NO PARAMETERS - THE SYSTEM CREATES CONTEXT IF REQUIRED  
                
            message2print = "{(Step 3B/70) got" + "; context2upu-got-by-copy2buffertoimage4upu: " + str(context2upu)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)                                  
            # {'COUNT2REFERENCE': 0, 'DATA2CALLBACK': {'MESSAGE': ...
                
            code2ack  = code2ack + '3B-'         
        except:
            message2print = "{(Step 3Z/70) failed" + "; context2upu-failed-by-copy2buffertoimage4upu: " + str(context2upu) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)             
            code2nack  = code2nack + '3Z-'
                
        task4copybuffertoimage2upu ['CONTEXT'] = context2upu           
            
        # ---------------------------------------------------------------------------------------------
        # (4) DEVICE - AUTOMATIC                       
        # ---------------------------------------------------------------------------------------------    
        device2upu = {}
            
        message2print = "{(Step 4A/70) trying" + "; device2upu-trying-by-copy2buffertoimage4upu: " + str(device2upu)  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print) 
        try:
            device2upu = get2upu4xi()                                                                           # THE SYSTEM CREATES DEVICE INFO, IF NECESSARY    
            
            message2print = "{(Step 4B/70) got" + "; device2upu-got-by-copy2buffertoimage4upu: " + str(device2upu)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)                                  
            # {'RESOLUTION2TIMER4PROFILING': 318, 
            #  'ID2PLATFORM': '0000000001509568', 
            #  'PROFILING2QUEUE': 'ACK', 
            #  'EXECUTE2KERNELS4OPENCL': 'ACK', 
            #  'NATIVE2WIDTHLONG4VECTOR': 2, 
            #  'MAX2UNITS4COMPUTE': 4, 
            #  'SIZE2MEMORY4GLOBAL': 3221225472, ...
               
            code2ack  = code2ack + '4B-'  
        except:
            message2print = "{(Step 4Z/70) failed" + "; device2upu-failed-by-copy2buffertoimage4upu: " + str(device2upu) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)             
            code2nack  = code2nack + '4Z-'
            
        task4copybuffertoimage2upu ['DEVICE'] = device2upu
            
                   
        # ---------------------------------------------------------------------------------------------
        # (5) IMAGE DESTINATION OBJECT - REQUIRED                         
        # ---------------------------------------------------------------------------------------------         
        imagedestinationinfo2upu = {}
             
        message2print = "{(Step 5A/70) trying" + "; imagedestinationinfo2upu-trying-by-copy2buffertoimage4upu: " + str(imagedestinationinfo2upu)  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print) 
        try:
            imagedestinationinfo2upu = get2imageinfo4upu()                                                                         
            
            message2print = "{(Step 5B/70) got" + "; imagedestinationinfo2upu-got-by-copy2buffertoimage4upu: " + str(imagedestinationinfo2upu)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)                                  
            # {'SIZE2QUEUE': 2147483646, 
            #  'FORMAT': {'TYPE': 'CL_UNORM_INT8', 
            #  'ORDER': 'CL_RGBA'}, 
            #  'QUEUE': <queue4xi.Queue4Xi object at 0x0000000004105358>, 
            #  'WIDTH': 800, 
            #  'CONTEXT': {'COUNT2REFERENCE': 0, 
            #  'DATA2CALLBACK': {'MESSAGE': '{(Step 1Z1/33) failed; create2context4upu-failed ; time: 04:41:15 10/04/11 Pacific Daylight Time}', 
            #  'CLASS': 'create2context4upu', 
            #  'METHOD': 'create2context4upu'}, 
            #  'FUNCTION2CALLBACK': <function trace2context4upu at 0x00000000037FF348>, 
            #  'ACK2RETURN': ' 2A-2C-2D-2E-2B-3B-4B-5B-6B-7B-7B-', 
            #  'INFO2PRIVATE': None, 
            #  'PLATFORM': {'list2platforms': ['DragonXi'],'ack2return': ' 2C-2E-2F-2G-', 'nack2return': ' ', 'available2platform': 1, 'max2entries': 1}, 
            #  'NACK2RETURN': ' ', 
            #  'SIZE2PRIVATE': 0, 
            #  'PROPERTIES': None}, 
            #  'PIPE': (<read-write PipeConnection, handle 524>, <read-write PipeConnection, handle 528>), 
            #  'COUNT2REFERENCE': 1, 
            #  'FUNCTION2CALLBACK': <function trace2image4upu at 0x0000000004645548>, 
            #  'COMMANDQUEUE': ...
            
            code2ack  = code2ack + '5B-'  
        except:
            message2print = "{(Step 5Z/70) failed" + "; imagedestinationinfo2upu-failed-by-copy2buffertoimage4upu: " + str(imagedestinationinfo2upu) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)             
            code2nack  = code2nack + '5Z-'          
                  
        task4copybuffertoimage2upu ['IMAGEDESTINATION'] = imagedestinationinfo2upu         
               
        # ---------------------------------------------------------------------------------------------
        # (6) BUFFER SOURCE OBJECT - REQUIRED                        
        # ---------------------------------------------------------------------------------------------                         
        buffersourceinfo2upu = {}
         
        message2print = "{(Step 6A/70) trying" + "; buffersourceinfo2upu-trying-by-copy2buffertoimage4upu: " + str(buffersourceinfo2upu)  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print) 
        try:
            buffersourceinfo2upu = get2bufferinfo4upu()                                                                         
            
            message2print = "{(Step 6B/70) got" + "; buffersourceinfo2upu-got-by-copy2buffertoimage4upu: " + str(buffersourceinfo2upu)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)                                  
            # {'CODE2ERROR': 0, 
            #  'COUNT2REFERENCE': 0, 
            #  'DATA2CALLBACK': {'MESSAGE': '{(Step 1Z1/33) failed; create2buffer4upu-failed ; time: 04:41:15 10/04/11 Pacific Daylight Time}', 
            #  'CLASS': 'create2buffer4upu', 
            #  'METHOD': 'create2buffer4upu'}, 
            #  'FUNCTION2CALLBACK': <function trace2buffer4upu at 0x0000000004631B48>, 
            #  'POINTER2BUFFER': <mmap.mmap object at 0x00000000040EBD98>, 
            #  'FLAGS': 0, 
            #  'CONTEXT': ...
            
            code2ack  = code2ack + '6B-'  
        except:
            message2print = "{(Step 6Z/70) failed" + "; buffersourceinfo2upu-failed-by-copy2buffertoimage4upu: " + str(buffersourceinfo2upu) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)             
            code2nack  = code2nack + '6Z-'          
                  
        task4copybuffertoimage2upu ['BUFFERSOURCE'] = buffersourceinfo2upu     
        
        # ---------------------------------------------------------------------------------------------
        # (7) QUEUE - REQUIRED BY UPUXI                          
        # ---------------------------------------------------------------------------------------------         
        queue2copybuffertoimage4upu = None
        
        message2print = "{(Step 7A/70) trying" + "; queue2copybuffertoimage4upu-trying-by-copy2buffertoimage4upu: " + str(queue2copybuffertoimage4upu)  + "; time: " + time4xi() + "}" 
        print4xi(class2print, method2print, message2print) 
        try:
            queue2copybuffertoimage4upu = Queue4Xi()                                                                              # THE SYSTEM CREATES QUEUE    
            
            message2print = "{(Step 7B/70) got" + "; queue2copybuffertoimage4upu-got-by-copy2buffertoimage4upu: " + str(queue2copybuffertoimage4upu)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)                                  
            # <queue4xi.Queue4Xi object at 0x000000000492B208> 
            
            code2ack  = code2ack + '7B-'  
        except:
            message2print = "{(Step 7Z/70) failed" + "; queue2copybuffertoimage4upu-failed-by-copy2buffertoimage4upu: " + str(queue2copybuffertoimage4upu) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)             
            code2nack  = code2nack + '7Z-'
            
        task4copybuffertoimage2upu ['QUEUE'] = queue2copybuffertoimage4upu
         
        # ---------------------------------------------------------------------------------------------
        # (8) PIPE - REQUIRED BY UPUXI                              
        # ---------------------------------------------------------------------------------------------         
        pipe2upu = None
       
        message2print = "{(Step 8A/70) trying" + "; pipe2upu-trying-by-copy2buffertoimage4upu: " + str(pipe2upu)  + "; time: " + time4xi() + "}" 
        print4xi(class2print, method2print, message2print) 
        try:
            pipe2upu = Pipe4Xi()                                                                                # THE SYSTEM CREATES PIPE    
            
            message2print = "{(Step 8B/70) got" + "; pipe2upu-got-by-copy2buffertoimage4upu: " + str(pipe2upu)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)                                  
            # (<read-write PipeConnection, handle 708>, <read-write PipeConnection, handle 712>) 
            
            code2ack  = code2ack + '8B-'  
        except:
            message2print = "{(Step 8Z/70) failed" + "; pipe2upu-failed-by-copy2buffertoimage4upu: " + str(pipe2upu) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)             
            code2nack  = code2nack + '8Z-'
            
        task4copybuffertoimage2upu ['PIPE'] = pipe2upu 
        
        # ---------------------------------------------------------------------------------------------
        # (9) COMMAND QUEUE - REQUIRED                              
        # ---------------------------------------------------------------------------------------------             
        commandqueueinfo2upu = {}
       
        message2print = "{(Step 9A/70) trying" + "; commandqueueinfo2upu-trying-by-copy2buffertoimage4upu: " + str(commandqueueinfo2upu)  + "; time: " + time4xi() + "}" 
        print4xi(class2print, method2print, message2print)  
                    
        try:
            commandqueueinfo2upu = get2commandqueueinfo4upu()                                                   # SYSTEM CREATES COMMAND QUEUE, IF NECESSARY   
            message2print = "{(Step 9B/70) got" + "; commandqueueinfo2upu-got-by-copy2buffertoimage4upu: " + str(commandqueueinfo2upu)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print) 
            # {'CODE2ERROR': 0, 'COUNT2REFERENCE': 0, 'DATA2CALLBACK': ...
            
            code2ack  = code2ack + '9B-'  
        except:
            message2print = "{(Step 9Z/70) failed" + "; commandqueueinfo2upu-failed-by-copy2buffertoimage4upu: " + str(commandqueueinfo2upu) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)             
            code2nack  = code2nack + '9Z-'
    
        task4copybuffertoimage2upu ['COMMANDQUEUE'] = commandqueueinfo2upu
        
        
        
        # ---------------------------------------------------------------------------------------------
        # (11) EVENT - REQUIRED                            
        # ---------------------------------------------------------------------------------------------             
        eventinfo2upu = {}
       
        message2print = "{(Step 11A/70) trying" + "; eventinfo2upu-trying-by-copy2buffertoimage4upu: " + str(eventinfo2upu)  + "; time: " + time4xi() + "}" 
        print4xi(class2print, method2print, message2print) 
                    
        try:
            eventinfo2upu = get2eventinfo4upu()                                                                    
            message2print = "{(Step 11B/70) got" + "; eventinfo2upu-got-by-copy2buffertoimage4upu: " + str(eventinfo2upu)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print) 
            # {'CODE2ERROR': 0, 'DATA2CALLBACK': ...
            
            code2ack  = code2ack + '11B-' 
        except:
            message2print = "{(Step 11Z/70) failed" + "; eventinfo2upu-failed-by-copy2buffertoimage4upu: " + str(eventinfo2upu) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)             
            code2nack  = code2nack + '11Z-'
    
        task4copybuffertoimage2upu ['EVENT'] = eventinfo2upu 
        
        
        # ---------------------------------------------------------------------------------------------
        # (12) EVENTWAITLIST - REQUIRED                              
        # ---------------------------------------------------------------------------------------------             
        eventwaitlist2upu = {}
       
        message2print = "{(Step 12A/70) trying" + "; eventwaitlist2upu-trying-by-copy2buffertoimage4upu: " + str(eventwaitlist2upu)  + "; time: " + time4xi() + "}" 
        print4xi(class2print, method2print, message2print) 
                    
        try:
            eventwaitlist2upu = {}                                                                   
            message2print = "{(Step 12B/70) set" + "; eventwaitlist2upu-set-by-copy2buffertoimage4upu: " + str(eventwaitlist2upu)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print) 
            # {}
            
            code2ack  = code2ack + '12B-' 
        except:
            message2print = "{(Step 12Z/70) failed" + "; eventwaitlist2upu-failed-by-copy2buffertoimage4upu: " + str(eventwaitlist2upu) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)             
            code2nack  = code2nack + '12Z-'
    
        task4copybuffertoimage2upu ['EVENTWAITLIST'] = eventwaitlist2upu                     
         
        # ---------------------------------------------------------------------------------------------
        # (13) EVENTWAITLISTQUANTITY - REQUIRED                              
        # ---------------------------------------------------------------------------------------------             
        eventwaitlistquantity2upu = 0
       
        message2print = "{(Step 13A/70) trying" + "; eventwaitlistquantity2upu-trying-by-copy2buffertoimage4upu: " + str(eventwaitlistquantity2upu)  + "; time: " + time4xi() + "}" 
        print4xi(class2print, method2print, message2print) 
                    
        try:
            keys2event = eventwaitlist2upu.keys()
            for key2event in keys2event:
                eventwaitlistquantity2upu = eventwaitlistquantity2upu + 1                                                                     
            message2print = "{(Step 13B/70) set" + "; eventwaitlistquantity2upu-set-by-copy2buffertoimage4upu: " + str(eventwaitlistquantity2upu)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print) 
            # 0 
            
            code2ack  = code2ack + '13B-' 
        except:
            message2print = "{(Step 13Z/70) failed" + "; eventwaitlistquantity2upu-failed-by-copy2buffertoimage4upu: " + str(eventwaitlistquantity2upu) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)             
            code2nack  = code2nack + '13Z-'
    
        task4copybuffertoimage2upu ['EVENTWAITLISTQUANTITY'] = eventwaitlistquantity2upu       
                                             
        
        # ---------------------------------------------------------------------------------------------
        # (15) ORIGIN FOR DESTINATION - REQUIRED                             
        # ---------------------------------------------------------------------------------------------              
        origin4destination2upu = {}
        
        message2print = "{(Step 15A/70) trying" + "; origin4destination2upu-trying-by-copy2buffertoimage4upu: " + str(origin4destination2upu)  + "; time: " + time4xi() + "}" 
        print4xi(class2print, method2print, message2print) 
                    
        try:                                                                                    
            origin4destination2upu['X'] = 100 
            origin4destination2upu['Y'] = 100  
            origin4destination2upu['Z'] = 100                                                                                                                          
            message2print = "{(Step 15B/70) set" + "; origin4destination2upu-set-by-copy2buffertoimage4upu to: " + str(origin4destination2upu)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print) 
            # {'Y': 100, 'X': 100, 'Z': 100}
            code2ack  = code2ack + '15B-' 
        except:
            message2print = "{(Step 15Z/70) failed" + "; origin4destination2upu-failed-by-copy2buffertoimage4upu: " + str(origin4destination2upu) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)             
            code2nack  = code2nack + '15Z-'
    
        task4copybuffertoimage2upu ['DESTINATIONORIGIN'] = origin4destination2upu
                    
        
        
        # --------------------------------------------------------------------------------------------------------
        # (16) REGION - REGION OF A IMAGE TO BE COPIED                         
        # --------------------------------------------------------------------------------------------------------             
        region4copy2upu = {}                                                                                                         
       
        message2print = "{(Step 16A/70) trying" + "; region4copy2upu-trying-by-copy2buffertoimage4upu: " + str(region4copy2upu)  + "; time: " + time4xi() + "}" 
        print4xi(class2print, method2print, message2print) 
                    
        try:
            region4copy2upu['X'] = 10 
            region4copy2upu['Y'] = 10  
            region4copy2upu['Z'] = 10                                                                           
            message2print = "{(Step 16B/70) set" + "; region4copy2upu-set-by-copy2buffertoimage4upu to: " + str(region4copy2upu)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print) 
            # {'Y': 10, 'X': 10, 'Z': 10}
            code2ack  = code2ack + '16B-'                
             
        except:
            message2print = "{(Step 16Z/70) failed" + "; region4copy2upu-failed-by-copy2buffertoimage4upu: " + str(region4copy2upu) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)             
            code2nack  = code2nack + '16Z-'
        
        task4copybuffertoimage2upu ['REGION'] = region4copy2upu              
        
        # --------------------------------------------------------------------------------------------------------
        # (17) OFFSET - WHERE TO BEGIN COPYING DATA FROM SOURCE BUFFER                   
        # --------------------------------------------------------------------------------------------------------             
        offset4copy2upu = 0                                                                                                         
        message2print = "{(Step 17A/70) set" + "; offset4copy2upu-set-by-copy2buffertoimage4upu to: " + str(offset4copy2upu)  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print) 
        # 0
        
        task4copybuffertoimage2upu ['OFFSET'] = offset4copy2upu            
                                
        # ---------------------------------------------------------------------------------------------
        # EOF PARAMETER PROVIDING                
        # ---------------------------------------------------------------------------------------------                                                                                                                               
        
        # ---------------------------------------------------------------------------------------------
        # ANALYZE TASKS - WHETHER AUTOMATICALLY GENERATED OR USER GIVEN 
        # ---------------------------------------------------------------------------------------------
        keys2task = task4copybuffertoimage2upu.keys()
        message2print = "{(Step 20/70) got" + "; keys2task-got-by-copy2buffertoimage4upu: " + str(sorted(keys2task))  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)            
        # ['BUFFERSOURCE',
        #  'COMMANDQUEUE',
        #  'CONTEXT',
        #  'DATA2CALLBACK',
        #  'DESTINATIONORIGIN',
        #  'DEVICE',
        #  'EVENT',
        #  'EVENTWAITLIST',
        #  'EVENTWAITLISTQUANTITY',
        #  'FUNCTION2CALLBACK',
        #  'IMAGEDESTINATION',
        #  'OFFSET',
        #  'PIPE',
        #  'PLATFORM',
        #  'QUEUE',
        #  'REGION']; time: 04:41:15 10/04/11 Pacific Daylight Time}
 
        if (('DATA2CALLBACK' in keys2task) and ('FUNCTION2CALLBACK' in keys2task)):
            pass
        else:                
            # ---------------------------------------------------------------------------------------------
            # (21B) TRACE - AUTOMATIC                      
            # ---------------------------------------------------------------------------------------------              
            function2callback4upu = trace2image4upu                                                             # CALLBACK FUNCTION FOR UPU
            
            message2print = "{(Step 21B1/70) initialized" + "; function2callback4upu-created-by-copy2buffertoimage4upu to: " + str(function2callback4upu)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)            
            # 
            
            code2ack  = code2ack + '21B1-'
           
            args2callback = {}                                                                                  # CREATE ARGUMENT DICTIONARY FOR CALLBACK FUNCTION             
            args2callback['CLASS']   = "copy2buffertoimage4upu"                                                        # MODULE OF UPUIZED OPENCL MODULE
            args2callback['METHOD']  = "copy2buffertoimage4upu"                                                        # FUNCTION OF UPUIZED OPENCL MODULE
            args2callback['MESSAGE'] = "{(Step 1Z1/33) failed" + "; copy2buffertoimage4upu-failed " + "; time: " + time4xi() + "}" # ERROR MESSAGE FOR UPUIZED OPENCL TO RETURN    

            message2print = "{(Step 21B2/70) initialized" + "; args2callback-created-by-copy2buffertoimage4upu to: " + str(args2callback)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)               
            # 

            code2ack  = code2ack + '21B2-'
        
            # ---------------------------------------------------------------------------------------------
            # (21C) CALLBACK - AUTOMATIC                      
            # ---------------------------------------------------------------------------------------------              
            task4copybuffertoimage2upu['DATA2CALLBACK'] = args2callback                                                # USER DATA PASSED TO UPUIZED OPENCL - SEVERAL ARGS PASSED IN ONE DICTIONARY
            task4copybuffertoimage2upu['FUNCTION2CALLBACK'] = function2callback4upu                                    # CALLBACK FUNCTION TO CALLED BY OPENCL IN EXCEPTIONS
            
            code2ack  = code2ack + '21C-'
            message2print = "{(Step 21C/70) initialized" + "; task4copybuffertoimage2upu-created-by-copy2buffertoimage4upu to: " + str(task4copybuffertoimage2upu)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)  
            #
             
        if 'PLATFORM' in keys2task:
            pass
        else:  
            # ---------------------------------------------------------------------------------------------
            # (22) PLATFORM - AUTOMATIC                   
            # ---------------------------------------------------------------------------------------------             
            platform2xi = {}
            message2print = "{(Step 22A/70) initialized" + "; platform2xi-created-by-copy2buffertoimage4upu to: " + str(platform2xi)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)   
            # 
            
            try:
                platform2xi = get2platformids4xi()                                                              # PLATFORM IS COMMON TO ALL DEVICES I.E TO DEVICE XI
                message2print = "{(Step 22B/70) got" + "; platform2xi-got-by-copy2buffertoimage4upu: " + str(platform2xi)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)
                # 
                 
                code2ack  = code2ack + '22B-'
            except:
                message2print = "{(Step 22Z/70) failed" + "; platform2xi-failed-by-copy2buffertoimage4upu: " + str(failed)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)                                                
                code2nack  = code2nack + '22Z-'
                 
            task4copybuffertoimage2upu['PLATFORM'] = platform2xi                                                       # PLATFORM OF SYSTEM 
          
        if 'CONTEXT' in keys2task:
            pass
        else:              
            # ---------------------------------------------------------------------------------------------
            # (23) CONTEXT - AUTOMATIC                 
            # ---------------------------------------------------------------------------------------------    
            context2upu = {}
            message2print = "{(Step 23A/70) trying" + "; context2upu-trying-by-copy2buffertoimage4upu: " + str(context2upu)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print) 
            try:
                context2upu = get2context4upu()                                                                 # NO PARAMETERS - THE SYSTEM CREATES CONTEXT IF REQUIRED  
                
                message2print = "{(Step 23B/70) got" + "; context2upu-got-by-copy2buffertoimage4upu: " + str(context2upu)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)                                  
                # 
                
                code2ack  = code2ack + '23B-'         
            except:
                message2print = "{(Step 23Z/70) failed" + "; context2upu-failed-by-copy2buffertoimage4upu: " + str(context2upu) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)             
                code2nack  = code2nack + '23Z-'
                
            task4copybuffertoimage2upu ['CONTEXT'] = context2upu           
        
        if 'DEVICE' in keys2task:
            pass
        else:                   
            # ---------------------------------------------------------------------------------------------
            # (24) DEVICE - AUTOMATIC                       
            # ---------------------------------------------------------------------------------------------    
            device2upu = {}
            
            message2print = "{(Step 24A/70) trying" + "; device2upu-trying-by-copy2buffertoimage4upu: " + str(device2upu)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print) 
            try:
                device2upu = get2upu4xi()                                                                       # THE SYSTEM CREATES DEVICE INFO, IF NECESSARY    
                
                message2print = "{(Step 24B/70) got" + "; device2upu-got-by-copy2buffertoimage4upu: " + str(device2upu)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)                                  
                # 
                
                code2ack  = code2ack + '24B-'  
            except:
                message2print = "{(Step 24Z/70) failed" + "; device2upu-failed-by-copy2buffertoimage4upu: " + str(device2upu) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)             
                code2nack  = code2nack + '24Z-'
            
            task4copybuffertoimage2upu ['DEVICE'] = device2upu
        
        if 'IMAGEDESTINATION' in keys2task:
            pass
        else:                      
            # ---------------------------------------------------------------------------------------------
            # (25) DESTINATION IMAGE - REQUIRED                            
            # ---------------------------------------------------------------------------------------------         
            destinationimageinfo2upu = {}
             
            message2print = "{(Step 25A/70) trying" + "; destinationimageinfo2upu-trying-by-copy2buffertoimage4upu: " + str(destinationimageinfo2upu)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print) 
            try:
                destinationimageinfo2upu = get2imageinfo4upu()                                                                         
                
                message2print = "{(Step 25B/70) got" + "; destinationimageinfo2upu-got-by-copy2buffertoimage4upu: " + str(destinationimageinfo2upu)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)                                  
                #                 
                code2ack  = code2ack + '25B-'  
            except:
                message2print = "{(Step 25Z/70) failed" + "; destinationimageinfo2upu-failed-by-copy2buffertoimage4upu: " + str(destinationimageinfo2upu) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)             
                code2nack  = code2nack + '25Z-'          
                      
            task4copybuffertoimage2upu ['IMAGEDESTINATION'] = destinationimageinfo2upu         
        
        if 'BUFFERSOURCE' in keys2task:
            pass
        else:                      
            # ---------------------------------------------------------------------------------------------
            # (26) SOURCE BUFFER - REQUIRED                            
            # ---------------------------------------------------------------------------------------------         
            sourcebufferinfo2upu = {}
             
            message2print = "{(Step 26A/70) trying" + "; sourcebufferinfo2upu-trying-by-copy2buffertoimage4upu: " + str(sourcebufferinfo2upu)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print) 
            try:
                sourcebufferinfo2upu = get2bufferinfo4upu()                                                                         
                
                message2print = "{(Step 26B/70) got" + "; sourcebufferinfo2upu-got-by-copy2buffertoimage4upu: " + str(sourcebufferinfo2upu)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)                                  
                #                 
                code2ack  = code2ack + '26B-'  
            except:
                message2print = "{(Step 26Z/70) failed" + "; sourcebufferinfo2upu-failed-by-copy2buffertoimage4upu: " + str(sourcebufferinfo2upu) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)             
                code2nack  = code2nack + '26Z-'          
                      
            task4copybuffertoimage2upu ['BUFFERSOURCE'] = sourcebufferinfo2upu    
                                    
        if 'QUEUE' in keys2task:
            pass
        else:   
            # ---------------------------------------------------------------------------------------------
            # (27) QUEUE - REQUIRED BY UPUXI                          
            # ---------------------------------------------------------------------------------------------         
            queue2copybuffertoimage4upu = None
            
            message2print = "{(Step 27A/70) trying" + "; queue2copybuffertoimage4upu-trying-by-copy2buffertoimage4upu: " + str(queue2copybuffertoimage4upu)  + "; time: " + time4xi() + "}" 
            print4xi(class2print, method2print, message2print) 
            try:
                queue2copybuffertoimage4upu = Queue4Xi()                                                                          # THE SYSTEM CREATES QUEUE    
                
                message2print = "{(Step 27B/70) got" + "; queue2copybuffertoimage4upu-got-by-copy2buffertoimage4upu: " + str(queue2copybuffertoimage4upu)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)                                  
                # 
                
                code2ack  = code2ack + '27B-'  
            except:
                message2print = "{(Step 27Z/70) failed" + "; queue2copybuffertoimage4upu-failed-by-copy2buffertoimage4upu: " + str(queue2copybuffertoimage4upu) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)             
                code2nack  = code2nack + '27Z-'
                
            task4copybuffertoimage2upu ['QUEUE'] = queue2copybuffertoimage4upu
            
        if 'PIPE' in keys2task:
            pass
        else:              
            # ---------------------------------------------------------------------------------------------
            # (28) PIPE - REQUIRED BY UPUXI                              
            # ---------------------------------------------------------------------------------------------         
            pipe2upu = None
           
            message2print = "{(Step 28A/70) trying" + "; pipe2upu-trying-by-copy2buffertoimage4upu: " + str(pipe2upu)  + "; time: " + time4xi() + "}" 
            print4xi(class2print, method2print, message2print) 
            try:
                pipe2upu = Pipe4Xi()                                                                            # THE SYSTEM CREATES PIPE    
                
                message2print = "{(Step 28B/70) got" + "; pipe2upu-got-by-copy2buffertoimage4upu: " + str(pipe2upu)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)                                  
                # 
                
                code2ack  = code2ack + '28B-'  
            except:
                message2print = "{(Step 28Z/70) failed" + "; pipe2upu-failed-by-copy2buffertoimage4upu: " + str(pipe2upu) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)             
                code2nack  = code2nack + '28Z-'
                
            task4copybuffertoimage2upu ['PIPE'] = pipe2upu 
            
        if 'COMMANDQUEUE' in keys2task:
            pass
        else:              
            # ---------------------------------------------------------------------------------------------
            # (28) COMMAND QUEUE - REQUIRED                              
            # ---------------------------------------------------------------------------------------------             
            commandqueueinfo2upu = {}
           
            message2print = "{(Step 29A/70) trying" + "; commandqueueinfo2upu-trying-by-copy2buffertoimage4upu: " + str(commandqueueinfo2upu)  + "; time: " + time4xi() + "}" 
            print4xi(class2print, method2print, message2print)  
                        
            try:
                commandqueueinfo2upu = get2commandqueueinfo4upu()                                               # SYSTEM CREATES COMMAND QUEUE, IF NECESSARY   
                message2print = "{(Step 29B/70) got" + "; commandqueueinfo2upu-got-by-copy2buffertoimage4upu: " + str(commandqueueinfo2upu)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print) 
                # 
                
                code2ack  = code2ack + '29B-'  
            except:
                message2print = "{(Step 29Z/70) failed" + "; commandqueueinfo2upu-failed-by-copy2buffertoimage4upu: " + str(commandqueueinfo2upu) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)             
                code2nack  = code2nack + '29Z-'
        
            task4copybuffertoimage2upu ['COMMANDQUEUE'] = commandqueueinfo2upu
        
             
        if 'EVENT' in keys2task:
            pass
        else:               
            # ---------------------------------------------------------------------------------------------
            # (31) EVENT - REQUIRED                            
            # ---------------------------------------------------------------------------------------------             
            eventinfo2upu = {}
           
            message2print = "{(Step 31A/70) trying" + "; eventinfo2upu-trying-by-copy2buffertoimage4upu: " + str(eventinfo2upu)  + "; time: " + time4xi() + "}" 
            print4xi(class2print, method2print, message2print) 
                        
            try:
                eventinfo2upu = get2eventinfo4upu()                                                                    
                message2print = "{(Step 31B/70) got" + "; eventinfo2upu-got-by-copy2buffertoimage4upu: " + str(eventinfo2upu)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print) 
                # 
                
                code2ack  = code2ack + '31B-' 
            except:
                message2print = "{(Step 31Z/70) failed" + "; eventinfo2upu-failed-by-copy2buffertoimage4upu: " + str(eventinfo2upu) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)             
                code2nack  = code2nack + '31Z-'
        
            task4copybuffertoimage2upu ['EVENT'] = eventinfo2upu 
        
             
        if 'EVENTWAITLIST' in keys2task:
            pass
        else:               
            # ---------------------------------------------------------------------------------------------
            # (32) EVENTWAITLIST - REQUIRED                              
            # ---------------------------------------------------------------------------------------------             
            eventwaitlist2upu = {}
           
            message2print = "{(Step 32A/70) trying" + "; eventwaitlist2upu-trying-by-copy2buffertoimage4upu: " + str(eventwaitlist2upu)  + "; time: " + time4xi() + "}" 
            print4xi(class2print, method2print, message2print) 
                        
            try:
                eventwaitlist2upu = {}                                                                   
                message2print = "{(Step 32B/70) set" + "; eventwaitlist2upu-set-by-copy2buffertoimage4upu: " + str(eventwaitlist2upu)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print) 
                # 
                
                code2ack  = code2ack + '32B-' 
            except:
                message2print = "{(Step 32Z/70) failed" + "; eventwaitlist2upu-failed-by-copy2buffertoimage4upu: " + str(eventwaitlist2upu) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)             
                code2nack  = code2nack + '32Z-'
        
            task4copybuffertoimage2upu ['EVENTWAITLIST'] = eventwaitlist2upu                     
              
        if 'EVENTWAITLISTQUANTITY' in keys2task:
            pass
        else:               
            # ---------------------------------------------------------------------------------------------
            # (33) EVENTWAITLISTQUANTITY - REQUIRED                              
            # ---------------------------------------------------------------------------------------------             
            eventwaitlistquantity2upu = 0
           
            message2print = "{(Step 33A/70) trying" + "; eventwaitlistquantity2upu-trying-by-copy2buffertoimage4upu: " + str(eventwaitlistquantity2upu)  + "; time: " + time4xi() + "}" 
            print4xi(class2print, method2print, message2print) 
                        
            try:
                keys2event = eventwaitlist2upu.keys()
                for key2event in keys2event:
                    eventwaitlistquantity2upu = eventwaitlistquantity2upu + 1                                                                     
                message2print = "{(Step 33B/70) set" + "; eventwaitlistquantity2upu-set-by-copy2buffertoimage4upu: " + str(eventwaitlistquantity2upu)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print) 
                # 0 
                
                code2ack  = code2ack + '33B-' 
            except:
                message2print = "{(Step 33Z/70) failed" + "; eventwaitlistquantity2upu-failed-by-copy2buffertoimage4upu: " + str(eventwaitlistquantity2upu) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)             
                code2nack  = code2nack + '33Z-'
        
            task4copybuffertoimage2upu ['EVENTWAITLISTQUANTITY'] = eventwaitlistquantity2upu       
                                             
  
             
        if 'DESTINATIONORIGIN' in keys2task:
            pass
        else:                    
            # ---------------------------------------------------------------------------------------------
            # (34) DESTINATIONORIGIN - REQUIRED                             
            # ---------------------------------------------------------------------------------------------             
            destination4origin2upu = {}
            
            message2print = "{(Step 34A/70) trying" + "; destination4origin2upu-trying-by-copy2buffertoimage4upu: " + str(destination4origin2upu)  + "; time: " + time4xi() + "}" 
            print4xi(class2print, method2print, message2print) 
                        
            try:                                                                                    
                destination4origin2upu['X'] = 0 
                destination4origin2upu['Y'] = 0  
                destination4origin2upu['Z'] = 0                                                                                                                          
                message2print = "{(Step 34B/70) set" + "; destination4origin2upu-set-by-copy2buffertoimage4upu to: " + str(destination4origin2upu)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print) 
                # 
                code2ack  = code2ack + '34B-' 
            except:
                message2print = "{(Step 34Z/70) failed" + "; destination4origin2upu-failed-by-copy2buffertoimage4upu: " + str(destination4origin2upu) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)             
                code2nack  = code2nack + '34Z-'
        
            task4copybuffertoimage2upu ['DESTINATIONORIGIN'] = destination4origin2upu
             
     
             
        if 'REGION' in keys2task:
            pass
        else:            
            # ---------------------------------------------------------------------------------------------
            # (36) REGION - REQUIRED                         
            # ---------------------------------------------------------------------------------------------             
            region2upu = {}
            
            message2print = "{(Step 36A/70) trying" + "; region2upu-trying-by-copy2buffertoimage4upu: " + str(region2upu)  + "; time: " + time4xi() + "}" 
            print4xi(class2print, method2print, message2print) 
                        
            try:                                                                                    
                region2upu['X'] = 160                                                                               # WIDTH  IN PIXELS TO WRITE 
                region2upu['Y'] = 90                                                                                # HEIGHT IN PIXELS TO WRITE
                region2upu['Z'] = 0                                                                                 # DEPTH  IN PIXELS TO WRITE                                                                                                                          
                message2print = "{(Step 36B/70) set" + "; region2upu-set-by-copy2buffertoimage4upu to: " + str(region2upu)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print) 
                # 
                code2ack  = code2ack + '36B-' 
            except:
                message2print = "{(Step 36Z/70) failed" + "; region2upu-failed-by-copy2buffertoimage4upu: " + str(region2upu) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)             
                code2nack  = code2nack + '36Z-'
        
            task4copybuffertoimage2upu ['REGION'] = region2upu
        
             
        if 'OFFSET' in keys2task:
            pass
        else:            
            # ---------------------------------------------------------------------------------------------
            # (37) OFFSET - WHERE TO BEGIN COPYING DATA FROM SOURCE BUFFER - REQUIRED                         
            # ---------------------------------------------------------------------------------------------             
            offset2upu = 0
            message2print = "{(Step 37A/70) set" + "; offset2upu-set-by-copy2buffertoimage4upu to: " + str(offset2upu)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print) 
            # 
            task4copybuffertoimage2upu ['OFFSET'] = offset2upu        
            code2ack  = code2ack + '37A-' 
                    
        # ---------------------------------------------------------------------------------------------
        # EOF TASK ANALYSIS
        # ---------------------------------------------------------------------------------------------
        keys2task = task4copybuffertoimage2upu.keys()
        message2print = "{(Step 39/70) set" + "; keys2task-set-by-copy2buffertoimage4upu to: " + str(sorted(keys2task))  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)            
        # ['BUFFERSOURCE',
        #  'COMMANDQUEUE',
        #  'CONTEXT',
        #  'DATA2CALLBACK',
        #  'DESTINATIONORIGIN',
        #  'DEVICE',
        #  'EVENT',
        #  'EVENTWAITLIST',
        #  'EVENTWAITLISTQUANTITY',
        #  'FUNCTION2CALLBACK',
        #  'IMAGEDESTINATION',
        #  'OFFSET',
        #  'PIPE',
        #  'PLATFORM',
        #  'QUEUE',
        #  'REGION']; time: 04:52:11 10/04/11 Pacific Daylight Time}
                


    # ---------------------------------------------------------------------------------------------
    # CONTROL AND COMMAND
    # ---------------------------------------------------------------------------------------------
    control2copybuffertoimage4upu = {}       
                                
    # ---------------------------------------------------------------------------------------------
    # CONSUMER AND PUBLISHER CHANNEL FOR CONTROL AND COMMAND
    # ---------------------------------------------------------------------------------------------
    ack4channels2upu = False
    channels2upu = ()
    message2print = "{(Step 40A/70) trying" + "channels2upu-trying-by-copy2buffertoimage4upu: " + str(channels2upu)  + "; time: " + time4xi() + "}" 
    print4xi(class2print, method2print, message2print)     
    try:
        # 
        consumerchannel2copybuffertoimage4upu, publisherchannel2copybuffertoimage4upu = task4copybuffertoimage2upu ['PIPE']

        message2print = "{(Step 40B/70) got" + "; consumerchannel2copybuffertoimage4upu-got-by-copy2buffertoimage4upu: " + str(consumerchannel2copybuffertoimage4upu)  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)            
        # <read-write PipeConnection, handle 704>

        message2print = "{(Step 40C/70) got" + "; publisherchannel2copybuffertoimage4upu-got-by-copy2buffertoimage4upu: " + str(publisherchannel2copybuffertoimage4upu)  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)            
        # <read-write PipeConnection, handle 708>                      
        ack4channels2upu = True
        
        control2copybuffertoimage4upu['CONSUMERCHANNEL']  = consumerchannel2copybuffertoimage4upu
        
        
        control2copybuffertoimage4upu['PUBLISHERCHANNEL'] = publisherchannel2copybuffertoimage4upu
        
        code2ack  = code2ack + '40D-' 
        
    except:
        ack4channels2upu = False
        code2nack  = code2nack + '40Z-'      
                        
    # ---------------------------------------------------------------------------------------------
    # QUEUE FOR CONTROL AND COMMAND
    # ---------------------------------------------------------------------------------------------
    ack4queue2copybuffertoimage4upu = False
    queue2copybuffertoimage4upu = None
    message2print = "{(Step 41A/70) trying" + "queue2copybuffertoimage4upu-trying-by-copy2buffertoimage4upu: " + str(queue2copybuffertoimage4upu)  + "; time: " + time4xi() + "}" 
    print4xi(class2print, method2print, message2print)     
    try:
        queue2copybuffertoimage4upu = task4copybuffertoimage2upu ['QUEUE']

        message2print = "{(Step 41B/70) got" + "; queue2copybuffertoimage4upu-got-by-copy2buffertoimage4upu: " + str(queue2copybuffertoimage4upu)  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)            
        # <queue4xi.Queue4Xi object at 0x0000000003DF2198>
              
        ack4queue2copybuffertoimage4upu = True
        control2copybuffertoimage4upu['QUEUE']  = queue2copybuffertoimage4upu        
        code2ack  = code2ack + '41C-'
    except:
        ack4queue2copybuffertoimage4upu = False  
        code2nack  = code2nack + '41Z-'
    
    
    # -----------------------------------------------------------------------------------------    
    # DUMMY PROCESS OBJECT = THREAD CREATION
    # -----------------------------------------------------------------------------------------           
    # dummy2image4upu = None
    # message2print = "{(Step 69A/70) trying" + "dummy2image4upu-trying-by-copy2buffertoimage4upu: " + str(dummy2image4upu)  + "; time: " + time4xi() + "}"   
    # print4xi(class2print, method2print, message2print)     
    # try:                            
        # dummy2image4upu = DummyProcess4Xi(target=manoeuvre2copybuffertoimage4upu,                                               # in this module to handle queue and pipe channels
        #                                   args =(queue2copybuffertoimage4upu,                                                         # caller puts and gets - does NOT call any functions
        #                                          consumerchannel2copybuffertoimage4upu, publisherchannel2copybuffertoimage4upu))                        # caller sends and receives - does NOT call any functions
        # message2print = "{(Step 69B/70) created" + "dummy2image4upu-created-by-copy2buffertoimage4upu: " + str(dummy2image4upu)  + "; time: " + time4xi() + "}" 
        # print4xi(class2print, method2print, message2print)  
        # code2ack  = code2ack + '69B-'
        
    # except:
        # code2nack  = code2nack + '69Z-'
        
    # ---------------------------------------------------------------------------------------------
    # RETURN CONTROL AND COMMAND DICTIONARY
    # ---------------------------------------------------------------------------------------------          
    message2print = "{(Step 70A/70) set" + "; code2ack-set-by-copy2buffertoimage4upu: " + str(code2ack) + "; time: " + time4xi() + "}"  
    print4xi(class2print, method2print, message2print)                                                             
    # 1B1-1B2-1C-2B-3B-4B-5B-6B-7B-8B-9B-11B-12B-13B-15B-16B-40D-41C- 
    message2print = "{(Step 70B/70) set" + "; code2nack-set-by-copy2buffertoimage4upu: " + str(code2nack) + "; time: " + time4xi() + "}" 
    print4xi(class2print, method2print, message2print)                                                             
    # 
        
    message2print = "{(Step 70C/70) returning" + "; control2copybuffertoimage4upu-returning-by-copy2buffertoimage4upu: " + str(control2copybuffertoimage4upu) + "; time: " + time4xi() + "}" 
    print4xi(class2print, method2print, message2print)                                                             
    # {'QUEUE': <queue4xi.Queue4Xi object at 0x0000000003DF2198>,
    #  'PUBLISHERCHANNEL': <read-write PipeConnection, handle 708>,
    #  'CONSUMERCHANNEL': <read-write PipeConnection, handle 704>}; time: 04:52:11 10/04/11 Pacific Daylight Time}

        
    return control2copybuffertoimage4upu
                    
#</function4xi> 


# -------------------------------------------------------------------------------------------------
#<function4xi function = "read2image4upu" usage = "
#                                                                                                              
#   # ---------------------------------------------------------------------------------------------
#   # (16) ESTABLISH CONTROL & COMMAND WITH READ2IMAGE4UPU                      
#   # ---------------------------------------------------------------------------------------------    
#   print('(Step 16A/700) OF UPU2IMAGE4XI5TEST - TRYING TO ESTABLISH CONTROL & COMMAND WITH READ2IMAGE4UPU ', time4xi())    
#   respon4read2image = {} 
#   task4read2image = None
#   try:                               
#       response4read2image = read2image4upu(task4read2image)                                                                                                          
#       print('(Step 16B/700) OF UPU2IMAGE4XI5TEST - ESTABLISHED CONTROL & COMMAND WITH READ2IMAGE4UPU : ', response4read2image)
#       queue2read4upu      = response4read2image['QUEUE']
#       channel2readpublish4upu = response4read2image['PUBLISHERCHANNEL']
#       channel2readconsume4upu = response4read2image['CONSUMERCHANNEL']
#   except:
#       print('(Step 16Z/700) OF UPU2IMAGE4XI5TEST *** FAILED TO ESTABLISH CONTROL & COMMAND WITH READ2IMAGE4UPU ', time4xi()) 
#   # --------------------------------------------------------------------------------------------------
#   # (17) SEND CONTROL & COMMAND COPY VIA PIPE FOR MANOEUVRE2IMAGE4UPU TO RECEIVE                 
#   # --------------------------------------------------------------------------------------------------      
#   print('(Step 17A/700) OF UPU2IMAGE4XI5TEST - TRYING TO SEND CONTROL & COMMAND VIA PIPE WITH READ2IMAGE4UPU ', time4xi())     
#   try:                               
#       message2readsend4upu = "<copy4xi>copy2read4upu</copy4xi>"
#       channel2readpublish4upu = response4read2image['PUBLISHERCHANNEL']
#       channel2readpublish4upu.send(message2readsend4upu)
#       print("(Step 17B/700) OF UPU2IMAGE4XI5TEST - SENT TO CHANNEL2PUBLISH4UPU: ", channel2readpublish4upu, " MESSAGE2SEND4UPU: " , message2readsend4upu)                                                                                                
#   except:
#       print('(Step 17Z1/700) OF UPU2IMAGE4XI5TEST *** FAILED TO SEND CONTROL & COMMAND VIA PIPE WITH READ2IMAGE4UPU ', time4xi())                                                
#   # ------------------------------------------------------------------------------------------
#   # (18) LET MANOEUVRE2IMAGE4UPU TO RECEIVE COPY AND AND SEND ROGER VIA PIPE                
#   # ------------------------------------------------------------------------------------------       
#   print("(Step 18C/700) OF UPU2IMAGE4XI5TEST - TRYING TO PERFORM TARGET FUNCTION MANOEUVRE2IMAGE4UPU ", time4xi()) 
#   try:        
#       response2manoeuvre4upu = manoeuvre2readimage4upu(queue2read4upu, channel2readconsume4upu, channel2readpublish4upu)
#       print("(Step 18D/700) OF UPU2IMAGE4XI5TEST - PERFORMED TARGET FUNCTION MANOEUVRE2IMAGE4UPU ", manoeuvre2readimage4upu)
#   except:
#       print("(Step 18Z/700) OF UPU2IMAGE4XI5TEST *** FAILED TO PERFORM TARGET FUNCTION MANOEUVRE2IMAGE4UPU ", time4xi())     
#   else:
#       pass # trust activated thread    
#   # ---------------------------------------------------------------------------------------------
#   # (19) RECEIVE CONTROL & COMMAND ROGER VIA PIPE                   
#   # ---------------------------------------------------------------------------------------------          
#   print('(Step 19A/700) OF UPU2IMAGE4XI5TEST - TRYING TO RECEIVE CONTROL & COMMAND VIA PIPE WITH READ2IMAGE4UPU ', time4xi())     
#   try:
#       channel2readconsume4upu = response4read2image['CONSUMERCHANNEL']                              
#       message2readreceive4upu = channel2readconsume4upu.recv()
#       print("(Step 19B/700) OF UPU2IMAGE4XI5TEST - RECEIVED FROM CHANNEL2CONSUME4UPU: ", channel2readconsume4upu, " CONTROL & COMMAND: ", message2readreceive4upu)                                                                                                  
#       # <roger4xi>roger2read4upu</roger4xi>
#   except:
#       print('(Step 19Z/700) OF UPU2IMAGE4XI5TEST *** FAILED TO RECEIVE CONTROL & COMMAND VIA PIPE  WITH READ2IMAGE4UPU ', time4xi())         
#   # ---------------------------------------------------------------------------------------------------
#   # (20) SEND CONTROL & COMMAND READY VIA PIPE FOR MANOEUVRE2IMAGE4UPU TO RECEIVE                
#   # ---------------------------------------------------------------------------------------------------      
#   print('(Step 20A/700) OF UPU2IMAGE4XI5TEST - TRYING TO SEND CONTROL & COMMAND VIA PIPE WITH READ2IMAGE4UPU ', time4xi())     
#   try:                               
#       message2readsend4upu = "<ready4xi>ready2read4upu</ready4xi>"
#       channel2readpublish4upu = response4read2image['PUBLISHERCHANNEL']
#       channel2readpublish4upu.send(message2readsend4upu)
#       print("(Step 20B/700) OF UPU2IMAGE4XI5TEST - SENT TO CHANNEL2PUBLISH4UPU: ", channel2readpublish4upu, " MESSAGE2SEND4UPU: " , message2readsend4upu)                                                                                                
#       # <ready4xi>ready2read4upu</ready4xi> 
#   except:
#        print('(Step 20Z/700) OF UPU2IMAGE4XI5TEST *** FAILED TO SEND CONTROL & COMMAND VIA PIPE WITH READ2IMAGE4UPU ', time4xi())                                                
#   # -----------------------------------------------------------------------------------
#   # (21) LET MANOEUVRE2IMAGE4UPU TO RECEIVE READY AND TO SEND READY                
#   # -----------------------------------------------------------------------------------       
#   print("(Step 21A/700) OF UPU2IMAGE4XI5TEST - TRYING TO PERFORM TARGET FUNCTION MANOEUVRE2IMAGE4UPU ", time4xi()) 
#   try:        
#       response2manoeuvre4upu = manoeuvre2readimage4upu(queue2read4upu, channel2readconsume4upu, channel2readpublish4upu)
#       print("(Step 21B/700) OF UPU2IMAGE4XI5TEST - PERFORMED TARGET FUNCTION MANOEUVRE2IMAGE4UPU ", manoeuvre2readimage4upu)
#   except:
#       print("(Step 21Z/700) OF UPU2IMAGE4XI5TEST *** FAILED TO PERFORM TARGET FUNCTION MANOEUVRE2IMAGE4UPU ", time4xi())     
#   # ---------------------------------------------------------------------------------------
#   # (22) RECEIVE CONTROL & COMMAND READY VIA PIPE                      
#   # ---------------------------------------------------------------------------------------          
#   print('(Step 22A/700) OF UPU2IMAGE4XI5TEST - TRYING TO RECEIVE CONTROL & COMMAND VIA PIPE WITH READ2IMAGE4UPU ', time4xi())     
#   try:
#       channel2readconsume4upu = response4read2image['CONSUMERCHANNEL']                              
#       message2readreceive4upu = channel2readconsume4upu.recv()
#       print("(Step 22B/700) OF UPU2IMAGE4XI5TEST - RECEIVED FROM CHANNEL2CONSUME4UPU: ", channel2readconsume4upu, " CONTROL & COMMAND: ", message2readreceive4upu)                                                                                                  
#       # <ready4xi>ready2read4upu</ready4xi> 
#   except:
#       print('(Step 22Z/700) OF UPU2IMAGE4XI5TEST *** FAILED TO RECEIVE CONTROL & COMMAND VIA PIPE  WITH READ2IMAGE4UPU ', time4xi())          
#   # -----------------------------------------------------------------------------------------
#   # (24) PUT IMAGE PATH TO QUEUE
#   # -----------------------------------------------------------------------------------------                          
#   print("(Step 24A/700) OF UPU2IMAGE4XI5TEST - TRYING TO PUT IMAGE PATH TO QUEUE ", time4xi())      
#   try:    
#       queue2read4dragon.put(path2dragon)             
#       print("(Step 24B/700) OF DRAGON2IMAGE4XI5TEST - PUT TO QUEUE IMAGE PATH: ", path2dragon) 
#       # ..//..//..//..//..//web//images//png//dragonxi.png  
#   except:
#       print("(Step 24Z/700) OF UPU2IMAGE4XI5TEST *** FAILED TO PUT IMAGE PATH TO QUEUE ", time4xi()) 
#   # ------------------------------------------------------------------------------------------------------
#   # (25) LET MANOEUVRE2IMAGE4UPU TO GET FROM QUEUE AND TO SEND EITHER ACK OR NACK VIA PIPE               
#   # -------------------------------------------------------------------------------------------------------       
#   print("(Step 25A/700) OF UPU2IMAGE4XI5TEST - TRYING TO PERFORM TARGET FUNCTION MANOEUVRE2IMAGE4UPU ", time4xi()) 
#   try:        
#       response2manoeuvre4upu = manoeuvre2readimage4upu(queue2read4upu, channel2readconsume4upu, channel2readpublish4upu)
#       print("(Step 25B/700) OF UPU2IMAGE4XI5TEST - PERFORMED TARGET FUNCTION MANOEUVRE2IMAGE4UPU ", manoeuvre2readimage4upu)
#   except:
#       print("(Step 25Z/700) OF UPU2IMAGE4XI5TEST *** FAILED TO PERFORM TARGET FUNCTION MANOEUVRE2IMAGE4UPU ", time4xi())     
#   # -----------------------------------------------------------------------------------------------
#   # (26) RECEIVE ACK OR NACK ABOUT IMAGE OPERATION VIA PIPE                
#   # -----------------------------------------------------------------------------------------------          
#   print('(Step 26A/700) OF UPU2IMAGE4XI5TEST - TRYING TO RECEIVE CONTROL & COMMAND ABOUT IMAGE OPERTION VIA PIPE ', time4xi())      
#   try: 
#       message2readreceive4upu = channel2readconsume4upu.recv()
#       print("(Step 26D/700) OF UPU2IMAGE4XI5TEST - RECEIVED FROM CHANNEL2CONSUME4UPU: ", channel2readconsume4upu, " CONTROL & COMMAND: ", message2readreceive4upu)                                                                                                   
#       # <ack4xi>ack2imageinfo4upu</ack4xi>
#   except:
#       print('(Step 26Z/700) OF UPU2IMAGE4XI5TEST *** FAILED TO RECEIVE CONTROL & COMMAND ABOUT IMAGE OPERTION VIA PIPE ', time4xi())               
#   # -------------------------------------------------------------------------------------------------------
#   # (27) LET MANOEUVRE2IMAGE4UPU GO TO STATE 0               
#   # -------------------------------------------------------------------------------------------------------       
#   print("(Step 27A/700) OF UPU2IMAGE4XI5TEST - TRYING TO PERFORM TARGET FUNCTION MANOEUVRE2IMAGE4UPU ", time4xi()) 
#   try:        
#       response2manoeuvre4upu = manoeuvre2readimage4upu(queue2read4upu, channel2readconsume4upu, channel2readpublish4upu)
#       print("(Step 27B/700) OF UPU2IMAGE4XI5TEST - GOT RESPONSE2MANOEUVRE4UPU ", response2manoeuvre4upu)
#   except:
#       print("(Step 27Z/700) OF UPU2IMAGE4XI5TEST *** FAILED TO PERFORM TARGET FUNCTION MANOEUVRE2IMAGE4UPU ", time4xi())   
#">

# -------------------------------------------------------------------------------------------------   
def read2image4upu(task4imageread2upu=None):

    '''
    reads 2D images for UPU
    '''
    global upu2image4xi                                                     # 2D-IMAGES FOR UPU  
    global control2readimage4upu                                              # COMMAND AND CONTROL FOR UPU IMAGES 
    
    # -----------------------------------------------------------------------------------------
    # (Step 1/33) PARAMETERS           
    # -----------------------------------------------------------------------------------------          
    class2print = "read2image4upu"
    method2print = "read2image4upu"

    message2print = "{(Step 1A/33) started" + "; read2image4upu-started: " + str(read2image4upu)  + "; time: " + time4xi() + "}" 
    print4xi(class2print, method2print, message2print)                                                                     #  
    # <function read2image4upu at 0x00000000039595C8> 
    
    message2print = "{(Step 1B/33) passed" + "; task4imageread2upu-to-read2image4upu: " + str(task4imageread2upu) + "; time: " + time4xi() + "}"
    print4xi(class2print, method2print, message2print)                                                              
    # None
            
    code2ack  = ' '
    code2nack = ' '
    
    # -----------------------------------------------------------------------------------------
    # CREATE UPU2IMAGE4XI IF NECESSARY       
    # ----------------------------------------------------------------------------------------- 
    try:
        if upu2image4xi is None:
            upu2image4xi = create2image4upu()                                                                              # INITIALIZE IMAGE FOR UPU 
            message2print = "{(Step 1C/70) created" + "; upu2image4xi-created-by-read2image4upu to: " + str(upu2image4xi)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)
            #             
        else:
            message2print = "{(Step 1D/70) skipped" + "; create2image4upu-skipped-by-read2image4upu: " + str(read2image4upu) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)
            # skipped                                 
    except:
        try:
            upu2image4xi = create2image4upu()
            message2print = "{(Step 1E/70) set" + "; upu2image4xi-set-by-read2image4upu to: " + str(upu2image4xi)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)
            #             
        except:
            upu2image4xi = {}
            message2print = "{(Step 1Z/70) yielded" + "; upu2image4xi-yielded-by-read2image4upu: " + str(upu2image4xi) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)  
            code2nack  = code2nack + '1Z-' 
    
    # -----------------------------------------------------------------------------------------
    # (Step 1/70) - CHECK PARAMETERS           
    # -----------------------------------------------------------------------------------------          
    try:             
        if task4imageread2upu is None:
            # -----------------------------------------------------------------------------------------
            # NO PARAMETERS GIVEN BY CALLER FOR READ - BUILD TASK! 
            # -----------------------------------------------------------------------------------------    
            
            task4imageread2upu = {}                                                                                                                                                                                       
            
            message2print = "{(Step 1A/70) initialized" + "; task4imageread2upu-created-by-read2image4upu to: " + str(task4imageread2upu)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)               
            # 
            
            # ---------------------------------------------------------------------------------------------
            # (1B) TRACE - AUTOMATIC                      
            # ---------------------------------------------------------------------------------------------              
            function2callback4upu = trace2image4upu                                                              # CALLBACK FUNCTION FOR UPU
            
            message2print = "{(Step 1B1/70) initialized" + "; function2callback4upu-created-by-read2image4upu to: " + str(function2callback4upu)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)            
            # <function trace2image4upu at 0x0000000004737548>;
            
            code2ack  = code2ack + '1B1-'
           
            args2callback = {}                                                                                   # CREATE ARGUMENT DICTIONARY FOR CALLBACK FUNCTION             
            args2callback['CLASS']   = "read2image4upu"                                                          # MODULE OF UPUIZED OPENCL MODULE
            args2callback['METHOD']  = "read2image4upu"                                                          # FUNCTION OF UPUIZED OPENCL MODULE
            args2callback['MESSAGE'] = "{(Step 1Z1/33) failed" + "; read2image4upu-failed " + "; time: " + time4xi() + "}" # ERROR MESSAGE FOR UPUIZED OPENCL TO RETURN    

            message2print = "{(Step 1B2/70) initialized" + "; args2callback-created-by-read2image4upu to: " + str(args2callback)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)               
            # {'MESSAGE': '{(Step 1Z1/33) failed; read2image4upu-failed ; time: 05:06:38 10/04/11 Pacific Daylight Time}', 
            #  'CLASS': 'read2image4upu', 
            #  'METHOD': 'read2image4upu'}; time: 05:06:38 10/04/11 Pacific Daylight Time} 

            code2ack  = code2ack + '1B2-'
            
            # ---------------------------------------------------------------------------------------------
            # (1C) CALLBACK - AUTOMATIC                      
            # ---------------------------------------------------------------------------------------------              
            task4imageread2upu['DATA2CALLBACK'] = args2callback                                                   # USER DATA PASSED TO UPUIZED OPENCL - SEVERAL ARGS PASSED IN ONE DICTIONARY
            task4imageread2upu['FUNCTION2CALLBACK'] = function2callback4upu                                       # CALLBACK FUNCTION TO CALLED BY OPENCL IN EXCEPTIONS
            
            code2ack  = code2ack + '1C-'
            message2print = "{(Step 1C/70) initialized" + "; task4imageread2upu-created-by-read2image4upu to: " + str(task4imageread2upu)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)  
            # {'FUNCTION2CALLBACK': <function trace2image4upu at 0x0000000004737548>, 
            #  'DATA2CALLBACK': {'MESSAGE': '{(Step 1Z1/33) failed; read2image4upu-failed ; time: 05:06:38 10/04/11 Pacific Daylight Time}', 
            #  'CLASS': 'read2image4upu', 
            #  'METHOD': 'read2image4upu'}}; time: 05:06:38 10/04/11 Pacific Daylight Time}
             
            
            # ---------------------------------------------------------------------------------------------
            # (2) PLATFORM - AUTOMATIC                   
            # ---------------------------------------------------------------------------------------------             
            platform2xi = {}
            message2print = "{(Step 2A/70) initialized" + "; platform2xi-created-by-read2image4upu to: " + str(platform2xi)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)   
            # {}
            
            try:
                platform2xi = get2platformids4xi()                                                                  # PLATFORM IS COMMON TO ALL DEVICES I.E TO DEVICE XI
                message2print = "{(Step 2B/70) got" + "; platform2xi-got-by-read2image4upu: " + str(platform2xi)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)
                # {'list2platforms': ['DragonXi'], 'ack2return': ' 2C-2E-2F-2G-', 'nack2return': ' ', 'available2platform': 1, 'max2entries': 1}
                 
                code2ack  = code2ack + '2B-'
            except:
                message2print = "{(Step 2Z/70) failed" + "; platform2xi-failed-by-read2image4upu: " + str(failed)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)                                                
                code2nack  = code2nack + '2Z-'
                 
            task4imageread2upu['PLATFORM'] = platform2xi                                                          # PLATFORM OF SYSTEM 
              
            
            # ---------------------------------------------------------------------------------------------
            # (3) CONTEXT - AUTOMATIC                 
            # ---------------------------------------------------------------------------------------------    
            context2upu = {}
            message2print = "{(Step 3A/70) trying" + "; context2upu-trying-by-read2image4upu: " + str(context2upu)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print) 
            try:
                context2upu = get2context4upu()                                                                     # NO PARAMETERS - THE SYSTEM CREATES CONTEXT IF REQUIRED  
                
                message2print = "{(Step 3B/70) got" + "; context2upu-got-by-read2image4upu: " + str(context2upu)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)                                  
                # {'COUNT2REFERENCE': 0, 
                #  'DATA2CALLBACK': {'MESSAGE': ...
    
                code2ack  = code2ack + '3B-'         
            except:
                message2print = "{(Step 3Z/70) failed" + "; context2upu-failed-by-read2image4upu: " + str(context2upu) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)             
                code2nack  = code2nack + '3Z-'
                
            task4imageread2upu ['CONTEXT'] = context2upu           
            
            # ---------------------------------------------------------------------------------------------
            # (4) DEVICE - AUTOMATIC                       
            # ---------------------------------------------------------------------------------------------    
            device2upu = {}
            
            message2print = "{(Step 4A/70) trying" + "; device2upu-trying-by-read2image4upu: " + str(device2upu)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print) 
            try:
                device2upu = get2upu4xi()                                                                           # THE SYSTEM CREATES DEVICE INFO, IF NECESSARY    
                
                message2print = "{(Step 4B/70) got" + "; device2upu-got-by-read2image4upu: " + str(device2upu)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)                                  
                # {'RESOLUTION2TIMER4PROFILING': 318, 
                #  'ID2PLATFORM': '0000000001509568', 
                #  'PROFILING2QUEUE': 'ACK', 
                #  'EXECUTE2KERNELS4OPENCL': 'ACK', 
                #  'NATIVE2WIDTHLONG4VECTOR': 2 ...
                
                code2ack  = code2ack + '4B-'  
            except:
                message2print = "{(Step 4Z/70) failed" + "; device2upu-failed-by-read2image4upu: " + str(device2upu) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)             
                code2nack  = code2nack + '4Z-'
            
            task4imageread2upu ['DEVICE'] = device2upu
            
                   
            # ---------------------------------------------------------------------------------------------
            # (5) IMAGE - REQUIRED                            
            # ---------------------------------------------------------------------------------------------         
            imageinfo2upu = {}
             
            message2print = "{(Step 5A/70) trying" + "; imageinfo2upu-trying-by-read2image4upu: " + str(imageinfo2upu)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print) 
            try:
                imageinfo2upu = get2imageinfo4upu()                                                                         
                
                message2print = "{(Step 5B/70) got" + "; imageinfo2upu-got-by-read2image4upu: " + str(imageinfo2upu)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)                                  
                # {'SIZE2QUEUE': 2147483646, 
                #  'FORMAT': {'TYPE': 'CL_UNORM_INT8', 
                #  'ORDER': 'CL_RGBA'}, 
                #  'QUEUE': <queue4xi.Queue4Xi object at 0x00000000041F9320>, 
                #  'WIDTH': 800, 
                #  'CONTEXT': {'COUNT2REFERENCE': 0, ...
                
                code2ack  = code2ack + '5B-'  
            except:
                message2print = "{(Step 5Z/70) failed" + "; imageinfo2upu-failed-by-read2image4upu: " + str(imageinfo2upu) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)             
                code2nack  = code2nack + '5Z-'          
                      
            task4imageread2upu ['IMAGE'] = imageinfo2upu         
                        

            # ---------------------------------------------------------------------------------------------
            # (6) QUEUE - REQUIRED BY UPUXI                          
            # ---------------------------------------------------------------------------------------------         
            queue2upu = None
            
            message2print = "{(Step 6A/70) trying" + "; queue2upu-trying-by-read2image4upu: " + str(queue2upu)  + "; time: " + time4xi() + "}" 
            print4xi(class2print, method2print, message2print) 
            try:
                queue2upu = Queue4Xi()                                                                                  # THE SYSTEM CREATES QUEUE    
                
                message2print = "{(Step 6B/70) got" + "; queue2upu-got-by-read2image4upu: " + str(queue2upu)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)                                  
                # <queue4xi.Queue4Xi object at 0x0000000004607160> 
                
                code2ack  = code2ack + '6B-'  
            except:
                message2print = "{(Step 6Z/70) failed" + "; queue2upu-failed-by-read2image4upu: " + str(queue2upu) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)             
                code2nack  = code2nack + '6Z-'
                
            task4imageread2upu ['QUEUE'] = queue2upu

            
            # ---------------------------------------------------------------------------------------------
            # (7) PIPE - REQUIRED BY UPUXI                              
            # ---------------------------------------------------------------------------------------------         
            pipe2upu = None
           
            message2print = "{(Step 7A/70) trying" + "; pipe2upu-trying-by-read2image4upu: " + str(pipe2upu)  + "; time: " + time4xi() + "}" 
            print4xi(class2print, method2print, message2print) 
            try:
                pipe2upu = Pipe4Xi()                                                                                    # THE SYSTEM CREATES PIPE    
                
                message2print = "{(Step 7B/70) got" + "; pipe2upu-got-by-read2image4upu: " + str(pipe2upu)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)                                  
                # (<read-write PipeConnection, handle 408>, <read-write PipeConnection, handle 412>) 
                
                code2ack  = code2ack + '7B-'  
            except:
                message2print = "{(Step 7Z/70) failed" + "; pipe2upu-failed-by-read2image4upu: " + str(pipe2upu) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)             
                code2nack  = code2nack + '7Z-'
                
            task4imageread2upu ['PIPE'] = pipe2upu 
            
            # ---------------------------------------------------------------------------------------------
            # (8) COMMAND QUEUE - REQUIRED                              
            # ---------------------------------------------------------------------------------------------             
            commandqueueinfo2upu = {}
           
            message2print = "{(Step 8A/70) trying" + "; commandqueueinfo2upu-trying-by-read2image4upu: " + str(commandqueueinfo2upu)  + "; time: " + time4xi() + "}" 
            print4xi(class2print, method2print, message2print)  
                        
            try:
                commandqueueinfo2upu = get2commandqueueinfo4upu()                                                    # SYSTEM CREATES COMMAND QUEUE, IF NECESSARY   
                message2print = "{(Step 8B/70) got" + "; commandqueueinfo2upu-got-by-read2image4upu: " + str(commandqueueinfo2upu)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print) 
                # {'CODE2ERROR': 0, 
                #  'COUNT2REFERENCE': 0, 
                #  'DATA2CALLBACK': {'MESSAGE': ...
                
                code2ack  = code2ack + '8B-'  
            except:
                message2print = "{(Step 8Z/70) failed" + "; commandqueueinfo2upu-failed-by-read2image4upu: " + str(commandqueueinfo2upu) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)             
                code2nack  = code2nack + '8Z-'
        
            task4imageread2upu ['COMMANDQUEUE'] = commandqueueinfo2upu
            
            
            
            # ---------------------------------------------------------------------------------------------
            # (9) BUFFER - AUTOMATIC BY UPUXI                               
            # ---------------------------------------------------------------------------------------------             
            bufferinfo2upu = {}
           
            message2print = "{(Step 9A/70) trying" + "; bufferinfo2upu-trying-by-read2image4upu: " + str(bufferinfo2upu)  + "; time: " + time4xi() + "}" 
            print4xi(class2print, method2print, message2print) 
                        
            try:
                bufferinfo2upu = get2bufferinfo4upu()                                                                   # SYSTEM CREATES BUFFER FOR UPU, IF NECESSARY
                message2print = "{(Step 9B/70) got" + "; bufferinfo2upu-got-by-read2image4upu: " + str(bufferinfo2upu)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print) 
                # {'CODE2ERROR': 0, 'COUNT2REFERENCE': 0, 'DATA2CALLBACK': {'MESSAGE': ...
                
                code2ack  = code2ack + '9B-' 
            except:
                message2print = "{(Step 9Z/70) failed" + "; bufferinfo2upu-failed-by-read2image4upu: " + str(bufferinfo2upu) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)             
                code2nack  = code2nack + '9Z-'
        
            task4imageread2upu ['BUFFER'] = bufferinfo2upu 
            
            # ---------------------------------------------------------------------------------------------
            # (10) EVENT - REQUIRED                            
            # ---------------------------------------------------------------------------------------------             
            eventinfo2upu = {}
           
            message2print = "{(Step 10A/70) trying" + "; eventinfo2upu-trying-by-read2image4upu: " + str(eventinfo2upu)  + "; time: " + time4xi() + "}" 
            print4xi(class2print, method2print, message2print) 
                        
            try:
                eventinfo2upu = get2eventinfo4upu()                                                                    
                message2print = "{(Step 10B/70) got" + "; eventinfo2upu-got-by-read2image4upu: " + str(eventinfo2upu)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print) 
                # {'CODE2ERROR': 0, 'DATA2CALLBACK': {'MESSAGE': ...
                
                code2ack  = code2ack + '10B-' 
            except:
                message2print = "{(Step 10Z/70) failed" + "; eventinfo2upu-failed-by-read2image4upu: " + str(eventinfo2upu) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)             
                code2nack  = code2nack + '10Z-'
        
            task4imageread2upu ['EVENT'] = eventinfo2upu 
            
            
            # ---------------------------------------------------------------------------------------------
            # (11) EVENTWAITLIST - REQUIRED                              
            # ---------------------------------------------------------------------------------------------             
            eventwaitlist2upu = {}
           
            message2print = "{(Step 11A/70) trying" + "; eventwaitlist2upu-trying-by-read2image4upu: " + str(eventwaitlist2upu)  + "; time: " + time4xi() + "}" 
            print4xi(class2print, method2print, message2print) 
                        
            try:
                eventwaitlist2upu = {}                                                                   
                message2print = "{(Step 11B/70) set" + "; eventwaitlist2upu-set-by-read2image4upu: " + str(eventwaitlist2upu)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print) 
                # {}
                
                code2ack  = code2ack + '11B-' 
            except:
                message2print = "{(Step 11Z/70) failed" + "; eventwaitlist2upu-failed-by-read2image4upu: " + str(eventwaitlist2upu) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)             
                code2nack  = code2nack + '11Z-'
        
            task4imageread2upu ['EVENTWAITLIST'] = eventwaitlist2upu                     

            
            # ---------------------------------------------------------------------------------------------
            # (12) EVENTWAITLISTQUANTITY - REQUIRED                               
            # ---------------------------------------------------------------------------------------------             
            eventwaitlistquantity2upu = 0
           
            message2print = "{(Step 12A/70) trying" + "; eventwaitlistquantity2upu-trying-by-read2image4upu: " + str(eventwaitlistquantity2upu)  + "; time: " + time4xi() + "}" 
            print4xi(class2print, method2print, message2print) 
                        
            try:
                keys2event = eventwaitlist2upu.keys()
                for key2event in keys2event:
                    eventwaitlistquantity2upu = eventwaitlistquantity2upu + 1                                                                   
                message2print = "{(Step 12B/70) set" + "; eventwaitlistquantity2upu-set-by-read2image4upu: " + str(eventwaitlistquantity2upu)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print) 
                # 0 
                
                code2ack  = code2ack + '12B-' 
            except:
                message2print = "{(Step 12Z/70) failed" + "; eventwaitlistquantity2upu-failed-by-read2image4upu: " + str(eventwaitlistquantity2upu) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)             
                code2nack  = code2nack + '12Z-'
        
            task4imageread2upu ['EVENTWAITLISTQUANTITY'] = eventwaitlistquantity2upu       
                                                 
            
            # ---------------------------------------------------------------------------------------------
            # (13) BLOCKING - REQUIRED                        
            # ---------------------------------------------------------------------------------------------             
            blocking2upu = ' '                                                                   
     
            message2print = "{(Step 13A/70) trying" + "; blocking2upu-trying-by-read2image4upu: " + str(blocking2upu)  + "; time: " + time4xi() + "}" 
            print4xi(class2print, method2print, message2print) 
                        
            try:
                blocking2upu = 'CL_FALSE'                                                          # the read command is non-blocking                                                              
                message2print = "{(Step 13B/70) set" + "; blocking2upu-set-by-read2image4upu to: " + str(blocking2upu)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print) 
                # CL_FALSE
                code2ack  = code2ack + '13B-' 
            except:
                message2print = "{(Step 13Z/70) failed" + "; blocking2upu-failed-by-read2image4upu: " + str(blocking2upu) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)             
                code2nack  = code2nack + '13Z-'
        
            task4imageread2upu ['BLOCKING'] = blocking2upu             
            
            # ---------------------------------------------------------------------------------------------
            # (14) ORIGIN - REQUIRED                             
            # ---------------------------------------------------------------------------------------------             
            origin2upu = {}
            
            message2print = "{(Step 14A/70) trying" + "; origin2upu-trying-by-read2image4upu: " + str(origin2upu)  + "; time: " + time4xi() + "}" 
            print4xi(class2print, method2print, message2print) 
                        
            try:                                                                                    
                origin2upu['X'] = 0 
                origin2upu['Y'] = 0  
                origin2upu['Z'] = 0                                                                                                                          
                message2print = "{(Step 14B/70) set" + "; origin2upu-set-by-read2image4upu to: " + str(origin2upu)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print) 
                # {'Y': 0, 'X': 0, 'Z': 0} 
                code2ack  = code2ack + '14B-' 
            except:
                message2print = "{(Step 14Z/70) failed" + "; origin2upu-failed-by-read2image4upu: " + str(origin2upu) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)             
                code2nack  = code2nack + '14Z-'
        
            task4imageread2upu ['ORIGIN'] = origin2upu
            
            
            # ---------------------------------------------------------------------------------------------
            # (15) REGION - REQUIRED                         
            # ---------------------------------------------------------------------------------------------             
            region2upu = {}
            
            message2print = "{(Step 15A/70) trying" + "; region2upu-trying-by-read2image4upu: " + str(region2upu)  + "; time: " + time4xi() + "}" 
            print4xi(class2print, method2print, message2print) 
                        
            try:                                                                                    
                region2upu['X'] = 160                                                                               # WIDTH  IN PIXELS TO READ 
                region2upu['Y'] = 90                                                                                # HEIGHT IN PIXELS TO READ 
                region2upu['Z'] = 0                                                                                 # DEPTH  IN PIXELS TO READ                                                                                                                          
                message2print = "{(Step 15B/70) set" + "; region2upu-set-by-read2image4upu to: " + str(region2upu)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print) 
                # {'Y': 90, 'X': 160, 'Z': 0}
                code2ack  = code2ack + '15B-' 
            except:
                message2print = "{(Step 15Z/70) failed" + "; region2upu-failed-by-read2image4upu: " + str(region2upu) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)             
                code2nack  = code2nack + '15Z-'
        
            task4imageread2upu ['REGION'] = region2upu
            
            
            # ---------------------------------------------------------------------------------------------
            # (16) ROWPITCH (STRIDE) - REQUIRED                              
            # ---------------------------------------------------------------------------------------------             
            # One can assume that memory representing an image or screen will have data laid out as one row of pixels followed directly by the next row of pixels. 
            # However many APIs (e.g. Direct3D) that return a pointer to image data add an extra block of data at the end of each row. 
            # int offset = x * bytesPerPixel + y * pitch;
            # When a video image is stored in memory, the memory buffer might contain extra padding bytes after each row of pixels. 
            # The padding bytes affect how the image is stored in memory, but do not affect how the image is displayed.
            # The stride is the number of bytes from one row of pixels in memory to the next row of pixels in memory. 
            # Stride is also called pitch. 
            # If padding bytes are present, the stride is wider than the width of the image, as shown in the following illustration.


            rowpitch2upu = 0
            size2element = 8                                                                                        # VOXEL (X,Y,Z) = SHOT IN PIXELS, R=1 PIXELS
            voxelquantity2layer = 10                                                                                # MAX NUMBER OF VOXELS IN EACH 2D LAYER 
            quantity2direction = 8                                                                                  # ONE DIRECTIONS IN 2D
            width2upu = 800
            
            message2print = "{(Step 16A/70) trying" + "; rowpitch2upu-trying-by-read2image4upu: " + str(rowpitch2upu)  + "; time: " + time4xi() + "}"  
            print4xi(class2print, method2print, message2print) 
            # 
                     
            try:                                                                                                    # X-AXEL IS THE WORST CASE TO SCAN WITH Y=0, Z=0
                rowpitch2upu = int(width2upu * size2element * voxelquantity2layer / quantity2direction)             # (800 * 8 * 10) / 8 = 8K PIXELS MAX TO SCAN 
                                                                                                                    # SPHERE IS SPARSE HAVING FEW SHOTS PER SCAN LINE 
                                                                                                                                         
                message2print = "{(Step 16B/70) set" + "; rowpitch2upu-set-by-read2image4upu to: " + str(rowpitch2upu)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print) 
                #  8000                                                                                          
                 
                code2ack  = code2ack + '16B-' 
            except:
                message2print = "{(Step 16Z/70) failed" + "; rowpitch2upu-failed-by-read2image4upu: " + str(rowpitch2upu) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)             
                # 
                
                code2nack  = code2nack + '16Z-'
        
            task4imageread2upu ['ROWPITCH'] = rowpitch2upu     
            
            # ----------------------------------------------------------------------------------------------------
            # (17) POINTER - REQUIRED - THE POINTER TO A BUFFER IN HOST MEMORY WHERE IMAGE DATA IS TO BE READ FROM                          
            # ----------------------------------------------------------------------------------------------------             
            pointer2upu = None
            height2upu = 800
            size2upu = int(width2upu * height2upu)                                                                  # CALCULATED MEMORY
            if size2upu == 0:
                size2upu = 4400                                                                                     # MINIMUM MEMORY
            
            message2print = "{(Step 17A/70) trying" + "; pointer2upu-trying-by-read2image4upu: " + str(pointer2upu)  + "; time: " + time4xi() + "}" 
            print4xi(class2print, method2print, message2print) 
            # 
                        
            try:
                arena2upu = Arena4Xi(size2upu)                                                                      # SYSTEM RESERVER MEMORY FOR IMAGE                             
                                                                                                                                         
                message2print = "{(Step 17B/70) got" + "; arena2upu-got-by-read2image4upu: " + str(arena2upu)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print) 
                # <arena4xi.Arena4Xi object at 0x0000000004449EF0>
                
                size2arena = arena2upu.size                                                                         # 
                              
                message2print = "{(Step 17C/70) got" + "; size2arena-got-by-read2buffer4upu: " + str(size2arena)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)
                # 640000
                
                buffer2arena = arena2upu.buffer
                
                message2print = "{(Step 17D/70) got" + "; buffer2arena-got-by-read2buffer4upu: " + str(buffer2arena)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)                
                # <mmap.mmap object at 0x00000000041A0C90>
                
                code2ack  = code2ack + '17D-' 
                
                if size2arena < size2upu:
                    size2upu = size2arena                                                                           # AVAILABLE MEMORY FOR IMAGE
                    message2print = "{(Step 17E/70) set" + "; size2upu-set-by-read2buffer4upu to: " + str(size2upu)  + "; time: " + time4xi() + "}"
                    print4xi(class2print, method2print, message2print)
                    # 
                    
                    code2ack  = code2ack + '17E-'
                else:
                    pass
                    
                pointer2upu = buffer2arena
                message2print = "{(Step 17F/70) set" + "; pointer2upu-set-by-read2buffer4upu to: " + str(pointer2upu)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)
                # <mmap.mmap object at 0x00000000041A0C90>
                
                code2ack  = code2ack + '17F-'       
            except:
                message2print = "{(Step 17Z/70) failed" + "; pointer2upu-failed-by-read2image4upu: " + str(pointer2upu) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)             
                # 
                
                code2nack  = code2nack + '17Z-'
        
            task4imageread2upu ['POINTER'] = pointer2upu                            
            task4imageread2upu ['SIZE']    = size2upu                                                               # AUTOMATIC BY UPUXI
            
            
            # --------------------------------------------------------------------------------------------------------
            # (18) SLICEPITCH - REQUIRED - THE SIZE IN BYTES OF THE 2D SLICE OF THE 3D REGION OF A 3D IMAGE BEING READ                         
            # --------------------------------------------------------------------------------------------------------             
            slicepitch2upu = None                                                                                   # SLICEPITCH IS ZERO FOR 2D IMAGES                           
           
            message2print = "{(Step 18A/70) trying" + "; slicepitch2upu-trying-by-read2image4upu: " + str(slicepitch2upu)  + "; time: " + time4xi() + "}" 
            print4xi(class2print, method2print, message2print) 
                        
            try:
                slicepitch2upu = 0                                                          
                message2print = "{(Step 18B/70) set" + "; slicepitch2upu-set-by-read2image4upu to: " + str(slicepitch2upu)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print) 
                # 0 

                code2ack  = code2ack + '18B-'                
                 
            except:
                message2print = "{(Step 18Z/70) failed" + "; slicepitch2upu-failed-by-read2image4upu: " + str(slicepitch2upu) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)             
                code2nack  = code2nack + '18Z-'
            
            task4imageread2upu ['SLICEPITCH'] = slicepitch2upu                  
            
            # ---------------------------------------------------------------------------------------------
            # EOF PARAMETER PROVIDING                
            # ---------------------------------------------------------------------------------------------                                                                                                                               
        else:
            pass
        # ---------------------------------------------------------------------------------------------
        # ANALYSE TASKS - WHETHER AUTOMATICALLY GENERATED OR USER GIVEN 
        # ---------------------------------------------------------------------------------------------
        keys2task = task4imageread2upu.keys()
        message2print = "{(Step 20/70) got" + "; keys2task-got-by-read2image4upu: " + str(sorted(keys2task))  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)            
        # ['BLOCKING',
        #  'BUFFER',
        #  'COMMANDQUEUE',
        #  'CONTEXT',
        #  'DATA2CALLBACK',
        #  'DEVICE',
        #  'EVENT',
        #  'EVENTWAITLIST',
        #  'EVENTWAITLISTQUANTITY',
        #  'FUNCTION2CALLBACK',
        #  'IMAGE',
        #  'ORIGIN',
        #  'PIPE',
        #  'PLATFORM',
        #  'POINTER',
        #  'QUEUE',
        #  'REGION',
        #  'ROWPITCH',
        #  'SIZE',
        #  'SLICEPITCH']; time: 05:25:42 10/04/11 Pacific Daylight Time}

        if (('DATA2CALLBACK' in keys2task) and ('FUNCTION2CALLBACK' in keys2task)):
            pass
        else:                
            # ---------------------------------------------------------------------------------------------
            # (21B) TRACE - AUTOMATIC                      
            # ---------------------------------------------------------------------------------------------              
            function2callback4upu = trace2image4upu                                                              # CALLBACK FUNCTION FOR UPU
            
            message2print = "{(Step 21B1/70) initialized" + "; function2callback4upu-created-by-read2image4upu to: " + str(function2callback4upu)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)            
            # 
            
            code2ack  = code2ack + '21B1-'
           
            args2callback = {}                                                                                   # CREATE ARGUMENT DICTIONARY FOR CALLBACK FUNCTION             
            args2callback['CLASS']   = "read2image4upu"                                                          # MODULE OF UPUIZED OPENCL MODULE
            args2callback['METHOD']  = "read2image4upu"                                                          # FUNCTION OF UPUIZED OPENCL MODULE
            args2callback['MESSAGE'] = "{(Step 1Z1/33) failed" + "; read2image4upu-failed " + "; time: " + time4xi() + "}" # ERROR MESSAGE FOR UPUIZED OPENCL TO RETURN    

            message2print = "{(Step 21B2/70) initialized" + "; args2callback-created-by-read2image4upu to: " + str(args2callback)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)               
            # 

            code2ack  = code2ack + '21B2-'
        
            # ---------------------------------------------------------------------------------------------
            # (21C) CALLBACK - AUTOMATIC                      
            # ---------------------------------------------------------------------------------------------              
            task4imageread2upu['DATA2CALLBACK'] = args2callback                                                   # USER DATA PASSED TO UPUIZED OPENCL - SEVERAL ARGS PASSED IN ONE DICTIONARY
            task4imageread2upu['FUNCTION2CALLBACK'] = function2callback4upu                                       # CALLBACK FUNCTION TO CALLED BY OPENCL IN EXCEPTIONS
            
            code2ack  = code2ack + '21C-'
            message2print = "{(Step 21C/70) initialized" + "; task4imageread2upu-created-by-read2image4upu to: " + str(task4imageread2upu)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)  
            #
             
        if 'PLATFORM' in keys2task:
            pass
        else:  
            # ---------------------------------------------------------------------------------------------
            # (22) PLATFORM - AUTOMATIC                   
            # ---------------------------------------------------------------------------------------------             
            platform2xi = {}
            message2print = "{(Step 22A/70) initialized" + "; platform2xi-created-by-read2image4upu to: " + str(platform2xi)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)   
            # 
            
            try:
                platform2xi = get2platformids4xi()                                                                  # PLATFORM IS COMMON TO ALL DEVICES I.E TO DEVICE XI
                message2print = "{(Step 22B/70) got" + "; platform2xi-got-by-read2image4upu: " + str(platform2xi)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)
                # 
                 
                code2ack  = code2ack + '22B-'
            except:
                message2print = "{(Step 22Z/70) failed" + "; platform2xi-failed-by-read2image4upu: " + str(failed)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)                                                
                code2nack  = code2nack + '22Z-'
                 
            task4imageread2upu['PLATFORM'] = platform2xi                                                          # PLATFORM OF SYSTEM 
          
        if 'CONTEXT' in keys2task:
            pass
        else:              
            # ---------------------------------------------------------------------------------------------
            # (23) CONTEXT - AUTOMATIC                 
            # ---------------------------------------------------------------------------------------------    
            context2upu = {}
            message2print = "{(Step 23A/70) trying" + "; context2upu-trying-by-read2image4upu: " + str(context2upu)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print) 
            try:
                context2upu = get2context4upu()                                                                     # NO PARAMETERS - THE SYSTEM CREATES CONTEXT IF REQUIRED  
                
                message2print = "{(Step 23B/70) got" + "; context2upu-got-by-read2image4upu: " + str(context2upu)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)                                  
                # 
                
                code2ack  = code2ack + '23B-'         
            except:
                message2print = "{(Step 23Z/70) failed" + "; context2upu-failed-by-read2image4upu: " + str(context2upu) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)             
                code2nack  = code2nack + '23Z-'
                
            task4imageread2upu ['CONTEXT'] = context2upu           
        
        if 'DEVICE' in keys2task:
            pass
        else:                   
            # ---------------------------------------------------------------------------------------------
            # (24) DEVICE - AUTOMATIC                       
            # ---------------------------------------------------------------------------------------------    
            device2upu = {}
            
            message2print = "{(Step 24A/70) trying" + "; device2upu-trying-by-read2image4upu: " + str(device2upu)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print) 
            try:
                device2upu = get2upu4xi()                                                                           # THE SYSTEM CREATES DEVICE INFO, IF NECESSARY    
                
                message2print = "{(Step 24B/70) got" + "; device2upu-got-by-read2image4upu: " + str(device2upu)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)                                  
                # 
                
                code2ack  = code2ack + '24B-'  
            except:
                message2print = "{(Step 24Z/70) failed" + "; device2upu-failed-by-read2image4upu: " + str(device2upu) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)             
                code2nack  = code2nack + '24Z-'
            
            task4imageread2upu ['DEVICE'] = device2upu
        
        if 'IMAGE' in keys2task:
            pass
        else:                      
            # ---------------------------------------------------------------------------------------------
            # (25) IMAGE - REQUIRED                            
            # ---------------------------------------------------------------------------------------------         
            imageinfo2upu = {}
             
            message2print = "{(Step 25A/70) trying" + "; imageinfo2upu-trying-by-read2image4upu: " + str(imageinfo2upu)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print) 
            try:
                imageinfo2upu = get2imageinfo4upu()                                                                         
                
                message2print = "{(Step 25B/70) got" + "; imageinfo2upu-got-by-read2image4upu: " + str(imageinfo2upu)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)                                  
                #                 
                code2ack  = code2ack + '25B-'  
            except:
                message2print = "{(Step 25Z/70) failed" + "; imageinfo2upu-failed-by-read2image4upu: " + str(imageinfo2upu) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)             
                code2nack  = code2nack + '25Z-'          
                      
            task4imageread2upu ['IMAGE'] = imageinfo2upu         
                    
        if 'QUEUE' in keys2task:
            pass
        else:   
            # ---------------------------------------------------------------------------------------------
            # (26) QUEUE - REQUIRED BY UPUXI                          
            # ---------------------------------------------------------------------------------------------         
            queue2upu = None
            
            message2print = "{(Step 26A/70) trying" + "; queue2upu-trying-by-read2image4upu: " + str(queue2upu)  + "; time: " + time4xi() + "}" 
            print4xi(class2print, method2print, message2print) 
            try:
                queue2upu = Queue4Xi()                                                                                  # THE SYSTEM CREATES QUEUE    
                
                message2print = "{(Step 26B/70) got" + "; queue2upu-got-by-read2image4upu: " + str(queue2upu)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)                                  
                # 
                
                code2ack  = code2ack + '26B-'  
            except:
                message2print = "{(Step 26Z/70) failed" + "; queue2upu-failed-by-read2image4upu: " + str(queue2upu) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)             
                code2nack  = code2nack + '26Z-'
                
            task4imageread2upu ['QUEUE'] = queue2upu
        
        if 'PIPE' in keys2task:
            pass
        else:              
            # ---------------------------------------------------------------------------------------------
            # (27) PIPE - REQUIRED BY UPUXI                              
            # ---------------------------------------------------------------------------------------------         
            pipe2upu = None
           
            message2print = "{(Step 27A/70) trying" + "; pipe2upu-trying-by-read2image4upu: " + str(pipe2upu)  + "; time: " + time4xi() + "}" 
            print4xi(class2print, method2print, message2print) 
            try:
                pipe2upu = Pipe4Xi()                                                                                    # THE SYSTEM CREATES PIPE    
                
                message2print = "{(Step 27B/70) got" + "; pipe2upu-got-by-read2image4upu: " + str(pipe2upu)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)                                  
                # 
                
                code2ack  = code2ack + '27B-'  
            except:
                message2print = "{(Step 27Z/70) failed" + "; pipe2upu-failed-by-read2image4upu: " + str(pipe2upu) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)             
                code2nack  = code2nack + '27Z-'
                
            task4imageread2upu ['PIPE'] = pipe2upu 
            
        if 'COMMANDQUEUE' in keys2task:
            pass
        else:              
            # ---------------------------------------------------------------------------------------------
            # (28) COMMAND QUEUE - REQUIRED                              
            # ---------------------------------------------------------------------------------------------             
            commandqueueinfo2upu = {}
           
            message2print = "{(Step 28A/70) trying" + "; commandqueueinfo2upu-trying-by-read2image4upu: " + str(commandqueueinfo2upu)  + "; time: " + time4xi() + "}" 
            print4xi(class2print, method2print, message2print)  
                        
            try:
                commandqueueinfo2upu = get2commandqueueinfo4upu()                                                    # SYSTEM CREATES COMMAND QUEUE, IF NECESSARY   
                message2print = "{(Step 28B/70) got" + "; commandqueueinfo2upu-got-by-read2image4upu: " + str(commandqueueinfo2upu)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print) 
                # 
                
                code2ack  = code2ack + '28B-'  
            except:
                message2print = "{(Step 28Z/70) failed" + "; commandqueueinfo2upu-failed-by-read2image4upu: " + str(commandqueueinfo2upu) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)             
                code2nack  = code2nack + '28Z-'
        
            task4imageread2upu ['COMMANDQUEUE'] = commandqueueinfo2upu
        
        
        if 'BUFFER' in keys2task:
            pass
        else:               
            # ---------------------------------------------------------------------------------------------
            # (29) BUFFER - AUTOMATIC BY UPUXI                               
            # ---------------------------------------------------------------------------------------------             
            bufferinfo2upu = {}
           
            message2print = "{(Step 29A/70) trying" + "; bufferinfo2upu-trying-by-read2image4upu: " + str(bufferinfo2upu)  + "; time: " + time4xi() + "}" 
            print4xi(class2print, method2print, message2print) 
                        
            try:
                bufferinfo2upu = get2bufferinfo4upu()                                                                   # SYSTEM CREATES BUFFER FOR UPU, IF NECESSARY
                message2print = "{(Step 29B/70) got" + "; bufferinfo2upu-got-by-read2image4upu: " + str(bufferinfo2upu)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print) 
                # 
                
                code2ack  = code2ack + '29B-' 
            except:
                message2print = "{(Step 29Z/70) failed" + "; bufferinfo2upu-failed-by-read2image4upu: " + str(bufferinfo2upu) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)             
                code2nack  = code2nack + '29Z-'
        
            task4imageread2upu ['BUFFER'] = bufferinfo2upu
             
        if 'EVENT' in keys2task:
            pass
        else:               
            # ---------------------------------------------------------------------------------------------
            # (30) EVENT - REQUIRED                            
            # ---------------------------------------------------------------------------------------------             
            eventinfo2upu = {}
           
            message2print = "{(Step 30A/70) trying" + "; eventinfo2upu-trying-by-read2image4upu: " + str(eventinfo2upu)  + "; time: " + time4xi() + "}" 
            print4xi(class2print, method2print, message2print) 
                        
            try:
                eventinfo2upu = get2eventinfo4upu()                                                                    
                message2print = "{(Step 30B/70) got" + "; eventinfo2upu-got-by-read2image4upu: " + str(eventinfo2upu)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print) 
                # 
                
                code2ack  = code2ack + '30B-' 
            except:
                message2print = "{(Step 30Z/70) failed" + "; eventinfo2upu-failed-by-read2image4upu: " + str(eventinfo2upu) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)             
                code2nack  = code2nack + '30Z-'
        
            task4imageread2upu ['EVENT'] = eventinfo2upu 
        
             
        if 'EVENTWAITLIST' in keys2task:
            pass
        else:               
            # ---------------------------------------------------------------------------------------------
            # (31) EVENTWAITLIST - REQUIRED                              
            # ---------------------------------------------------------------------------------------------             
            eventwaitlist2upu = {}
           
            message2print = "{(Step 31A/70) trying" + "; eventwaitlist2upu-trying-by-read2image4upu: " + str(eventwaitlist2upu)  + "; time: " + time4xi() + "}" 
            print4xi(class2print, method2print, message2print) 
                        
            try:
                eventwaitlist2upu = {}                                                                   
                message2print = "{(Step 31B/70) set" + "; eventwaitlist2upu-set-by-read2image4upu: " + str(eventwaitlist2upu)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print) 
                # 
                
                code2ack  = code2ack + '31B-' 
            except:
                message2print = "{(Step 31Z/70) failed" + "; eventwaitlist2upu-failed-by-read2image4upu: " + str(eventwaitlist2upu) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)             
                code2nack  = code2nack + '31Z-'
        
            task4imageread2upu ['EVENTWAITLIST'] = eventwaitlist2upu                     
              
        if 'EVENTWAITLISTQUANTITY' in keys2task:
            pass
        else:               
            # ---------------------------------------------------------------------------------------------
            # (32) EVENTWAITLISTQUANTITY - REQUIRED                              
            # ---------------------------------------------------------------------------------------------             
            eventwaitlistquantity2upu = 0
           
            message2print = "{(Step 32A/70) trying" + "; eventwaitlistquantity2upu-trying-by-read2image4upu: " + str(eventwaitlistquantity2upu)  + "; time: " + time4xi() + "}" 
            print4xi(class2print, method2print, message2print) 
                        
            try:
                keys2event = eventwaitlist2upu.keys()
                for key2event in keys2event:
                    eventwaitlistquantity2upu = eventwaitlistquantity2upu + 1                                                                      
                message2print = "{(Step 32B/70) set" + "; eventwaitlistquantity2upu-set-by-read2image4upu: " + str(eventwaitlistquantity2upu)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print) 
                # 0
                
                code2ack  = code2ack + '32B-' 
            except:
                message2print = "{(Step 32Z/70) failed" + "; eventwaitlistquantity2upu-failed-by-read2image4upu: " + str(eventwaitlistquantity2upu) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)             
                code2nack  = code2nack + '32Z-'
        
            task4imageread2upu ['EVENTWAITLISTQUANTITY'] = eventwaitlistquantity2upu       
                                                 
                 
        if 'BLOCKING' in keys2task:
            pass
        else:              
            # ---------------------------------------------------------------------------------------------
            # (33) BLOCKING - REQUIRED                        
            # ---------------------------------------------------------------------------------------------             
            blocking2upu = ' '                                                                   
            message2print = "{(Step 33A/70) trying" + "; blocking2upu-trying-by-read2image4upu: " + str(blocking2upu)  + "; time: " + time4xi() + "}" 
            print4xi(class2print, method2print, message2print) 
                        
            try:
                blocking2upu = 'CL_FALSE'                                                          # the read command is non-blocking                                                              
                message2print = "{(Step 33B/70) set" + "; blocking2upu-set-by-read2image4upu to: " + str(blocking2upu)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print) 
                # 
                code2ack  = code2ack + '33B-' 
            except:
                message2print = "{(Step 33Z/70) failed" + "; blocking2upu-failed-by-read2image4upu: " + str(blocking2upu) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)             
                code2nack  = code2nack + '33Z-'
        
            task4imageread2upu ['BLOCKING'] = blocking2upu             
             
        if 'ORIGIN' in keys2task:
            pass
        else:                    
            # ---------------------------------------------------------------------------------------------
            # (34) ORIGIN - REQUIRED                             
            # ---------------------------------------------------------------------------------------------             
            origin2upu = {}
            
            message2print = "{(Step 34A/70) trying" + "; origin2upu-trying-by-read2image4upu: " + str(origin2upu)  + "; time: " + time4xi() + "}" 
            print4xi(class2print, method2print, message2print) 
                        
            try:                                                                                    
                origin2upu['X'] = 0 
                origin2upu['Y'] = 0  
                origin2upu['Z'] = 0                                                                                                                          
                message2print = "{(Step 34B/70) set" + "; origin2upu-set-by-read2image4upu to: " + str(origin2upu)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print) 
                # 
                code2ack  = code2ack + '34B-' 
            except:
                message2print = "{(Step 34Z/70) failed" + "; origin2upu-failed-by-read2image4upu: " + str(origin2upu) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)             
                code2nack  = code2nack + '34Z-'
        
            task4imageread2upu ['ORIGIN'] = origin2upu
            
             
        if 'REGION' in keys2task:
            pass
        else:            
            # ---------------------------------------------------------------------------------------------
            # (35) REGION - REQUIRED                         
            # ---------------------------------------------------------------------------------------------             
            region2upu = {}
            
            message2print = "{(Step 35A/70) trying" + "; region2upu-trying-by-read2image4upu: " + str(region2upu)  + "; time: " + time4xi() + "}" 
            print4xi(class2print, method2print, message2print) 
                            
            try:                                                                                    
                region2upu['X'] = 160                                                                               # WIDTH  IN PIXELS TO READ 
                region2upu['Y'] = 90                                                                                # HEIGHT IN PIXELS TO READ 
                region2upu['Z'] = 0                                                                                 # DEPTH  IN PIXELS TO READ                                                                                                                          
                message2print = "{(Step 35B/70) set" + "; region2upu-set-by-read2image4upu to: " + str(region2upu)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print) 
                # 
                code2ack  = code2ack + '35B-' 
            except:
                message2print = "{(Step 35Z/70) failed" + "; region2upu-failed-by-read2image4upu: " + str(region2upu) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)             
                code2nack  = code2nack + '35Z-'
        
            task4imageread2upu ['REGION'] = region2upu
        
            
             
        if 'ROWPITCH' in keys2task:
            pass
        else:               
            # ---------------------------------------------------------------------------------------------
            # (36) ROWPITCH (STRIDE) - REQUIRED                              
            # ---------------------------------------------------------------------------------------------             
            # One can assume that memory representing an image or screen will have data laid out as one row of pixels followed directly by the next row of pixels. 
            # However many APIs (e.g. Direct3D) that return a pointer to image data add an extra block of data at the end of each row. 
            # int offset = x * bytesPerPixel + y * pitch;
            # When a video image is stored in memory, the memory buffer might contain extra padding bytes after each row of pixels. 
            # The padding bytes affect how the image is stored in memory, but do not affect how the image is displayed.
            # The stride is the number of bytes from one row of pixels in memory to the next row of pixels in memory. 
            # Stride is also called pitch. 
            # If padding bytes are present, the stride is wider than the width of the image, as shown in the following illustration.
     
            rowpitch2upu = 0
            size2element = 1                                                                                        # VOXEL (X,Y,Z) = SHOT IN PIXELS, R=1 PIXELS
            voxelquantity2layer = 1                                                                                 # MAX NUMBER OF VOXELS IN EACH 2D LAYER 
            quantity2direction = 1                                                                                  # ONE DIRECTIONS IN 2D
            
            message2print = "{(Step 36A/70) trying" + "; rowpitch2upu-trying-by-read2image4upu: " + str(rowpitch2upu)  + "; time: " + time4xi() + "}" 
            print4xi(class2print, method2print, message2print) 
            # 
                        
            try:                                                                                                    # X-AXEL IS THE WORST CASE TO SCAN WITH Y=0, Z=0
                rowpitch2upu = int(width2upu * size2element * voxelquantity2layer / quantity2direction)             # (800 * 8 * 10) / 8 = 8K PIXELS MAX TO SCAN 
                                                                                                                    # SPHERE IS SPARSE HAVING FEW SHOTS PER SCAN LINE 
                                                                                                                                         
                message2print = "{(Step 36B/70) set" + "; rowpitch2upu-set-by-read2image4upu to: " + str(rowpitch2upu)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print) 
                #                                                                                             
                 
                code2ack  = code2ack + '36B-' 
            except:
                message2print = "{(Step 36Z/70) failed" + "; rowpitch2upu-failed-by-read2image4upu: " + str(rowpitch2upu) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)             
                # 
                
                code2nack  = code2nack + '36Z-'
        
            task4imageread2upu ['ROWPITCH'] = rowpitch2upu     
             
        if 'POINTER' in keys2task:
            pass
        else:                
            # ----------------------------------------------------------------------------------------------------
            # (37) POINTER - REQUIRED - THE POINTER TO A BUFFER IN HOST MEMORY WHERE IMAGE DATA IS TO BE READ FROM                          
            # ----------------------------------------------------------------------------------------------------             
            pointer2upu = None
            size2upu = int(width2upu * height2upu)                                                                  # CALCULATED MEMORY
            if size2upu == 0:
                size2upu = 4400                                                                                     # MINIMUM MEMORY
            
            message2print = "{(Step 37A/70) trying" + "; pointer2upu-trying-by-read2image4upu: " + str(pointer2upu)  + "; time: " + time4xi() + "}" 
            print4xi(class2print, method2print, message2print) 
            # 
                        
            try:
                arena2upu = Arena4Xi(size2upu)                                                                      # SYSTEM RESERVER MEMORY FOR IMAGE                             
                                                                                                                                         
                message2print = "{(Step 37B/70) got" + "; arena2upu-got-by-read2image4upu: " + str(arena2upu)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print) 
                # 
                
                size2arena = arena2upu.size                                                                         # 
                              
                message2print = "{(Step 37C/70) got" + "; size2arena-got-by-read2buffer4upu: " + str(size2arena)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)
                # 
                
                buffer2arena = arena2upu.buffer
                
                message2print = "{(Step 37D/70) got" + "; buffer2arena-got-by-read2buffer4upu: " + str(buffer2arena)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)                
                # 
                
                code2ack  = code2ack + '37D-' 
                
                if size2arena < size2upu:
                    size2upu = size2arena                                                                           # AVAILABLE MEMORY FOR IMAGE
                    message2print = "{(Step 37E/70) set" + "; size2upu-set-by-read2buffer4upu to: " + str(size2upu)  + "; time: " + time4xi() + "}"
                    print4xi(class2print, method2print, message2print)
                    # 
                    
                    code2ack  = code2ack + '37E-'
                else:
                    pass
                    
                pointer2upu = buffer2arena
                message2print = "{(Step 37F/70) set" + "; pointer2upu-set-by-read2buffer4upu to: " + str(pointer2upu)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)
                # 
                
                code2ack  = code2ack + '37F-'       
            except:
                message2print = "{(Step 37Z/70) failed" + "; pointer2upu-failed-by-read2image4upu: " + str(pointer2upu) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)             
                # 
                
                code2nack  = code2nack + '37Z-'
        
            task4imageread2upu ['POINTER'] = pointer2upu                            
            task4imageread2upu ['SIZE']    = size2upu                                                               # AUTOMATIC BY UPUXI
        
             
        if 'SLICEPITCH' in keys2task:
            pass
        else:               
            # --------------------------------------------------------------------------------------------------------
            # (38) SLICEPITCH - REQUIRED - THE SIZE IN BYTES OF THE 2D SLICE OF THE 3D REGION OF A 3D IMAGE BEING READ                         
            # --------------------------------------------------------------------------------------------------------             
            slicepitch2upu = None                                                                                   # SLICEPITCH IS ZERO FOR 2D IMAGES                           
           
            message2print = "{(Step 38A/70) trying" + "; slicepitch2upu-trying-by-read2image4upu: " + str(slicepitch2upu)  + "; time: " + time4xi() + "}" 
            print4xi(class2print, method2print, message2print) 
                        
            try:
                slicepitch2upu = 0                                                          
                message2print = "{(Step 38B/70) set" + "; slicepitch2upu-set-by-read2image4upu to: " + str(slicepitch2upu)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print) 
                # 
                code2ack  = code2ack + '38B-'                
                 
            except:
                message2print = "{(Step 38Z/70) failed" + "; slicepitch2upu-failed-by-read2image4upu: " + str(slicepitch2upu) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)             
                code2nack  = code2nack + '38Z-'
            
            task4imageread2upu ['SLICEPITCH'] = slicepitch2upu
                              
        # ---------------------------------------------------------------------------------------------
        # EOF TASK ANALYSIS
        # ---------------------------------------------------------------------------------------------
        keys2task = task4imageread2upu.keys()
        message2print = "{(Step 39/70) set" + "; keys2task-set-by-read2image4upu to: " + str(sorted(keys2task))  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)            
        # ['BLOCKING',
        #  'BUFFER',
        #  'COMMANDQUEUE',
        #  'CONTEXT',
        #  'DATA2CALLBACK',
        #  'DEVICE',
        #  'EVENT',
        #  'EVENTWAITLIST',
        #  'EVENTWAITLISTQUANTITY',
        #  'FUNCTION2CALLBACK',
        #  'IMAGE',
        #  'ORIGIN',
        #  'PIPE',
        #  'PLATFORM',
        #  'POINTER',
        #  'QUEUE',
        #  'REGION',
        #  'ROWPITCH',
        #  'SIZE',
        #  'SLICEPITCH']; time: 05:25:42 10/04/11 Pacific Daylight Time}                
    except: 
        message2print = "{(Step 39Z/70) yielded" + "; task4imageread2upu-yielded-by-read2image4upu: " + str(task4imageread2upu) + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)  
        # 
        
        code2nack  = code2nack + '39Z-' 

    # ---------------------------------------------------------------------------------------------
    # CONTROL AND COMMAND
    # ---------------------------------------------------------------------------------------------
    control2readimage4upu = {}       
                                
    # ---------------------------------------------------------------------------------------------
    # CONSUMER AND PUBLISHER CHANNEL FOR CONTROL AND COMMAND
    # ---------------------------------------------------------------------------------------------
    ack4channels2upu = False
    channels2upu = ()
    message2print = "{(Step 40A/70) trying" + "channels2upu-trying-by-read2image4upu: " + str(channels2upu)  + "; time: " + time4xi() + "}" 
    print4xi(class2print, method2print, message2print)     
    try:
        # (<read-write PipeConnection, handle 520>, <read-write PipeConnection, handle 524>) 
        consumerchannel2upu, publisherchannel2upu = task4imageread2upu ['PIPE']

        message2print = "{(Step 40B/70) got" + "; consumerchannel2upu-got-by-read2image4upu: " + str(consumerchannel2upu)  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)            
        # 

        message2print = "{(Step 40C/70) got" + "; publisherchannel2upu-got-by-read2image4upu: " + str(publisherchannel2upu)  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)            
        #                      
        ack4channels2upu = True
        
        control2readimage4upu['CONSUMERCHANNEL']  = consumerchannel2upu
        
        
        control2readimage4upu['PUBLISHERCHANNEL'] = publisherchannel2upu
        
        code2ack  = code2ack + '40D-' 
        
    except:
        ack4channels2upu = False
        code2nack  = code2nack + '40Z-'      
                        
    # ---------------------------------------------------------------------------------------------
    # QUEUE FOR CONTROL AND COMMAND
    # ---------------------------------------------------------------------------------------------
    ack4queue2upu = False
    queue2upu = None
    message2print = "{(Step 41A/70) trying" + "; queue2upu-trying-by-read2image4upu: " + str(queue2upu)  + "; time: " + time4xi() + "}" 
    print4xi(class2print, method2print, message2print)     
    try:
        queue2upu = task4imageread2upu ['QUEUE']

        message2print = "{(Step 41B/70) got" + "; queue2upu-got-by-read2image4upu: " + str(queue2upu)  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)            
        # 
              
        ack4queue2upu = True
        control2readimage4upu['QUEUE']  = queue2upu        
        code2ack  = code2ack + '41C-'
    except:
        ack4queue2upu = False  
        code2nack  = code2nack + '41Z-'
    
    
    # -----------------------------------------------------------------------------------------    
    # DUMMY PROCESS OBJECT = THREAD CREATION
    # -----------------------------------------------------------------------------------------           
    # dummy2image4upu = None
    # message2print = "{(Step 69A/70) trying" + "dummy2image4upu-trying-by-read2image4upu: " + str(dummy2image4upu)  + "; time: " + time4xi() + "}"   
    # print4xi(class2print, method2print, message2print)     
    # try:                            
        # dummy2image4upu = DummyProcess4Xi(target=manoeuvre2readimage4upu,                                               # in this module to handle queue and pipe channels
        #                                   args =(queue2upu,                                                         # caller puts and gets - does NOT call any functions
        #                                          consumerchannel2upu, publisherchannel2upu))                        # caller sends and receives - does NOT call any functions
        # message2print = "{(Step 69B/70) created" + "dummy2image4upu-created-by-read2image4upu: " + str(dummy2image4upu)  + "; time: " + time4xi() + "}" 
        # print4xi(class2print, method2print, message2print)  
        # code2ack  = code2ack + '69B-'

    # except:
        # code2nack  = code2nack + '69Z-'
        
    # ---------------------------------------------------------------------------------------------
    # RETURN CONTROL AND COMMAND DICTIONARY
    # ---------------------------------------------------------------------------------------------          
    message2print = "{(Step 70A/70) set" + "; code2ack-set-by-read2image4upu: " + str(code2ack) + "; time: " + time4xi() + "}"  
    print4xi(class2print, method2print, message2print)                                                             
    # 1B1-1B2-1C-2B-3B-4B-5B-6B-7B-8B-9B-10B-11B-12B-13B-14B-15B-16B-17D-17F-18B-40D-41C-
     
    message2print = "{(Step 70B/70) set" + "; code2nack-set-by-read2image4upu: " + str(code2nack) + "; time: " + time4xi() + "}" 
    print4xi(class2print, method2print, message2print)                                                             
    #   
        
    message2print = "{(Step 70C/70) returning" + "; control2readimage4upu-returning-by-read2image4upu: " + str(control2readimage4upu) + "; time: " + time4xi() + "}" 
    print4xi(class2print, method2print, message2print)                                                             
    # {'QUEUE': <queue4xi.Queue4Xi object at 0x0000000003FE3B70>,
    #  'PUBLISHERCHANNEL': <read-write PipeConnection, handle 416>, 
    #  'CONSUMERCHANNEL': <read-write PipeConnection, handle 412>}
        
    return control2readimage4upu
                    
#</function4xi> 

# -------------------------------------------------------------------------------------------------
#<function4xi function = "map2image4upu" usage = "
#                                                                                                              
#   # ---------------------------------------------------------------------------------------------
#   # ESTABLISH CONTROL & COMMAND                 
#   # ---------------------------------------------------------------------------------------------    
#   print('(Step 92A/700) OF UPU2IMAGE4XI5TEST - TRYING TO ESTABLISH CONTROL & COMMAND WITH MAP2IMAGE4UPU ', time4xi())           
#                    
#   response4map2image = {} 
#   task4map2image = None                                                                                  # WE ONLY WANT TO ESTABLISH CONTROL AND CONTROL FIRST
#
#   try:                               
#       response4map2image = map2image4upu(task4map2image)                                                                                                          
#       print('(Step 92B/700) OF UPU2IMAGE4XI5TEST - ESTABLISHED CONTROL & COMMAND WITH MAP2IMAGE4UPU : ', response4map2image)
#    
#       queue4map2image4upu      = response4map2image['QUEUE']
#       channel4map2imagepublish4upu = response4map2image['PUBLISHERCHANNEL']
#       channel4map2imageconsume4upu = response4map2image['CONSUMERCHANNEL']
#   except:
#       print('(Step 92Z/700) OF UPU2IMAGE4XI5TEST *** FAILED TO ESTABLISH CONTROL & COMMAND WITH MAP2IMAGE4UPU ', time4xi()) 
#        
#   # -----------------------------------------------------------------------------------------------------------
#   # SEND CONTROL & COMMAND COPY VIA PIPE FOR MANOEUVRE2COPYIMAGE4UPU TO RECEIVE                 
#   # -----------------------------------------------------------------------------------------------------------      
#   print('(Step 93A/700) OF UPU2IMAGE4XI5TEST - TRYING TO SEND CONTROL & COMMAND VIA PIPE WITH MAP2IMAGE4UPU ', time4xi())      
#   try:                               
#       message2send4upu = "<copy4xi>copy2mapimage4upu</copy4xi>"
#       channel4map2imagepublish4upu.send(message2send4upu)
#       print("(Step 93B/700) OF UPU2IMAGE4XI5TEST - SENT TO CHANNEL2PUBLISH4UPU: ", channel4map2imagepublish4upu, " MESSAGE2SEND4UPU: " , message2send4upu)                                                                                                
#   except:
#       print('(Step 93Z1/700) OF UPU2IMAGE4XI5TEST *** FAILED TO SEND CONTROL & COMMAND VIA PIPE WITH MAP2IMAGE4UPU ', time4xi())   
# 
#   # ---------------------------------------------------------------------------------------------------------------------
#   # SIMULATE MANOEUVRE2COPYIMAGE4UPU  - LET IT TO RECEIVE COPY AND TO SEND ROGER VIA PIPE                
#   # ---------------------------------------------------------------------------------------------------------------------       
#   if ack2simulate4upu is True:
#       print("(Step 94A/700) OF UPU2IMAGE4XI5TEST - TRYING TO PERFORM TARGET FUNCTION MANOEUVRE2COPYIMAGE4UPU ", time4xi())   
#       try:        
#           response2manoeuvre4copy = manoeuvre2mapimage4upu(queue4map2image4upu, channel4map2imageconsume4upu, channel4map2imagepublish4upu)
#           print("(Step 94B/700) OF UPU2IMAGE4XI5TEST - PERFORMED MANOEUVRE2COPYIMAGE4UPU, RESPONSE2MANOEUVRE4COPY: ", response2manoeuvre4copy)
#        except:
#           print("(Step 94Z/700) OF UPU2IMAGE4XI5TEST *** FAILED TO PERFORM TARGET FUNCTION MANOEUVRE2COPYIMAGE4UPU ", time4xi())     
#   else:
#       pass # trust activated thread    
#
#   # ---------------------------------------------------------------------------------------------
#   # RECEIVE CONTROL & COMMAND ROGER VIA PIPE                  
#   # ---------------------------------------------------------------------------------------------          
#   print('(Step 95A/700) OF UPU2IMAGE4XI5TEST - TRYING TO RECEIVE CONTROL & COMMAND VIA PIPE WITH MAP2IMAGE4UPU ', time4xi())      
#   try:
#       message2receive4upu = channel4map2imageconsume4upu.recv()
#       print("(Step 95B/700) OF UPU2IMAGE4XI5TEST - RECEIVED FROM CHANNEL2CONSUME4UPU: ", channel4map2imageconsume4upu, " CONTROL & COMMAND: ", message2receive4upu)                                                                                                  
#   except:
#       print('(Step 95Z/700) OF UPU2IMAGE4XI5TEST *** FAILED TO RECEIVE CONTROL & COMMAND VIA PIPE  WITH MAP2IMAGE4UPU ', time4xi())          
#   
#   # ------------------------------------------------------------------------------------------------------------
#   # SEND CONTROL & COMMAND READY VIA PIPE FOR MANOEUVRE2COPYIMAGE4UPU TO RECEIVE                
#   # ------------------------------------------------------------------------------------------------------------      
#   print('(Step 96A/700) OF UPU2IMAGE4XI5TEST - TRYING TO SEND CONTROL & COMMAND VIA PIPE WITH MAP2IMAGE4UPU ', time4xi())     
#   try:                               
#       message2send4upu = "<ready4xi>ready2mapimage4upu</ready4xi>"
#       channel4map2imagepublish4upu.send(message2send4upu)
#       print("(Step 96B/700) OF UPU2IMAGE4XI5TEST - SENT TO CHANNEL2PUBLISH4UPU: ", channel4map2imagepublish4upu, " MESSAGE2SEND4UPU: " , message2send4upu)                                                                                                
#   except:
#       print('(Step 84Z/700) OF UPU2IMAGE4XI5TEST *** FAILED TO SEND CONTROL & COMMAND VIA PIPE WITH MAP2IMAGE4UPU ', time4xi())                                                
# 
#   # -------------------------------------------------------------------------------------------------------
#   # SIMULATE MANOEUVRE2COPYIMAGE4UPU  - LET IT RECEIVE READY AND SEND READY                
#   # -------------------------------------------------------------------------------------------------------       
#   if ack2simulate4upu is True:
#       print("(Step 97A/700) OF UPU2IMAGE4XI5TEST - TRYING TO PERFORM TARGET FUNCTION MANOEUVRE2COPYIMAGE4UPU ", time4xi())  
#       try:        
#           response2manoeuvre4copy = manoeuvre2mapimage4upu(queue4map2image4upu, channel4map2imageconsume4upu, channel4map2imagepublish4upu)
#           print("(Step 97B/700) OF UPU2IMAGE4XI5TEST - PERFORMED TARGET FUNCTION MANOEUVRE2COPYIMAGE4UPU ", manoeuvre2copyimage4upu)
#       except:
#           print("(Step 97Z/700) OF UPU2IMAGE4XI5TEST *** FAILED TO PERFORM TARGET FUNCTION MANOEUVRE2COPYIMAGE4UPU ", time4xi())     
#   else:
#       pass # trust activated thread    
#            
#    
#   # ---------------------------------------------------------------------------------------
#   # RECEIVE CONTROL & COMMAND READY VIA PIPE                
#   # ---------------------------------------------------------------------------------------          
#   print('(Step 98A/700) OF UPU2IMAGE4XI5TEST - TRYING TO RECEIVE CONTROL & COMMAND VIA PIPE WITH MAP2IMAGE4UPU ', time4xi())     
#   try:
#       message2receive4upu = channel4map2imageconsume4upu.recv()
#       print("(Step 98B/700) OF UPU2IMAGE4XI5TEST - RECEIVED FROM CHANNEL2CONSUME4UPU: ", channel4map2imageconsume4upu, " CONTROL & COMMAND: ", message2receive4upu)                                                                                                  
#   except:
#       print('(Step 86Z/700) OF UPU2IMAGE4XI5TEST *** FAILED TO RECEIVE CONTROL & COMMAND VIA PIPE  WITH MAP2IMAGE4UPU ', time4xi())          
#                         
#   # -----------------------------------------------------------------------------------------
#   # IMAGE UPUXI COPY OPERATIONS 
#   # -----------------------------------------------------------------------------------------         
#   print('(Step 99A/700) OF UPU2IMAGE4XI5TEST - TRYING FILE OPERATIONS WITH IMAGE UPUXI ', time4xi())       
#   try:        
#       path2dragon = '..//..//..//..//..//web//images//png//dragonxi.png'
#       file2dragon = open(path2dragon)
#       print("(Step 99B/700) OF UPU2IMAGE4XI5TEST - FILENO: ", file2upu.fileno( ))  
#       print("(Step 99C/700) OF UPU2IMAGE4XI5TEST - MODE: ", file2upu.mode)
#       print("(Step 99D/700) OF UPU2IMAGE4XI5TEST - NAME: ", file2upu.name)
#       offset2upu = 10
#       file2upu.seek(offset2upu)
#       print("(Step 99E/700) OF UPU2IMAGE4XI5TEST - PERFORMED SEEK WITH OFFSET: ", offset2upu)
#       file2upu.close()
#       print("(Step 99F/700) OF DRAGON2IMAGE4XI5TEST - CLOSED IMAGE FILE: ", path2dragon)  
#   except:
#       print("(Step 87Z/700) OF DRAGON2IMAGE4XI5TEST *** FAILED TO PERFORM FILE OPERATIONS WITH IMAGE: ", path2dragon)        
# 
#   # -----------------------------------------------------------------------------------------
#   # PUT IMAGE PATH TO QUEUE
#   # -----------------------------------------------------------------------------------------                          
#   print("(Step 100A/700) OF UPU2IMAGE4XI5TEST - TRYING TO PUT IMAGE PATH TO QUEUE ", time4xi())      
#   try:    
#       queue4map2image4dragon.put(path2dragon)             
#       print("(Step 100B/700) OF DRAGON2IMAGE4XI5TEST - PUT TO QUEUE IMAGE PATH: ", path2dragon) 
#   except:
#       print("(Step 88Z/700) OF UPU2IMAGE4XI5TEST *** FAILED TO PUT IMAGE PATH TO QUEUE ", time4xi()) 
#   
#   # -------------------------------------------------------------------------------------------------------------------------------------
#   # SIMULATE MANOEUVRE2COPYIMAGE4UPU  - STATE III - GETS FROM QUEUE AND SENDS EITHER ACK OR NACK VIA PIPE               
#   # -------------------------------------------------------------------------------------------------------------------------------------       
#   if ack2simulate4upu is True:
#       print("(Step 101A/700) OF UPU2IMAGE4XI5TEST - TRYING TO PERFORM TARGET FUNCTION MANOEUVRE2COPYIMAGE4UPU ", time4xi()) 
#       try:        
#           response2manoeuvre4copy = manoeuvre2mapimage4upu(queue4map2image4upu, channel4map2imageconsume4upu, channel4map2imagepublish4upu)
#           print("(Step 101B/700) OF UPU2IMAGE4XI5TEST - PERFORMED TARGET FUNCTION MANOEUVRE2COPYIMAGE4UPU ", manoeuvre2copyimage4upu)
#       except:
#           print("(Step 101Z/700) OF UPU2IMAGE4XI5TEST *** FAILED TO PERFORM TARGET FUNCTION MANOEUVRE2COPYIMAGE4UPU ", time4xi())     
#   else:
#       pass # trust activated thread                     
#                 
#                                  
#   # ----------------------------------------------------------------------------------------------------
#   # RECEIVE ACK OR NACK ABOUT IMAGE OPERATION CONTROL & COMMAND VIA PIPE                
#   # ----------------------------------------------------------------------------------------------------          
#   print('(Step 102A/700) OF UPU2IMAGE4XI5TEST - TRYING TO RECEIVE CONTROL & COMMAND ABOUT IMAGE OPERTION VIA PIPE ', time4xi())      
#   try: 
#       message2receive4upu = channel4map2imageconsume4upu.recv()
#       print("(Step 102B/700) OF UPU2IMAGE4XI5TEST - RECEIVED FROM CHANNEL2CONSUME4UPU: ", channel4map2imageconsume4upu, " CONTROL & COMMAND: ", message2receive4upu)                                                                                                   
#   except:
#       print('(Step 90Z/700) OF UPU2IMAGE4XI5TEST *** FAILED TO RECEIVE CONTROL & COMMAND ABOUT IMAGE OPERTION VIA PIPE ', time4xi())               
#
#   
#   # ----------------------------------------------------------------------------------------------------------------
#   # SIMULATE MANOEUVRE2COPYIMAGE4UPU  - LET MANOEUVRE2COPYIMAGE4UPU TO GO TO STATE 0               
#   # ----------------------------------------------------------------------------------------------------------------       
#   if ack2simulate4upu is True:
#       print("(Step 103A/700) OF UPU2IMAGE4XI5TEST - TRYING TO PERFORM TARGET FUNCTION MANOEUVRE2COPYIMAGE4UPU ", time4xi())  
#       try:        
#           response2manoeuvre4copy = manoeuvre2mapimage4upu(queue4map2image4upu, channel4map2imageconsume4upu, channel4map2imagepublish4upu)
#           print("(Step 103B/700) OF UPU2IMAGE4XI5TEST - GOT RESPONSE2MANOEUVRE4UPU ", response2manoeuvre4copy)
#       except:
#           print("(Step 103Z/700) OF UPU2IMAGE4XI5TEST *** FAILED TO PERFORM TARGET FUNCTION MANOEUVRE2COPYIMAGE4UPU ", time4xi())     
#   else:
#       pass # trust activated thread  
#                                           
#">

# -------------------------------------------------------------------------------------------------   
def map2image4upu(task4imagemap2upu=None):

    '''
    maps 2D images for UPU
    '''
    global upu2image4xi                                                                                             # 2D-IMAGES FOR UPU  
    global control2mapimage4upu                                                                                     # COMMAND AND CONTROL FOR UPU IMAGES 
    
    # -----------------------------------------------------------------------------------------
    # (Step 1/33) PARAMETERS           
    # -----------------------------------------------------------------------------------------          
    class2print = "map2image4upu"
    method2print = "map2image4upu"

    message2print = "{(Step 1A/33) started" + "; map2image4upu-started: " + str(map2image4upu)  + "; time: " + time4xi() + "}" 
    print4xi(class2print, method2print, message2print)                                                                     #  
    # 
    
    message2print = "{(Step 1B/33) passed" + "; task4imagemap2upu-to-map2image4upu: " + str(task4imagemap2upu) + "; time: " + time4xi() + "}"
    print4xi(class2print, method2print, message2print)                                                              
    # 
            
    code2ack  = ' '
    code2nack = ' '
    
    # -----------------------------------------------------------------------------------------
    # CREATE UPU2IMAGE4XI IF NECESSARY       
    # ----------------------------------------------------------------------------------------- 
    try:
        if upu2image4xi is None:
            upu2image4xi = create2image4upu()                                                                              # INITIALIZE IMAGE FOR UPU 
            message2print = "{(Step 1C/70) created" + "; upu2image4xi-created-by-map2image4upu to: " + str(upu2image4xi)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)
            #             
        else:
            message2print = "{(Step 1D/70) skipped" + "; create2image4upu-skipped-by-map2image4upu: " + str(map2image4upu) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)
            #                                
    except:
        try:
            upu2image4xi = create2image4upu()
            message2print = "{(Step 1E/70) set" + "; upu2image4xi-set-by-map2image4upu to: " + str(upu2image4xi)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)
            #             
        except:
            upu2image4xi = {}
            message2print = "{(Step 1Z/70) yielded" + "; upu2image4xi-yielded-by-map2image4upu: " + str(upu2image4xi) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)  
            code2nack  = code2nack + '1Z-' 
    
    # -----------------------------------------------------------------------------------------
    # (Step 1/70) - CHECK PARAMETERS           
    # -----------------------------------------------------------------------------------------          
    try:             
        if task4imagemap2upu is None:
            # -----------------------------------------------------------------------------------------
            # NO PARAMETERS GIVEN BY CALLER FOR MAP - BUILD TASK! 
            # -----------------------------------------------------------------------------------------    
            
            task4imagemap2upu = {}                                                                                                                                                                                       
            
            message2print = "{(Step 1A/70) initialized" + "; task4imagemap2upu-created-by-map2image4upu to: " + str(task4imagemap2upu)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)               
            # 
            
            # ---------------------------------------------------------------------------------------------
            # (1B) TRACE - AUTOMATIC                      
            # ---------------------------------------------------------------------------------------------              
            function2callback4upu = trace2image4upu                                                              # CALLBACK FUNCTION FOR UPU
            
            message2print = "{(Step 1B1/70) initialized" + "; function2callback4upu-created-by-map2image4upu to: " + str(function2callback4upu)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)            
            # 
            
            code2ack  = code2ack + '1B1-'
           
            args2callback = {}                                                                                   # CREATE ARGUMENT DICTIONARY FOR CALLBACK FUNCTION             
            args2callback['CLASS']   = "map2image4upu"                                                          # MODULE OF UPUIZED OPENCL MODULE
            args2callback['METHOD']  = "map2image4upu"                                                          # FUNCTION OF UPUIZED OPENCL MODULE
            args2callback['MESSAGE'] = "{(Step 1Z1/33) failed" + "; map2image4upu-failed " + "; time: " + time4xi() + "}" # ERROR MESSAGE FOR UPUIZED OPENCL TO RETURN    

            message2print = "{(Step 1B2/70) initialized" + "; args2callback-created-by-map2image4upu to: " + str(args2callback)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)               
            # 
            code2ack  = code2ack + '1B2-'
            
            # ---------------------------------------------------------------------------------------------
            # (1C) CALLBACK - AUTOMATIC                      
            # ---------------------------------------------------------------------------------------------              
            task4imagemap2upu['DATA2CALLBACK'] = args2callback                                                   # USER DATA PASSED TO UPUIZED OPENCL - SEVERAL ARGS PASSED IN ONE DICTIONARY
            task4imagemap2upu['FUNCTION2CALLBACK'] = function2callback4upu                                       # CALLBACK FUNCTION TO CALLED BY OPENCL IN EXCEPTIONS
            
            code2ack  = code2ack + '1C-'
            message2print = "{(Step 1C/70) initialized" + "; task4imagemap2upu-created-by-map2image4upu to: " + str(task4imagemap2upu)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)  
            # 
            
            # ---------------------------------------------------------------------------------------------
            # (2) PLATFORM - AUTOMATIC                   
            # ---------------------------------------------------------------------------------------------             
            platform2xi = {}
            message2print = "{(Step 2A/70) initialized" + "; platform2xi-created-by-map2image4upu to: " + str(platform2xi)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)   
            # 
            
            try:
                platform2xi = get2platformids4xi()                                                                  # PLATFORM IS COMMON TO ALL DEVICES I.E TO DEVICE XI
                message2print = "{(Step 2B/70) got" + "; platform2xi-got-by-map2image4upu: " + str(platform2xi)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)
                # 
                code2ack  = code2ack + '2B-'
            except:
                message2print = "{(Step 2Z/70) failed" + "; platform2xi-failed-by-map2image4upu: " + str(failed)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)                                                
                code2nack  = code2nack + '2Z-'
                 
            task4imagemap2upu['PLATFORM'] = platform2xi                                                          # PLATFORM OF SYSTEM 
              
            
            # ---------------------------------------------------------------------------------------------
            # (3) CONTEXT - AUTOMATIC                 
            # ---------------------------------------------------------------------------------------------    
            context2upu = {}
            message2print = "{(Step 3A/70) trying" + "; context2upu-trying-by-map2image4upu: " + str(context2upu)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print) 
            try:
                context2upu = get2context4upu()                                                                     # NO PARAMETERS - THE SYSTEM CREATES CONTEXT IF REQUIRED  
                
                message2print = "{(Step 3B/70) got" + "; context2upu-got-by-map2image4upu: " + str(context2upu)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)                                  
                # 
                
                code2ack  = code2ack + '3B-'         
            except:
                message2print = "{(Step 3Z/70) failed" + "; context2upu-failed-by-map2image4upu: " + str(context2upu) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)             
                code2nack  = code2nack + '3Z-'
                
            task4imagemap2upu ['CONTEXT'] = context2upu           
            
            # ---------------------------------------------------------------------------------------------
            # (4) DEVICE - AUTOMATIC                       
            # ---------------------------------------------------------------------------------------------    
            device2upu = {}
            
            message2print = "{(Step 4A/70) trying" + "; device2upu-trying-by-map2image4upu: " + str(device2upu)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print) 
            try:
                device2upu = get2upu4xi()                                                                           # THE SYSTEM CREATES DEVICE INFO, IF NECESSARY    
                
                message2print = "{(Step 4B/70) got" + "; device2upu-got-by-map2image4upu: " + str(device2upu)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)                                  
                # 
                code2ack  = code2ack + '4B-'  
            except:
                message2print = "{(Step 4Z/70) failed" + "; device2upu-failed-by-map2image4upu: " + str(device2upu) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)             
                code2nack  = code2nack + '4Z-'
            
            task4imagemap2upu ['DEVICE'] = device2upu
            
                   
            # ---------------------------------------------------------------------------------------------
            # (5) IMAGE - REQUIRED                            
            # ---------------------------------------------------------------------------------------------         
            imageinfo2upu = {}
             
            message2print = "{(Step 5A/70) trying" + "; imageinfo2upu-trying-by-map2image4upu: " + str(imageinfo2upu)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print) 
            try:
                imageinfo2upu = get2imageinfo4upu()                                                                         
                
                message2print = "{(Step 5B/70) got" + "; imageinfo2upu-got-by-map2image4upu: " + str(imageinfo2upu)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)                                  
                # 
                code2ack  = code2ack + '5B-'  
            except:
                message2print = "{(Step 5Z/70) failed" + "; imageinfo2upu-failed-by-map2image4upu: " + str(imageinfo2upu) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)             
                code2nack  = code2nack + '5Z-'          
                      
            task4imagemap2upu ['IMAGE'] = imageinfo2upu         
                        

            # ---------------------------------------------------------------------------------------------
            # (6) QUEUE - REQUIRED BY UPUXI                          
            # ---------------------------------------------------------------------------------------------         
            queue2upu = None
            
            message2print = "{(Step 6A/70) trying" + "; queue2upu-trying-by-map2image4upu: " + str(queue2upu)  + "; time: " + time4xi() + "}" 
            print4xi(class2print, method2print, message2print) 
            try:
                queue2upu = Queue4Xi()                                                                                  # THE SYSTEM CREATES QUEUE    
                
                message2print = "{(Step 6B/70) got" + "; queue2upu-got-by-map2image4upu: " + str(queue2upu)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)                                  
                # 
                
                code2ack  = code2ack + '6B-'  
            except:
                message2print = "{(Step 6Z/70) failed" + "; queue2upu-failed-by-map2image4upu: " + str(queue2upu) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)             
                code2nack  = code2nack + '6Z-'
                
            task4imagemap2upu ['QUEUE'] = queue2upu

            
            # ---------------------------------------------------------------------------------------------
            # (7) PIPE - REQUIRED BY UPUXI                              
            # ---------------------------------------------------------------------------------------------         
            pipe2upu = None
           
            message2print = "{(Step 7A/70) trying" + "; pipe2upu-trying-by-map2image4upu: " + str(pipe2upu)  + "; time: " + time4xi() + "}" 
            print4xi(class2print, method2print, message2print) 
            try:
                pipe2upu = Pipe4Xi()                                                                                    # THE SYSTEM CREATES PIPE    
                
                message2print = "{(Step 7B/70) got" + "; pipe2upu-got-by-map2image4upu: " + str(pipe2upu)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)                                  
                # 
                
                code2ack  = code2ack + '7B-'  
            except:
                message2print = "{(Step 7Z/70) failed" + "; pipe2upu-failed-by-map2image4upu: " + str(pipe2upu) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)             
                code2nack  = code2nack + '7Z-'
                
            task4imagemap2upu ['PIPE'] = pipe2upu 
            
            # ---------------------------------------------------------------------------------------------
            # (8) COMMAND QUEUE - REQUIRED                              
            # ---------------------------------------------------------------------------------------------             
            commandqueueinfo2upu = {}
           
            message2print = "{(Step 8A/70) trying" + "; commandqueueinfo2upu-trying-by-map2image4upu: " + str(commandqueueinfo2upu)  + "; time: " + time4xi() + "}" 
            print4xi(class2print, method2print, message2print)  
                        
            try:
                commandqueueinfo2upu = get2commandqueueinfo4upu()                                                    # SYSTEM CREATES COMMAND QUEUE, IF NECESSARY   
                message2print = "{(Step 8B/70) got" + "; commandqueueinfo2upu-got-by-map2image4upu: " + str(commandqueueinfo2upu)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print) 
                # 
                
                code2ack  = code2ack + '8B-'  
            except:
                message2print = "{(Step 8Z/70) failed" + "; commandqueueinfo2upu-failed-by-map2image4upu: " + str(commandqueueinfo2upu) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)             
                code2nack  = code2nack + '8Z-'
        
            task4imagemap2upu ['COMMANDQUEUE'] = commandqueueinfo2upu
            
            
            
            # ---------------------------------------------------------------------------------------------
            # (9) BUFFER - AUTOMATIC BY UPUXI                               
            # ---------------------------------------------------------------------------------------------             
            bufferinfo2upu = {}
           
            message2print = "{(Step 9A/70) trying" + "; bufferinfo2upu-trying-by-map2image4upu: " + str(bufferinfo2upu)  + "; time: " + time4xi() + "}" 
            print4xi(class2print, method2print, message2print) 
                        
            try:
                bufferinfo2upu = get2bufferinfo4upu()                                                                   # SYSTEM CREATES BUFFER FOR UPU, IF NECESSARY
                message2print = "{(Step 9B/70) got" + "; bufferinfo2upu-got-by-map2image4upu: " + str(bufferinfo2upu)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print) 
                # 
                
                code2ack  = code2ack + '9B-' 
            except:
                message2print = "{(Step 9Z/70) failed" + "; bufferinfo2upu-failed-by-map2image4upu: " + str(bufferinfo2upu) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)             
                code2nack  = code2nack + '9Z-'
        
            task4imagemap2upu ['BUFFER'] = bufferinfo2upu 
            
            # ---------------------------------------------------------------------------------------------
            # (10) EVENT - REQUIRED                            
            # ---------------------------------------------------------------------------------------------             
            eventinfo2upu = {}
           
            message2print = "{(Step 10A/70) trying" + "; eventinfo2upu-trying-by-map2image4upu: " + str(eventinfo2upu)  + "; time: " + time4xi() + "}" 
            print4xi(class2print, method2print, message2print) 
                        
            try:
                eventinfo2upu = get2eventinfo4upu()                                                                    
                message2print = "{(Step 10B/70) got" + "; eventinfo2upu-got-by-map2image4upu: " + str(eventinfo2upu)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print) 
                # 
                
                code2ack  = code2ack + '10B-' 
            except:
                message2print = "{(Step 10Z/70) failed" + "; eventinfo2upu-failed-by-map2image4upu: " + str(eventinfo2upu) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)             
                code2nack  = code2nack + '10Z-'
        
            task4imagemap2upu ['EVENT'] = eventinfo2upu 
            
            
            # ---------------------------------------------------------------------------------------------
            # (11) EVENTWAITLIST - REQUIRED                              
            # ---------------------------------------------------------------------------------------------             
            eventwaitlist2upu = {}
           
            message2print = "{(Step 11A/70) trying" + "; eventwaitlist2upu-trying-by-map2image4upu: " + str(eventwaitlist2upu)  + "; time: " + time4xi() + "}" 
            print4xi(class2print, method2print, message2print) 
                        
            try:
                eventwaitlist2upu = {}                                                                   
                message2print = "{(Step 11B/70) set" + "; eventwaitlist2upu-set-by-map2image4upu: " + str(eventwaitlist2upu)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print) 
                # 
                
                code2ack  = code2ack + '11B-' 
            except:
                message2print = "{(Step 11Z/70) failed" + "; eventwaitlist2upu-failed-by-map2image4upu: " + str(eventwaitlist2upu) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)             
                code2nack  = code2nack + '11Z-'
        
            task4imagemap2upu ['EVENTWAITLIST'] = eventwaitlist2upu                     

            
            # ---------------------------------------------------------------------------------------------
            # (12) EVENTWAITLISTQUANTITY - REQUIRED                              
            # ---------------------------------------------------------------------------------------------             
            eventwaitlistquantity2upu = 0
           
            message2print = "{(Step 12A/70) trying" + "; eventwaitlistquantity2upu-trying-by-map2image4upu: " + str(eventwaitlistquantity2upu)  + "; time: " + time4xi() + "}" 
            print4xi(class2print, method2print, message2print) 
                        
            try:
                keys2event = eventwaitlist2upu.keys()
                for key2event in keys2event:
                    eventwaitlistquantity2upu = eventwaitlistquantity2upu + 1                                                                      
                message2print = "{(Step 12B/70) set" + "; eventwaitlistquantity2upu-set-by-map2image4upu: " + str(eventwaitlistquantity2upu)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print) 
                # 0 
                
                code2ack  = code2ack + '12B-' 
            except:
                message2print = "{(Step 12Z/70) failed" + "; eventwaitlistquantity2upu-failed-by-map2image4upu: " + str(eventwaitlistquantity2upu) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)             
                code2nack  = code2nack + '12Z-'
        
            task4imagemap2upu ['EVENTWAITLISTQUANTITY'] = eventwaitlistquantity2upu       
                                                 
            
            # ---------------------------------------------------------------------------------------------
            # (13) BLOCKING - REQUIRED                        
            # ---------------------------------------------------------------------------------------------             
            blocking2upu = ' '                                                                   
     
            message2print = "{(Step 13A/70) trying" + "; blocking2upu-trying-by-map2image4upu: " + str(blocking2upu)  + "; time: " + time4xi() + "}" 
            print4xi(class2print, method2print, message2print) 
                        
            try:
                blocking2upu = 'CL_FALSE'                                                          # the map command is non-blocking                                                              
                message2print = "{(Step 13B/70) set" + "; blocking2upu-set-by-map2image4upu to: " + str(blocking2upu)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print) 
                # 
                
                code2ack  = code2ack + '13B-' 
            except:
                message2print = "{(Step 13Z/70) failed" + "; blocking2upu-failed-by-map2image4upu: " + str(blocking2upu) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)             
                code2nack  = code2nack + '13Z-'
        
            task4imagemap2upu ['BLOCKING'] = blocking2upu             
            
            # ---------------------------------------------------------------------------------------------
            # (14) ORIGIN - REQUIRED                             
            # ---------------------------------------------------------------------------------------------             
            origin2upu = {}
            
            message2print = "{(Step 14A/70) trying" + "; origin2upu-trying-by-map2image4upu: " + str(origin2upu)  + "; time: " + time4xi() + "}" 
            print4xi(class2print, method2print, message2print) 
                        
            try:                                                                                    
                origin2upu['X'] = 0 
                origin2upu['Y'] = 0  
                origin2upu['Z'] = 0                                                                                                                          
                message2print = "{(Step 14B/70) set" + "; origin2upu-set-by-map2image4upu to: " + str(origin2upu)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print) 
                # 
                 
                code2ack  = code2ack + '14B-' 
            except:
                message2print = "{(Step 14Z/70) failed" + "; origin2upu-failed-by-map2image4upu: " + str(origin2upu) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)             
                code2nack  = code2nack + '14Z-'
        
            task4imagemap2upu ['ORIGIN'] = origin2upu
            
            
            # ---------------------------------------------------------------------------------------------
            # (15) REGION - REQUIRED                         
            # ---------------------------------------------------------------------------------------------             
            region2upu = {}
            
            message2print = "{(Step 15A/70) trying" + "; region2upu-trying-by-map2image4upu: " + str(region2upu)  + "; time: " + time4xi() + "}" 
            print4xi(class2print, method2print, message2print) 
                        
            try:                                                                                    
                region2upu['X'] = 160                                                                               # WIDTH  IN PIXELS TO MAP 
                region2upu['Y'] = 90                                                                                # HEIGHT IN PIXELS TO MAP 
                region2upu['Z'] = 0                                                                                 # DEPTH  IN PIXELS TO MAP                                                                                                                          
                message2print = "{(Step 15B/70) set" + "; region2upu-set-by-map2image4upu to: " + str(region2upu)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print) 
                # 
                
                code2ack  = code2ack + '15B-' 
            except:
                message2print = "{(Step 15Z/70) failed" + "; region2upu-failed-by-map2image4upu: " + str(region2upu) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)             
                code2nack  = code2nack + '15Z-'
        
            task4imagemap2upu ['REGION'] = region2upu
            
            
            # ---------------------------------------------------------------------------------------------
            # (16) ROWPITCH (STRIDE) - REQUIRED                              
            # ---------------------------------------------------------------------------------------------             
            # One can assume that memory representing an image or screen will have data laid out as one row of pixels followed directly by the next row of pixels. 
            # However many APIs (e.g. Direct3D) that return a pointer to image data add an extra block of data at the end of each row. 
            # int offset = x * bytesPerPixel + y * pitch;
            # When a video image is stored in memory, the memory buffer might contain extra padding bytes after each row of pixels. 
            # The padding bytes affect how the image is stored in memory, but do not affect how the image is displayed.
            # The stride is the number of bytes from one row of pixels in memory to the next row of pixels in memory. 
            # Stride is also called pitch. 
            # If padding bytes are present, the stride is wider than the width of the image, as shown in the following illustration.


            rowpitch2upu = 0
            size2element = 8                                                                                        # VOXEL (X,Y,Z) = SHOT IN PIXELS, R=1 PIXELS
            voxelquantity2layer = 10                                                                                # MAX NUMBER OF VOXELS IN EACH 2D LAYER 
            quantity2direction = 8                                                                                  # ONE DIRECTIONS IN 2D
            width2upu = 800
            
            message2print = "{(Step 16A/70) trying" + "; rowpitch2upu-trying-by-map2image4upu: " + str(rowpitch2upu)  + "; time: " + time4xi() + "}"  
            print4xi(class2print, method2print, message2print) 
            # 
                     
            try:                                                                                                    # X-AXEL IS THE WORST CASE TO SCAN WITH Y=0, Z=0
                rowpitch2upu = int(width2upu * size2element * voxelquantity2layer / quantity2direction)             # (800 * 8 * 10) / 8 = 8K PIXELS MAX TO SCAN 
                                                                                                                    # SPHERE IS SPARSE HAVING FEW SHOTS PER SCAN LINE 
                                                                                                                                         
                message2print = "{(Step 16B/70) set" + "; rowpitch2upu-set-by-map2image4upu to: " + str(rowpitch2upu)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print) 
                #  8000                                                                                          
                 
                code2ack  = code2ack + '16B-' 
            except:
                message2print = "{(Step 16Z/70) failed" + "; rowpitch2upu-failed-by-map2image4upu: " + str(rowpitch2upu) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)             
                # 
                
                code2nack  = code2nack + '16Z-'
        
            task4imagemap2upu ['ROWPITCH'] = rowpitch2upu     
            
            # --------------------------------------------------------------------------------------------------------
            # (18) SLICEPITCH - REQUIRED - THE SIZE IN BYTES OF THE 2D SLICE OF THE 3D REGION OF A 3D IMAGE BEING MAP                         
            # --------------------------------------------------------------------------------------------------------             
            slicepitch2upu = None                                                                                   # SLICEPITCH IS ZERO FOR 2D IMAGES                           
           
            message2print = "{(Step 18A/70) trying" + "; slicepitch2upu-trying-by-map2image4upu: " + str(slicepitch2upu)  + "; time: " + time4xi() + "}" 
            print4xi(class2print, method2print, message2print) 
                        
            try:
                slicepitch2upu = 0                                                          
                message2print = "{(Step 18B/70) set" + "; slicepitch2upu-set-by-map2image4upu to: " + str(slicepitch2upu)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print) 
                # 0 

                code2ack  = code2ack + '18B-'                
                 
            except:
                message2print = "{(Step 18Z/70) failed" + "; slicepitch2upu-failed-by-map2image4upu: " + str(slicepitch2upu) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)             
                code2nack  = code2nack + '18Z-'
            
            task4imagemap2upu ['SLICEPITCH'] = slicepitch2upu                  
            
            # ---------------------------------------------------------------------------------------------
            # (19) FLAGS - REQUIRED                        
            # ---------------------------------------------------------------------------------------------             
            flags2upu = 'CL_MAP_READ'                                                                
     
            message2print = "{(Step 19A/70) trying" + "; flags2upu-trying-by-map2image4upu: " + str(flags2upu)  + "; time: " + time4xi() + "}" 
            print4xi(class2print, method2print, message2print) 
                        
            try:
                flags2upu = 'CL_MAP_WRITE'  # THE REGION SPECIFIED BY (ORIGIN, REGION) IN THE IMAGE OBJECT IS BEING MAPPED FOR WRITING                                                                                                                     
                message2print = "{(Step 19B/70) set" + "; flags2upu-set-by-map2image4upu to: " + str(flags2upu)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print) 
                # 
                
                code2ack  = code2ack + '19B-' 
            except:
                message2print = "{(Step 19Z/70) failed" + "; flags2upu-failed-by-map2image4upu: " + str(flags2upu) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)             
                code2nack  = code2nack + '19Z-'
        
            task4imagemap2upu ['FLAGS'] = flags2upu 
                                  
            # ---------------------------------------------------------------------------------------------
            # EOF PARAMETER PROVIDING                
            # ---------------------------------------------------------------------------------------------                                                                                                                               
        else:
            pass
            
        # ---------------------------------------------------------------------------------------------
        # ANALYSE TASKS - WHETHER AUTOMATICALLY GENERATED OR USER GIVEN 
        # ---------------------------------------------------------------------------------------------
        keys2task = task4imagemap2upu.keys()
        message2print = "{(Step 20/70) got" + "; keys2task-got-by-map2image4upu: " + str(sorted(keys2task))  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)            
        # ['BLOCKING',
        #  'BUFFER',
        #  'COMMANDQUEUE',
        #  'CONTEXT',
        #  'DATA2CALLBACK',
        #  'DEVICE',
        #  'EVENT',
        #  'EVENTWAITLIST',
        #  'EVENTWAITLISTQUANTITY',
        #  'FLAGS',
        #  'FUNCTION2CALLBACK',
        #  'IMAGE',
        #  'ORIGIN',
        #  'PIPE',
        #  'PLATFORM',
        #  'QUEUE',
        #  'REGION',
        #  'ROWPITCH',
        #  'SLICEPITCH']; time: 01:28:44 10/05/11 Pacific Daylight Time}
 

        if (('DATA2CALLBACK' in keys2task) and ('FUNCTION2CALLBACK' in keys2task)):
            pass
        else:                
            # ---------------------------------------------------------------------------------------------
            # (21B) TRACE - AUTOMATIC                      
            # ---------------------------------------------------------------------------------------------              
            function2callback4upu = trace2image4upu                                                              # CALLBACK FUNCTION FOR UPU
            
            message2print = "{(Step 21B1/70) initialized" + "; function2callback4upu-created-by-map2image4upu to: " + str(function2callback4upu)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)            
            # 
            
            code2ack  = code2ack + '21B1-'
           
            args2callback = {}                                                                                   # CREATE ARGUMENT DICTIONARY FOR CALLBACK FUNCTION             
            args2callback['CLASS']   = "map2image4upu"                                                          # MODULE OF UPUIZED OPENCL MODULE
            args2callback['METHOD']  = "map2image4upu"                                                          # FUNCTION OF UPUIZED OPENCL MODULE
            args2callback['MESSAGE'] = "{(Step 1Z1/33) failed" + "; map2image4upu-failed " + "; time: " + time4xi() + "}" # ERROR MESSAGE FOR UPUIZED OPENCL TO RETURN    

            message2print = "{(Step 21B2/70) initialized" + "; args2callback-created-by-map2image4upu to: " + str(args2callback)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)               
            # 

            code2ack  = code2ack + '21B2-'
        
            # ---------------------------------------------------------------------------------------------
            # (21C) CALLBACK - AUTOMATIC                      
            # ---------------------------------------------------------------------------------------------              
            task4imagemap2upu['DATA2CALLBACK'] = args2callback                                                   # USER DATA PASSED TO UPUIZED OPENCL - SEVERAL ARGS PASSED IN ONE DICTIONARY
            task4imagemap2upu['FUNCTION2CALLBACK'] = function2callback4upu                                       # CALLBACK FUNCTION TO CALLED BY OPENCL IN EXCEPTIONS
            
            code2ack  = code2ack + '21C-'
            message2print = "{(Step 21C/70) initialized" + "; task4imagemap2upu-created-by-map2image4upu to: " + str(task4imagemap2upu)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)  
            #
             
        if 'PLATFORM' in keys2task:
            pass
        else:  
            # ---------------------------------------------------------------------------------------------
            # (22) PLATFORM - AUTOMATIC                   
            # ---------------------------------------------------------------------------------------------             
            platform2xi = {}
            message2print = "{(Step 22A/70) initialized" + "; platform2xi-created-by-map2image4upu to: " + str(platform2xi)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)   
            # 
            
            try:
                platform2xi = get2platformids4xi()                                                                  # PLATFORM IS COMMON TO ALL DEVICES I.E TO DEVICE XI
                message2print = "{(Step 22B/70) got" + "; platform2xi-got-by-map2image4upu: " + str(platform2xi)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)
                # 
                 
                code2ack  = code2ack + '22B-'
            except:
                message2print = "{(Step 22Z/70) failed" + "; platform2xi-failed-by-map2image4upu: " + str(failed)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)                                                
                code2nack  = code2nack + '22Z-'
                 
            task4imagemap2upu['PLATFORM'] = platform2xi                                                          # PLATFORM OF SYSTEM 
          
        if 'CONTEXT' in keys2task:
            pass
        else:              
            # ---------------------------------------------------------------------------------------------
            # (23) CONTEXT - AUTOMATIC                 
            # ---------------------------------------------------------------------------------------------    
            context2upu = {}
            message2print = "{(Step 23A/70) trying" + "; context2upu-trying-by-map2image4upu: " + str(context2upu)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print) 
            try:
                context2upu = get2context4upu()                                                                     # NO PARAMETERS - THE SYSTEM CREATES CONTEXT IF REQUIRED  
                
                message2print = "{(Step 23B/70) got" + "; context2upu-got-by-map2image4upu: " + str(context2upu)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)                                  
                # 
                
                code2ack  = code2ack + '23B-'         
            except:
                message2print = "{(Step 23Z/70) failed" + "; context2upu-failed-by-map2image4upu: " + str(context2upu) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)             
                code2nack  = code2nack + '23Z-'
                
            task4imagemap2upu ['CONTEXT'] = context2upu           
        
        if 'DEVICE' in keys2task:
            pass
        else:                   
            # ---------------------------------------------------------------------------------------------
            # (24) DEVICE - AUTOMATIC                       
            # ---------------------------------------------------------------------------------------------    
            device2upu = {}
            
            message2print = "{(Step 24A/70) trying" + "; device2upu-trying-by-map2image4upu: " + str(device2upu)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print) 
            try:
                device2upu = get2upu4xi()                                                                           # THE SYSTEM CREATES DEVICE INFO, IF NECESSARY    
                
                message2print = "{(Step 24B/70) got" + "; device2upu-got-by-map2image4upu: " + str(device2upu)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)                                  
                # 
                
                code2ack  = code2ack + '24B-'  
            except:
                message2print = "{(Step 24Z/70) failed" + "; device2upu-failed-by-map2image4upu: " + str(device2upu) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)             
                code2nack  = code2nack + '24Z-'
            
            task4imagemap2upu ['DEVICE'] = device2upu
        
        if 'IMAGE' in keys2task:
            pass
        else:                      
            # ---------------------------------------------------------------------------------------------
            # (25) IMAGE - REQUIRED                            
            # ---------------------------------------------------------------------------------------------         
            imageinfo2upu = {}
             
            message2print = "{(Step 25A/70) trying" + "; imageinfo2upu-trying-by-map2image4upu: " + str(imageinfo2upu)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print) 
            try:
                imageinfo2upu = get2imageinfo4upu()                                                                         
                
                message2print = "{(Step 25B/70) got" + "; imageinfo2upu-got-by-map2image4upu: " + str(imageinfo2upu)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)                                  
                #                 
                code2ack  = code2ack + '25B-'  
            except:
                message2print = "{(Step 25Z/70) failed" + "; imageinfo2upu-failed-by-map2image4upu: " + str(imageinfo2upu) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)             
                code2nack  = code2nack + '25Z-'          
                      
            task4imagemap2upu ['IMAGE'] = imageinfo2upu         
                    
        if 'QUEUE' in keys2task:
            pass
        else:   
            # ---------------------------------------------------------------------------------------------
            # (26) QUEUE - REQUIRED BY UPUXI                          
            # ---------------------------------------------------------------------------------------------         
            queue2upu = None
            
            message2print = "{(Step 26A/70) trying" + "; queue2upu-trying-by-map2image4upu: " + str(queue2upu)  + "; time: " + time4xi() + "}" 
            print4xi(class2print, method2print, message2print) 
            try:
                queue2upu = Queue4Xi()                                                                                  # THE SYSTEM CREATES QUEUE    
                
                message2print = "{(Step 26B/70) got" + "; queue2upu-got-by-map2image4upu: " + str(queue2upu)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)                                  
                # 
                
                code2ack  = code2ack + '26B-'  
            except:
                message2print = "{(Step 26Z/70) failed" + "; queue2upu-failed-by-map2image4upu: " + str(queue2upu) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)             
                code2nack  = code2nack + '26Z-'
                
            task4imagemap2upu ['QUEUE'] = queue2upu
        
        if 'PIPE' in keys2task:
            pass
        else:              
            # ---------------------------------------------------------------------------------------------
            # (27) PIPE - REQUIRED BY UPUXI                              
            # ---------------------------------------------------------------------------------------------         
            pipe2upu = None
           
            message2print = "{(Step 27A/70) trying" + "; pipe2upu-trying-by-map2image4upu: " + str(pipe2upu)  + "; time: " + time4xi() + "}" 
            print4xi(class2print, method2print, message2print) 
            try:
                pipe2upu = Pipe4Xi()                                                                                    # THE SYSTEM CREATES PIPE    
                
                message2print = "{(Step 27B/70) got" + "; pipe2upu-got-by-map2image4upu: " + str(pipe2upu)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)                                  
                # 
                
                code2ack  = code2ack + '27B-'  
            except:
                message2print = "{(Step 27Z/70) failed" + "; pipe2upu-failed-by-map2image4upu: " + str(pipe2upu) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)             
                code2nack  = code2nack + '27Z-'
                
            task4imagemap2upu ['PIPE'] = pipe2upu 
            
        if 'COMMANDQUEUE' in keys2task:
            pass
        else:              
            # ---------------------------------------------------------------------------------------------
            # (28) COMMAND QUEUE - REQUIRED                              
            # ---------------------------------------------------------------------------------------------             
            commandqueueinfo2upu = {}
           
            message2print = "{(Step 28A/70) trying" + "; commandqueueinfo2upu-trying-by-map2image4upu: " + str(commandqueueinfo2upu)  + "; time: " + time4xi() + "}" 
            print4xi(class2print, method2print, message2print)  
                        
            try:
                commandqueueinfo2upu = get2commandqueueinfo4upu()                                                    # SYSTEM CREATES COMMAND QUEUE, IF NECESSARY   
                message2print = "{(Step 28B/70) got" + "; commandqueueinfo2upu-got-by-map2image4upu: " + str(commandqueueinfo2upu)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print) 
                # 
                
                code2ack  = code2ack + '28B-'  
            except:
                message2print = "{(Step 28Z/70) failed" + "; commandqueueinfo2upu-failed-by-map2image4upu: " + str(commandqueueinfo2upu) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)             
                code2nack  = code2nack + '28Z-'
        
            task4imagemap2upu ['COMMANDQUEUE'] = commandqueueinfo2upu
        
        
        if 'BUFFER' in keys2task:
            pass
        else:               
            # ---------------------------------------------------------------------------------------------
            # (29) BUFFER - AUTOMATIC BY UPUXI                               
            # ---------------------------------------------------------------------------------------------             
            bufferinfo2upu = {}
           
            message2print = "{(Step 29A/70) trying" + "; bufferinfo2upu-trying-by-map2image4upu: " + str(bufferinfo2upu)  + "; time: " + time4xi() + "}" 
            print4xi(class2print, method2print, message2print) 
                        
            try:
                bufferinfo2upu = get2bufferinfo4upu()                                                                   # SYSTEM CREATES BUFFER FOR UPU, IF NECESSARY
                message2print = "{(Step 29B/70) got" + "; bufferinfo2upu-got-by-map2image4upu: " + str(bufferinfo2upu)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print) 
                # 
                
                code2ack  = code2ack + '29B-' 
            except:
                message2print = "{(Step 29Z/70) failed" + "; bufferinfo2upu-failed-by-map2image4upu: " + str(bufferinfo2upu) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)             
                code2nack  = code2nack + '29Z-'
        
            task4imagemap2upu ['BUFFER'] = bufferinfo2upu
             
        if 'EVENT' in keys2task:
            pass
        else:               
            # ---------------------------------------------------------------------------------------------
            # (30) EVENT - REQUIRED                            
            # ---------------------------------------------------------------------------------------------             
            eventinfo2upu = {}
           
            message2print = "{(Step 30A/70) trying" + "; eventinfo2upu-trying-by-map2image4upu: " + str(eventinfo2upu)  + "; time: " + time4xi() + "}" 
            print4xi(class2print, method2print, message2print) 
                        
            try:
                eventinfo2upu = get2eventinfo4upu()                                                                    
                message2print = "{(Step 30B/70) got" + "; eventinfo2upu-got-by-map2image4upu: " + str(eventinfo2upu)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print) 
                # 
                
                code2ack  = code2ack + '30B-' 
            except:
                message2print = "{(Step 30Z/70) failed" + "; eventinfo2upu-failed-by-map2image4upu: " + str(eventinfo2upu) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)             
                code2nack  = code2nack + '30Z-'
        
            task4imagemap2upu ['EVENT'] = eventinfo2upu 
        
             
        if 'EVENTWAITLIST' in keys2task:
            pass
        else:               
            # ---------------------------------------------------------------------------------------------
            # (31) EVENTWAITLIST - REQUIRED                              
            # ---------------------------------------------------------------------------------------------             
            eventwaitlist2upu = {}
           
            message2print = "{(Step 31A/70) trying" + "; eventwaitlist2upu-trying-by-map2image4upu: " + str(eventwaitlist2upu)  + "; time: " + time4xi() + "}" 
            print4xi(class2print, method2print, message2print) 
                        
            try:
                eventwaitlist2upu = {}                                                                   
                message2print = "{(Step 31B/70) set" + "; eventwaitlist2upu-set-by-map2image4upu: " + str(eventwaitlist2upu)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print) 
                # 
                
                code2ack  = code2ack + '31B-' 
            except:
                message2print = "{(Step 31Z/70) failed" + "; eventwaitlist2upu-failed-by-map2image4upu: " + str(eventwaitlist2upu) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)             
                code2nack  = code2nack + '31Z-'
        
            task4imagemap2upu ['EVENTWAITLIST'] = eventwaitlist2upu                     
              
        if 'EVENTWAITLISTQUANTITY' in keys2task:
            pass
        else:               
            # ---------------------------------------------------------------------------------------------
            # (32) EVENTWAITLISTQUANTITY - REQUIRED                             
            # ---------------------------------------------------------------------------------------------             
            eventwaitlistquantity2upu = 0
           
            message2print = "{(Step 32A/70) trying" + "; eventwaitlistquantity2upu-trying-by-map2image4upu: " + str(eventwaitlistquantity2upu)  + "; time: " + time4xi() + "}" 
            print4xi(class2print, method2print, message2print) 
                        
            try:
                keys2event = eventwaitlist2upu.keys()
                for key2event in keys2event:
                    eventwaitlistquantity2upu = eventwaitlistquantity2upu + 1                                                                   
                message2print = "{(Step 32B/70) set" + "; eventwaitlistquantity2upu-set-by-map2image4upu: " + str(eventwaitlistquantity2upu)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print) 
                # 
                
                code2ack  = code2ack + '32B-' 
            except:
                message2print = "{(Step 32Z/70) failed" + "; eventwaitlistquantity2upu-failed-by-map2image4upu: " + str(eventwaitlistquantity2upu) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)             
                code2nack  = code2nack + '32Z-'
        
            task4imagemap2upu ['EVENTWAITLISTQUANTITY'] = eventwaitlistquantity2upu       
                                                 
                 
        if 'BLOCKING' in keys2task:
            pass
        else:              
            # ---------------------------------------------------------------------------------------------
            # (33) BLOCKING - REQUIRED                        
            # ---------------------------------------------------------------------------------------------             
            blocking2upu = ' '                                                                   
            message2print = "{(Step 33A/70) trying" + "; blocking2upu-trying-by-map2image4upu: " + str(blocking2upu)  + "; time: " + time4xi() + "}" 
            print4xi(class2print, method2print, message2print) 
                        
            try:
                blocking2upu = 'CL_FALSE'                                                          # the map command is non-blocking                                                              
                message2print = "{(Step 33B/70) set" + "; blocking2upu-set-by-map2image4upu to: " + str(blocking2upu)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print) 
                # 
                code2ack  = code2ack + '33B-' 
            except:
                message2print = "{(Step 33Z/70) failed" + "; blocking2upu-failed-by-map2image4upu: " + str(blocking2upu) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)             
                code2nack  = code2nack + '33Z-'
        
            task4imagemap2upu ['BLOCKING'] = blocking2upu             
             
        if 'ORIGIN' in keys2task:
            pass
        else:                    
            # ---------------------------------------------------------------------------------------------
            # (34) ORIGIN - REQUIRED                             
            # ---------------------------------------------------------------------------------------------             
            origin2upu = {}
            
            message2print = "{(Step 34A/70) trying" + "; origin2upu-trying-by-map2image4upu: " + str(origin2upu)  + "; time: " + time4xi() + "}" 
            print4xi(class2print, method2print, message2print) 
                        
            try:                                                                                    
                origin2upu['X'] = 0 
                origin2upu['Y'] = 0  
                origin2upu['Z'] = 0                                                                                                                          
                message2print = "{(Step 34B/70) set" + "; origin2upu-set-by-map2image4upu to: " + str(origin2upu)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print) 
                # 
                code2ack  = code2ack + '34B-' 
            except:
                message2print = "{(Step 34Z/70) failed" + "; origin2upu-failed-by-map2image4upu: " + str(origin2upu) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)             
                code2nack  = code2nack + '34Z-'
        
            task4imagemap2upu ['ORIGIN'] = origin2upu
            
             
        if 'REGION' in keys2task:
            pass
        else:            
            # ---------------------------------------------------------------------------------------------
            # (35) REGION - REQUIRED                         
            # ---------------------------------------------------------------------------------------------             
            region2upu = {}
            
            message2print = "{(Step 35A/70) trying" + "; region2upu-trying-by-map2image4upu: " + str(region2upu)  + "; time: " + time4xi() + "}" 
            print4xi(class2print, method2print, message2print) 
                            
            try:                                                                                    
                region2upu['X'] = 160                                                                               # WIDTH  IN PIXELS TO MAP 
                region2upu['Y'] = 90                                                                                # HEIGHT IN PIXELS TO MAP 
                region2upu['Z'] = 0                                                                                 # DEPTH  IN PIXELS TO MAP                                                                                                                          
                message2print = "{(Step 35B/70) set" + "; region2upu-set-by-map2image4upu to: " + str(region2upu)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print) 
                # 
                code2ack  = code2ack + '35B-' 
            except:
                message2print = "{(Step 35Z/70) failed" + "; region2upu-failed-by-map2image4upu: " + str(region2upu) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)             
                code2nack  = code2nack + '35Z-'
        
            task4imagemap2upu ['REGION'] = region2upu
        
            
             
        if 'ROWPITCH' in keys2task:
            pass
        else:               
            # ---------------------------------------------------------------------------------------------
            # (36) ROWPITCH (STRIDE) - REQUIRED                              
            # ---------------------------------------------------------------------------------------------             
            # One can assume that memory representing an image or screen will have data laid out as one row of pixels followed directly by the next row of pixels. 
            # However many APIs (e.g. Direct3D) that return a pointer to image data add an extra block of data at the end of each row. 
            # int offset = x * bytesPerPixel + y * pitch;
            # When a video image is stored in memory, the memory buffer might contain extra padding bytes after each row of pixels. 
            # The padding bytes affect how the image is stored in memory, but do not affect how the image is displayed.
            # The stride is the number of bytes from one row of pixels in memory to the next row of pixels in memory. 
            # Stride is also called pitch. 
            # If padding bytes are present, the stride is wider than the width of the image, as shown in the following illustration.
     
            rowpitch2upu = 0
            size2element = 1                                                                                        # VOXEL (X,Y,Z) = SHOT IN PIXELS, R=1 PIXELS
            voxelquantity2layer = 1                                                                                 # MAX NUMBER OF VOXELS IN EACH 2D LAYER 
            quantity2direction = 1                                                                                  # ONE DIRECTIONS IN 2D
            
            message2print = "{(Step 36A/70) trying" + "; rowpitch2upu-trying-by-map2image4upu: " + str(rowpitch2upu)  + "; time: " + time4xi() + "}" 
            print4xi(class2print, method2print, message2print) 
            # 
                        
            try:                                                                                                    # X-AXEL IS THE WORST CASE TO SCAN WITH Y=0, Z=0
                rowpitch2upu = int(width2upu * size2element * voxelquantity2layer / quantity2direction)             # (800 * 8 * 10) / 8 = 8K PIXELS MAX TO SCAN 
                                                                                                                    # SPHERE IS SPARSE HAVING FEW SHOTS PER SCAN LINE 
                                                                                                                                         
                message2print = "{(Step 36B/70) set" + "; rowpitch2upu-set-by-map2image4upu to: " + str(rowpitch2upu)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print) 
                #                                                                                             
                 
                code2ack  = code2ack + '36B-' 
            except:
                message2print = "{(Step 36Z/70) failed" + "; rowpitch2upu-failed-by-map2image4upu: " + str(rowpitch2upu) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)             
                # 
                
                code2nack  = code2nack + '36Z-'
        
            task4imagemap2upu ['ROWPITCH'] = rowpitch2upu     
   
        if 'SLICEPITCH' in keys2task:
            pass
        else:               
            # --------------------------------------------------------------------------------------------------------
            # (38) SLICEPITCH - REQUIRED - THE SIZE IN BYTES OF THE 2D SLICE OF THE 3D REGION OF A 3D IMAGE BEING MAP                         
            # --------------------------------------------------------------------------------------------------------             
            slicepitch2upu = None                                                                                   # SLICEPITCH IS ZERO FOR 2D IMAGES                           
           
            message2print = "{(Step 38A/70) trying" + "; slicepitch2upu-trying-by-map2image4upu: " + str(slicepitch2upu)  + "; time: " + time4xi() + "}" 
            print4xi(class2print, method2print, message2print) 
                        
            try:
                slicepitch2upu = 0                                                          
                message2print = "{(Step 38B/70) set" + "; slicepitch2upu-set-by-map2image4upu to: " + str(slicepitch2upu)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print) 
                # 
                code2ack  = code2ack + '38B-'                
                 
            except:
                message2print = "{(Step 38Z/70) failed" + "; slicepitch2upu-failed-by-map2image4upu: " + str(slicepitch2upu) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)             
                code2nack  = code2nack + '38Z-'
            
            task4imagemap2upu ['SLICEPITCH'] = slicepitch2upu
   
        if 'FLAGS' in keys2task:
            pass
        else:               
            # --------------------------------------------------------------------------------------------------------
            # (39) FLAGS - REQUIRED                   
            # --------------------------------------------------------------------------------------------------------             
            flags2upu = 'CL_MAP_READ'                                                                                                             
           
            message2print = "{(Step 39A/70) trying" + "; flags2upu-trying-by-map2image4upu: " + str(flags2upu)  + "; time: " + time4xi() + "}" 
            print4xi(class2print, method2print, message2print) 
                        
            try:
                flags2upu = 'CL_MAP_WRITE'                                                          
                message2print = "{(Step 39B/70) set" + "; flags2upu-set-by-map2image4upu to: " + str(flags2upu)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print) 
                # 
                code2ack  = code2ack + '39B-'                
                 
            except:
                message2print = "{(Step 39Z/70) failed" + "; flags2upu-failed-by-map2image4upu: " + str(flags2upu) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)             
                code2nack  = code2nack + '39Z-'
            
            task4imagemap2upu ['FLAGS'] = flags2upu
                                          
        # ---------------------------------------------------------------------------------------------
        # EOF TASK ANALYSIS
        # ---------------------------------------------------------------------------------------------
        keys2task = task4imagemap2upu.keys()
        message2print = "{(Step 40A/70) set" + "; keys2task-set-by-map2image4upu to: " + str(sorted(keys2task))  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)            
        # ['BLOCKING',
        #  'BUFFER',
        #  'COMMANDQUEUE',
        #  'CONTEXT',
        #  'DATA2CALLBACK',
        #  'DEVICE',
        #  'EVENT',
        #  'EVENTWAITLIST',
        #  'EVENTWAITLISTQUANTITY',
        #  'FLAGS',
        #  'FUNCTION2CALLBACK',
        #  'IMAGE',
        #  'ORIGIN',
        #  'PIPE',
        #  'PLATFORM',
        #  'QUEUE',
        #  'REGION',
        #  'ROWPITCH',
        #  'SLICEPITCH']; time: 01:28:44 10/05/11 Pacific Daylight Time}

                       
    except: 
        message2print = "{(Step 40Z/70) yielded" + "; task4imagemap2upu-yielded-by-map2image4upu: " + str(task4imagemap2upu) + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)  
        # 
        
        code2nack  = code2nack + '40Z-' 

    # ---------------------------------------------------------------------------------------------
    # CONTROL AND COMMAND
    # ---------------------------------------------------------------------------------------------
    control2mapimage4upu = {}       
                                
    # ---------------------------------------------------------------------------------------------
    # CONSUMER AND PUBLISHER CHANNEL FOR CONTROL AND COMMAND
    # ---------------------------------------------------------------------------------------------
    ack4channels2upu = False
    channels2upu = ()
    message2print = "{(Step 40A/70) trying" + "channels2upu-trying-by-map2image4upu: " + str(channels2upu)  + "; time: " + time4xi() + "}" 
    print4xi(class2print, method2print, message2print)     
    try:
        # 
        consumerchannel2upu, publisherchannel2upu = task4imagemap2upu ['PIPE']

        message2print = "{(Step 40B/70) got" + "; consumerchannel2upu-got-by-map2image4upu: " + str(consumerchannel2upu)  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)            
        # <read-write PipeConnection, handle 748>

        message2print = "{(Step 40C/70) got" + "; publisherchannel2upu-got-by-map2image4upu: " + str(publisherchannel2upu)  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)            
        # <read-write PipeConnection, handle 752>                     
        ack4channels2upu = True
        
        control2mapimage4upu['CONSUMERCHANNEL']  = consumerchannel2upu
        
        
        control2mapimage4upu['PUBLISHERCHANNEL'] = publisherchannel2upu
        
        code2ack  = code2ack + '40D-' 
        
    except:
        ack4channels2upu = False
        code2nack  = code2nack + '40Z-'      
                        
    # ---------------------------------------------------------------------------------------------
    # QUEUE FOR CONTROL AND COMMAND
    # ---------------------------------------------------------------------------------------------
    ack4queue2upu = False
    queue2upu = None
    message2print = "{(Step 41A/70) trying" + "queue2upu-trying-by-map2image4upu: " + str(queue2upu)  + "; time: " + time4xi() + "}" 
    print4xi(class2print, method2print, message2print)     
    try:
        queue2upu = task4imagemap2upu ['QUEUE']

        message2print = "{(Step 41B/70) got" + "; queue2upu-got-by-map2image4upu: " + str(queue2upu)  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)            
        # <queue4xi.Queue4Xi object at 0x0000000004B3FB70> 
              
        ack4queue2upu = True
        control2mapimage4upu['QUEUE']  = queue2upu        
        code2ack  = code2ack + '41C-'
    except:
        ack4queue2upu = False  
        code2nack  = code2nack + '41Z-'
    
    
    # -----------------------------------------------------------------------------------------    
    # DUMMY PROCESS OBJECT = THREAD CREATION
    # -----------------------------------------------------------------------------------------           
    # dummy2image4upu = None
    # message2print = "{(Step 69A/70) trying" + "dummy2image4upu-trying-by-map2image4upu: " + str(dummy2image4upu)  + "; time: " + time4xi() + "}"   
    # print4xi(class2print, method2print, message2print)     
    # try:                            
        # dummy2image4upu = DummyProcess4Xi(target=manoeuvre2mapimage4upu,                                            # in this module to handle queue and pipe channels
        #                                   args =(queue2upu,                                                         # caller puts and gets - does NOT call any functions
        #                                          consumerchannel2upu, publisherchannel2upu))                        # caller sends and receives - does NOT call any functions
        # message2print = "{(Step 69B/70) created" + "dummy2image4upu-created-by-map2image4upu: " + str(dummy2image4upu)  + "; time: " + time4xi() + "}" 
        # print4xi(class2print, method2print, message2print)  
        # code2ack  = code2ack + '69B-'
        
    # except:
        # code2nack  = code2nack + '69Z-'
        
    # ---------------------------------------------------------------------------------------------
    # RETURN CONTROL AND COMMAND DICTIONARY
    # ---------------------------------------------------------------------------------------------          
    message2print = "{(Step 70A/70) set" + "; code2ack-set-by-map2image4upu: " + str(code2ack) + "; time: " + time4xi() + "}"  
    print4xi(class2print, method2print, message2print)                                                             
    # 
    message2print = "{(Step 70B/70) set" + "; code2nack-set-by-map2image4upu: " + str(code2nack) + "; time: " + time4xi() + "}" 
    print4xi(class2print, method2print, message2print)                                                             
    # 1B1-1B2-1C-2B-3B-4B-5B-6B-7B-8B-9B-10B-11B-12B-13B-14B-15B-16B-18B-19B-40D-41C- 
        
    message2print = "{(Step 70C/70) returning" + "; control2mapimage4upu-returning-by-map2image4upu: " + str(control2mapimage4upu) + "; time: " + time4xi() + "}" 
    print4xi(class2print, method2print, message2print)                                                             
    # {'QUEUE': <queue4xi.Queue4Xi object at 0x0000000004B3FB70>,
    #  'PUBLISHERCHANNEL': <read-write PipeConnection, handle 752>,
    #  'CONSUMERCHANNEL': <read-write PipeConnection, handle 748>}; time: 01:28:44 10/05/11 Pacific Daylight Time}

        
    return control2mapimage4upu
                    
#</function4xi> 

# -----------------------------------------------------------------------------------------------------------------------
#<function4xi function = "manoeuvre2copyimage4upu" usage = "
#   # ---------------------------------------------------------------------------------------------------------------------
#   # CALL MANOEUVRE2COPYIMAGE4UPU - LET IT TO RECEIVE COPY MESSAGE AND TO SEND ROGER MESSAGE VIA PIPE                
#   # ---------------------------------------------------------------------------------------------------------------------       
#   if ack2simulate4upu is True:
#       print("(Step 42A/700) OF UPU2IMAGE4XI5TEST - TRYING TO PERFORM TARGET FUNCTION MANOEUVRE2COPYIMAGE4UPU ", time4xi())   
#       try:        
#           response2manoeuvre4copy = manoeuvre2copyimage4upu(queue2copy4upu, channel2copyconsume4upu, channel2copypublish4upu)
#           print("(Step 42B/700) OF UPU2IMAGE4XI5TEST - PERFORMED MANOEUVRE2COPYIMAGE4UPU, RESPONSE2MANOEUVRE4COPY: ", response2manoeuvre4copy)
#           # {'MESSAGE2SEND4UPU': '<roger4xi>roger2copy4upu</roger4xi>', 'ACK2INFO4UPU': False, 'MESSAGE2GET4UPU': '<default4xi>default2copy4upu</default4xi>', 
#           #  'MESSAGE2RECEIVE4UPU': '<copy4xi>copy2copy4upu</copy4xi>', 
#           'CODE2ACK4UPU': ' 0D-1C-', 'MESSAGE2PUT4UPU': '<default4xi>default2copy4upu</default4xi>', 'CODE2NACK4UPU': ' ', 'STATE2MANOEUVRE4UPU': 1}
#        except:
#           print("(Step 42Z/700) OF UPU2IMAGE4XI5TEST *** FAILED TO PERFORM TARGET FUNCTION MANOEUVRE2COPYIMAGE4UPU ", time4xi())     
#   else:
#       pass # trust activated thread
#">            

# ----------------------------------------------------------------------------------------------------------------------- 
def manoeuvre2copyimage4upu(queue2copymanoeuvre4upu, consumerchannel2copymanoeuvre4upu, publisherchannel2copymanoeuvre4upu):
    '''
    manoeuvres copy control and command
    '''
    global state2copyimage4upu                                                                                           # STATE FOR COMMAND AND CONTROL
    global max2copyimagestate4upu                                                                                        # MAX FOR STATE
    global ack2copyimageinfo4upu                                                                                         # ACK FOR IMAGE INFO
    global message2copyimageget4upu                                 
    global message2copyimageput4upu
    global message2copyimagereceive4upu
    global message2copyimagesend4upu
    
    # -----------------------------------------------------------------------------------------
    # (Step 0/33) PARAMETERS            
    # -----------------------------------------------------------------------------------------          
    class2print  = "manoeuvre2copyimage4upu" 
    method2print = "manoeuvre2copyimage4upu" 

    code2ack  = ' '
    code2nack = ' '
    
    message2print = "{(Step 0A/33) started" + "; manoeuvre2copyimage4upu-started: " + str(manoeuvre2copyimage4upu)  + "; time: " + time4xi() + "}" 
    print4xi(class2print, method2print, message2print)                                                                     #  
    # <function manoeuvre2copyimage4upu at 0x0000000003DB8C48>  
    
    message2print = "{(Step 0B/33) passed" + "; queue2copymanoeuvre4upu-to-manoeuvre2copyimage4upu: " + str(queue2copymanoeuvre4upu) + "; time: " + time4xi() + "}"
    print4xi(class2print, method2print, message2print)                                                              
    # <queue4xi.Queue4Xi object at 0x0000000003AD9400>
    
    message2print = "{(Step 0C/33) passed" + "; consumerchannel2copymanoeuvre4upu-to-manoeuvre2copyimage4upu: " + str(consumerchannel2copymanoeuvre4upu) + "; time: " + time4xi() + "}"
    print4xi(class2print, method2print, message2print)                                                              
    # <copy-write PipeConnection, handle 408>
    
    message2print = "{(Step 0D/33) passed" + "; publisherchannel2copymanoeuvre4upu-to-manoeuvre2copyimage4upu: " + str(publisherchannel2copymanoeuvre4upu) + "; time: " + time4xi() + "}"
    print4xi(class2print, method2print, message2print)                                                              
    # <copy-write PipeConnection, handle 412>     
    
    code2ack  = code2ack + '0D-' 
    response2manouvre4copy = {}
                    
    # ------------------------------------------------------------------------------------------------------------------------
    # SETTINGS
    # ------------------------------------------------------------------------------------------------------------------------    
    message2print = "{(Step 0E/70) try" + "; state2copyimage4upu-try-by-manoeuvre2copyimage4upu"  + "; time: " + time4xi() + "}" 
    print4xi(class2print, method2print, message2print)     
    try:
        if state2copyimage4upu is None:
            state2copyimage4upu = 0
            message2print = "{(Step 0F/33) initialized" + "; state2copyimage4upu-initialized-by-manoeuvre2copyimage4upu to: " + str(state2copyimage4upu)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)    
        else:
            message2print = "{(Step 0G/33) used" + "; state2copyimage4upu-used-by-manoeuvre2copyimage4upu: " + str(state2copyimage4upu)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print) 
            # 1; 2; 3;  
    except:
        state2copyimage4upu = 0
        message2print = "{(Step 0Z1/33) initialized" + "; state2copyimage4upu-initialized-by-manoeuvre2copyimage4upu to: " + str(state2copyimage4upu)  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)
        # 0
        ack2copyimageinfo4upu        = False
        message2print = "{(Step 0Z2/33) initialized" + "; ack2copyimageinfo4upu-initialized-by-manoeuvre2copyimage4upu to: " + str(ack2copyimageinfo4upu)  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)                                                     
        # False
        message2copyimageget4upu     = '<default4xi>default2copy4upu</default4xi>'
        message2print = "{(Step 0Z3/33) initialized" + "; message2copyimageget4upu-initialized-by-manoeuvre2copyimage4upu to: " + str(message2copyimageget4upu)  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)                                               
        # <default4xi>default2copy4upu</default4xi>
        message2copyimageput4upu     = '<default4xi>default2copy4upu</default4xi>'
        message2print = "{(Step 0Z4/33) initialized" + "; message2copyimageput4upu-initialized-by-manoeuvre2copyimage4upu to: " + str(message2copyimageput4upu)  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)              
        # <default4xi>default2copy4upu</default4xi>
        message2copyimagereceive4upu = '<default4xi>default2copy4upu</default4xi>'
        message2print = "{(Step 0Z5/33) initialized" + "; message2copyimagereceive4upu-initialized-by-manoeuvre2copyimage4upu to: " + str(message2copyimagereceive4upu)  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)              
        # <default4xi>default2copy4upu</default4xi>
        message2copyimagesend4upu    = '<default4xi>default2copy4upu</default4xi>'
        message2print = "{(Step 0Z6/33) initialized" + "; message2copyimagesend4upu-initialized-by-manoeuvre2copyimage4upu to: " + str(message2copyimagesend4upu)  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)              
        # <default4xi>default2copy4upu</default4xi>
        max2copyimagestate4upu = 3
        message2print = "{(Step 0Z7/33) initialized" + ";  max2copyimagestate4upu-initialized-by-manoeuvre2copyimage4upu to: " + str( max2copyimagestate4upu)  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)  
        # 
                    
    # ------------------------------------------------------------------------------------------------------------------------
    # STATE UPDATE
    # ------------------------------------------------------------------------------------------------------------------------                      
    state2copyimage4upu = state2copyimage4upu + 1                                                                   
    message2print = "{(Step 0H/33) added" + "; state2copyimage4upu-added-by-manoeuvre2copyimage4upu: " + str(state2copyimage4upu) + "; time: " + time4xi() + "}"
    print4xi(class2print, method2print, message2print)
    # 1; 2; 3; 4  
    if state2copyimage4upu > max2copyimagestate4upu:
        state2copyimage4upu = 0
        message2print = "{(Step 0I/33) set" + "; state2copyimage4upu-set-by-manoeuvre2copyimage4upu to: " + str(state2copyimage4upu)  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)
        # 0
        ack2copyimageinfo4upu        = False
        message2print = "{(Step 0J/33) set" + "; ack2copyimageinfo4upu-set-by-manoeuvre2copyimage4upu to: " + str(ack2copyimageinfo4upu)  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)                                                     
        # False
        message2copyimageget4upu     = '<default4xi>default2copy4upu</default4xi>'
        message2print = "{(Step 0K/33) set" + "; message2copyimageget4upu-set-by-manoeuvre2copyimage4upu to: " + str(message2copyimageget4upu)  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)                                               
        # <default4xi>default2copy4upu</default4xi>
        message2copyimageput4upu     = '<default4xi>default2copy4upu</default4xi>'
        message2print = "{(Step 0L/33) set" + "; message2copyimageput4upu-set-by-manoeuvre2copyimage4upu to: " + str(message2copyimageput4upu)  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)              
        # <default4xi>default2copy4upu</default4xi>
        message2copyimagereceive4upu = '<default4xi>default2copy4upu</default4xi>'
        message2print = "{(Step 0M/33) set" + "; message2copyimagereceive4upu-set-by-manoeuvre2copyimage4upu to: " + str(message2copyimagereceive4upu)  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)              
        # <default4xi>default2copy4upu</default4xi>
        message2copyimagesend4upu    = '<default4xi>default2copy4upu</default4xi>'
        message2print = "{(Step 0N/33) set" + "; message2copyimagesend4upu-set-by-manoeuvre2copyimage4upu to: " + str(message2copyimagesend4upu)  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)              
        # <default4xi>default2copy4upu</default4xi>
        max2copyimagestate4upu = 3
        message2print = "{(Step 0O/33) initialized" + ";  max2copyimagestate4upu-set-by-manoeuvre2copyimage4upu to: " + str( max2copyimagestate4upu)  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)  
        #     
                    
    # ------------------------------------------------------------------------------------------------------------------------
    # STATE I - RECEIVE CONTROL AND COMMAND COPY MESSAGE VIA PIPE AND SEND ROGER OR NEGATIVE BACK 
    # ------------------------------------------------------------------------------------------------------------------------
    if state2copyimage4upu == 1:
        message2print = "{(Step 1A/70) try" + "; receive-try-by-manoeuvre2copyimage4upu"  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)
        message2copyimagereceive4upu = ''
        try:
            # ----------------------------------------------------------------------------------------------------------------
            # RECEIVE CONTROL AND COMMAND MESSAGE VIA PIPE
            # ----------------------------------------------------------------------------------------------------------------         
            message2copyimagereceive4upu = consumerchannel2copymanoeuvre4upu.recv()
            message2print = "{(Step 1B/33) received" + "; message2copyimagereceive4upu-received-by-manoeuvre2copyimage4upu: " + str(message2copyimagereceive4upu) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)  
            # <copy4xi>copy2copy4upu</copy4xi>
                         
            if message2copyimagereceive4upu == '<copy4xi>copy2copy4upu</copy4xi>':
                message2copyimagesend4upu = '<roger4xi>roger2copy4upu</roger4xi>'
            else:
                message2copyimagesend4upu = '<negative4xi><negative2copy4upu</<negative4xi>'
            # ----------------------------------------------------------------------------------------------------------------
            # SEND ROGER OR NEGATIVE BACK 
            # ----------------------------------------------------------------------------------------------------------------                                
            publisherchannel2copymanoeuvre4upu.send(message2copyimagesend4upu)
            
            message2print = "{(Step 1C/33) sent" + "; message2copyimagesend4upu-sent-by-manoeuvre2copyimage4upu: " + str(message2copyimagesend4upu)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)          
            # <roger4xi>roger2copy4upu</roger4xi>
            
            code2ack  = code2ack + '1C-'
        except:
            message2print = "{(Step 1Z/33) failed" + "; message2copyimagereceive4upu-failed-by-manoeuvre2copyimage4upu: " + str(message2copyimagereceive4upu) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)             
            code2nack  = code2nack + '1Z-' 
    else:
        pass

                    
    # ------------------------------------------------------------------------------------------------------------------------
    # STATE II - RECEIVE CONTROL AND COMMAND READY MESSAGE VIA PIPE AND SEND READY OR NEGATIVE BACK 
    # ------------------------------------------------------------------------------------------------------------------------
    if state2copyimage4upu == 2:
        message2print = "{(Step 2A/70) try" + "; receive-try-by-manoeuvre2copyimage4upu"  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)
        message2copyimagereceive4upu = ''
        try:
            # ----------------------------------------------------------------------------------------------------------------
            # RECEIVE CONTROL AND COMMAND MESSAGE VIA PIPE
            # ----------------------------------------------------------------------------------------------------------------         
            message2copyimagereceive4upu = consumerchannel2copymanoeuvre4upu.recv()
            message2print = "{(Step 2B/33) received" + "; message2copyimagereceive4upu-received-by-manoeuvre2copyimage4upu: " + str(message2copyimagereceive4upu) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)  
            # <ready4xi>ready2copy4upu</ready4xi>
                         
            if message2copyimagereceive4upu == '<ready4xi>ready2copy4upu</ready4xi>':
                message2copyimagesend4upu = '<ready4xi>ready2copy4upu</ready4xi>'
            else:
                message2copyimagesend4upu = '<negative4xi><negative2copy4upu</<negative4xi>'
            # ----------------------------------------------------------------------------------------------------------------
            # SEND READY OR NEGATIVE BACK 
            # ----------------------------------------------------------------------------------------------------------------                                
            publisherchannel2copymanoeuvre4upu.send(message2copyimagesend4upu)
            
            message2print = "{(Step 2C/33) sent" + "; message2copyimagesend4upu-sent-by-manoeuvre2copyimage4upu: " + str(message2copyimagesend4upu)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)          
            # 
            
            code2ack  = code2ack + '2C-'
        except:
            message2print = "{(Step 2Z/33) failed" + "; message2copyimagereceive4upu-failed-by-manoeuvre2copyimage4upu: " + str(message2copyimagereceive4upu) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)             
            code2nack  = code2nack + '2Z-' 
    else:
        pass        
                
    # ------------------------------------------------------------------------------------------------------------------------
    # STATE III - GET IMAGE INFORMATION SUCH AS IMAGE PATH VIA QUEUE AND SEND ACK OR NACK VIA PIPE
    # ------------------------------------------------------------------------------------------------------------------------
    if state2copyimage4upu == 3:    
        info2image4upu = None
        message2print = "{(Step 3A/70) try" + "; info2image4upu-try-by-manoeuvre2copyimage4upu"  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)
        try: 
            # ----------------------------------------------------------------------------------------------------------------
            # GET IMAGE INFORMATION SUCH AS IMAGE PATH VIA QUEUE
            # ----------------------------------------------------------------------------------------------------------------        
            info2image4upu = queue2copymanoeuvre4upu.get()
            
            message2print = "{(Step 3B/33) got" + "; info2image4upu-got-by-manoeuvre2copyimage4upu: " + str(info2image4upu) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)  
            # ..//..//..//..//..//web//images//png//dragonxi.png
            if info2image4upu is None:
                code2nack  = code2nack + '3Z1-'
                ack2copyimageinfo4upu = False 
            else:    
                code2ack  = code2ack + '3C-'
                ack2copyimageinfo4upu = True
        except:
            message2print = "{(Step 3Z1/33) failed" + "; queue2copymanoeuvre4upu-failed-by-manoeuvre2copyimage4upu: " + str(queue2copymanoeuvre4upu) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)             
            code2nack  = code2nack + '3Z1-'
            
        message2copyimagesend4upu = ' '
        try:
            # ----------------------------------------------------------------------------------------------------------------
            # SEND CONTROL INFO ABOUT IMAGE OPERATION - EITHER ACK OR NACK  
            # ----------------------------------------------------------------------------------------------------------------         
            if ack2copyimageinfo4upu is True:
                message2copyimagesend4upu = '<ack4xi>ack2imageinfo4upu</ack4xi>'
            else:
                message2copyimagesend4upu = '<nack4xi>nack2imageinfo4upu</nack4xi>'
            message2print = "{(Step 4B/33) set" + "; message2copyimagesend4upu-set-by-manoeuvre2copyimage4upu to: " + str(message2copyimagesend4upu)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print) 
            # <ack4xi>ack2imageinfo4upu</ack4xi>       
        except:
            message2print = "{(Step 4Z1/33) failed" + "; message2copyimagesend4upu-failed-by-manoeuvre2copyimage4upu: " + str(message2copyimagesend4upu) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)             
            
            code2nack  = code2nack + '4Z1-'    
        try:                
            message2print = "{(Step 4C/70) using" + "; publisherchannel2copymanoeuvre4upu-calling-by-manoeuvre2copyimage4upu: " + str(publisherchannel2copymanoeuvre4upu) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)    
            # <copy-write PipeConnection, handle 412>
            message2print = "{{(Step 4D/70) passing" + "; message2copyimagesend4upu-passing-to-send-by-manoeuvre2copyimage4upu: " + str(message2copyimagesend4upu) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)
            #              
        except:
            message2print = "{(Step 4Z1/33) failed" + "; message2copyimagesend4upu-failed-by-manoeuvre2copyimage4upu: " + str(message2copyimagesend4upu) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)             
            code2nack  = code2nack + '4Z2-'                
        try:    
            publisherchannel2copymanoeuvre4upu.send(message2copyimagesend4upu)
            
            message2print = "{(Step 4E/33) sent" + "; message2copyimagesend4upu-sent-by-manoeuvre2copyimage4upu: " + str(message2copyimagesend4upu)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)          
            # <ack4xi>ack2imageinfo4upu</ack4xi>            
            code2ack  = code2ack + '4E-'
        except:
            message2print = "{(Step 4Z3/33) failed" + "; message2copyimagesend4upu-failed-by-manoeuvre2copyimage4upu: " + str(message2copyimagesend4upu) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)             
            code2nack  = code2nack + '4Z3-'                
    else:
        pass
        

            
    # ------------------------------------------------------------------------------------------------------------------------
    # RETURN 
    # ------------------------------------------------------------------------------------------------------------------------
    response2manouvre4copy['ACK2INFO4UPU']        = ack2copyimageinfo4upu
    response2manouvre4copy['CODE2ACK4UPU']        = code2ack
    response2manouvre4copy['CODE2NACK4UPU']       = code2nack
    response2manouvre4copy['MESSAGE2GET4UPU']     = message2copyimageget4upu
    response2manouvre4copy['MESSAGE2PUT4UPU']     = message2copyimageput4upu
    response2manouvre4copy['MESSAGE2RECEIVE4UPU'] = message2copyimagereceive4upu
    response2manouvre4copy['MESSAGE2SEND4UPU']    = message2copyimagesend4upu
    response2manouvre4copy['STATE2MANOEUVRE4UPU'] = state2copyimage4upu
                          
    message2print = "{(Step 33/33) returning" + "; response2manouvre4copy-returning-by-manoeuvre2copyimage4upu: " + str(response2manouvre4copy) + "; time: " + time4xi() + "}" 
    print4xi(class2print, method2print, message2print)                                                             
    #1 {'MESSAGE2SEND4UPU': '<roger4xi>roger2copy4upu</roger4xi>',
    #   'ACK2INFO4UPU': False,
    #   'MESSAGE2GET4UPU': '<default4xi>default2copy4upu</default4xi>',
    #   'MESSAGE2RECEIVE4UPU': '<copy4xi>copy2copy4upu</copy4xi>',
    #   'CODE2ACK4UPU': ' 0D-1C-',
    #   'MESSAGE2PUT4UPU': '<default4xi>default2copy4upu</default4xi>',
    #   'CODE2NACK4UPU': ' ',
    #   'STATE2MANOEUVRE4UPU': 1}; time: 05:25:15 10/03/11 Pacific Daylight Time}

 
    #2 {'MESSAGE2SEND4UPU': '<ready4xi>ready2copy4upu</ready4xi>',
    #   'ACK2INFO4UPU': False,
    #   'MESSAGE2GET4UPU': '<default4xi>default2copy4upu</default4xi>',
    #   'MESSAGE2RECEIVE4UPU': '<ready4xi>ready2copy4upu</ready4xi>',
    #   'CODE2ACK4UPU': ' 0D-2C-',
    #   'MESSAGE2PUT4UPU': '<default4xi>default2copy4upu</default4xi>',
    #   'CODE2NACK4UPU': ' ',
    #   'STATE2MANOEUVRE4UPU': 2}; time: 05:25:15 10/03/11 Pacific Daylight Time}

   
    #3 {'MESSAGE2SEND4UPU': '<ack4xi>ack2imageinfo4upu</ack4xi>',
    #   'ACK2INFO4UPU': True,
    #   'MESSAGE2GET4UPU': '<default4xi>default2copy4upu</default4xi>',
    #   'MESSAGE2RECEIVE4UPU': '<ready4xi>ready2copy4upu</ready4xi>',
    #   'CODE2ACK4UPU': ' 0D-3C-4E-',
    #   'MESSAGE2PUT4UPU': '<default4xi>default2copy4upu</default4xi>',
    #   'CODE2NACK4UPU': ' ', 'STATE2MANOEUVRE4UPU': 3}; time: 05:25:15 10/03/11 Pacific Daylight Time}

    
    #4 {'MESSAGE2SEND4UPU': '<default4xi>default2copy4upu</default4xi>',
    #   'ACK2INFO4UPU': False,
    #   'MESSAGE2GET4UPU': '<default4xi>default2copy4upu</default4xi>',
    #   'MESSAGE2RECEIVE4UPU': '<default4xi>default2copy4upu</default4xi>',
    #   'CODE2ACK4UPU': ' 0D-',
    #   'MESSAGE2PUT4UPU': '<default4xi>default2copy4upu</default4xi>',
    #   'CODE2NACK4UPU': ' ', 'STATE2MANOEUVRE4UPU': 0}; time: 05:25:15 10/03/11 Pacific Daylight Time}

    return response2manouvre4copy           
          
#</function4xi>


# -----------------------------------------------------------------------------------------------------------------------
#<function4xi function = "manoeuvre2copyimagetobuffer4upu" usage = "
#   # ---------------------------------------------------------------------------------------------------------------------
#   # CALL MANOEUVRE2COPYIMAGE4UPU  - LET IT TO RECEIVE COPY AND TO SEND ROGER VIA PIPE                
#   # ---------------------------------------------------------------------------------------------------------------------       
#   if ack2simulate4upu is True:
#       print("(Step 62A/700) OF UPU2IMAGE4XI5TEST - TRYING TO PERFORM TARGET FUNCTION MANOEUVRE2COPYIMAGE4UPU ", time4xi())   
#       try:        
#           response2manoeuvre4copy = manoeuvre2copyimagetobuffer4upu(queue4copy2imagetobuffer4upu, channel4copy2imagetobufferconsume4upu, channel4copy2imagetobufferpublish4upu)
#           print("(Step 62B/700) OF UPU2IMAGE4XI5TEST - PERFORMED MANOEUVRE2COPYIMAGE4UPU, RESPONSE2MANOEUVRE4COPY: ", response2manoeuvre4copy)
#           # { 'MESSAGE2SEND4UPU': '<roger4xi>roger2copyimagetobuffer4upu</roger4xi>',
#           #   'ACK2INFO4UPU': False,
#           #   'MESSAGE2GET4UPU': '<default4xi>default2copyimagetobuffer4upu</default4xi>',
#           #   'MESSAGE2RECEIVE4UPU': '<copy4xi>copy2copyimagetobuffer4upu</copy4xi>',
#           #   'CODE2ACK4UPU': ' 0D-1C-',
#           #   'MESSAGE2PUT4UPU': '<default4xi>default2copyimagetobuffer4upu</default4xi>',
#           #   'CODE2NACK4UPU': ' ',
#           #   'STATE2MANOEUVRE4UPU': 1}
#        except:
#           print("(Step 62Z/700) OF UPU2IMAGE4XI5TEST *** FAILED TO PERFORM TARGET FUNCTION MANOEUVRE2COPYIMAGE4UPU ", time4xi())     
#   else:
#       pass # trust activated thread    
#">

# ----------------------------------------------------------------------------------------------------------------------- 
def manoeuvre2copyimagetobuffer4upu(queue2copyimagetobuffermanoeuvre4upu, consumerchannel2copyimagetobuffermanoeuvre4upu, publisherchannel2copyimagetobuffermanoeuvre4upu):
    '''
    manoeuvres copy control and command for copy2imagetobuffer4upu
    '''
    global state2copyimagetobuffer4upu                                                                                           # STATE FOR COMMAND AND CONTROL
    global max2copyimagetobufferstate4upu                                                                                        # MAX FOR STATE
    global ack2copyimagetobufferinfo4upu                                                                                         # ACK FOR IMAGE INFO
    global message2copyimagetobufferget4upu                                 
    global message2copyimagetobufferput4upu
    global message2copyimagetobufferreceive4upu
    global message2copyimagetobuffersend4upu
    
    # -----------------------------------------------------------------------------------------
    # (Step 0/33) PARAMETERS            
    # -----------------------------------------------------------------------------------------          
    class2print  = "manoeuvre2copyimagetobuffer4upu" 
    method2print = "manoeuvre2copyimagetobuffer4upu" 

    code2ack  = ' '
    code2nack = ' '
    
    message2print = "{(Step 0A/33) started" + "; manoeuvre2copyimagetobuffer4upu-started: " + str(manoeuvre2copyimagetobuffer4upu)  + "; time: " + time4xi() + "}" 
    print4xi(class2print, method2print, message2print)                                                                     #  
    # 
    
    message2print = "{(Step 0B/33) passed" + "; queue2copyimagetobuffermanoeuvre4upu-to-manoeuvre2copyimagetobuffer4upu: " + str(queue2copyimagetobuffermanoeuvre4upu) + "; time: " + time4xi() + "}"
    print4xi(class2print, method2print, message2print)                                                              
    # 
    
    message2print = "{(Step 0C/33) passed" + "; consumerchannel2copyimagetobuffermanoeuvre4upu-to-manoeuvre2copyimagetobuffer4upu: " + str(consumerchannel2copyimagetobuffermanoeuvre4upu) + "; time: " + time4xi() + "}"
    print4xi(class2print, method2print, message2print)                                                              
    # 
    
    message2print = "{(Step 0D/33) passed" + "; publisherchannel2copyimagetobuffermanoeuvre4upu-to-manoeuvre2copyimagetobuffer4upu: " + str(publisherchannel2copyimagetobuffermanoeuvre4upu) + "; time: " + time4xi() + "}"
    print4xi(class2print, method2print, message2print)                                                              
    # 
    
    code2ack  = code2ack + '0D-' 
    response2manouvre4copyimagetobuffer = {}
                    
    # ------------------------------------------------------------------------------------------------------------------------
    # SETTINGS
    # ------------------------------------------------------------------------------------------------------------------------    
    message2print = "{(Step 0E/70) try" + "; state2copyimagetobuffer4upu-try-by-manoeuvre2copyimagetobuffer4upu"  + "; time: " + time4xi() + "}" 
    print4xi(class2print, method2print, message2print)     
    try:
        if state2copyimagetobuffer4upu is None:
            state2copyimagetobuffer4upu = 0
            message2print = "{(Step 0F/33) initialized" + "; state2copyimagetobuffer4upu-initialized-by-manoeuvre2copyimagetobuffer4upu to: " + str(state2copyimagetobuffer4upu)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)    
        else:
            message2print = "{(Step 0G/33) used" + "; state2copyimagetobuffer4upu-used-by-manoeuvre2copyimagetobuffer4upu: " + str(state2copyimagetobuffer4upu)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print) 
            # 1; 2; 3;  
    except:
        state2copyimagetobuffer4upu = 0
        message2print = "{(Step 0Z1/33) initialized" + "; state2copyimagetobuffer4upu-initialized-by-manoeuvre2copyimagetobuffer4upu to: " + str(state2copyimagetobuffer4upu)  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)
        # 0
        ack2copyimagetobufferinfo4upu        = False
        message2print = "{(Step 0Z2/33) initialized" + "; ack2copyimagetobufferinfo4upu-initialized-by-manoeuvre2copyimagetobuffer4upu to: " + str(ack2copyimagetobufferinfo4upu)  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)                                                     
        # False
        message2copyimagetobufferget4upu     = '<default4xi>default2copyimagetobuffer4upu</default4xi>'
        message2print = "{(Step 0Z3/33) initialized" + "; message2copyimagetobufferget4upu-initialized-by-manoeuvre2copyimagetobuffer4upu to: " + str(message2copyimagetobufferget4upu)  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)                                               
        # 
        message2copyimagetobufferput4upu     = '<default4xi>default2copyimagetobuffer4upu</default4xi>'
        message2print = "{(Step 0Z4/33) initialized" + "; message2copyimagetobufferput4upu-initialized-by-manoeuvre2copyimagetobuffer4upu to: " + str(message2copyimagetobufferput4upu)  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)              
        # 
        message2copyimagetobufferreceive4upu = '<default4xi>default2copyimagetobuffer4upu</default4xi>'
        message2print = "{(Step 0Z5/33) initialized" + "; message2copyimagetobufferreceive4upu-initialized-by-manoeuvre2copyimagetobuffer4upu to: " + str(message2copyimagetobufferreceive4upu)  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)              
        # 
        message2copyimagetobuffersend4upu    = '<default4xi>default2copyimagetobuffer4upu</default4xi>'
        message2print = "{(Step 0Z6/33) initialized" + "; message2copyimagetobuffersend4upu-initialized-by-manoeuvre2copyimagetobuffer4upu to: " + str(message2copyimagetobuffersend4upu)  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)              
        # 
        max2copyimagetobufferstate4upu = 3
        message2print = "{(Step 0Z7/33) initialized" + ";  max2copyimagetobufferstate4upu-initialized-by-manoeuvre2copyimagetobuffer4upu to: " + str( max2copyimagetobufferstate4upu)  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)  
        # 
                    
    # ------------------------------------------------------------------------------------------------------------------------
    # STATE UPDATE
    # ------------------------------------------------------------------------------------------------------------------------                      
    state2copyimagetobuffer4upu = state2copyimagetobuffer4upu + 1                                                                   
    message2print = "{(Step 0H/33) added" + "; state2copyimagetobuffer4upu-added-by-manoeuvre2copyimagetobuffer4upu: " + str(state2copyimagetobuffer4upu) + "; time: " + time4xi() + "}"
    print4xi(class2print, method2print, message2print)
    # 1; 2; 3; 4 
     
    if state2copyimagetobuffer4upu > max2copyimagetobufferstate4upu:
        state2copyimagetobuffer4upu = 0
        message2print = "{(Step 0I/33) set" + "; state2copyimagetobuffer4upu-set-by-manoeuvre2copyimagetobuffer4upu to: " + str(state2copyimagetobuffer4upu)  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)
        # 
        ack2copyimagetobufferinfo4upu        = False
        message2print = "{(Step 0J/33) set" + "; ack2copyimagetobufferinfo4upu-set-by-manoeuvre2copyimagetobuffer4upu to: " + str(ack2copyimagetobufferinfo4upu)  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)                                                     
        # 
        message2copyimagetobufferget4upu     = '<default4xi>default2copyimagetobuffer4upu</default4xi>'
        message2print = "{(Step 0K/33) set" + "; message2copyimagetobufferget4upu-set-by-manoeuvre2copyimagetobuffer4upu to: " + str(message2copyimagetobufferget4upu)  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)                                               
        # 
        message2copyimagetobufferput4upu     = '<default4xi>default2copyimagetobuffer4upu</default4xi>'
        message2print = "{(Step 0L/33) set" + "; message2copyimagetobufferput4upu-set-by-manoeuvre2copyimagetobuffer4upu to: " + str(message2copyimagetobufferput4upu)  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)              
        # 
        message2copyimagetobufferreceive4upu = '<default4xi>default2copyimagetobuffer4upu</default4xi>'
        message2print = "{(Step 0M/33) set" + "; message2copyimagetobufferreceive4upu-set-by-manoeuvre2copyimagetobuffer4upu to: " + str(message2copyimagetobufferreceive4upu)  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)              
        # 
        message2copyimagetobuffersend4upu    = '<default4xi>default2copyimagetobuffer4upu</default4xi>'
        message2print = "{(Step 0N/33) set" + "; message2copyimagetobuffersend4upu-set-by-manoeuvre2copyimagetobuffer4upu to: " + str(message2copyimagetobuffersend4upu)  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)              
        # 
        max2copyimagetobufferstate4upu = 3
        message2print = "{(Step 0O/33) initialized" + ";  max2copyimagetobufferstate4upu-set-by-manoeuvre2copyimagetobuffer4upu to: " + str( max2copyimagetobufferstate4upu)  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)  
        #     
                    
    # ------------------------------------------------------------------------------------------------------------------------
    # STATE I - RECEIVE CONTROL AND COMMAND COPY MESSAGE VIA PIPE AND SEND ROGER OR NEGATIVE BACK 
    # ------------------------------------------------------------------------------------------------------------------------
    if state2copyimagetobuffer4upu == 1:
        message2print = "{(Step 1A/70) try" + "; receive-try-by-manoeuvre2copyimagetobuffer4upu"  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)
        message2copyimagetobufferreceive4upu = ''
        try:
            # ----------------------------------------------------------------------------------------------------------------
            # RECEIVE CONTROL AND COMMAND MESSAGE VIA PIPE
            # ----------------------------------------------------------------------------------------------------------------         
            message2copyimagetobufferreceive4upu = consumerchannel2copyimagetobuffermanoeuvre4upu.recv()
            message2print = "{(Step 1B/33) received" + "; message2copyimagetobufferreceive4upu-received-by-manoeuvre2copyimagetobuffer4upu: " + str(message2copyimagetobufferreceive4upu) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)  
            # 
                         
            if message2copyimagetobufferreceive4upu == '<copy4xi>copy2copyimagetobuffer4upu</copy4xi>': 
                message2copyimagetobuffersend4upu = '<roger4xi>roger2copyimagetobuffer4upu</roger4xi>'
            else:
                message2copyimagetobuffersend4upu = '<negative4xi><negative2copyimagetobuffer4upu</<negative4xi>'
            # ----------------------------------------------------------------------------------------------------------------
            # SEND ROGER OR NEGATIVE BACK 
            # ----------------------------------------------------------------------------------------------------------------                                
            publisherchannel2copyimagetobuffermanoeuvre4upu.send(message2copyimagetobuffersend4upu)
            
            message2print = "{(Step 1C/33) sent" + "; message2copyimagetobuffersend4upu-sent-by-manoeuvre2copyimagetobuffer4upu: " + str(message2copyimagetobuffersend4upu)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)          
            # 
            
            code2ack  = code2ack + '1C-'
        except:
            message2print = "{(Step 1Z/33) failed" + "; message2copyimagetobufferreceive4upu-failed-by-manoeuvre2copyimagetobuffer4upu: " + str(message2copyimagetobufferreceive4upu) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)             
            code2nack  = code2nack + '1Z-' 
    else:
        pass

                    
    # ------------------------------------------------------------------------------------------------------------------------
    # STATE II - RECEIVE CONTROL AND COMMAND READY MESSAGE VIA PIPE AND SEND READY OR NEGATIVE BACK 
    # ------------------------------------------------------------------------------------------------------------------------
    if state2copyimagetobuffer4upu == 2:
        message2print = "{(Step 2A/70) try" + "; receive-try-by-manoeuvre2copyimagetobuffer4upu"  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)
        message2copyimagetobufferreceive4upu = ''
        try:
            # ----------------------------------------------------------------------------------------------------------------
            # RECEIVE CONTROL AND COMMAND MESSAGE VIA PIPE
            # ----------------------------------------------------------------------------------------------------------------         
            message2copyimagetobufferreceive4upu = consumerchannel2copyimagetobuffermanoeuvre4upu.recv()
            message2print = "{(Step 2B/33) received" + "; message2copyimagetobufferreceive4upu-received-by-manoeuvre2copyimagetobuffer4upu: " + str(message2copyimagetobufferreceive4upu) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)  
            # 
                         
            if message2copyimagetobufferreceive4upu == '<ready4xi>ready2copyimagetobuffer4upu</ready4xi>':
                message2copyimagetobuffersend4upu = '<ready4xi>ready2copyimagetobuffer4upu</ready4xi>'
            else:
                message2copyimagetobuffersend4upu = '<negative4xi><negative2copyimagetobuffer4upu</<negative4xi>'
            # ----------------------------------------------------------------------------------------------------------------
            # SEND READY OR NEGATIVE BACK 
            # ----------------------------------------------------------------------------------------------------------------                                
            publisherchannel2copyimagetobuffermanoeuvre4upu.send(message2copyimagetobuffersend4upu)
            
            message2print = "{(Step 2C/33) sent" + "; message2copyimagetobuffersend4upu-sent-by-manoeuvre2copyimagetobuffer4upu: " + str(message2copyimagetobuffersend4upu)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)          
            # 
            
            code2ack  = code2ack + '2C-'
        except:
            message2print = "{(Step 2Z/33) failed" + "; message2copyimagetobufferreceive4upu-failed-by-manoeuvre2copyimagetobuffer4upu: " + str(message2copyimagetobufferreceive4upu) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)             
            code2nack  = code2nack + '2Z-' 
    else:
        pass        
                
    # ------------------------------------------------------------------------------------------------------------------------
    # STATE III - GET IMAGE INFORMATION SUCH AS IMAGE PATH VIA QUEUE AND SEND ACK OR NACK VIA PIPE
    # ------------------------------------------------------------------------------------------------------------------------
    if state2copyimagetobuffer4upu == 3:    
        info2image4upu = None
        message2print = "{(Step 3A/70) try" + "; info2image4upu-try-by-manoeuvre2copyimagetobuffer4upu"  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)
        try: 
            # ----------------------------------------------------------------------------------------------------------------
            # GET IMAGE INFORMATION SUCH AS IMAGE PATH VIA QUEUE
            # ----------------------------------------------------------------------------------------------------------------        
            info2image4upu = queue2copyimagetobuffermanoeuvre4upu.get()
            
            message2print = "{(Step 3B/33) got" + "; info2image4upu-got-by-manoeuvre2copyimagetobuffer4upu: " + str(info2image4upu) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)  
            # ..//..//..//..//..//web//images//png//dragonxi.png
            if info2image4upu is None:
                code2nack  = code2nack + '3Z1-'
                ack2copyimagetobufferinfo4upu = False 
            else:    
                code2ack  = code2ack + '3C-'
                ack2copyimagetobufferinfo4upu = True
        except:
            message2print = "{(Step 3Z1/33) failed" + "; queue2copyimagetobuffermanoeuvre4upu-failed-by-manoeuvre2copyimagetobuffer4upu: " + str(queue2copyimagetobuffermanoeuvre4upu) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)             
            code2nack  = code2nack + '3Z1-'
            
        message2copyimagetobuffersend4upu = ' '
        try:
            # ----------------------------------------------------------------------------------------------------------------
            # SEND CONTROL INFO ABOUT IMAGE OPERATION - EITHER ACK OR NACK  
            # ----------------------------------------------------------------------------------------------------------------         
            if ack2copyimagetobufferinfo4upu is True:
                message2copyimagetobuffersend4upu = '<ack4xi>ack2imageinfo4upu</ack4xi>'
            else:
                message2copyimagetobuffersend4upu = '<nack4xi>nack2imageinfo4upu</nack4xi>'
            message2print = "{(Step 4B/33) set" + "; message2copyimagetobuffersend4upu-set-by-manoeuvre2copyimagetobuffer4upu to: " + str(message2copyimagetobuffersend4upu)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print) 
            # <ack4xi>ack2imageinfo4upu</ack4xi>       
        except:
            message2print = "{(Step 4Z1/33) failed" + "; message2copyimagetobuffersend4upu-failed-by-manoeuvre2copyimagetobuffer4upu: " + str(message2copyimagetobuffersend4upu) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)             
            
            code2nack  = code2nack + '4Z1-'    
        try:                
            message2print = "{(Step 4C/70) using" + "; publisherchannel2copyimagetobuffermanoeuvre4upu-calling-by-manoeuvre2copyimagetobuffer4upu: " + str(publisherchannel2copyimagetobuffermanoeuvre4upu) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)    
            # <copy-write PipeConnection, handle 412>
            message2print = "{{(Step 4D/70) passing" + "; message2copyimagetobuffersend4upu-passing-to-send-by-manoeuvre2copyimagetobuffer4upu: " + str(message2copyimagetobuffersend4upu) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)
            #              
        except:
            message2print = "{(Step 4Z1/33) failed" + "; message2copyimagetobuffersend4upu-failed-by-manoeuvre2copyimagetobuffer4upu: " + str(message2copyimagetobuffersend4upu) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)             
            code2nack  = code2nack + '4Z2-'                
        try:    
            publisherchannel2copyimagetobuffermanoeuvre4upu.send(message2copyimagetobuffersend4upu)
            
            message2print = "{(Step 4E/33) sent" + "; message2copyimagetobuffersend4upu-sent-by-manoeuvre2copyimagetobuffer4upu: " + str(message2copyimagetobuffersend4upu)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)          
            # <ack4xi>ack2imageinfo4upu</ack4xi>            
            code2ack  = code2ack + '4E-'
        except:
            message2print = "{(Step 4Z3/33) failed" + "; message2copyimagetobuffersend4upu-failed-by-manoeuvre2copyimagetobuffer4upu: " + str(message2copyimagetobuffersend4upu) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)             
            code2nack  = code2nack + '4Z3-'                
    else:
        pass
        

            
    # ------------------------------------------------------------------------------------------------------------------------
    # RETURN 
    # ------------------------------------------------------------------------------------------------------------------------
    response2manouvre4copyimagetobuffer['ACK2INFO4UPU']        = ack2copyimagetobufferinfo4upu
    response2manouvre4copyimagetobuffer['CODE2ACK4UPU']        = code2ack
    response2manouvre4copyimagetobuffer['CODE2NACK4UPU']       = code2nack
    response2manouvre4copyimagetobuffer['MESSAGE2GET4UPU']     = message2copyimagetobufferget4upu
    response2manouvre4copyimagetobuffer['MESSAGE2PUT4UPU']     = message2copyimagetobufferput4upu
    response2manouvre4copyimagetobuffer['MESSAGE2RECEIVE4UPU'] = message2copyimagetobufferreceive4upu
    response2manouvre4copyimagetobuffer['MESSAGE2SEND4UPU']    = message2copyimagetobuffersend4upu
    response2manouvre4copyimagetobuffer['STATE2MANOEUVRE4UPU'] = state2copyimagetobuffer4upu
                          
    message2print = "{(Step 33/33) returning" + "; response2manouvre4copyimagetobuffer-returning-by-manoeuvre2copyimagetobuffer4upu: " + str(response2manouvre4copyimagetobuffer) + "; time: " + time4xi() + "}" 
    print4xi(class2print, method2print, message2print)                                                             
    #1 {'MESSAGE2SEND4UPU': '<roger4xi>roger2copyimagetobuffer4upu</roger4xi>',
    #   'ACK2INFO4UPU': False,
    #   'MESSAGE2GET4UPU': '<default4xi>default2copyimagetobuffer4upu</default4xi>',
    #   'MESSAGE2RECEIVE4UPU': '<copy4xi>copy2copyimagetobuffer4upu</copy4xi>',
    #   'CODE2ACK4UPU': ' 0D-1C-',
    #   'MESSAGE2PUT4UPU': '<default4xi>default2copyimagetobuffer4upu</default4xi>',
    #   'CODE2NACK4UPU': ' ',
    #   'STATE2MANOEUVRE4UPU': 1}; time: 02:00:20 10/04/11 Pacific Daylight Time}

    
    #2 {'MESSAGE2SEND4UPU': '<ready4xi>ready2copyimagetobuffer4upu</ready4xi>',
    #   'ACK2INFO4UPU': False,
    #   'MESSAGE2GET4UPU': '<default4xi>default2copyimagetobuffer4upu</default4xi>',
    #   'MESSAGE2RECEIVE4UPU': '<ready4xi>ready2copyimagetobuffer4upu</ready4xi>',
    #   'CODE2ACK4UPU': ' 0D-2C-',
    #   'MESSAGE2PUT4UPU': '<default4xi>default2copyimagetobuffer4upu</default4xi>',
    #   'CODE2NACK4UPU': ' ',
    #   'STATE2MANOEUVRE4UPU': 2}; time: 02:10:33 10/04/11 Pacific Daylight Time}


    #3 {'MESSAGE2SEND4UPU': '<ack4xi>ack2imageinfo4upu</ack4xi>',
    #   'ACK2INFO4UPU': True,
    #   'MESSAGE2GET4UPU': '<default4xi>default2copyimagetobuffer4upu</default4xi>',
    #   'MESSAGE2RECEIVE4UPU': '<ready4xi>ready2copyimagetobuffer4upu</ready4xi>',
    #   'CODE2ACK4UPU': ' 0D-3C-4E-',
    #   'MESSAGE2PUT4UPU': '<default4xi>default2copyimagetobuffer4upu</default4xi>',
    #   'CODE2NACK4UPU': ' ',
    #   'STATE2MANOEUVRE4UPU': 3}; time: 02:10:33 10/04/11 Pacific Daylight Time}


    #4 {'MESSAGE2SEND4UPU': '<default4xi>default2copyimagetobuffer4upu</default4xi>',
    #   'ACK2INFO4UPU': False,
    #   'MESSAGE2GET4UPU': '<default4xi>default2copyimagetobuffer4upu</default4xi>',
    #   'MESSAGE2RECEIVE4UPU': '<default4xi>default2copyimagetobuffer4upu</default4xi>',
    #   'CODE2ACK4UPU': ' 0D-',
    #   'MESSAGE2PUT4UPU': '<default4xi>default2copyimagetobuffer4upu</default4xi>',
    #   'CODE2NACK4UPU': ' ',
    #   'STATE2MANOEUVRE4UPU': 0}; time: 02:10:33 10/04/11 Pacific Daylight Time}


    return response2manouvre4copyimagetobuffer           
          
#</function4xi>

# -----------------------------------------------------------------------------------------------------------------------
#<function4xi function = "manoeuvre2copybuffertoimage4upu" usage = "
#   # ---------------------------------------------------------------------------------------------------------------------
#   # SIMULATE MANOEUVRE2COPYIMAGE4UPU - LET IT TO RECEIVE COPY AND TO SEND ROGER VIA PIPE                
#   # ---------------------------------------------------------------------------------------------------------------------       
#   if ack2simulate4upu is True:
#       print("(Step 82A/700) OF UPU2IMAGE4XI5TEST - TRYING TO PERFORM TARGET FUNCTION MANOEUVRE2COPYIMAGE4UPU ", time4xi())   
#       try:        
#           response2manoeuvre4copy = manoeuvre2copybuffertoimage4upu(queue4copybuffertoimage4upu, channel4copybuffertoimageconsume4upu, channel4copybuffertoimagepublish4upu)
#           print("(Step 82B/700) OF UPU2IMAGE4XI5TEST - PERFORMED MANOEUVRE2COPYIMAGE4UPU, RESPONSE2MANOEUVRE4COPY: ", response2manoeuvre4copy)
#           # {'MESSAGE2SEND4UPU': '<roger4xi>roger2copybuffertoimage4upu</roger4xi>',
#           #   'ACK2INFO4UPU': False,
#           #   'MESSAGE2GET4UPU': '<default4xi>default2copybuffertoimage4upu</default4xi>',
#           #   'MESSAGE2RECEIVE4UPU': '<copy4xi>copy2copybuffertoimage4upu</copy4xi>',
#           #   'CODE2ACK4UPU': ' 0D-1C-',
#           #   'MESSAGE2PUT4UPU': '<default4xi>default2copybuffertoimage4upu</default4xi>',
#           #   'CODE2NACK4UPU': ' ',
#           #   'STATE2MANOEUVRE4UPU': 1}
#       except:
#           print("(Step 82Z/700) OF UPU2IMAGE4XI5TEST *** FAILED TO PERFORM TARGET FUNCTION MANOEUVRE2COPYIMAGE4UPU ", time4xi())     
#   else:
#       pass # trust activated thread    
#">

# ----------------------------------------------------------------------------------------------------------------------- 
def manoeuvre2copybuffertoimage4upu(queue2copybuffertoimagemanoeuvre4upu, consumerchannel2copybuffertoimagemanoeuvre4upu, publisherchannel2copybuffertoimagemanoeuvre4upu):
    '''
    manoeuvres copy control and command for copy2buffertoimage4upu
    '''
    global state2copybuffertoimage4upu                                                                                           # STATE FOR COMMAND AND CONTROL
    global max2copybuffertoimagestate4upu                                                                                        # MAX FOR STATE
    global ack2copybuffertoimageinfo4upu                                                                                         # ACK FOR IMAGE INFO
    global message2copybuffertoimageget4upu                                 
    global message2copybuffertoimageput4upu
    global message2copybuffertoimagereceive4upu
    global message2copybuffertoimagesend4upu
    
    # -----------------------------------------------------------------------------------------
    # (Step 0/33) PARAMETERS            
    # -----------------------------------------------------------------------------------------          
    class2print  = "manoeuvre2copybuffertoimage4upu" 
    method2print = "manoeuvre2copybuffertoimage4upu" 

    code2ack  = ' '
    code2nack = ' '
    
    message2print = "{(Step 0A/33) started" + "; manoeuvre2copybuffertoimage4upu-started: " + str(manoeuvre2copybuffertoimage4upu)  + "; time: " + time4xi() + "}" 
    print4xi(class2print, method2print, message2print)                                                                     #  
    # 
    
    message2print = "{(Step 0B/33) passed" + "; queue2copybuffertoimagemanoeuvre4upu-to-manoeuvre2copybuffertoimage4upu: " + str(queue2copybuffertoimagemanoeuvre4upu) + "; time: " + time4xi() + "}"
    print4xi(class2print, method2print, message2print)                                                              
    # 
    
    message2print = "{(Step 0C/33) passed" + "; consumerchannel2copybuffertoimagemanoeuvre4upu-to-manoeuvre2copybuffertoimage4upu: " + str(consumerchannel2copybuffertoimagemanoeuvre4upu) + "; time: " + time4xi() + "}"
    print4xi(class2print, method2print, message2print)                                                              
    # 
    
    message2print = "{(Step 0D/33) passed" + "; publisherchannel2copybuffertoimagemanoeuvre4upu-to-manoeuvre2copybuffertoimage4upu: " + str(publisherchannel2copybuffertoimagemanoeuvre4upu) + "; time: " + time4xi() + "}"
    print4xi(class2print, method2print, message2print)                                                              
    # 
    
    code2ack  = code2ack + '0D-' 
    response2manouvre4copybuffertoimage = {}
                    
    # ------------------------------------------------------------------------------------------------------------------------
    # SETTINGS
    # ------------------------------------------------------------------------------------------------------------------------    
    message2print = "{(Step 0E/70) try" + "; state2copybuffertoimage4upu-try-by-manoeuvre2copybuffertoimage4upu"  + "; time: " + time4xi() + "}" 
    print4xi(class2print, method2print, message2print)     
    try:
        if state2copybuffertoimage4upu is None:
            state2copybuffertoimage4upu = 0
            message2print = "{(Step 0F/33) initialized" + "; state2copybuffertoimage4upu-initialized-by-manoeuvre2copybuffertoimage4upu to: " + str(state2copybuffertoimage4upu)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)    
        else:
            message2print = "{(Step 0G/33) used" + "; state2copybuffertoimage4upu-used-by-manoeuvre2copybuffertoimage4upu: " + str(state2copybuffertoimage4upu)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print) 
            # 1; 2; 3;  
    except:
        state2copybuffertoimage4upu = 0
        message2print = "{(Step 0Z1/33) initialized" + "; state2copybuffertoimage4upu-initialized-by-manoeuvre2copybuffertoimage4upu to: " + str(state2copybuffertoimage4upu)  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)
        # 0
        ack2copybuffertoimageinfo4upu        = False
        message2print = "{(Step 0Z2/33) initialized" + "; ack2copybuffertoimageinfo4upu-initialized-by-manoeuvre2copybuffertoimage4upu to: " + str(ack2copybuffertoimageinfo4upu)  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)                                                     
        # False
        message2copybuffertoimageget4upu     = '<default4xi>default2copybuffertoimage4upu</default4xi>'
        message2print = "{(Step 0Z3/33) initialized" + "; message2copybuffertoimageget4upu-initialized-by-manoeuvre2copybuffertoimage4upu to: " + str(message2copybuffertoimageget4upu)  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)                                               
        # 
        message2copybuffertoimageput4upu     = '<default4xi>default2copybuffertoimage4upu</default4xi>'
        message2print = "{(Step 0Z4/33) initialized" + "; message2copybuffertoimageput4upu-initialized-by-manoeuvre2copybuffertoimage4upu to: " + str(message2copybuffertoimageput4upu)  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)              
        # 
        message2copybuffertoimagereceive4upu = '<default4xi>default2copybuffertoimage4upu</default4xi>'
        message2print = "{(Step 0Z5/33) initialized" + "; message2copybuffertoimagereceive4upu-initialized-by-manoeuvre2copybuffertoimage4upu to: " + str(message2copybuffertoimagereceive4upu)  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)              
        # 
        message2copybuffertoimagesend4upu    = '<default4xi>default2copybuffertoimage4upu</default4xi>'
        message2print = "{(Step 0Z6/33) initialized" + "; message2copybuffertoimagesend4upu-initialized-by-manoeuvre2copybuffertoimage4upu to: " + str(message2copybuffertoimagesend4upu)  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)              
        # 
        max2copybuffertoimagestate4upu = 3
        message2print = "{(Step 0Z7/33) initialized" + ";  max2copybuffertoimagestate4upu-initialized-by-manoeuvre2copybuffertoimage4upu to: " + str( max2copybuffertoimagestate4upu)  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)  
        # 
                    
    # ------------------------------------------------------------------------------------------------------------------------
    # STATE UPDATE
    # ------------------------------------------------------------------------------------------------------------------------                      
    state2copybuffertoimage4upu = state2copybuffertoimage4upu + 1                                                                   
    message2print = "{(Step 0H/33) added" + "; state2copybuffertoimage4upu-added-by-manoeuvre2copybuffertoimage4upu: " + str(state2copybuffertoimage4upu) + "; time: " + time4xi() + "}"
    print4xi(class2print, method2print, message2print)
    # 1; 2; 3; 4 
     
    if state2copybuffertoimage4upu > max2copybuffertoimagestate4upu:
        state2copybuffertoimage4upu = 0
        message2print = "{(Step 0I/33) set" + "; state2copybuffertoimage4upu-set-by-manoeuvre2copybuffertoimage4upu to: " + str(state2copybuffertoimage4upu)  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)
        # 
        ack2copybuffertoimageinfo4upu        = False
        message2print = "{(Step 0J/33) set" + "; ack2copybuffertoimageinfo4upu-set-by-manoeuvre2copybuffertoimage4upu to: " + str(ack2copybuffertoimageinfo4upu)  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)                                                     
        # 
        message2copybuffertoimageget4upu     = '<default4xi>default2copybuffertoimage4upu</default4xi>'
        message2print = "{(Step 0K/33) set" + "; message2copybuffertoimageget4upu-set-by-manoeuvre2copybuffertoimage4upu to: " + str(message2copybuffertoimageget4upu)  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)                                               
        # 
        message2copybuffertoimageput4upu     = '<default4xi>default2copybuffertoimage4upu</default4xi>'
        message2print = "{(Step 0L/33) set" + "; message2copybuffertoimageput4upu-set-by-manoeuvre2copybuffertoimage4upu to: " + str(message2copybuffertoimageput4upu)  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)              
        # 
        message2copybuffertoimagereceive4upu = '<default4xi>default2copybuffertoimage4upu</default4xi>'
        message2print = "{(Step 0M/33) set" + "; message2copybuffertoimagereceive4upu-set-by-manoeuvre2copybuffertoimage4upu to: " + str(message2copybuffertoimagereceive4upu)  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)              
        # 
        message2copybuffertoimagesend4upu    = '<default4xi>default2copybuffertoimage4upu</default4xi>'
        message2print = "{(Step 0N/33) set" + "; message2copybuffertoimagesend4upu-set-by-manoeuvre2copybuffertoimage4upu to: " + str(message2copybuffertoimagesend4upu)  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)              
        # 
        max2copybuffertoimagestate4upu = 3
        message2print = "{(Step 0O/33) initialized" + ";  max2copybuffertoimagestate4upu-set-by-manoeuvre2copybuffertoimage4upu to: " + str( max2copybuffertoimagestate4upu)  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)  
        #     
                    
    # ------------------------------------------------------------------------------------------------------------------------
    # STATE I - RECEIVE CONTROL AND COMMAND COPY MESSAGE VIA PIPE AND SEND ROGER OR NEGATIVE BACK 
    # ------------------------------------------------------------------------------------------------------------------------
    if state2copybuffertoimage4upu == 1:
        message2print = "{(Step 1A/70) try" + "; receive-try-by-manoeuvre2copybuffertoimage4upu"  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)
        message2copybuffertoimagereceive4upu = ''
        try:
            # ----------------------------------------------------------------------------------------------------------------
            # RECEIVE CONTROL AND COMMAND MESSAGE VIA PIPE
            # ----------------------------------------------------------------------------------------------------------------         
            message2copybuffertoimagereceive4upu = consumerchannel2copybuffertoimagemanoeuvre4upu.recv()
            message2print = "{(Step 1B/33) received" + "; message2copybuffertoimagereceive4upu-received-by-manoeuvre2copybuffertoimage4upu: " + str(message2copybuffertoimagereceive4upu) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)  
            # 
                         
            if message2copybuffertoimagereceive4upu == '<copy4xi>copy2copybuffertoimage4upu</copy4xi>':
                message2copybuffertoimagesend4upu = '<roger4xi>roger2copybuffertoimage4upu</roger4xi>'
            else:
                message2copybuffertoimagesend4upu = '<negative4xi><negative2copybuffertoimage4upu</<negative4xi>'
            # ----------------------------------------------------------------------------------------------------------------
            # SEND ROGER OR NEGATIVE BACK 
            # ----------------------------------------------------------------------------------------------------------------                                
            publisherchannel2copybuffertoimagemanoeuvre4upu.send(message2copybuffertoimagesend4upu)
            
            message2print = "{(Step 1C/33) sent" + "; message2copybuffertoimagesend4upu-sent-by-manoeuvre2copybuffertoimage4upu: " + str(message2copybuffertoimagesend4upu)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)          
            # 
            
            code2ack  = code2ack + '1C-'
        except:
            message2print = "{(Step 1Z/33) failed" + "; message2copybuffertoimagereceive4upu-failed-by-manoeuvre2copybuffertoimage4upu: " + str(message2copybuffertoimagereceive4upu) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)             
            code2nack  = code2nack + '1Z-' 
    else:
        pass

                    
    # ------------------------------------------------------------------------------------------------------------------------
    # STATE II - RECEIVE CONTROL AND COMMAND READY MESSAGE VIA PIPE AND SEND READY OR NEGATIVE BACK 
    # ------------------------------------------------------------------------------------------------------------------------
    if state2copybuffertoimage4upu == 2:
        message2print = "{(Step 2A/70) try" + "; receive-try-by-manoeuvre2copybuffertoimage4upu"  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)
        message2copybuffertoimagereceive4upu = ''
        try:
            # ----------------------------------------------------------------------------------------------------------------
            # RECEIVE CONTROL AND COMMAND MESSAGE VIA PIPE
            # ----------------------------------------------------------------------------------------------------------------         
            message2copybuffertoimagereceive4upu = consumerchannel2copybuffertoimagemanoeuvre4upu.recv()
            message2print = "{(Step 2B/33) received" + "; message2copybuffertoimagereceive4upu-received-by-manoeuvre2copybuffertoimage4upu: " + str(message2copybuffertoimagereceive4upu) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)  
            # 
                         
            if message2copybuffertoimagereceive4upu == '<ready4xi>ready2copybuffertoimage4upu</ready4xi>':
                message2copybuffertoimagesend4upu = '<ready4xi>ready2copybuffertoimage4upu</ready4xi>'
            else:
                message2copybuffertoimagesend4upu = '<negative4xi><negative2copybuffertoimage4upu</<negative4xi>'
            # ----------------------------------------------------------------------------------------------------------------
            # SEND READY OR NEGATIVE BACK 
            # ----------------------------------------------------------------------------------------------------------------                                
            publisherchannel2copybuffertoimagemanoeuvre4upu.send(message2copybuffertoimagesend4upu)
            
            message2print = "{(Step 2C/33) sent" + "; message2copybuffertoimagesend4upu-sent-by-manoeuvre2copybuffertoimage4upu: " + str(message2copybuffertoimagesend4upu)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)          
            # 
            
            code2ack  = code2ack + '2C-'
        except:
            message2print = "{(Step 2Z/33) failed" + "; message2copybuffertoimagereceive4upu-failed-by-manoeuvre2copybuffertoimage4upu: " + str(message2copybuffertoimagereceive4upu) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)             
            code2nack  = code2nack + '2Z-' 
    else:
        pass        
                
    # ------------------------------------------------------------------------------------------------------------------------
    # STATE III - GET IMAGE INFORMATION SUCH AS IMAGE PATH VIA QUEUE AND SEND ACK OR NACK VIA PIPE
    # ------------------------------------------------------------------------------------------------------------------------
    if state2copybuffertoimage4upu == 3:    
        info2image4upu = None
        message2print = "{(Step 3A/70) try" + "; info2image4upu-try-by-manoeuvre2copybuffertoimage4upu"  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)
        try: 
            # ----------------------------------------------------------------------------------------------------------------
            # GET IMAGE INFORMATION SUCH AS IMAGE PATH VIA QUEUE
            # ----------------------------------------------------------------------------------------------------------------        
            info2image4upu = queue2copybuffertoimagemanoeuvre4upu.get()
            
            message2print = "{(Step 3B/33) got" + "; info2image4upu-got-by-manoeuvre2copybuffertoimage4upu: " + str(info2image4upu) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)  
            # ..//..//..//..//..//web//images//png//dragonxi.png
            if info2image4upu is None:
                code2nack  = code2nack + '3Z1-'
                ack2copybuffertoimageinfo4upu = False 
            else:    
                code2ack  = code2ack + '3C-'
                ack2copybuffertoimageinfo4upu = True
        except:
            message2print = "{(Step 3Z1/33) failed" + "; queue2copybuffertoimagemanoeuvre4upu-failed-by-manoeuvre2copybuffertoimage4upu: " + str(queue2copybuffertoimagemanoeuvre4upu) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)             
            code2nack  = code2nack + '3Z1-'
            
        message2copybuffertoimagesend4upu = ' '
        try:
            # ----------------------------------------------------------------------------------------------------------------
            # SEND CONTROL INFO ABOUT IMAGE OPERATION - EITHER ACK OR NACK  
            # ----------------------------------------------------------------------------------------------------------------         
            if ack2copybuffertoimageinfo4upu is True:
                message2copybuffertoimagesend4upu = '<ack4xi>ack2imageinfo4upu</ack4xi>'
            else:
                message2copybuffertoimagesend4upu = '<nack4xi>nack2imageinfo4upu</nack4xi>'
            message2print = "{(Step 4B/33) set" + "; message2copybuffertoimagesend4upu-set-by-manoeuvre2copybuffertoimage4upu to: " + str(message2copybuffertoimagesend4upu)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print) 
            # <ack4xi>ack2imageinfo4upu</ack4xi>       
        except:
            message2print = "{(Step 4Z1/33) failed" + "; message2copybuffertoimagesend4upu-failed-by-manoeuvre2copybuffertoimage4upu: " + str(message2copybuffertoimagesend4upu) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)             
            
            code2nack  = code2nack + '4Z1-'    
        try:                
            message2print = "{(Step 4C/70) using" + "; publisherchannel2copybuffertoimagemanoeuvre4upu-calling-by-manoeuvre2copybuffertoimage4upu: " + str(publisherchannel2copybuffertoimagemanoeuvre4upu) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)    
            # <copy-write PipeConnection, handle 412>
            message2print = "{{(Step 4D/70) passing" + "; message2copybuffertoimagesend4upu-passing-to-send-by-manoeuvre2copybuffertoimage4upu: " + str(message2copybuffertoimagesend4upu) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)
            #              
        except:
            message2print = "{(Step 4Z1/33) failed" + "; message2copybuffertoimagesend4upu-failed-by-manoeuvre2copybuffertoimage4upu: " + str(message2copybuffertoimagesend4upu) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)             
            code2nack  = code2nack + '4Z2-'                
        try:    
            publisherchannel2copybuffertoimagemanoeuvre4upu.send(message2copybuffertoimagesend4upu)
            
            message2print = "{(Step 4E/33) sent" + "; message2copybuffertoimagesend4upu-sent-by-manoeuvre2copybuffertoimage4upu: " + str(message2copybuffertoimagesend4upu)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)          
            # <ack4xi>ack2imageinfo4upu</ack4xi>            
            code2ack  = code2ack + '4E-'
        except:
            message2print = "{(Step 4Z3/33) failed" + "; message2copybuffertoimagesend4upu-failed-by-manoeuvre2copybuffertoimage4upu: " + str(message2copybuffertoimagesend4upu) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)             
            code2nack  = code2nack + '4Z3-'                
    else:
        pass
        

            
    # ------------------------------------------------------------------------------------------------------------------------
    # RETURN 
    # ------------------------------------------------------------------------------------------------------------------------
    response2manouvre4copybuffertoimage['ACK2INFO4UPU']        = ack2copybuffertoimageinfo4upu
    response2manouvre4copybuffertoimage['CODE2ACK4UPU']        = code2ack
    response2manouvre4copybuffertoimage['CODE2NACK4UPU']       = code2nack
    response2manouvre4copybuffertoimage['MESSAGE2GET4UPU']     = message2copybuffertoimageget4upu
    response2manouvre4copybuffertoimage['MESSAGE2PUT4UPU']     = message2copybuffertoimageput4upu
    response2manouvre4copybuffertoimage['MESSAGE2RECEIVE4UPU'] = message2copybuffertoimagereceive4upu
    response2manouvre4copybuffertoimage['MESSAGE2SEND4UPU']    = message2copybuffertoimagesend4upu
    response2manouvre4copybuffertoimage['STATE2MANOEUVRE4UPU'] = state2copybuffertoimage4upu
                          
    message2print = "{(Step 33/33) returning" + "; response2manouvre4copybuffertoimage-returning-by-manoeuvre2copybuffertoimage4upu: " + str(response2manouvre4copybuffertoimage) + "; time: " + time4xi() + "}" 
    print4xi(class2print, method2print, message2print)                                                             
    #1 {'MESSAGE2SEND4UPU': '<roger4xi>roger2copybuffertoimage4upu</roger4xi>',
    #   'ACK2INFO4UPU': False,
    #   'MESSAGE2GET4UPU': '<default4xi>default2copybuffertoimage4upu</default4xi>',
    #   'MESSAGE2RECEIVE4UPU': '<copy4xi>copy2copybuffertoimage4upu</copy4xi>',
    #   'CODE2ACK4UPU': ' 0D-1C-',
    #   'MESSAGE2PUT4UPU': '<default4xi>default2copybuffertoimage4upu</default4xi>',
    #   'CODE2NACK4UPU': ' ',
    #   'STATE2MANOEUVRE4UPU': 1}; time: 02:18:55 10/04/11 Pacific Daylight Time}

    
    #2 {'MESSAGE2SEND4UPU': '<ready4xi>ready2copybuffertoimage4upu</ready4xi>',
    #   'ACK2INFO4UPU': False,
    #   'MESSAGE2GET4UPU': '<default4xi>default2copybuffertoimage4upu</default4xi>',
    #   'MESSAGE2RECEIVE4UPU': '<ready4xi>ready2copybuffertoimage4upu</ready4xi>',
    #   'CODE2ACK4UPU': ' 0D-2C-',
    #   'MESSAGE2PUT4UPU': '<default4xi>default2copybuffertoimage4upu</default4xi>',
    #   'CODE2NACK4UPU': ' ',
    #   'STATE2MANOEUVRE4UPU': 2}; time: 02:18:55 10/04/11 Pacific Daylight Time}

    #3 {'MESSAGE2SEND4UPU': '<ack4xi>ack2imageinfo4upu</ack4xi>',
    #   'ACK2INFO4UPU': True,
    #   'MESSAGE2GET4UPU': '<default4xi>default2copybuffertoimage4upu</default4xi>',
    #   'MESSAGE2RECEIVE4UPU': '<ready4xi>ready2copybuffertoimage4upu</ready4xi>',
    #   'CODE2ACK4UPU': ' 0D-3C-4E-',
    #   'MESSAGE2PUT4UPU': '<default4xi>default2copybuffertoimage4upu</default4xi>',
    #   'CODE2NACK4UPU': ' ',
    #   'STATE2MANOEUVRE4UPU': 3}; time: 02:18:55 10/04/11 Pacific Daylight Time}
 

    #4 {'MESSAGE2SEND4UPU': '<default4xi>default2copybuffertoimage4upu</default4xi>',
    #   'ACK2INFO4UPU': False,
    #   'MESSAGE2GET4UPU': '<default4xi>default2copybuffertoimage4upu</default4xi>',
    #   'MESSAGE2RECEIVE4UPU': '<default4xi>default2copybuffertoimage4upu</default4xi>',
    #   'CODE2ACK4UPU': ' 0D-',
    #   'MESSAGE2PUT4UPU': '<default4xi>default2copybuffertoimage4upu</default4xi>',
    #   'CODE2NACK4UPU': ' ',
    #   'STATE2MANOEUVRE4UPU': 0}

    return response2manouvre4copybuffertoimage           
          
#</function4xi>



# -----------------------------------------------------------------------------------------------------------------------
#<function4xi function = "manoeuvre2readimage4upu" usage = "control function for read2image4upu function in this module">

# ----------------------------------------------------------------------------------------------------------------------- 
def manoeuvre2readimage4upu(queue2readmanoeuvre4upu, consumerchannel2readmanoeuvre4upu, publisherchannel2readmanoeuvre4upu):
    '''
    manoeuvres read control and command
    '''
    global state2read4upu                                                                                           # STATE FOR COMMAND AND CONTROL
    global max2readstate4upu                                                                                        # MAX FOR STATE
    global ack2readinfo4upu                                                                                         # ACK FOR IMAGE INFO
    global message2readget4upu                                 
    global message2readput4upu
    global message2readreceive4upu
    global message2readsend4upu
    
    # -----------------------------------------------------------------------------------------
    # (Step 0/33) PARAMETERS            
    # -----------------------------------------------------------------------------------------          
    class2print  = "manoeuvre2readimage4upu" 
    method2print = "manoeuvre2readimage4upu" 

    code2ack  = ' '
    code2nack = ' '
    
    message2print = "{(Step 0A/33) started" + "; manoeuvre2readimage4upu-started: " + str(manoeuvre2readimage4upu)  + "; time: " + time4xi() + "}" 
    print4xi(class2print, method2print, message2print)                                                                     #  
    # <function manoeuvre2readimage4upu at 0x0000000003DB8C48>  
    
    message2print = "{(Step 0B/33) passed" + "; queue2readmanoeuvre4upu-to-manoeuvre2readimage4upu: " + str(queue2readmanoeuvre4upu) + "; time: " + time4xi() + "}"
    print4xi(class2print, method2print, message2print)                                                              
    # <queue4xi.Queue4Xi object at 0x0000000003AD9400>
    
    message2print = "{(Step 0C/33) passed" + "; consumerchannel2readmanoeuvre4upu-to-manoeuvre2readimage4upu: " + str(consumerchannel2readmanoeuvre4upu) + "; time: " + time4xi() + "}"
    print4xi(class2print, method2print, message2print)                                                              
    # <read-write PipeConnection, handle 408>
    
    message2print = "{(Step 0D/33) passed" + "; publisherchannel2readmanoeuvre4upu-to-manoeuvre2readimage4upu: " + str(publisherchannel2readmanoeuvre4upu) + "; time: " + time4xi() + "}"
    print4xi(class2print, method2print, message2print)                                                              
    # <read-write PipeConnection, handle 412>     
    
    code2ack  = code2ack + '0D-' 
    response2manouvre4read = {}
                    
    # ------------------------------------------------------------------------------------------------------------------------
    # SETTINGS
    # ------------------------------------------------------------------------------------------------------------------------    
    message2print = "{(Step 0E/70) try" + "; state2read4upu-try-by-manoeuvre2readimage4upu"  + "; time: " + time4xi() + "}" 
    print4xi(class2print, method2print, message2print)     
    try:
        if state2read4upu is None:
            state2read4upu = 0
            message2print = "{(Step 0F/33) initialized" + "; state2read4upu-initialized-by-manoeuvre2readimage4upu to: " + str(state2read4upu)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)    
        else:
            message2print = "{(Step 0G/33) used" + "; state2read4upu-used-by-manoeuvre2readimage4upu: " + str(state2read4upu)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print) 
            # 1; 2; 3;  
    except:
        state2read4upu = 0
        message2print = "{(Step 0Z1/33) initialized" + "; state2read4upu-initialized-by-manoeuvre2readimage4upu to: " + str(state2read4upu)  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)
        # 0
        ack2readinfo4upu        = False
        message2print = "{(Step 0Z2/33) initialized" + "; ack2readinfo4upu-initialized-by-manoeuvre2readimage4upu to: " + str(ack2readinfo4upu)  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)                                                     
        # False
        message2readget4upu     = '<default4xi>default2read4upu</default4xi>'
        message2print = "{(Step 0Z3/33) initialized" + "; message2readget4upu-initialized-by-manoeuvre2readimage4upu to: " + str(message2readget4upu)  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)                                               
        # <default4xi>default2read4upu</default4xi>
        message2readput4upu     = '<default4xi>default2read4upu</default4xi>'
        message2print = "{(Step 0Z4/33) initialized" + "; message2readput4upu-initialized-by-manoeuvre2readimage4upu to: " + str(message2readput4upu)  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)              
        # <default4xi>default2read4upu</default4xi>
        message2readreceive4upu = '<default4xi>default2read4upu</default4xi>'
        message2print = "{(Step 0Z5/33) initialized" + "; message2readreceive4upu-initialized-by-manoeuvre2readimage4upu to: " + str(message2readreceive4upu)  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)              
        # <default4xi>default2read4upu</default4xi>
        message2readsend4upu    = '<default4xi>default2read4upu</default4xi>'
        message2print = "{(Step 0Z6/33) initialized" + "; message2readsend4upu-initialized-by-manoeuvre2readimage4upu to: " + str(message2readsend4upu)  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)              
        # <default4xi>default2read4upu</default4xi>
        max2readstate4upu = 3
        message2print = "{(Step 0Z7/33) initialized" + ";  max2readstate4upu-initialized-by-manoeuvre2readimage4upu to: " + str( max2readstate4upu)  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)  
        # 
                    
    # ------------------------------------------------------------------------------------------------------------------------
    # STATE UPDATE
    # ------------------------------------------------------------------------------------------------------------------------                      
    state2read4upu = state2read4upu + 1                                                                   
    message2print = "{(Step 0H/33) added" + "; state2read4upu-added-by-manoeuvre2readimage4upu: " + str(state2read4upu) + "; time: " + time4xi() + "}"
    print4xi(class2print, method2print, message2print)
    # 1; 2; 3; 4  
    if state2read4upu > max2readstate4upu:
        state2read4upu = 0
        message2print = "{(Step 0I/33) set" + "; state2read4upu-set-by-manoeuvre2readimage4upu to: " + str(state2read4upu)  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)
        # 0
        ack2readinfo4upu        = False
        message2print = "{(Step 0J/33) set" + "; ack2readinfo4upu-set-by-manoeuvre2readimage4upu to: " + str(ack2readinfo4upu)  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)                                                     
        # False
        message2readget4upu     = '<default4xi>default2read4upu</default4xi>'
        message2print = "{(Step 0K/33) set" + "; message2readget4upu-set-by-manoeuvre2readimage4upu to: " + str(message2readget4upu)  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)                                               
        # <default4xi>default2read4upu</default4xi>
        message2readput4upu     = '<default4xi>default2read4upu</default4xi>'
        message2print = "{(Step 0L/33) set" + "; message2readput4upu-set-by-manoeuvre2readimage4upu to: " + str(message2readput4upu)  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)              
        # <default4xi>default2read4upu</default4xi>
        message2readreceive4upu = '<default4xi>default2read4upu</default4xi>'
        message2print = "{(Step 0M/33) set" + "; message2readreceive4upu-set-by-manoeuvre2readimage4upu to: " + str(message2readreceive4upu)  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)              
        # <default4xi>default2read4upu</default4xi>
        message2readsend4upu    = '<default4xi>default2read4upu</default4xi>'
        message2print = "{(Step 0N/33) set" + "; message2readsend4upu-set-by-manoeuvre2readimage4upu to: " + str(message2readsend4upu)  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)              
        # <default4xi>default2read4upu</default4xi>
        max2readstate4upu = 3
        message2print = "{(Step 0O/33) initialized" + ";  max2readstate4upu-set-by-manoeuvre2readimage4upu to: " + str( max2readstate4upu)  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)  
        #     
                    
    # ------------------------------------------------------------------------------------------------------------------------
    # STATE I - RECEIVE CONTROL AND COMMAND COPY MESSAGE VIA PIPE AND SEND ROGER OR NEGATIVE BACK 
    # ------------------------------------------------------------------------------------------------------------------------
    if state2read4upu == 1:
        message2print = "{(Step 1A/70) try" + "; receive-try-by-manoeuvre2readimage4upu"  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)
        message2readreceive4upu = ''
        try:
            # ----------------------------------------------------------------------------------------------------------------
            # RECEIVE CONTROL AND COMMAND MESSAGE VIA PIPE
            # ----------------------------------------------------------------------------------------------------------------         
            message2readreceive4upu = consumerchannel2readmanoeuvre4upu.recv()
            message2print = "{(Step 1B/33) received" + "; message2readreceive4upu-received-by-manoeuvre2readimage4upu: " + str(message2readreceive4upu) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)  
            # <copy4xi>copy2read4upu</copy4xi>
                         
            if message2readreceive4upu == '<copy4xi>copy2read4upu</copy4xi>':
                message2readsend4upu = '<roger4xi>roger2read4upu</roger4xi>'
            else:
                message2readsend4upu = '<negative4xi><negative2read4upu</<negative4xi>'
            # ----------------------------------------------------------------------------------------------------------------
            # SEND ROGER OR NEGATIVE BACK 
            # ----------------------------------------------------------------------------------------------------------------                                
            publisherchannel2readmanoeuvre4upu.send(message2readsend4upu)
            
            message2print = "{(Step 1C/33) sent" + "; message2readsend4upu-sent-by-manoeuvre2readimage4upu: " + str(message2readsend4upu)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)          
            # <roger4xi>roger2read4upu</roger4xi>
            
            code2ack  = code2ack + '1C-'
        except:
            message2print = "{(Step 1Z/33) failed" + "; message2readreceive4upu-failed-by-manoeuvre2readimage4upu: " + str(message2readreceive4upu) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)             
            code2nack  = code2nack + '1Z-' 
    else:
        pass

                    
    # ------------------------------------------------------------------------------------------------------------------------
    # STATE II - RECEIVE CONTROL AND COMMAND READY MESSAGE VIA PIPE AND SEND READY OR NEGATIVE BACK 
    # ------------------------------------------------------------------------------------------------------------------------
    if state2read4upu == 2:
        message2print = "{(Step 2A/70) try" + "; receive-try-by-manoeuvre2readimage4upu"  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)
        message2readreceive4upu = ''
        try:
            # ----------------------------------------------------------------------------------------------------------------
            # RECEIVE CONTROL AND COMMAND MESSAGE VIA PIPE
            # ----------------------------------------------------------------------------------------------------------------         
            message2readreceive4upu = consumerchannel2readmanoeuvre4upu.recv()
            message2print = "{(Step 2B/33) received" + "; message2readreceive4upu-received-by-manoeuvre2readimage4upu: " + str(message2readreceive4upu) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)  
            # <ready4xi>ready2read4upu</ready4xi> 
                         
            if message2readreceive4upu == '<ready4xi>ready2read4upu</ready4xi>':
                message2readsend4upu = '<ready4xi>ready2read4upu</ready4xi>'
            else:
                message2readsend4upu = '<negative4xi><negative2ready4upu</<negative4xi>'
            # ----------------------------------------------------------------------------------------------------------------
            # SEND ROGER OR NEGATIVE BACK 
            # ----------------------------------------------------------------------------------------------------------------                                
            publisherchannel2readmanoeuvre4upu.send(message2readsend4upu)
            
            message2print = "{(Step 2C/33) sent" + "; message2readsend4upu-sent-by-manoeuvre2readimage4upu: " + str(message2readsend4upu)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)          
            # <ready4xi>ready2read4upu</ready4xi>
            
            code2ack  = code2ack + '2C-'
        except:
            message2print = "{(Step 2Z/33) failed" + "; message2readreceive4upu-failed-by-manoeuvre2readimage4upu: " + str(message2readreceive4upu) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)             
            code2nack  = code2nack + '2Z-' 
    else:
        pass        
                
    # ------------------------------------------------------------------------------------------------------------------------
    # STATE III - GET IMAGE INFORMATION SUCH AS IMAGE PATH VIA QUEUE AND SEND ACK OR NACK VIA PIPE
    # ------------------------------------------------------------------------------------------------------------------------
    if state2read4upu == 3:    
        info2image4upu = None
        message2print = "{(Step 3A/70) try" + "; info2image4upu-try-by-manoeuvre2readimage4upu"  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)
        try: 
            # ----------------------------------------------------------------------------------------------------------------
            # GET IMAGE INFORMATION SUCH AS IMAGE PATH VIA QUEUE
            # ----------------------------------------------------------------------------------------------------------------        
            info2image4upu = queue2readmanoeuvre4upu.get()
            
            message2print = "{(Step 3B/33) got" + "; info2image4upu-got-by-manoeuvre2readimage4upu: " + str(info2image4upu) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)  
            # ..//..//..//..//..//web//images//png//dragonxi.png
            if info2image4upu is None:
                code2nack  = code2nack + '3Z1-'
                ack2readinfo4upu = False 
            else:    
                code2ack  = code2ack + '3C-'
                ack2readinfo4upu = True
        except:
            message2print = "{(Step 3Z1/33) failed" + "; queue2readmanoeuvre4upu-failed-by-manoeuvre2readimage4upu: " + str(queue2readmanoeuvre4upu) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)             
            code2nack  = code2nack + '3Z1-'
            
        message2readsend4upu = ' '
        try:
            # ----------------------------------------------------------------------------------------------------------------
            # SEND CONTROL INFO ABOUT IMAGE OPERATION - EITHER ACK OR NACK  
            # ----------------------------------------------------------------------------------------------------------------         
            if ack2readinfo4upu is True:
                message2readsend4upu = '<ack4xi>ack2imageinfo4upu</ack4xi>'
            else:
                message2readsend4upu = '<nack4xi>nack2imageinfo4upu</nack4xi>'
            message2print = "{(Step 4B/33) set" + "; message2readsend4upu-set-by-manoeuvre2readimage4upu to: " + str(message2readsend4upu)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print) 
            # <ack4xi>ack2imageinfo4upu</ack4xi>       
        except:
            message2print = "{(Step 4Z1/33) failed" + "; message2readsend4upu-failed-by-manoeuvre2readimage4upu: " + str(message2readsend4upu) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)             
            
            code2nack  = code2nack + '4Z1-'    
        try:                
            message2print = "{(Step 4C/70) using" + "; publisherchannel2readmanoeuvre4upu-calling-by-manoeuvre2readimage4upu: " + str(publisherchannel2readmanoeuvre4upu) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)    
            # <read-write PipeConnection, handle 412>
            message2print = "{{(Step 4D/70) passing" + "; message2readsend4upu-passing-to-send-by-manoeuvre2readimage4upu: " + str(message2readsend4upu) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)
            #              
        except:
            message2print = "{(Step 4Z1/33) failed" + "; message2readsend4upu-failed-by-manoeuvre2readimage4upu: " + str(message2readsend4upu) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)             
            code2nack  = code2nack + '4Z2-'                
        try:    
            publisherchannel2readmanoeuvre4upu.send(message2readsend4upu)
            
            message2print = "{(Step 4E/33) sent" + "; message2readsend4upu-sent-by-manoeuvre2readimage4upu: " + str(message2readsend4upu)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)          
            # <ack4xi>ack2imageinfo4upu</ack4xi>            
            code2ack  = code2ack + '4E-'
        except:
            message2print = "{(Step 4Z3/33) failed" + "; message2readsend4upu-failed-by-manoeuvre2readimage4upu: " + str(message2readsend4upu) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)             
            code2nack  = code2nack + '4Z3-'                
    else:
        pass
        

            
    # ------------------------------------------------------------------------------------------------------------------------
    # RETURN 
    # ------------------------------------------------------------------------------------------------------------------------
    response2manouvre4read['ACK2INFO4UPU']        = ack2readinfo4upu
    response2manouvre4read['CODE2ACK4UPU']        = code2ack
    response2manouvre4read['CODE2NACK4UPU']       = code2nack
    response2manouvre4read['MESSAGE2GET4UPU']     = message2readget4upu
    response2manouvre4read['MESSAGE2PUT4UPU']     = message2readput4upu
    response2manouvre4read['MESSAGE2RECEIVE4UPU'] = message2readreceive4upu
    response2manouvre4read['MESSAGE2SEND4UPU']    = message2readsend4upu
    response2manouvre4read['STATE2MANOEUVRE4UPU'] = state2read4upu
                          
    message2print = "{(Step 33/33) returning" + "; response2manouvre4read-returning-by-manoeuvre2readimage4upu: " + str(response2manouvre4read) + "; time: " + time4xi() + "}" 
    print4xi(class2print, method2print, message2print)                                                             
    #1 {'MESSAGE2SEND4UPU': '<roger4xi>roger2read4upu</roger4xi>',
    #  'ACK2INFO4UPU': False,
    #  'MESSAGE2GET4UPU': '<default4xi>default2read4upu</default4xi>',
    #  'MESSAGE2RECEIVE4UPU': '<copy4xi>copy2read4upu</copy4xi>',
    #  'CODE2ACK4UPU': ' 0D-1C-',
    #  'MESSAGE2PUT4UPU': '<default4xi>default2read4upu</default4xi>',
    #  'CODE2NACK4UPU': ' ', 'STATE2MANOEUVRE4UPU': 1}; time: 01:44:40 09/29/11 Pacific Daylight Time}
 
    #2 {'MESSAGE2SEND4UPU': '<ready4xi>ready2read4upu</ready4xi>',
    #  'ACK2INFO4UPU': False,
    #  'MESSAGE2GET4UPU': '<default4xi>default2read4upu</default4xi>',
    #  'MESSAGE2RECEIVE4UPU': '<ready4xi>ready2read4upu</ready4xi>',
    #  'CODE2ACK4UPU': ' 0D-2C-',
    #  'MESSAGE2PUT4UPU': '<default4xi>default2read4upu</default4xi>',
    #  'CODE2NACK4UPU': ' ', 'STATE2MANOEUVRE4UPU': 2}; time: 01:44:40 09/29/11 Pacific Daylight Time}
    
    #3 {'MESSAGE2SEND4UPU': '<ack4xi>ack2imageinfo4upu</ack4xi>',
    #  'ACK2INFO4UPU': True,
    #  'MESSAGE2GET4UPU': '<default4xi>default2read4upu</default4xi>',
    #  'MESSAGE2RECEIVE4UPU': '<ready4xi>ready2read4upu</ready4xi>',
    #  'CODE2ACK4UPU': ' 0D-3C-4E-',
    #  'MESSAGE2PUT4UPU': '<default4xi>default2read4upu</default4xi>',
    #  'CODE2NACK4UPU': ' ',
    #  'STATE2MANOEUVRE4UPU': 3}; time: 01:44:40 09/29/11 Pacific Daylight Time}
    
    #4 {'MESSAGE2SEND4UPU': '<default4xi>default2read4upu</default4xi>',
    #  'ACK2INFO4UPU': False,
    #  'MESSAGE2GET4UPU': '<default4xi>default2read4upu</default4xi>',
    #  'MESSAGE2RECEIVE4UPU': '<default4xi>default2read4upu</default4xi>',
    #  'CODE2ACK4UPU': ' 0D-',
    #  'MESSAGE2PUT4UPU': '<default4xi>default2read4upu</default4xi>',
    #  'CODE2NACK4UPU': ' ',
    #  'STATE2MANOEUVRE4UPU': 0}; time: 02:11:49 09/29/11 Pacific Daylight Time}


 
    
    
    return response2manouvre4read           
          
#</function4xi>


# -------------------------------------------------------------------------------------------------------------------------
#<function4xi function = "manoeuvre2writeimage4upu" usage = "control function for write2image4upu function in this module">

# ------------------------------------------------------------------------------------------------------------------------- 
def manoeuvre2writeimage4upu(queue2writemanoeuvre4upu, consumerchannel2writemanoeuvre4upu, publisherchannel2writemanoeuvre4upu):
    '''
    manoeuvres control and command
    '''
    global state2write4upu                                                       # STATE FOR COMMAND AND CONTROL
    global max2writestate4upu                                                    # MAX FOR STATE
    global ack2writeinfo4upu                                                     # ACK FOR IMAGE INFO
    global message2writeget4upu                                 
    global message2writeput4upu
    global message2writereceive4upu
    global message2writesend4upu
    
    # -----------------------------------------------------------------------------------------
    # (Step 0/33) PARAMETERS            
    # -----------------------------------------------------------------------------------------          
    class2print  = "manoeuvre2writeimage4upu" 
    method2print = "manoeuvre2writeimage4upu" 

    code2ack  = ' '
    code2nack = ' '
    
    message2print = "{(Step 0A/33) started" + "; manoeuvre2writeimage4upu-started: " + str(manoeuvre2writeimage4upu)  + "; time: " + time4xi() + "}" 
    print4xi(class2print, method2print, message2print)                                                                      
    # <function manoeuvre2writeimage4upu at 0x0000000003DB8C48>  
    
    message2print = "{(Step 0B/33) passed" + "; queue2writemanoeuvre4upu-to-manoeuvre2writeimage4upu: " + str(queue2writemanoeuvre4upu) + "; time: " + time4xi() + "}"
    print4xi(class2print, method2print, message2print)                                                              
    # <queue4xi.Queue4Xi object at 0x000000000422D9E8>
    
    message2print = "{(Step 0C/33) passed" + "; consumerchannel2writemanoeuvre4upu-to-manoeuvre2writeimage4upu: " + str(consumerchannel2writemanoeuvre4upu) + "; time: " + time4xi() + "}"
    print4xi(class2print, method2print, message2print)                                                              
    # read-write PipeConnection, handle 568>
    
    message2print = "{(Step 0D/33) passed" + "; publisherchannel2writemanoeuvre4upu-to-manoeuvre2writeimage4upu: " + str(publisherchannel2writemanoeuvre4upu) + "; time: " + time4xi() + "}"
    print4xi(class2print, method2print, message2print)                                                              
    # <read-write PipeConnection, handle 572> 
    
    code2ack  = code2ack + '0D-' 
    response2manouvre4write = {}
                    
    # ------------------------------------------------------------------------------------------------------------------------
    # SETTINGS
    # ------------------------------------------------------------------------------------------------------------------------    
    message2print = "{(Step 0E/70) try" + "; state2write4upu-try-by-manoeuvre2writeimage4upu"  + "; time: " + time4xi() + "}" 
    print4xi(class2print, method2print, message2print)     
    try:
        if state2write4upu is None:
            state2write4upu = 0
            message2print = "{(Step 0F/33) initialized" + "; state2write4upu-initialized-by-manoeuvre2writeimage4upu to: " + str(state2write4upu)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)    
        else:
            message2print = "{(Step 0G/33) used" + "; state2write4upu-used-by-manoeuvre2writeimage4upu: " + str(state2write4upu)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print) 
            # 1; 2; 3;  
    except:
        state2write4upu = 0
        message2print = "{(Step 0Z1/33) initialized" + "; state2write4upu-initialized-by-manoeuvre2writeimage4upu to: " + str(state2write4upu)  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)
        # 0
        ack2writeinfo4upu        = False
        message2print = "{(Step 0Z2/33) initialized" + "; ack2writeinfo4upu-initialized-by-manoeuvre2writeimage4upu to: " + str(ack2writeinfo4upu)  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)                                                     
        # False
        message2writeget4upu     = '<default4xi>default2write4upu</default4xi>'
        message2print = "{(Step 0Z3/33) initialized" + "; message2writeget4upu-initialized-by-manoeuvre2writeimage4upu to: " + str(message2writeget4upu)  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)                                               
        # <default4xi>default2write4upu</default4xi>
        message2writeput4upu     = '<default4xi>default2write4upu</default4xi>'
        message2print = "{(Step 0Z4/33) initialized" + "; message2writeput4upu-initialized-by-manoeuvre2writeimage4upu to: " + str(message2writeput4upu)  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)              
        # <default4xi>default2write4upu</default4xi>
        message2writereceive4upu = '<default4xi>default2write4upu</default4xi>'
        message2print = "{(Step 0Z5/33) initialized" + "; message2writereceive4upu-initialized-by-manoeuvre2writeimage4upu to: " + str(message2writereceive4upu)  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)              
        # <default4xi>default2write4upu</default4xi>
        message2writesend4upu    = '<default4xi>default2write4upu</default4xi>'
        message2print = "{(Step 0Z6/33) initialized" + "; message2writesend4upu-initialized-by-manoeuvre2writeimage4upu to: " + str(message2writesend4upu)  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)              
        # <default4xi>default2write4upu</default4xi>
        max2writestate4upu = 3
        message2print = "{(Step 0Z7/33) initialized" + ";  max2writestate4upu-initialized-by-manoeuvre2writeimage4upu to: " + str( max2writestate4upu)  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)  
        # 
                    
    # ------------------------------------------------------------------------------------------------------------------------
    # STATE UPDATE
    # ------------------------------------------------------------------------------------------------------------------------                      
    state2write4upu = state2write4upu + 1                                                                   
    message2print = "{(Step 0H/33) added" + "; state2write4upu-added-by-manoeuvre2writeimage4upu: " + str(state2write4upu) + "; time: " + time4xi() + "}"
    print4xi(class2print, method2print, message2print)
    # 1; 2; 3; 4  
    if state2write4upu > max2writestate4upu:
        state2write4upu = 0
        message2print = "{(Step 0I/33) set" + "; state2write4upu-set-by-manoeuvre2writeimage4upu to: " + str(state2write4upu)  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)
        # 0
        ack2writeinfo4upu        = False
        message2print = "{(Step 0J/33) set" + "; ack2writeinfo4upu-set-by-manoeuvre2writeimage4upu to: " + str(ack2writeinfo4upu)  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)                                                     
        # False
        message2writeget4upu     = '<default4xi>default2write4upu</default4xi>'
        message2print = "{(Step 0K/33) set" + "; message2writeget4upu-set-by-manoeuvre2writeimage4upu to: " + str(message2writeget4upu)  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)                                               
        # <default4xi>default2write4upu</default4xi>
        message2writeput4upu     = '<default4xi>default2write4upu</default4xi>'
        message2print = "{(Step 0L/33) set" + "; message2writeput4upu-set-by-manoeuvre2writeimage4upu to: " + str(message2writeput4upu)  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)              
        # <default4xi>default2write4upu</default4xi>
        message2writereceive4upu = '<default4xi>default2write4upu</default4xi>'
        message2print = "{(Step 0M/33) set" + "; message2writereceive4upu-set-by-manoeuvre2writeimage4upu to: " + str(message2writereceive4upu)  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)              
        # <default4xi>default2write4upu</default4xi>
        message2writesend4upu    = '<default4xi>default2write4upu</default4xi>'
        message2print = "{(Step 0N/33) set" + "; message2writesend4upu-set-by-manoeuvre2writeimage4upu to: " + str(message2writesend4upu)  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)              
        # <default4xi>default2write4upu</default4xi>
        max2writestate4upu = 3
        message2print = "{(Step 0O/33) initialized" + ";  max2writestate4upu-set-by-manoeuvre2writeimage4upu to: " + str( max2writestate4upu)  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)  
        #     
                    
    # ------------------------------------------------------------------------------------------------------------------------
    # STATE I - RECEIVE CONTROL AND COMMAND COPY MESSAGE VIA PIPE AND SEND ROGER OR NEGATIVE BACK 
    # ------------------------------------------------------------------------------------------------------------------------
    if state2write4upu == 1:
        message2print = "{(Step 1A/70) try" + "; receive-try-by-manoeuvre2writeimage4upu"  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)
        message2writereceive4upu = ''
        try:
            # ----------------------------------------------------------------------------------------------------------------
            # RECEIVE CONTROL AND COMMAND MESSAGE VIA PIPE
            # ----------------------------------------------------------------------------------------------------------------         
            message2writereceive4upu = consumerchannel2writemanoeuvre4upu.recv()
            message2print = "{(Step 1B/33) received" + "; message2writereceive4upu-received-by-manoeuvre2writeimage4upu: " + str(message2writereceive4upu) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)  
            # <copy4xi>copy2write4upu</copy4xi>
                         
            if message2writereceive4upu == '<copy4xi>copy2write4upu</copy4xi>':
                message2writesend4upu = '<roger4xi>roger2write4upu</roger4xi>'
            else:
                message2writesend4upu = '<negative4xi><negative2write4upu</<negative4xi>'
            # ----------------------------------------------------------------------------------------------------------------
            # SEND ROGER OR NEGATIVE BACK 
            # ----------------------------------------------------------------------------------------------------------------                                
            publisherchannel2writemanoeuvre4upu.send(message2writesend4upu)
            
            message2print = "{(Step 1C/33) sent" + "; message2writesend4upu-sent-by-manoeuvre2writeimage4upu: " + str(message2writesend4upu)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)          
            # <roger4xi>roger2write4upu</roger4xi>
            
            code2ack  = code2ack + '1C-'
        except:
            message2print = "{(Step 1Z/33) failed" + "; message2writereceive4upu-failed-by-manoeuvre2writeimage4upu: " + str(message2writereceive4upu) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)             
            code2nack  = code2nack + '1Z-' 
    else:
        pass

                    
    # ------------------------------------------------------------------------------------------------------------------------
    # STATE II - RECEIVE CONTROL AND COMMAND READY MESSAGE VIA PIPE AND SEND READY OR NEGATIVE BACK 
    # ------------------------------------------------------------------------------------------------------------------------
    if state2write4upu == 2:
        message2print = "{(Step 2A/70) try" + "; receive-try-by-manoeuvre2writeimage4upu"  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)
        message2writereceive4upu = ''
        try:
            # ----------------------------------------------------------------------------------------------------------------
            # RECEIVE CONTROL AND COMMAND MESSAGE VIA PIPE
            # ----------------------------------------------------------------------------------------------------------------         
            message2writereceive4upu = consumerchannel2writemanoeuvre4upu.recv()
            message2print = "{(Step 2B/33) received" + "; message2writereceive4upu-received-by-manoeuvre2writeimage4upu: " + str(message2writereceive4upu) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)  
            # <ready4xi>ready2write4upu</ready4xi>
                         
            if message2writereceive4upu == '<ready4xi>ready2write4upu</ready4xi>':
                message2writesend4upu = '<ready4xi>ready2write4upu</ready4xi>'
            else:
                message2writesend4upu = '<negative4xi><negative2ready4upu</<negative4xi>'
            # ----------------------------------------------------------------------------------------------------------------
            # SEND ROGER OR NEGATIVE BACK 
            # ----------------------------------------------------------------------------------------------------------------                                
            publisherchannel2writemanoeuvre4upu.send(message2writesend4upu)
            
            message2print = "{(Step 2C/33) sent" + "; message2writesend4upu-sent-by-manoeuvre2writeimage4upu: " + str(message2writesend4upu)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)          
            # <ready4xi>ready2write4upu</ready4xi>
            
            code2ack  = code2ack + '2C-'
        except:
            message2print = "{(Step 2Z/33) failed" + "; message2writereceive4upu-failed-by-manoeuvre2writeimage4upu: " + str(message2writereceive4upu) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)             
            code2nack  = code2nack + '2Z-' 
    else:
        pass        
                
    # ------------------------------------------------------------------------------------------------------------------------
    # STATE III - GET IMAGE INFORMATION SUCH AS IMAGE PATH VIA QUEUE AND SEND ACK OR NACK VIA PIPE
    # ------------------------------------------------------------------------------------------------------------------------
    if state2write4upu == 3:    
        info2image4upu = None
        message2print = "{(Step 3A/70) try" + "; info2image4upu-try-by-manoeuvre2writeimage4upu"  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)
        try: 
            # ----------------------------------------------------------------------------------------------------------------
            # GET IMAGE INFORMATION SUCH AS IMAGE PATH VIA QUEUE
            # ----------------------------------------------------------------------------------------------------------------        
            info2image4upu = queue2writemanoeuvre4upu.get()
            
            message2print = "{(Step 3B/33) got" + "; info2image4upu-got-by-manoeuvre2writeimage4upu: " + str(info2image4upu) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)  
            # ..//..//..//..//..//web//images//png//dragonxi.png
            if info2image4upu is None:
                code2nack  = code2nack + '3Z1-'
                ack2writeinfo4upu = False 
            else:    
                code2ack  = code2ack + '3C-'
                ack2writeinfo4upu = True
        except:
            message2print = "{(Step 3Z1/33) failed" + "; queue2writemanoeuvre4upu-failed-by-manoeuvre2writeimage4upu: " + str(queue2writemanoeuvre4upu) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)             
            code2nack  = code2nack + '3Z1-'
            
        message2writesend4upu = ' '
        try:
            # ----------------------------------------------------------------------------------------------------------------
            # SEND CONTROL INFO ABOUT IMAGE OPERATION - EITHER ACK OR NACK  
            # ----------------------------------------------------------------------------------------------------------------         
            if ack2writeinfo4upu is True:
                message2writesend4upu = '<ack4xi>ack2imageinfo4upu</ack4xi>'
            else:
                message2writesend4upu = '<nack4xi>nack2imageinfo4upu</nack4xi>'
            message2print = "{(Step 4B/33) set" + "; message2writesend4upu-set-by-manoeuvre2writeimage4upu to: " + str(message2writesend4upu)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print) 
            # <ack4xi>ack2imageinfo4upu</ack4xi>       
        except:
            message2print = "{(Step 4Z1/33) failed" + "; message2writesend4upu-failed-by-manoeuvre2writeimage4upu: " + str(message2writesend4upu) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)             
            
            code2nack  = code2nack + '4Z1-'    
        try:                
            message2print = "{(Step 4C/70) using" + "; publisherchannel2writemanoeuvre4upu-calling-by-manoeuvre2writeimage4upu: " + str(publisherchannel2writemanoeuvre4upu) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)    
            # <read-write PipeConnection, handle 412>
            message2print = "{{(Step 4D/70) passing" + "; message2writesend4upu-passing-to-send-by-manoeuvre2writeimage4upu: " + str(message2writesend4upu) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)
            #              
        except:
            message2print = "{(Step 4Z1/33) failed" + "; message2writesend4upu-failed-by-manoeuvre2writeimage4upu: " + str(message2writesend4upu) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)             
            code2nack  = code2nack + '4Z2-'                
        try:    
            publisherchannel2writemanoeuvre4upu.send(message2writesend4upu)
            
            message2print = "{(Step 4E/33) sent" + "; message2writesend4upu-sent-by-manoeuvre2writeimage4upu: " + str(message2writesend4upu)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)          
            # <ack4xi>ack2imageinfo4upu</ack4xi>          
            code2ack  = code2ack + '4E-'
        except:
            message2print = "{(Step 4Z3/33) failed" + "; message2writesend4upu-failed-by-manoeuvre2writeimage4upu: " + str(message2writesend4upu) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)             
            code2nack  = code2nack + '4Z3-'                
    else:
        pass
        

            
    # ------------------------------------------------------------------------------------------------------------------------
    # RETURN 
    # ------------------------------------------------------------------------------------------------------------------------
    response2manouvre4write['ACK2INFO4UPU']        = ack2writeinfo4upu
    response2manouvre4write['CODE2ACK4UPU']        = code2ack
    response2manouvre4write['CODE2NACK4UPU']       = code2nack
    response2manouvre4write['MESSAGE2GET4UPU']     = message2writeget4upu
    response2manouvre4write['MESSAGE2PUT4UPU']     = message2writeput4upu
    response2manouvre4write['MESSAGE2RECEIVE4UPU'] = message2writereceive4upu
    response2manouvre4write['MESSAGE2SEND4UPU']    = message2writesend4upu
    response2manouvre4write['STATE2MANOEUVRE4UPU'] = state2write4upu
                          
    message2print = "{(Step 33/33) returning" + "; response2manouvre4write-returning-by-manoeuvre2writeimage4upu: " + str(response2manouvre4write) + "; time: " + time4xi() + "}" 
    print4xi(class2print, method2print, message2print)                                                             
    #1
    # {'MESSAGE2SEND4UPU': '<roger4xi>roger2write4upu</roger4xi>', 
    #  'ACK2INFO4UPU': False, 'MESSAGE2GET4UPU': '<default4xi>default2write4upu</default4xi>', 
    #  'MESSAGE2RECEIVE4UPU': '<copy4xi>copy2write4upu</copy4xi>', 
    #  'CODE2ACK4UPU': ' 0D-1C-', 
    #  'MESSAGE2PUT4UPU': '<default4xi>default2write4upu</default4xi>', 
    #  'CODE2NACK4UPU': ' ', 
    #  'STATE2MANOEUVRE4UPU': 1}; time: 00:11:53 10/03/11 Pacific Daylight Time}
    #2
    # {'MESSAGE2SEND4UPU': '<ready4xi>ready2write4upu</ready4xi>', 
    #  'ACK2INFO4UPU': False, 'MESSAGE2GET4UPU': '<default4xi>default2write4upu</default4xi>', 
    #  'MESSAGE2RECEIVE4UPU': '<ready4xi>ready2write4upu</ready4xi>', 
    #  'CODE2ACK4UPU': ' 0D-2C-', 
    #  'MESSAGE2PUT4UPU': '<default4xi>default2write4upu</default4xi>', 
    #  'CODE2NACK4UPU': ' ', 
    #  'STATE2MANOEUVRE4UPU': 2}; time: 00:11:53 10/03/11 Pacific Daylight Time}
    #3
    # {'MESSAGE2SEND4UPU': '<ack4xi>ack2imageinfo4upu</ack4xi>', 
    #  'ACK2INFO4UPU': True, 'MESSAGE2GET4UPU': '<default4xi>default2write4upu</default4xi>', 
    #  'MESSAGE2RECEIVE4UPU': '<ready4xi>ready2write4upu</ready4xi>', 
    #  'CODE2ACK4UPU': ' 0D-3C-4E-', 
    #  'MESSAGE2PUT4UPU': '<default4xi>default2write4upu</default4xi>', 
    #  'CODE2NACK4UPU': ' ', 
    #  'STATE2MANOEUVRE4UPU': 3}; time: 00:11:53 10/03/11 Pacific Daylight Time}

    return response2manouvre4write           
          
#</function4xi>

# -------------------------------------------------------------------------------------------------------------------------
#<function4xi function = "manoeuvre2mapimage4upu" usage = "control function for map2image4upu function in this module">

# ------------------------------------------------------------------------------------------------------------------------- 
def manoeuvre2mapimage4upu(queue2mapmanoeuvre4upu, consumerchannel2mapmanoeuvre4upu, publisherchannel2mapmanoeuvre4upu):
    '''
    manoeuvres control and command
    '''
    global state2map4upu                                                       # STATE FOR COMMAND AND CONTROL
    global max2mapstate4upu                                                    # MAX FOR STATE
    global ack2mapinfo4upu                                                     # ACK FOR IMAGE INFO
    global message2mapget4upu                                 
    global message2mapput4upu
    global message2mapreceive4upu
    global message2mapsend4upu
    
    # -----------------------------------------------------------------------------------------
    # (Step 0/33) PARAMETERS            
    # -----------------------------------------------------------------------------------------          
    class2print  = "manoeuvre2mapimage4upu" 
    method2print = "manoeuvre2mapimage4upu" 

    code2ack  = ' '
    code2nack = ' '
    
    message2print = "{(Step 0A/33) started" + "; manoeuvre2mapimage4upu-started: " + str(manoeuvre2mapimage4upu)  + "; time: " + time4xi() + "}" 
    print4xi(class2print, method2print, message2print)                                                                      
    #  
    
    message2print = "{(Step 0B/33) passed" + "; queue2mapmanoeuvre4upu-to-manoeuvre2mapimage4upu: " + str(queue2mapmanoeuvre4upu) + "; time: " + time4xi() + "}"
    print4xi(class2print, method2print, message2print)                                                              
    # 
    
    message2print = "{(Step 0C/33) passed" + "; consumerchannel2mapmanoeuvre4upu-to-manoeuvre2mapimage4upu: " + str(consumerchannel2mapmanoeuvre4upu) + "; time: " + time4xi() + "}"
    print4xi(class2print, method2print, message2print)                                                              
    # 
    
    message2print = "{(Step 0D/33) passed" + "; publisherchannel2mapmanoeuvre4upu-to-manoeuvre2mapimage4upu: " + str(publisherchannel2mapmanoeuvre4upu) + "; time: " + time4xi() + "}"
    print4xi(class2print, method2print, message2print)                                                              
    # 
    
    code2ack  = code2ack + '0D-' 
    response2manouvre4map = {}
                    
    # ------------------------------------------------------------------------------------------------------------------------
    # SETTINGS
    # ------------------------------------------------------------------------------------------------------------------------    
    message2print = "{(Step 0E/70) try" + "; state2map4upu-try-by-manoeuvre2mapimage4upu"  + "; time: " + time4xi() + "}" 
    print4xi(class2print, method2print, message2print)     
    try:
        if state2map4upu is None:
            state2map4upu = 0
            message2print = "{(Step 0F/33) initialized" + "; state2map4upu-initialized-by-manoeuvre2mapimage4upu to: " + str(state2map4upu)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)    
        else:
            message2print = "{(Step 0G/33) used" + "; state2map4upu-used-by-manoeuvre2mapimage4upu: " + str(state2map4upu)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print) 
            # 1; 2; 3;  
    except:
        state2map4upu = 0
        message2print = "{(Step 0Z1/33) initialized" + "; state2map4upu-initialized-by-manoeuvre2mapimage4upu to: " + str(state2map4upu)  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)
        # 0
        ack2mapinfo4upu        = False
        message2print = "{(Step 0Z2/33) initialized" + "; ack2mapinfo4upu-initialized-by-manoeuvre2mapimage4upu to: " + str(ack2mapinfo4upu)  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)                                                     
        # False
        message2mapget4upu     = '<default4xi>default2map4upu</default4xi>'
        message2print = "{(Step 0Z3/33) initialized" + "; message2mapget4upu-initialized-by-manoeuvre2mapimage4upu to: " + str(message2mapget4upu)  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)                                               
        # <default4xi>default2map4upu</default4xi>
        message2mapput4upu     = '<default4xi>default2map4upu</default4xi>'
        message2print = "{(Step 0Z4/33) initialized" + "; message2mapput4upu-initialized-by-manoeuvre2mapimage4upu to: " + str(message2mapput4upu)  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)              
        # <default4xi>default2map4upu</default4xi>
        message2mapreceive4upu = '<default4xi>default2map4upu</default4xi>'
        message2print = "{(Step 0Z5/33) initialized" + "; message2mapreceive4upu-initialized-by-manoeuvre2mapimage4upu to: " + str(message2mapreceive4upu)  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)              
        # <default4xi>default2map4upu</default4xi>
        message2mapsend4upu    = '<default4xi>default2map4upu</default4xi>'
        message2print = "{(Step 0Z6/33) initialized" + "; message2mapsend4upu-initialized-by-manoeuvre2mapimage4upu to: " + str(message2mapsend4upu)  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)              
        # <default4xi>default2map4upu</default4xi>
        max2mapstate4upu = 3
        message2print = "{(Step 0Z7/33) initialized" + ";  max2mapstate4upu-initialized-by-manoeuvre2mapimage4upu to: " + str( max2mapstate4upu)  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)  
        # 
                    
    # ------------------------------------------------------------------------------------------------------------------------
    # STATE UPDATE
    # ------------------------------------------------------------------------------------------------------------------------                      
    state2map4upu = state2map4upu + 1                                                                   
    message2print = "{(Step 0H/33) added" + "; state2map4upu-added-by-manoeuvre2mapimage4upu: " + str(state2map4upu) + "; time: " + time4xi() + "}"
    print4xi(class2print, method2print, message2print)
    # 1; 2; 3; 4  
    if state2map4upu > max2mapstate4upu:
        state2map4upu = 0
        message2print = "{(Step 0I/33) set" + "; state2map4upu-set-by-manoeuvre2mapimage4upu to: " + str(state2map4upu)  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)
        # 0
        ack2mapinfo4upu        = False
        message2print = "{(Step 0J/33) set" + "; ack2mapinfo4upu-set-by-manoeuvre2mapimage4upu to: " + str(ack2mapinfo4upu)  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)                                                     
        # False
        message2mapget4upu     = '<default4xi>default2map4upu</default4xi>'
        message2print = "{(Step 0K/33) set" + "; message2mapget4upu-set-by-manoeuvre2mapimage4upu to: " + str(message2mapget4upu)  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)                                               
        # <default4xi>default2map4upu</default4xi>
        message2mapput4upu     = '<default4xi>default2map4upu</default4xi>'
        message2print = "{(Step 0L/33) set" + "; message2mapput4upu-set-by-manoeuvre2mapimage4upu to: " + str(message2mapput4upu)  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)              
        # <default4xi>default2map4upu</default4xi>
        message2mapreceive4upu = '<default4xi>default2map4upu</default4xi>'
        message2print = "{(Step 0M/33) set" + "; message2mapreceive4upu-set-by-manoeuvre2mapimage4upu to: " + str(message2mapreceive4upu)  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)              
        # <default4xi>default2map4upu</default4xi>
        message2mapsend4upu    = '<default4xi>default2map4upu</default4xi>'
        message2print = "{(Step 0N/33) set" + "; message2mapsend4upu-set-by-manoeuvre2mapimage4upu to: " + str(message2mapsend4upu)  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)              
        # <default4xi>default2map4upu</default4xi>
        max2mapstate4upu = 3
        message2print = "{(Step 0O/33) initialized" + ";  max2mapstate4upu-set-by-manoeuvre2mapimage4upu to: " + str( max2mapstate4upu)  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)  
        #     
                    
    # ------------------------------------------------------------------------------------------------------------------------
    # STATE I - RECEIVE CONTROL AND COMMAND COPY MESSAGE VIA PIPE AND SEND ROGER OR NEGATIVE BACK 
    # ------------------------------------------------------------------------------------------------------------------------
    if state2map4upu == 1:
        message2print = "{(Step 1A/70) try" + "; receive-try-by-manoeuvre2mapimage4upu"  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)
        message2mapreceive4upu = ''
        try:
            # ----------------------------------------------------------------------------------------------------------------
            # RECEIVE CONTROL AND COMMAND MESSAGE VIA PIPE
            # ----------------------------------------------------------------------------------------------------------------         
            message2mapreceive4upu = consumerchannel2mapmanoeuvre4upu.recv()
            message2print = "{(Step 1B/33) received" + "; message2mapreceive4upu-received-by-manoeuvre2mapimage4upu: " + str(message2mapreceive4upu) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)  
            # 
                         
            if message2mapreceive4upu == '<copy4xi>copy2mapimage4upu</copy4xi>':
                message2mapsend4upu = '<roger4xi>roger2mapimage4upu</roger4xi>'
            else:
                message2mapsend4upu = '<negative4xi><negative2mapimage4upu</<negative4xi>'
            # ----------------------------------------------------------------------------------------------------------------
            # SEND ROGER OR NEGATIVE BACK 
            # ----------------------------------------------------------------------------------------------------------------                                
            publisherchannel2mapmanoeuvre4upu.send(message2mapsend4upu)
            
            message2print = "{(Step 1C/33) sent" + "; message2mapsend4upu-sent-by-manoeuvre2mapimage4upu: " + str(message2mapsend4upu)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)          
            # 
            
            code2ack  = code2ack + '1C-'
        except:
            message2print = "{(Step 1Z/33) failed" + "; message2mapreceive4upu-failed-by-manoeuvre2mapimage4upu: " + str(message2mapreceive4upu) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)             
            code2nack  = code2nack + '1Z-' 
    else:
        pass

                    
    # ------------------------------------------------------------------------------------------------------------------------
    # STATE II - RECEIVE CONTROL AND COMMAND READY MESSAGE VIA PIPE AND SEND READY OR NEGATIVE BACK 
    # ------------------------------------------------------------------------------------------------------------------------
    if state2map4upu == 2:
        message2print = "{(Step 2A/70) try" + "; receive-try-by-manoeuvre2mapimage4upu"  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)
        message2mapreceive4upu = ''
        try:
            # ----------------------------------------------------------------------------------------------------------------
            # RECEIVE CONTROL AND COMMAND MESSAGE VIA PIPE
            # ----------------------------------------------------------------------------------------------------------------         
            message2mapreceive4upu = consumerchannel2mapmanoeuvre4upu.recv()
            message2print = "{(Step 2B/33) received" + "; message2mapreceive4upu-received-by-manoeuvre2mapimage4upu: " + str(message2mapreceive4upu) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)  
            # 
                         
            if message2mapreceive4upu == '<ready4xi>ready2mapimage4upu</ready4xi>':
                message2mapsend4upu = '<ready4xi>ready2mapimage4upu</ready4xi>'
            else:
                message2mapsend4upu = '<negative4xi><negative2ready4upu</<negative4xi>'
            # ----------------------------------------------------------------------------------------------------------------
            # SEND ROGER OR NEGATIVE BACK 
            # ----------------------------------------------------------------------------------------------------------------                                
            publisherchannel2mapmanoeuvre4upu.send(message2mapsend4upu)
            
            message2print = "{(Step 2C/33) sent" + "; message2mapsend4upu-sent-by-manoeuvre2mapimage4upu: " + str(message2mapsend4upu)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)          
            # 
            
            code2ack  = code2ack + '2C-'
        except:
            message2print = "{(Step 2Z/33) failed" + "; message2mapreceive4upu-failed-by-manoeuvre2mapimage4upu: " + str(message2mapreceive4upu) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)             
            code2nack  = code2nack + '2Z-' 
    else:
        pass        
                
    # ------------------------------------------------------------------------------------------------------------------------
    # STATE III - GET IMAGE INFORMATION SUCH AS IMAGE PATH VIA QUEUE AND SEND ACK OR NACK VIA PIPE
    # ------------------------------------------------------------------------------------------------------------------------
    if state2map4upu == 3:    
        info2image4upu = None
        message2print = "{(Step 3A/70) try" + "; info2image4upu-try-by-manoeuvre2mapimage4upu"  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)
        try: 
            # ----------------------------------------------------------------------------------------------------------------
            # GET IMAGE INFORMATION SUCH AS IMAGE PATH VIA QUEUE
            # ----------------------------------------------------------------------------------------------------------------        
            info2image4upu = queue2mapmanoeuvre4upu.get()
            
            message2print = "{(Step 3B/33) got" + "; info2image4upu-got-by-manoeuvre2mapimage4upu: " + str(info2image4upu) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)  
            # 
            if info2image4upu is None:
                code2nack  = code2nack + '3Z1-'
                ack2mapinfo4upu = False 
            else:    
                code2ack  = code2ack + '3C-'
                ack2mapinfo4upu = True
        except:
            message2print = "{(Step 3Z1/33) failed" + "; queue2mapmanoeuvre4upu-failed-by-manoeuvre2mapimage4upu: " + str(queue2mapmanoeuvre4upu) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)             
            code2nack  = code2nack + '3Z1-'
            
        message2mapsend4upu = ' '
        try:
            # ----------------------------------------------------------------------------------------------------------------
            # SEND CONTROL INFO ABOUT IMAGE OPERATION - EITHER ACK OR NACK  
            # ----------------------------------------------------------------------------------------------------------------         
            if ack2mapinfo4upu is True:
                message2mapsend4upu = '<ack4xi>ack2imageinfo4upu</ack4xi>'
            else:
                message2mapsend4upu = '<nack4xi>nack2imageinfo4upu</nack4xi>'
            message2print = "{(Step 4B/33) set" + "; message2mapsend4upu-set-by-manoeuvre2mapimage4upu to: " + str(message2mapsend4upu)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print) 
            #      
        except:
            message2print = "{(Step 4Z1/33) failed" + "; message2mapsend4upu-failed-by-manoeuvre2mapimage4upu: " + str(message2mapsend4upu) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)             
            
            code2nack  = code2nack + '4Z1-'    
        try:                
            message2print = "{(Step 4C/70) using" + "; publisherchannel2mapmanoeuvre4upu-calling-by-manoeuvre2mapimage4upu: " + str(publisherchannel2mapmanoeuvre4upu) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)    
            # 
            message2print = "{{(Step 4D/70) passing" + "; message2mapsend4upu-passing-to-send-by-manoeuvre2mapimage4upu: " + str(message2mapsend4upu) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)
            #              
        except:
            message2print = "{(Step 4Z1/33) failed" + "; message2mapsend4upu-failed-by-manoeuvre2mapimage4upu: " + str(message2mapsend4upu) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)             
            code2nack  = code2nack + '4Z2-'                
        try:    
            publisherchannel2mapmanoeuvre4upu.send(message2mapsend4upu)
            
            message2print = "{(Step 4E/33) sent" + "; message2mapsend4upu-sent-by-manoeuvre2mapimage4upu: " + str(message2mapsend4upu)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)          
            #          
            code2ack  = code2ack + '4E-'
        except:
            message2print = "{(Step 4Z3/33) failed" + "; message2mapsend4upu-failed-by-manoeuvre2mapimage4upu: " + str(message2mapsend4upu) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)             
            code2nack  = code2nack + '4Z3-'                
    else:
        pass
        

            
    # ------------------------------------------------------------------------------------------------------------------------
    # RETURN 
    # ------------------------------------------------------------------------------------------------------------------------
    response2manouvre4map['ACK2INFO4UPU']        = ack2mapinfo4upu
    response2manouvre4map['CODE2ACK4UPU']        = code2ack
    response2manouvre4map['CODE2NACK4UPU']       = code2nack
    response2manouvre4map['MESSAGE2GET4UPU']     = message2mapget4upu
    response2manouvre4map['MESSAGE2PUT4UPU']     = message2mapput4upu
    response2manouvre4map['MESSAGE2RECEIVE4UPU'] = message2mapreceive4upu
    response2manouvre4map['MESSAGE2SEND4UPU']    = message2mapsend4upu
    response2manouvre4map['STATE2MANOEUVRE4UPU'] = state2map4upu
                          
    message2print = "{(Step 33/33) returning" + "; response2manouvre4map-returning-by-manoeuvre2mapimage4upu: " + str(response2manouvre4map) + "; time: " + time4xi() + "}" 
    print4xi(class2print, method2print, message2print)                                                             
    #1 {'MESSAGE2SEND4UPU': '<roger4xi>roger2mapimage4upu</roger4xi>', 
    #  'ACK2INFO4UPU': False, 
    #  'MESSAGE2GET4UPU': '<default4xi>default2map4upu</default4xi>', 
    #  'MESSAGE2RECEIVE4UPU': '<copy4xi>copy2mapimage4upu</copy4xi>', 
    #  'CODE2ACK4UPU': ' 0D-1C-', 
    #  'MESSAGE2PUT4UPU': '<default4xi>default2map4upu</default4xi>', 
    #  'CODE2NACK4UPU': ' ', 
    #  'STATE2MANOEUVRE4UPU': 1}
     
    #2 {'MESSAGE2SEND4UPU': '<ready4xi>ready2mapimage4upu</ready4xi>', 
    #  'ACK2INFO4UPU': False, 
    #  'MESSAGE2GET4UPU': '<default4xi>default2map4upu</default4xi>', 
    #  'MESSAGE2RECEIVE4UPU': '<ready4xi>ready2mapimage4upu</ready4xi>', 
    #  'CODE2ACK4UPU': ' 0D-2C-', 
    #  'MESSAGE2PUT4UPU': '<default4xi>default2map4upu</default4xi>', 
    #  'CODE2NACK4UPU': ' ', 
    #  'STATE2MANOEUVRE4UPU': 2}; time: 06:49:48 10/04/11 Pacific Daylight Time}
     
    #3 {'MESSAGE2SEND4UPU': '<ack4xi>ack2imageinfo4upu</ack4xi>', 
    #  'ACK2INFO4UPU': True, 
    #  'MESSAGE2GET4UPU': '<default4xi>default2map4upu</default4xi>', 
    #  'MESSAGE2RECEIVE4UPU': '<ready4xi>ready2mapimage4upu</ready4xi>', 
    #  'CODE2ACK4UPU': ' 0D-3C-4E-', 
    #  'MESSAGE2PUT4UPU': '<default4xi>default2map4upu</default4xi>', 
    #  'CODE2NACK4UPU': ' ', 
    #  'STATE2MANOEUVRE4UPU': 3}; time: 06:49:48 10/04/11 Pacific Daylight Time}
    
    #4 {'MESSAGE2SEND4UPU': '<default4xi>default2map4upu</default4xi>', 
    #  'ACK2INFO4UPU': False, 
    #  'MESSAGE2GET4UPU': '<default4xi>default2map4upu</default4xi>', 
    #  'MESSAGE2RECEIVE4UPU': '<default4xi>default2map4upu</default4xi>', 
    #  'CODE2ACK4UPU': ' 0D-', 
    #  'MESSAGE2PUT4UPU': '<default4xi>default2map4upu</default4xi>', 
    #  'CODE2NACK4UPU': ' ', 'STATE2MANOEUVRE4UPU': 0}; time: 06:49:48 10/04/11 Pacific Daylight Time}

    return response2manouvre4map           
          
#</function4xi>

# -------------------------------------------------------------------------------------------------
#<function4xi function = "write2image4upu" usage = "
#                                                                                                              
#   # ---------------------------------------------------------------------------------------------
#   # (16) ESTABLISH CONTROL & COMMAND WITH READ2IMAGE4UPU                      
#   # ---------------------------------------------------------------------------------------------    
#   print('(Step 16A/700) OF UPU2IMAGE4XI5TEST - TRYING TO ESTABLISH CONTROL & COMMAND WITH READ2IMAGE4UPU ', time4xi())    
#   respon4write2image = {} 
#   task4write2image = None
#   try:                               
#       response4write2image = write2image4upu(task4write2image)                                                                                                          
#       print('(Step 16B/700) OF UPU2IMAGE4XI5TEST - ESTABLISHED CONTROL & COMMAND WITH READ2IMAGE4UPU : ', response4write2image)
#       queue2write4upu      = response4write2image['QUEUE']
#       channel2publish4upu = response4write2image['PUBLISHERCHANNEL']
#       channel2consume4upu = response4write2image['CONSUMERCHANNEL']
#   except:
#       print('(Step 16Z/700) OF UPU2IMAGE4XI5TEST *** FAILED TO ESTABLISH CONTROL & COMMAND WITH READ2IMAGE4UPU ', time4xi()) 
#   # --------------------------------------------------------------------------------------------------
#   # (17) SEND CONTROL & COMMAND COPY VIA PIPE FOR MANOEUVRE2IMAGE4UPU TO RECEIVE                 
#   # --------------------------------------------------------------------------------------------------      
#   print('(Step 17A/700) OF UPU2IMAGE4XI5TEST - TRYING TO SEND CONTROL & COMMAND VIA PIPE WITH READ2IMAGE4UPU ', time4xi())     
#   try:                               
#       message2writesend4upu = "<copy4xi>copy2write4upu</copy4xi>"
#       channel2publish4upu = response4write2image['PUBLISHERCHANNEL']
#       channel2publish4upu.send(message2writesend4upu)
#       print("(Step 17B/700) OF UPU2IMAGE4XI5TEST - SENT TO CHANNEL2PUBLISH4UPU: ", channel2publish4upu, " MESSAGE2SEND4UPU: " , message2writesend4upu)                                                                                                
#   except:
#       print('(Step 17Z1/700) OF UPU2IMAGE4XI5TEST *** FAILED TO SEND CONTROL & COMMAND VIA PIPE WITH READ2IMAGE4UPU ', time4xi())                                                
#   # ------------------------------------------------------------------------------------------
#   # (18) LET MANOEUVRE2IMAGE4UPU TO RECEIVE COPY AND AND SEND ROGER VIA PIPE                
#   # ------------------------------------------------------------------------------------------       
#   print("(Step 18C/700) OF UPU2IMAGE4XI5TEST - TRYING TO PERFORM TARGET FUNCTION MANOEUVRE2IMAGE4UPU ", time4xi()) 
#   try:        
#       response2manoeuvre4upu = manoeuvre2readimage4upu(queue2write4upu, channel2consume4upu, channel2publish4upu)
#       print("(Step 18D/700) OF UPU2IMAGE4XI5TEST - PERFORMED TARGET FUNCTION MANOEUVRE2IMAGE4UPU ", manoeuvre2readimage4upu)
#   except:
#       print("(Step 18Z/700) OF UPU2IMAGE4XI5TEST *** FAILED TO PERFORM TARGET FUNCTION MANOEUVRE2IMAGE4UPU ", time4xi())     
#   else:
#       pass # trust activated thread    
#   # ---------------------------------------------------------------------------------------------
#   # (19) RECEIVE CONTROL & COMMAND ROGER VIA PIPE                   
#   # ---------------------------------------------------------------------------------------------          
#   print('(Step 19A/700) OF UPU2IMAGE4XI5TEST - TRYING TO RECEIVE CONTROL & COMMAND VIA PIPE WITH READ2IMAGE4UPU ', time4xi())     
#   try:
#       channel2consume4upu = response4write2image['CONSUMERCHANNEL']                              
#       message2writereceive4upu = channel2consume4upu.recv()
#       print("(Step 19B/700) OF UPU2IMAGE4XI5TEST - RECEIVED FROM CHANNEL2CONSUME4UPU: ", channel2consume4upu, " CONTROL & COMMAND: ", message2writereceive4upu)                                                                                                  
#       # <roger4xi>roger2write4upu</roger4xi>
#   except:
#       print('(Step 19Z/700) OF UPU2IMAGE4XI5TEST *** FAILED TO RECEIVE CONTROL & COMMAND VIA PIPE  WITH READ2IMAGE4UPU ', time4xi())         
#   # ---------------------------------------------------------------------------------------------------
#   # (20) SEND CONTROL & COMMAND READY VIA PIPE FOR MANOEUVRE2IMAGE4UPU TO RECEIVE                
#   # ---------------------------------------------------------------------------------------------------      
#   print('(Step 20A/700) OF UPU2IMAGE4XI5TEST - TRYING TO SEND CONTROL & COMMAND VIA PIPE WITH READ2IMAGE4UPU ', time4xi())     
#   try:                               
#       message2writesend4upu = "<write4xi>write2write4upu</write4xi>"
#       channel2publish4upu = response4write2image['PUBLISHERCHANNEL']
#       channel2publish4upu.send(message2writesend4upu)
#       print("(Step 20B/700) OF UPU2IMAGE4XI5TEST - SENT TO CHANNEL2PUBLISH4UPU: ", channel2publish4upu, " MESSAGE2SEND4UPU: " , message2writesend4upu)                                                                                                
#       # <write4xi>write2write4upu</write4xi> 
#   except:
#        print('(Step 20Z/700) OF UPU2IMAGE4XI5TEST *** FAILED TO SEND CONTROL & COMMAND VIA PIPE WITH READ2IMAGE4UPU ', time4xi())                                                
#   # -----------------------------------------------------------------------------------
#   # (21) LET MANOEUVRE2IMAGE4UPU TO RECEIVE READY AND TO SEND READY                
#   # -----------------------------------------------------------------------------------       
#   print("(Step 21A/700) OF UPU2IMAGE4XI5TEST - TRYING TO PERFORM TARGET FUNCTION MANOEUVRE2IMAGE4UPU ", time4xi()) 
#   try:        
#       response2manoeuvre4upu = manoeuvre2readimage4upu(queue2write4upu, channel2consume4upu, channel2publish4upu)
#       print("(Step 21B/700) OF UPU2IMAGE4XI5TEST - PERFORMED TARGET FUNCTION MANOEUVRE2IMAGE4UPU ", manoeuvre2readimage4upu)
#   except:
#       print("(Step 21Z/700) OF UPU2IMAGE4XI5TEST *** FAILED TO PERFORM TARGET FUNCTION MANOEUVRE2IMAGE4UPU ", time4xi())     
#   # ---------------------------------------------------------------------------------------
#   # (22) RECEIVE CONTROL & COMMAND READY VIA PIPE                      
#   # ---------------------------------------------------------------------------------------          
#   print('(Step 22A/700) OF UPU2IMAGE4XI5TEST - TRYING TO RECEIVE CONTROL & COMMAND VIA PIPE WITH READ2IMAGE4UPU ', time4xi())     
#   try:
#       channel2consume4upu = response4write2image['CONSUMERCHANNEL']                              
#       message2writereceive4upu = channel2consume4upu.recv()
#       print("(Step 22B/700) OF UPU2IMAGE4XI5TEST - RECEIVED FROM CHANNEL2CONSUME4UPU: ", channel2consume4upu, " CONTROL & COMMAND: ", message2writereceive4upu)                                                                                                  
#       # <write4xi>write2write4upu</write4xi> 
#   except:
#       print('(Step 22Z/700) OF UPU2IMAGE4XI5TEST *** FAILED TO RECEIVE CONTROL & COMMAND VIA PIPE  WITH READ2IMAGE4UPU ', time4xi())          
#   # -----------------------------------------------------------------------------------------
#   # (24) PUT IMAGE PATH TO QUEUE
#   # -----------------------------------------------------------------------------------------                          
#   print("(Step 24A/700) OF UPU2IMAGE4XI5TEST - TRYING TO PUT IMAGE PATH TO QUEUE ", time4xi())      
#   try:    
#       queue2write4dragon.put(path2dragon)             
#       print("(Step 24B/700) OF DRAGON2IMAGE4XI5TEST - PUT TO QUEUE IMAGE PATH: ", path2dragon) 
#       # ..//..//..//..//..//web//images//png//dragonxi.png  
#   except:
#       print("(Step 24Z/700) OF UPU2IMAGE4XI5TEST *** FAILED TO PUT IMAGE PATH TO QUEUE ", time4xi()) 
#   # ------------------------------------------------------------------------------------------------------
#   # (25) LET MANOEUVRE2IMAGE4UPU TO GET FROM QUEUE AND TO SEND EITHER ACK OR NACK VIA PIPE               
#   # -------------------------------------------------------------------------------------------------------       
#   print("(Step 25A/700) OF UPU2IMAGE4XI5TEST - TRYING TO PERFORM TARGET FUNCTION MANOEUVRE2IMAGE4UPU ", time4xi()) 
#   try:        
#       response2manoeuvre4upu = manoeuvre2readimage4upu(queue2write4upu, channel2consume4upu, channel2publish4upu)
#       print("(Step 25B/700) OF UPU2IMAGE4XI5TEST - PERFORMED TARGET FUNCTION MANOEUVRE2IMAGE4UPU ", manoeuvre2readimage4upu)
#   except:
#       print("(Step 25Z/700) OF UPU2IMAGE4XI5TEST *** FAILED TO PERFORM TARGET FUNCTION MANOEUVRE2IMAGE4UPU ", time4xi())     
#   # -----------------------------------------------------------------------------------------------
#   # (26) RECEIVE ACK OR NACK ABOUT IMAGE OPERATION VIA PIPE                
#   # -----------------------------------------------------------------------------------------------          
#   print('(Step 26A/700) OF UPU2IMAGE4XI5TEST - TRYING TO RECEIVE CONTROL & COMMAND ABOUT IMAGE OPERTION VIA PIPE ', time4xi())      
#   try: 
#       message2writereceive4upu = channel2consume4upu.recv()
#       print("(Step 26D/700) OF UPU2IMAGE4XI5TEST - RECEIVED FROM CHANNEL2CONSUME4UPU: ", channel2consume4upu, " CONTROL & COMMAND: ", message2writereceive4upu)                                                                                                   
#       # <ack4xi>ack2imageinfo4upu</ack4xi>
#   except:
#       print('(Step 26Z/700) OF UPU2IMAGE4XI5TEST *** FAILED TO RECEIVE CONTROL & COMMAND ABOUT IMAGE OPERTION VIA PIPE ', time4xi())               
#   # -------------------------------------------------------------------------------------------------------
#   # (27) LET MANOEUVRE2IMAGE4UPU GO TO STATE 0               
#   # -------------------------------------------------------------------------------------------------------       
#   print("(Step 27A/700) OF UPU2IMAGE4XI5TEST - TRYING TO PERFORM TARGET FUNCTION MANOEUVRE2IMAGE4UPU ", time4xi()) 
#   try:        
#       response2manoeuvre4upu = manoeuvre2readimage4upu(queue2write4upu, channel2consume4upu, channel2publish4upu)
#       print("(Step 27B/700) OF UPU2IMAGE4XI5TEST - GOT RESPONSE2MANOEUVRE4UPU ", response2manoeuvre4upu)
#   except:
#       print("(Step 27Z/700) OF UPU2IMAGE4XI5TEST *** FAILED TO PERFORM TARGET FUNCTION MANOEUVRE2IMAGE4UPU ", time4xi())   

#">

# -------------------------------------------------------------------------------------------------   
def write2image4upu(task4imagewrite2upu=None):

    '''
    writes 2D images for UPU
    '''
    global upu2image4xi                                                     # 2D-IMAGES FOR UPU  
    global control2writeimage4upu                                              # COMMAND AND CONTROL FOR UPU IMAGES 
    
    # -----------------------------------------------------------------------------------------
    # (Step 1/33) PARAMETERS        
    # -----------------------------------------------------------------------------------------          
    class2print = "write2image4upu"
    method2print = "write2image4upu"

    message2print = "{(Step 1A/33) started" + "; write2image4upu-started: " + str(write2image4upu)  + "; time: " + time4xi() + "}" 
    print4xi(class2print, method2print, message2print)                                                                     #  
    # <function write2image4upu at 0x00000000038E8EC8>
    
    message2print = "{(Step 1B/33) passed" + "; task4imagewrite2upu-to-write2image4upu: " + str(task4imagewrite2upu) + "; time: " + time4xi() + "}"
    print4xi(class2print, method2print, message2print)                                                              
    # None
            
    code2ack  = ' '
    code2nack = ' '
    
    # -----------------------------------------------------------------------------------------
    # CREATE UPU2IMAGE4XI IF NECESSARY       
    # ----------------------------------------------------------------------------------------- 
    try:
        if upu2image4xi is None:
            upu2image4xi = create2image4upu()                                                                              # INITIALIZE IMAGE FOR UPU 
            message2print = "{(Step 1C/70) created" + "; upu2image4xi-created-by-write2image4upu to: " + str(upu2image4xi)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)
            #             
        else:
            message2print = "{(Step 1D/70) skipped" + "; create2image4upu-skipped-by-write2image4upu: " + str(write2image4upu) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)
            # create2image4upu-skipped-by-write2image4upu: <function write2image4upu at 0x00000000038E8EC8>                                 
    except:
        try:
            upu2image4xi = create2image4upu()
            message2print = "{(Step 1E/70) set" + "; upu2image4xi-set-by-write2image4upu to: " + str(upu2image4xi)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)
            #             
        except:
            upu2image4xi = {}
            message2print = "{(Step 1Z/70) yielded" + "; upu2image4xi-yielded-by-write2image4upu: " + str(upu2image4xi) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)  
            code2nack  = code2nack + '1Z-' 
    
    # -----------------------------------------------------------------------------------------
    # (Step 1/70) - CHECK PARAMETERS           
    # -----------------------------------------------------------------------------------------          
    try:             
        if task4imagewrite2upu is None:
            # -----------------------------------------------------------------------------------------
            # NO PARAMETERS GIVEN BY CALLER FOR READ - BUILD TASK! 
            # -----------------------------------------------------------------------------------------    
            
            task4imagewrite2upu = {}                                                                                                                                                                                       
            
            message2print = "{(Step 1A/70) initialized" + "; task4imagewrite2upu-created-by-write2image4upu to: " + str(task4imagewrite2upu)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)               
            # 
            
            # ---------------------------------------------------------------------------------------------
            # (1B) TRACE - AUTOMATIC                      
            # ---------------------------------------------------------------------------------------------              
            function2callback4upu = trace2image4upu                                                                 # CALLBACK FUNCTION FOR UPU
            
            message2print = "{(Step 1B1/70) initialized" + "; function2callback4upu-created-by-write2image4upu to: " + str(function2callback4upu)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)            
            # <function trace2image4upu at 0x00000000038E2A48>
            
            code2ack  = code2ack + '1B1-'
           
            args2callback = {}                                                                                      # CREATE ARGUMENT DICTIONARY FOR CALLBACK FUNCTION             
            args2callback['CLASS']   = "write2image4upu"                                                            # MODULE OF UPUIZED OPENCL MODULE
            args2callback['METHOD']  = "write2image4upu"                                                            # FUNCTION OF UPUIZED OPENCL MODULE
            args2callback['MESSAGE'] = "{(Step 1Z1/33) failed" + "; write2image4upu-failed " + "; time: " + time4xi() + "}" # ERROR MESSAGE FOR UPUIZED OPENCL TO RETURN    

            message2print = "{(Step 1B2/70) initialized" + "; args2callback-created-by-write2image4upu to: " + str(args2callback)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)               
            # {'MESSAGE': '{(Step 1Z1/33) failed; write2image4upu-failed ; time: 02:32:29 10/05/11 Pacific Daylight Time}', 
            #  'CLASS': 'write2image4upu', 
            #  'METHOD': 'write2image4upu'}; time: 02:32:29 10/05/11 Pacific Daylight Time}

            code2ack  = code2ack + '1B2-'
            
            # ---------------------------------------------------------------------------------------------
            # (1C) CALLBACK - AUTOMATIC                      
            # ---------------------------------------------------------------------------------------------              
            task4imagewrite2upu['DATA2CALLBACK'] = args2callback                                                    # USER DATA PASSED TO UPUIZED OPENCL - SEVERAL ARGS PASSED IN ONE DICTIONARY
            task4imagewrite2upu['FUNCTION2CALLBACK'] = function2callback4upu                                        # CALLBACK FUNCTION TO CALLED BY OPENCL IN EXCEPTIONS
            
            code2ack  = code2ack + '1C-'
            message2print = "{(Step 1C/70) initialized" + "; task4imagewrite2upu-created-by-write2image4upu to: " + str(task4imagewrite2upu)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)  
            # {'FUNCTION2CALLBACK': <function trace2image4upu at 0x00000000038E2A48>,
            #  'DATA2CALLBACK': {'MESSAGE': '{(Step 1Z1/33) failed; write2image4upu-failed ; time: 02:32:29 10/05/11 Pacific Daylight Time}',
            #  'CLASS': 'write2image4upu',
            #  'METHOD': 'write2image4upu'}}; time: 02:32:29 10/05/11 Pacific Daylight Time}

            
            # ---------------------------------------------------------------------------------------------
            # (2) PLATFORM - AUTOMATIC                   
            # ---------------------------------------------------------------------------------------------             
            platform2xi = {}
            message2print = "{(Step 2A/70) initialized" + "; platform2xi-created-by-write2image4upu to: " + str(platform2xi)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)   
            # {}
            
            try:
                platform2xi = get2platformids4xi()                                                                  # PLATFORM IS COMMON TO ALL DEVICES I.E TO DEVICE XI
                message2print = "{(Step 2B/70) got" + "; platform2xi-got-by-write2image4upu: " + str(platform2xi)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)
                # {'list2platforms': ['DragonXi'], 'ack2return': ' 2C-2E-2F-2G-', 'nack2return': ' ', 'available2platform': 1, 'max2entries': 1}; time: 02:32:29 10/05/11 Pacific Daylight Time}

                code2ack  = code2ack + '2B-'
            except:
                message2print = "{(Step 2Z/70) failed" + "; platform2xi-failed-by-write2image4upu: " + str(failed)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)                                                
                code2nack  = code2nack + '2Z-'
                 
            task4imagewrite2upu['PLATFORM'] = platform2xi                                                           # PLATFORM OF SYSTEM 
              
            
            # ---------------------------------------------------------------------------------------------
            # (3) CONTEXT - AUTOMATIC                 
            # ---------------------------------------------------------------------------------------------    
            context2upu = {}
            message2print = "{(Step 3A/70) trying" + "; context2upu-trying-by-write2image4upu: " + str(context2upu)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print) 
            try:
                context2upu = get2context4upu()                                                                     # NO PARAMETERS - THE SYSTEM CREATES CONTEXT IF REQUIRED  
                
                message2print = "{(Step 3B/70) got" + "; context2upu-got-by-write2image4upu: " + str(context2upu)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)                                  
                # {'COUNT2REFERENCE': 0, 
                #  'DATA2CALLBACK': {'MESSAGE': '{(Step 1Z1/33) failed; create2context4upu-failed ; time: 02:32:29 10/05/11 Pacific Daylight Time}', 
                #  'CLASS': 'create2context4upu', ...
   
                code2ack  = code2ack + '3B-'         
            except:
                message2print = "{(Step 3Z/70) failed" + "; context2upu-failed-by-write2image4upu: " + str(context2upu) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)             
                code2nack  = code2nack + '3Z-'
                
            task4imagewrite2upu ['CONTEXT'] = context2upu           
            
            # ---------------------------------------------------------------------------------------------
            # (4) DEVICE - AUTOMATIC                       
            # ---------------------------------------------------------------------------------------------    
            device2upu = {}
            
            message2print = "{(Step 4A/70) trying" + "; device2upu-trying-by-write2image4upu: " + str(device2upu)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print) 
            try:
                device2upu = get2upu4xi()                                                                           # THE SYSTEM CREATES DEVICE INFO, IF NECESSARY    
                
                message2print = "{(Step 4B/70) got" + "; device2upu-got-by-write2image4upu: " + str(device2upu)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)                                  
                # {'RESOLUTION2TIMER4PROFILING': 318, 'ID2PLATFORM': '0000000001509568' ...
               
                code2ack  = code2ack + '4B-'  
            except:
                message2print = "{(Step 4Z/70) failed" + "; device2upu-failed-by-write2image4upu: " + str(device2upu) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)             
                code2nack  = code2nack + '4Z-'
            
            task4imagewrite2upu ['DEVICE'] = device2upu
            
                   
            # ---------------------------------------------------------------------------------------------
            # (5) IMAGE - REQUIRED                            
            # ---------------------------------------------------------------------------------------------         
            imageinfo2upu = {}
             
            message2print = "{(Step 5A/70) trying" + "; imageinfo2upu-trying-by-write2image4upu: " + str(imageinfo2upu)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print) 
            try:
                imageinfo2upu = get2imageinfo4upu()                                                                         
                
                message2print = "{(Step 5B/70) got" + "; imageinfo2upu-got-by-write2image4upu: " + str(imageinfo2upu)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)                                  
                # {'SIZE2QUEUE': 2147483646, 
                #  'FORMAT': {'TYPE': 'CL_UNORM_INT8', 
                #  'ORDER': 'CL_RGBA'}, 
                #  'QUEUE': <queue4xi.Queue4Xi object at 0x0000000004494E10>, 
                #  'WIDTH': 800, 
                #  'CONTEXT': {'COUNT2REFERENCE': 0, 
                #  'DATA2CALLBACK': {'MESSAGE': '{(Step 1Z1/33) failed; create2context4upu-failed ; time: 02:32:29 10/05/11 Pacific Daylight Time}', 
                #  'CLASS': 'create2context4upu', 
                #  'METHOD': 'create2context4upu'}, 
                #  'FUNCTION2CALLBACK': <function trace2context4upu at 0x000000000387E348>, 
                #  'ACK2RETURN': ' 2A-2C-2D-2E-2B-3B-4B-5B-6B-7B-7B-', 
                #  'INFO2PRIVATE': None, 
                #  'PLATFORM': {'list2platforms': ['DragonXi'], 'ack2return': ' 2C-2E-2F-2G-', 'nack2return': ' ', 'available2platform': 1, 'max2entries': 1}, 
                #  'NACK2RETURN': ' ', 
                #  'SIZE2PRIVATE': 0, 
                #  'PROPERTIES': None}, 
                #  'PIPE': (<read-write PipeConnection, handle 520>, <read-write PipeConnection, handle 524>), 
                #  'COUNT2REFERENCE': 1, 
                #  'FUNCTION2CALLBACK': <function trace2image4upu at 0x00000000038E2A48>, 
                #  'COMMANDQUEUE': ...

                code2ack  = code2ack + '5B-'  
            except:
                message2print = "{(Step 5Z/70) failed" + "; imageinfo2upu-failed-by-write2image4upu: " + str(imageinfo2upu) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)             
                code2nack  = code2nack + '5Z-'          
                      
            task4imagewrite2upu ['IMAGE'] = imageinfo2upu         
                        

            # ---------------------------------------------------------------------------------------------
            # (6) QUEUE - REQUIRED BY UPUXI                          
            # ---------------------------------------------------------------------------------------------         
            queue2upu = None
            
            message2print = "{(Step 6A/70) trying" + "; queue2upu-trying-by-write2image4upu: " + str(queue2upu)  + "; time: " + time4xi() + "}" 
            print4xi(class2print, method2print, message2print) 
            try:
                queue2upu = Queue4Xi()                                                                              # THE SYSTEM CREATES QUEUE    
                
                message2print = "{(Step 6B/70) got" + "; queue2upu-got-by-write2image4upu: " + str(queue2upu)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)                                  
                # <queue4xi.Queue4Xi object at 0x0000000003D4FA90>
                
                code2ack  = code2ack + '6B-'  
            except:
                message2print = "{(Step 6Z/70) failed" + "; queue2upu-failed-by-write2image4upu: " + str(queue2upu) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)             
                code2nack  = code2nack + '6Z-'
                
            task4imagewrite2upu ['QUEUE'] = queue2upu

            
            # ---------------------------------------------------------------------------------------------
            # (7) PIPE - REQUIRED BY UPUXI                              
            # ---------------------------------------------------------------------------------------------         
            pipe2upu = None
           
            message2print = "{(Step 7A/70) trying" + "; pipe2upu-trying-by-write2image4upu: " + str(pipe2upu)  + "; time: " + time4xi() + "}" 
            print4xi(class2print, method2print, message2print) 
            try:
                pipe2upu = Pipe4Xi()                                                                                # THE SYSTEM CREATES PIPE    
                
                message2print = "{(Step 7B/70) got" + "; pipe2upu-got-by-write2image4upu: " + str(pipe2upu)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)                                  
                # (<read-write PipeConnection, handle 568>, <read-write PipeConnection, handle 572>)
                
                code2ack  = code2ack + '7B-'  
            except:
                message2print = "{(Step 7Z/70) failed" + "; pipe2upu-failed-by-write2image4upu: " + str(pipe2upu) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)             
                code2nack  = code2nack + '7Z-'
                
            task4imagewrite2upu ['PIPE'] = pipe2upu 
            
            # ---------------------------------------------------------------------------------------------
            # (8) COMMAND QUEUE - REQUIRED                              
            # ---------------------------------------------------------------------------------------------             
            commandqueueinfo2upu = {}
           
            message2print = "{(Step 8A/70) trying" + "; commandqueueinfo2upu-trying-by-write2image4upu: " + str(commandqueueinfo2upu)  + "; time: " + time4xi() + "}" 
            print4xi(class2print, method2print, message2print)  
                        
            try:
                commandqueueinfo2upu = get2commandqueueinfo4upu()                                                   # SYSTEM CREATES COMMAND QUEUE, IF NECESSARY   
                message2print = "{(Step 8B/70) got" + "; commandqueueinfo2upu-got-by-write2image4upu: " + str(commandqueueinfo2upu)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print) 
                # {'CODE2ERROR': 0, 
                #  'COUNT2REFERENCE': 0, 
                #  'DATA2CALLBACK': {'MESSAGE': '{(Step 1Z1/33) failed; create2commandqueue4upu-failed ; time: 02:32:29 10/05/11 Pacific Daylight Time}', 
                #  'CLASS': 'create2commandqueue4upu', 
                #  'METHOD': 'create2commandqueue4upu'}, 
                #  'FUNCTION2CALLBACK': <function trace2commandqueue4upu at 0x000000000388DA48> ...
                
                code2ack  = code2ack + '8B-'  
            except:
                message2print = "{(Step 8Z/70) failed" + "; commandqueueinfo2upu-failed-by-write2image4upu: " + str(commandqueueinfo2upu) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)             
                code2nack  = code2nack + '8Z-'
        
            task4imagewrite2upu ['COMMANDQUEUE'] = commandqueueinfo2upu
            
            
            
            # ---------------------------------------------------------------------------------------------
            # (9) BUFFER - AUTOMATIC BY UPUXI                               
            # ---------------------------------------------------------------------------------------------             
            bufferinfo2upu = {}
           
            message2print = "{(Step 9A/70) trying" + "; bufferinfo2upu-trying-by-write2image4upu: " + str(bufferinfo2upu)  + "; time: " + time4xi() + "}" 
            print4xi(class2print, method2print, message2print) 
                        
            try:
                bufferinfo2upu = get2bufferinfo4upu()                                                               # SYSTEM CREATES BUFFER FOR UPU, IF NECESSARY
                message2print = "{(Step 9B/70) got" + "; bufferinfo2upu-got-by-write2image4upu: " + str(bufferinfo2upu)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print) 
                # {'CODE2ERROR': 0, 
                #  'COUNT2REFERENCE': 0, 
                #  'DATA2CALLBACK': {'MESSAGE': '{(Step 1Z1/33) failed; create2buffer4upu-failed ; time: 02:32:28 10/05/11 Pacific Daylight Time}', 
                #  'CLASS': 'create2buffer4upu', 
                #  'METHOD': 'create2buffer4upu'}, ...
                
                code2ack  = code2ack + '9B-' 
            except:
                message2print = "{(Step 9Z/70) failed" + "; bufferinfo2upu-failed-by-write2image4upu: " + str(bufferinfo2upu) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)             
                code2nack  = code2nack + '9Z-'
        
            task4imagewrite2upu ['BUFFER'] = bufferinfo2upu 
            
            # ---------------------------------------------------------------------------------------------
            # (10) EVENT - REQUIRED                            
            # ---------------------------------------------------------------------------------------------             
            eventinfo2upu = {}
           
            message2print = "{(Step 10A/70) trying" + "; eventinfo2upu-trying-by-write2image4upu: " + str(eventinfo2upu)  + "; time: " + time4xi() + "}" 
            print4xi(class2print, method2print, message2print) 
                        
            try:
                eventinfo2upu = get2eventinfo4upu()                                                                    
                message2print = "{(Step 10B/70) got" + "; eventinfo2upu-got-by-write2image4upu: " + str(eventinfo2upu)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print) 
                #  {'CODE2ERROR': 0, 
                #  'DATA2CALLBACK': {'MESSAGE': '{(Step 1Z1/33) failed; create2event4upu-failed ; time: 02:32:28 10/05/11 Pacific Daylight Time}', 
                #  'CLASS': 'create2event4upu', 
                #  'METHOD': 'create2event4upu'}, 
                #  'FUNCTION2CALLBACK': <function trace2event4upu at 0x00000000038A3F48>, ...
                
                code2ack  = code2ack + '10B-' 
            except:
                message2print = "{(Step 10Z/70) failed" + "; eventinfo2upu-failed-by-write2image4upu: " + str(eventinfo2upu) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)             
                code2nack  = code2nack + '10Z-'
        
            task4imagewrite2upu ['EVENT'] = eventinfo2upu 
            
            
            # ---------------------------------------------------------------------------------------------
            # (11) EVENTWAITLIST - REQUIRED                              
            # ---------------------------------------------------------------------------------------------             
            eventwaitlist2upu = {}
           
            message2print = "{(Step 11A/70) trying" + "; eventwaitlist2upu-trying-by-write2image4upu: " + str(eventwaitlist2upu)  + "; time: " + time4xi() + "}" 
            print4xi(class2print, method2print, message2print) 
                        
            try:
                eventwaitlist2upu = {}                                                                   
                message2print = "{(Step 11B/70) set" + "; eventwaitlist2upu-set-by-write2image4upu: " + str(eventwaitlist2upu)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print) 
                # {} 
                
                code2ack  = code2ack + '11B-' 
            except:
                message2print = "{(Step 11Z/70) failed" + "; eventwaitlist2upu-failed-by-write2image4upu: " + str(eventwaitlist2upu) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)             
                code2nack  = code2nack + '11Z-'
        
            task4imagewrite2upu ['EVENTWAITLIST'] = eventwaitlist2upu                     

            
            # ---------------------------------------------------------------------------------------------
            # (12) EVENTWAITLISTQUANTITY - REQUIRED                               
            # ---------------------------------------------------------------------------------------------             
            eventwaitlistquantity2upu = 0
           
            message2print = "{(Step 12A/70) trying" + "; eventwaitlistquantity2upu-trying-by-write2image4upu: " + str(eventwaitlistquantity2upu)  + "; time: " + time4xi() + "}" 
            print4xi(class2print, method2print, message2print) 
                        
            try:
                keys2event = eventwaitlist2upu.keys()
                for key2event in keys2event:
                    eventwaitlistquantity2upu = eventwaitlistquantity2upu + 1                                                                   
                message2print = "{(Step 12B/70) set" + "; eventwaitlistquantity2upu-set-by-write2image4upu: " + str(eventwaitlistquantity2upu)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print) 
                # 0 
                
                code2ack  = code2ack + '12B-' 
            except:
                message2print = "{(Step 12Z/70) failed" + "; eventwaitlistquantity2upu-failed-by-write2image4upu: " + str(eventwaitlistquantity2upu) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)             
                code2nack  = code2nack + '12Z-'
        
            task4imagewrite2upu ['EVENTWAITLISTQUANTITY'] = eventwaitlistquantity2upu       
                                                 
            
            # ---------------------------------------------------------------------------------------------
            # (13) BLOCKING - REQUIRED                        
            # ---------------------------------------------------------------------------------------------             
            blocking2upu = ' '                                                                   
     
            message2print = "{(Step 13A/70) trying" + "; blocking2upu-trying-by-write2image4upu: " + str(blocking2upu)  + "; time: " + time4xi() + "}" 
            print4xi(class2print, method2print, message2print) 
                        
            try:
                blocking2upu = 'CL_FALSE'                                                                           # the write command is non-blocking                                                              
                message2print = "{(Step 13B/70) set" + "; blocking2upu-set-by-write2image4upu to: " + str(blocking2upu)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print) 
                # CL_FALSE 
                code2ack  = code2ack + '13B-' 
            except:
                message2print = "{(Step 13Z/70) failed" + "; blocking2upu-failed-by-write2image4upu: " + str(blocking2upu) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)             
                code2nack  = code2nack + '13Z-'
        
            task4imagewrite2upu ['BLOCKING'] = blocking2upu             
            
            # ---------------------------------------------------------------------------------------------
            # (14) ORIGIN - REQUIRED                             
            # ---------------------------------------------------------------------------------------------             
            origin2upu = {}
            
            message2print = "{(Step 14A/70) trying" + "; origin2upu-trying-by-write2image4upu: " + str(origin2upu)  + "; time: " + time4xi() + "}" 
            print4xi(class2print, method2print, message2print) 
                        
            try:                                                                                    
                origin2upu['X'] = 0 
                origin2upu['Y'] = 0  
                origin2upu['Z'] = 0                                                                                                                          
                message2print = "{(Step 14B/70) set" + "; origin2upu-set-by-write2image4upu to: " + str(origin2upu)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print) 
                # {'Y': 0, 'X': 0, 'Z': 0} 
                code2ack  = code2ack + '14B-' 
            except:
                message2print = "{(Step 14Z/70) failed" + "; origin2upu-failed-by-write2image4upu: " + str(origin2upu) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)             
                code2nack  = code2nack + '14Z-'
        
            task4imagewrite2upu ['ORIGIN'] = origin2upu
            
            
            # ---------------------------------------------------------------------------------------------
            # (15) REGION - REQUIRED                         
            # ---------------------------------------------------------------------------------------------             
            region2upu = {}
            
            message2print = "{(Step 15A/70) trying" + "; region2upu-trying-by-write2image4upu: " + str(region2upu)  + "; time: " + time4xi() + "}" 
            print4xi(class2print, method2print, message2print) 
                        
            try:                                                                                    
                region2upu['X'] = 160                                                                               # WIDTH  IN PIXELS TO READ 
                region2upu['Y'] = 90                                                                                # HEIGHT IN PIXELS TO READ 
                region2upu['Z'] = 0                                                                                 # DEPTH  IN PIXELS TO READ                                                                                                                          
                message2print = "{(Step 15B/70) set" + "; region2upu-set-by-write2image4upu to: " + str(region2upu)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print) 
                # {'Y': 90, 'X': 160, 'Z': 0}
                code2ack  = code2ack + '15B-' 
            except:
                message2print = "{(Step 15Z/70) failed" + "; region2upu-failed-by-write2image4upu: " + str(region2upu) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)             
                code2nack  = code2nack + '15Z-'
        
            task4imagewrite2upu ['REGION'] = region2upu
            
            
            # ---------------------------------------------------------------------------------------------
            # (16) ROWPITCH (STRIDE) - REQUIRED                              
            # ---------------------------------------------------------------------------------------------             
            # One can assume that memory representing an image or screen will have data laid out as one row of pixels followed directly by the next row of pixels. 
            # However many APIs (e.g. Direct3D) that return a pointer to image data add an extra block of data at the end of each row. 
            # int offset = x * bytesPerPixel + y * pitch;
            # When a video image is stored in memory, the memory buffer might contain extra padding bytes after each row of pixels. 
            # The padding bytes affect how the image is stored in memory, but do not affect how the image is displayed.
            # The stride is the number of bytes from one row of pixels in memory to the next row of pixels in memory. 
            # Stride is also called pitch. 
            # If padding bytes are present, the stride is wider than the width of the image, as shown in the following illustration.

            height2upu = 800
            width2upu = 800
            rowpitch2upu = 0
            size2element = 1                                                                                        # VOXEL (X,Y,Z) = SHOT IN PIXELS, R=1 PIXELS
            voxelquantity2layer = 1                                                                                 # MAX NUMBER OF VOXELS IN EACH 2D LAYER 
            quantity2direction = 1                                                                                  # ONE DIRECTIONS IN 2D
            
            message2print = "{(Step 16A/70) trying" + "; rowpitch2upu-trying-by-write2image4upu: " + str(rowpitch2upu)  + "; time: " + time4xi() + "}" 
            print4xi(class2print, method2print, message2print) 
            #  
                        
            try:                                                                                                    # X-AXEL IS THE WORST CASE TO SCAN WITH Y=0, Z=0
                rowpitch2upu = int(width2upu * size2element * voxelquantity2layer / quantity2direction)             # (800 * 8 * 10) / 8 = 8K PIXELS MAX TO SCAN 
                                                                                                                    # SPHERE IS SPARSE HAVING FEW SHOTS PER SCAN LINE 
                                                                                                                                         
                message2print = "{(Step 16B/70) set" + "; rowpitch2upu-set-by-write2image4upu to: " + str(rowpitch2upu)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print) 
                # 800                                                                                            
                 
                code2ack  = code2ack + '16B-' 
            except:
                message2print = "{(Step 16Z/70) failed" + "; rowpitch2upu-failed-by-write2image4upu: " + str(rowpitch2upu) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)             
                # 
                
                code2nack  = code2nack + '16Z-'
        
            task4imagewrite2upu ['ROWPITCH'] = rowpitch2upu     
            
            # ----------------------------------------------------------------------------------------------------
            # (17) POINTER - REQUIRED - THE POINTER TO A BUFFER IN HOST MEMORY WHERE IMAGE DATA IS TO BE READ FROM                          
            # ----------------------------------------------------------------------------------------------------             
            pointer2upu = None
            size2upu = int(width2upu * height2upu)                                                                  # CALCULATED MEMORY
            if size2upu == 0:
                size2upu = 4400                                                                                     # MINIMUM MEMORY
            
            message2print = "{(Step 17A/70) trying" + "; pointer2upu-trying-by-write2image4upu: " + str(pointer2upu)  + "; time: " + time4xi() + "}" 
            print4xi(class2print, method2print, message2print) 
            # 
                        
            try:
                arena2upu = Arena4Xi(size2upu)                                                                      # SYSTEM RESERVER MEMORY FOR IMAGE                             
                                                                                                                                         
                message2print = "{(Step 17B/70) got" + "; arena2upu-got-by-write2image4upu: " + str(arena2upu)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print) 
                # <arena4xi.Arena4Xi object at 0x00000000041348D0>
                
                size2arena = arena2upu.size                                                                         # 
                              
                message2print = "{(Step 17C/70) got" + "; size2arena-got-by-write2buffer4upu: " + str(size2arena)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)
                # 640000 
                
                buffer2arena = arena2upu.buffer
                
                message2print = "{(Step 17D/70) got" + "; buffer2arena-got-by-write2buffer4upu: " + str(buffer2arena)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)                
                # <mmap.mmap object at 0x000000000421A450> 
                
                code2ack  = code2ack + '17D-' 
                
                if size2arena < size2upu:
                    size2upu = size2arena                                                                           # AVAILABLE MEMORY FOR IMAGE
                    message2print = "{(Step 17E/70) set" + "; size2upu-set-by-write2buffer4upu to: " + str(size2upu)  + "; time: " + time4xi() + "}"
                    print4xi(class2print, method2print, message2print)
                    # 
                    
                    code2ack  = code2ack + '17E-'
                else:
                    pass
                    
                pointer2upu = buffer2arena
                message2print = "{(Step 17F/70) set" + "; pointer2upu-set-by-write2buffer4upu to: " + str(pointer2upu)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)
                # <mmap.mmap object at 0x000000000421A450>
                
                code2ack  = code2ack + '17F-'       
            except:
                message2print = "{(Step 17Z/70) failed" + "; pointer2upu-failed-by-write2image4upu: " + str(pointer2upu) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)             
                # 
                
                code2nack  = code2nack + '17Z-'
        
            task4imagewrite2upu ['POINTER'] = pointer2upu                            
            task4imagewrite2upu ['SIZE']    = size2upu                                                               # AUTOMATIC BY UPUXI
            
            
            # --------------------------------------------------------------------------------------------------------
            # (18) SLICEPITCH - REQUIRED - THE SIZE IN BYTES OF THE 2D SLICE OF THE 3D REGION OF A 3D IMAGE BEING READ                         
            # --------------------------------------------------------------------------------------------------------             
            slicepitch2upu = None                                                                                   # SLICEPITCH IS ZERO FOR 2D IMAGES                           
           
            message2print = "{(Step 18A/70) trying" + "; slicepitch2upu-trying-by-write2image4upu: " + str(slicepitch2upu)  + "; time: " + time4xi() + "}" 
            print4xi(class2print, method2print, message2print) 
                        
            try:
                slicepitch2upu = 0                                                          
                message2print = "{(Step 18B/70) set" + "; slicepitch2upu-set-by-write2image4upu to: " + str(slicepitch2upu)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print) 
                # 0

                code2ack  = code2ack + '18B-'                
                 
            except:
                message2print = "{(Step 18Z/70) failed" + "; slicepitch2upu-failed-by-write2image4upu: " + str(slicepitch2upu) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)             
                code2nack  = code2nack + '18Z-'
            
            task4imagewrite2upu ['SLICEPITCH'] = slicepitch2upu                  
            
            # ---------------------------------------------------------------------------------------------
            # EOF PARAMETER PROVIDING                
            # ---------------------------------------------------------------------------------------------                                                                                                                               
        else:
            pass
            
        # ---------------------------------------------------------------------------------------------
        # CHECK PARAMETERS - WHETHER AUTOMATICALLY GENERATED OR USER GIVEN 
        # ---------------------------------------------------------------------------------------------
        keys2task = task4imagewrite2upu.keys()
        message2print = "{(Step 20/70) using" + "; keys2task-using-by-write2image4upu: " + str(sorted(keys2task))  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)            
        # ['BLOCKING',
        #  'BUFFER',
        #  'COMMANDQUEUE',
        #  'CONTEXT',
        #  'DATA2CALLBACK',
        #  'DEVICE',
        #  'EVENT',
        #  'EVENTWAITLIST',
        #  'EVENTWAITLISTQUANTITY',
        #  'FUNCTION2CALLBACK',
        #  'IMAGE',
        #  'ORIGIN',
        #  'PIPE',
        #  'PLATFORM',
        #  'POINTER',
        #  'QUEUE',
        #  'REGION',
        #  'ROWPITCH',
        #  'SIZE',
        #  'SLICEPITCH']; time: 02:45:31 10/05/11 Pacific Daylight Time}

        if (('DATA2CALLBACK' in keys2task) and ('FUNCTION2CALLBACK' in keys2task)):
            pass
        else:                
            # ---------------------------------------------------------------------------------------------
            # (21B) TRACE - AUTOMATIC                      
            # ---------------------------------------------------------------------------------------------              
            function2callback4upu = trace2image4upu                                                             # CALLBACK FUNCTION FOR UPU
            
            message2print = "{(Step 21B1/70) initialized" + "; function2callback4upu-created-by-write2image4upu to: " + str(function2callback4upu)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)            
            # 
            
            code2ack  = code2ack + '21B1-'
           
            args2callback = {}                                                                                  # CREATE ARGUMENT DICTIONARY FOR CALLBACK FUNCTION             
            args2callback['CLASS']   = "write2image4upu"                                                        # MODULE OF UPUIZED OPENCL MODULE
            args2callback['METHOD']  = "write2image4upu"                                                        # FUNCTION OF UPUIZED OPENCL MODULE
            args2callback['MESSAGE'] = "{(Step 1Z1/33) failed" + "; write2image4upu-failed " + "; time: " + time4xi() + "}" # ERROR MESSAGE FOR UPUIZED OPENCL TO RETURN    
            message2print = "{(Step 21B2/70) initialized" + "; args2callback-created-by-write2image4upu to: " + str(args2callback)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)               
            # 
    
            code2ack  = code2ack + '21B2-'
        
            # ---------------------------------------------------------------------------------------------
            # (21C) CALLBACK - AUTOMATIC                      
            # ---------------------------------------------------------------------------------------------              
            task4imagewrite2upu['DATA2CALLBACK'] = args2callback                                                # USER DATA PASSED TO UPUIZED OPENCL - SEVERAL ARGS PASSED IN ONE DICTIONARY
            task4imagewrite2upu['FUNCTION2CALLBACK'] = function2callback4upu                                    # CALLBACK FUNCTION TO CALLED BY OPENCL IN EXCEPTIONS
            
            code2ack  = code2ack + '21C-'
            message2print = "{(Step 21C/70) initialized" + "; task4imagewrite2upu-created-by-write2image4upu to: " + str(task4imagewrite2upu)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)  
            #
             
        if 'PLATFORM' in keys2task:
            pass
        else:  
            # ---------------------------------------------------------------------------------------------
            # (22) PLATFORM - AUTOMATIC                   
            # ---------------------------------------------------------------------------------------------             
            platform2xi = {}
            message2print = "{(Step 22A/70) initialized" + "; platform2xi-created-by-write2image4upu to: " + str(platform2xi)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)   
            # 
                
            try:
                platform2xi = get2platformids4xi()                                                              # PLATFORM IS COMMON TO ALL DEVICES I.E TO DEVICE XI
                message2print = "{(Step 22B/70) got" + "; platform2xi-got-by-write2image4upu: " + str(platform2xi)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)
                # 
                     
                code2ack  = code2ack + '22B-'
            except:
                message2print = "{(Step 22Z/70) failed" + "; platform2xi-failed-by-write2image4upu: " + str(failed)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)                                                
                code2nack  = code2nack + '22Z-'
                     
            task4imagewrite2upu['PLATFORM'] = platform2xi                                                       # PLATFORM OF SYSTEM 
              
        if 'CONTEXT' in keys2task:
            pass
        else:              
            # ---------------------------------------------------------------------------------------------
            # (23) CONTEXT - AUTOMATIC                 
            # ---------------------------------------------------------------------------------------------    
            context2upu = {}
            message2print = "{(Step 23A/70) trying" + "; context2upu-trying-by-write2image4upu: " + str(context2upu)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print) 
            try:
                context2upu = get2context4upu()                                                                 # NO PARAMETERS - THE SYSTEM CREATES CONTEXT IF REQUIRED  
                
                message2print = "{(Step 23B/70) got" + "; context2upu-got-by-write2image4upu: " + str(context2upu)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)                                  
                # 
                
                code2ack  = code2ack + '23B-'         
            except:
                message2print = "{(Step 23Z/70) failed" + "; context2upu-failed-by-write2image4upu: " + str(context2upu) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)             
                code2nack  = code2nack + '23Z-'
                
            task4imagewrite2upu ['CONTEXT'] = context2upu           
        
        if 'DEVICE' in keys2task:
            pass
        else:                   
            # ---------------------------------------------------------------------------------------------
            # (24) DEVICE - AUTOMATIC                       
            # ---------------------------------------------------------------------------------------------    
            device2upu = {}
            
            message2print = "{(Step 24A/70) trying" + "; device2upu-trying-by-write2image4upu: " + str(device2upu)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print) 
            try:
                device2upu = get2upu4xi()                                                                       # THE SYSTEM CREATES DEVICE INFO, IF NECESSARY    
                
                message2print = "{(Step 24B/70) got" + "; device2upu-got-by-write2image4upu: " + str(device2upu)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)                                  
                # 
                
                code2ack  = code2ack + '24B-'  
            except:
                message2print = "{(Step 24Z/70) failed" + "; device2upu-failed-by-write2image4upu: " + str(device2upu) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)             
                code2nack  = code2nack + '24Z-'
            
            task4imagewrite2upu ['DEVICE'] = device2upu
        
        if 'IMAGE' in keys2task:
            pass
        else:                      
            # ---------------------------------------------------------------------------------------------
            # (25) IMAGE - REQUIRED                            
            # ---------------------------------------------------------------------------------------------         
            imageinfo2upu = {}
             
            message2print = "{(Step 25A/70) trying" + "; imageinfo2upu-trying-by-write2image4upu: " + str(imageinfo2upu)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print) 
            try:
                imageinfo2upu = get2imageinfo4upu()                                                                         
                
                message2print = "{(Step 25B/70) got" + "; imageinfo2upu-got-by-write2image4upu: " + str(imageinfo2upu)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)                                  
                #                 
                code2ack  = code2ack + '25B-'  
            except:
                message2print = "{(Step 25Z/70) failed" + "; imageinfo2upu-failed-by-write2image4upu: " + str(imageinfo2upu) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)             
                code2nack  = code2nack + '25Z-'          
                      
            task4imagewrite2upu ['IMAGE'] = imageinfo2upu         
                    
        if 'QUEUE' in keys2task:
            pass
        else:   
            # ---------------------------------------------------------------------------------------------
            # (26) QUEUE - REQUIRED BY UPUXI                          
            # ---------------------------------------------------------------------------------------------         
            queue2upu = None
            
            message2print = "{(Step 26A/70) trying" + "; queue2upu-trying-by-write2image4upu: " + str(queue2upu)  + "; time: " + time4xi() + "}" 
            print4xi(class2print, method2print, message2print) 
            try:
                queue2upu = Queue4Xi()                                                                          # THE SYSTEM CREATES QUEUE    
                
                message2print = "{(Step 26B/70) got" + "; queue2upu-got-by-write2image4upu: " + str(queue2upu)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)                                  
                # 
                
                code2ack  = code2ack + '26B-'  
            except:
                message2print = "{(Step 26Z/70) failed" + "; queue2upu-failed-by-write2image4upu: " + str(queue2upu) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)             
                code2nack  = code2nack + '26Z-'
                
            task4imagewrite2upu ['QUEUE'] = queue2upu

        if 'PIPE' in keys2task:
            pass
        else:              
            # ---------------------------------------------------------------------------------------------
            # (27) PIPE - REQUIRED BY UPUXI                              
            # ---------------------------------------------------------------------------------------------         
            pipe2upu = None
           
            message2print = "{(Step 27A/70) trying" + "; pipe2upu-trying-by-write2image4upu: " + str(pipe2upu)  + "; time: " + time4xi() + "}" 
            print4xi(class2print, method2print, message2print) 
            try:
                pipe2upu = Pipe4Xi()                                                                            # THE SYSTEM CREATES PIPE    
                
                message2print = "{(Step 27B/70) got" + "; pipe2upu-got-by-write2image4upu: " + str(pipe2upu)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)                                  
                # 
                
                code2ack  = code2ack + '27B-'  
            except:
                message2print = "{(Step 27Z/70) failed" + "; pipe2upu-failed-by-write2image4upu: " + str(pipe2upu) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)             
                code2nack  = code2nack + '27Z-'
                
            task4imagewrite2upu ['PIPE'] = pipe2upu 
            
        if 'COMMANDQUEUE' in keys2task:
            pass
        else:              
            # ---------------------------------------------------------------------------------------------
            # (28) COMMAND QUEUE - REQUIRED                              
            # ---------------------------------------------------------------------------------------------             
            commandqueueinfo2upu = {}
           
            message2print = "{(Step 28A/70) trying" + "; commandqueueinfo2upu-trying-by-write2image4upu: " + str(commandqueueinfo2upu)  + "; time: " + time4xi() + "}" 
            print4xi(class2print, method2print, message2print)  
                        
            try:
                commandqueueinfo2upu = get2commandqueueinfo4upu()                                               # SYSTEM CREATES COMMAND QUEUE, IF NECESSARY   
                message2print = "{(Step 28B/70) got" + "; commandqueueinfo2upu-got-by-write2image4upu: " + str(commandqueueinfo2upu)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print) 
                # 
                
                code2ack  = code2ack + '28B-'  
            except:
                message2print = "{(Step 28Z/70) failed" + "; commandqueueinfo2upu-failed-by-write2image4upu: " + str(commandqueueinfo2upu) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)             
                code2nack  = code2nack + '28Z-'
        
            task4imagewrite2upu ['COMMANDQUEUE'] = commandqueueinfo2upu
            
            
        if 'BUFFER' in keys2task:
            pass
        else:               
            # ---------------------------------------------------------------------------------------------
            # (29) BUFFER - AUTOMATIC BY UPUXI                               
            # ---------------------------------------------------------------------------------------------             
            bufferinfo2upu = {}
           
            message2print = "{(Step 29A/70) trying" + "; bufferinfo2upu-trying-by-write2image4upu: " + str(bufferinfo2upu)  + "; time: " + time4xi() + "}" 
            print4xi(class2print, method2print, message2print) 
                        
            try:
                bufferinfo2upu = get2bufferinfo4upu()                                                           # SYSTEM CREATES BUFFER FOR UPU, IF NECESSARY
                message2print = "{(Step 29B/70) got" + "; bufferinfo2upu-got-by-write2image4upu: " + str(bufferinfo2upu)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print) 
                # 
                
                code2ack  = code2ack + '29B-' 
            except:
                message2print = "{(Step 29Z/70) failed" + "; bufferinfo2upu-failed-by-write2image4upu: " + str(bufferinfo2upu) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)             
                code2nack  = code2nack + '29Z-'
        
            task4imagewrite2upu ['BUFFER'] = bufferinfo2upu
             
        if 'EVENT' in keys2task:
            pass
        else:               
            # ---------------------------------------------------------------------------------------------
            # (30) EVENT - REQUIRED                            
            # ---------------------------------------------------------------------------------------------             
            eventinfo2upu = {}
           
            message2print = "{(Step 30A/70) trying" + "; eventinfo2upu-trying-by-write2image4upu: " + str(eventinfo2upu)  + "; time: " + time4xi() + "}" 
            print4xi(class2print, method2print, message2print) 
                        
            try:
                eventinfo2upu = get2eventinfo4upu()                                                                    
                message2print = "{(Step 30B/70) got" + "; eventinfo2upu-got-by-write2image4upu: " + str(eventinfo2upu)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print) 
                # 
                
                code2ack  = code2ack + '30B-' 
            except:
                message2print = "{(Step 30Z/70) failed" + "; eventinfo2upu-failed-by-write2image4upu: " + str(eventinfo2upu) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)             
                code2nack  = code2nack + '30Z-'
        
            task4imagewrite2upu ['EVENT'] = eventinfo2upu 
        
             
        if 'EVENTWAITLIST' in keys2task:
            pass
        else:               
            # ---------------------------------------------------------------------------------------------
            # (31) EVENTWAITLIST - REQUIRED                              
            # ---------------------------------------------------------------------------------------------             
            eventwaitlist2upu = {}
           
            message2print = "{(Step 31A/70) trying" + "; eventwaitlist2upu-trying-by-write2image4upu: " + str(eventwaitlist2upu)  + "; time: " + time4xi() + "}" 
            print4xi(class2print, method2print, message2print) 
                        
            try:
                eventwaitlist2upu = {}                                                                   
                message2print = "{(Step 31B/70) set" + "; eventwaitlist2upu-set-by-write2image4upu: " + str(eventwaitlist2upu)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print) 
                # 
                
                code2ack  = code2ack + '31B-' 
            except:
                message2print = "{(Step 31Z/70) failed" + "; eventwaitlist2upu-failed-by-write2image4upu: " + str(eventwaitlist2upu) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)             
                code2nack  = code2nack + '31Z-'
        
            task4imagewrite2upu ['EVENTWAITLIST'] = eventwaitlist2upu                     

                 
        if 'EVENTWAITLISTQUANTITY' in keys2task:
            pass
        else:               
            # ---------------------------------------------------------------------------------------------
            # (32) EVENTWAITLISTQUANTITY - REQUIRED                               
            # ---------------------------------------------------------------------------------------------             
            eventwaitlistquantity2upu = 0
           
            message2print = "{(Step 32A/70) trying" + "; eventwaitlistquantity2upu-trying-by-write2image4upu: " + str(eventwaitlistquantity2upu)  + "; time: " + time4xi() + "}" 
            print4xi(class2print, method2print, message2print) 
                        
            try:
                keys2event = eventwaitlist2upu.keys()
                for key2event in keys2event:
                    eventwaitlistquantity2upu = eventwaitlistquantity2upu + 1                                                                     
                message2print = "{(Step 32B/70) set" + "; eventwaitlistquantity2upu-set-by-write2image4upu: " + str(eventwaitlistquantity2upu)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print) 
                # 
                
                code2ack  = code2ack + '32B-' 
            except:
                message2print = "{(Step 32Z/70) failed" + "; eventwaitlistquantity2upu-failed-by-write2image4upu: " + str(eventwaitlistquantity2upu) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)             
                code2nack  = code2nack + '32Z-'
        
            task4imagewrite2upu ['EVENTWAITLISTQUANTITY'] = eventwaitlistquantity2upu       
                                                 
                 
        if 'BLOCKING' in keys2task:
            pass
        else:              
            # ---------------------------------------------------------------------------------------------
            # (33) BLOCKING - REQUIRED                        
            # ---------------------------------------------------------------------------------------------             
            blocking2upu = ' '                                                                   
            message2print = "{(Step 33A/70) trying" + "; blocking2upu-trying-by-write2image4upu: " + str(blocking2upu)  + "; time: " + time4xi() + "}" 
            print4xi(class2print, method2print, message2print) 
                        
            try:
                blocking2upu = 'CL_FALSE'                                                                       # the write command is non-blocking                                                              
                message2print = "{(Step 33B/70) set" + "; blocking2upu-set-by-write2image4upu to: " + str(blocking2upu)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print) 
                # 
                code2ack  = code2ack + '33B-' 
            except:
                message2print = "{(Step 33Z/70) failed" + "; blocking2upu-failed-by-write2image4upu: " + str(blocking2upu) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)             
                code2nack  = code2nack + '33Z-'
        
            task4imagewrite2upu ['BLOCKING'] = blocking2upu             
                 
        if 'ORIGIN' in keys2task:
            pass
        else:                    
            # ---------------------------------------------------------------------------------------------
            # (34) ORIGIN - REQUIRED                             
            # ---------------------------------------------------------------------------------------------             
            origin2upu = {}
            
            message2print = "{(Step 34A/70) trying" + "; origin2upu-trying-by-write2image4upu: " + str(origin2upu)  + "; time: " + time4xi() + "}" 
            print4xi(class2print, method2print, message2print) 
                        
            try:                                                                                    
                origin2upu['X'] = 0 
                origin2upu['Y'] = 0  
                origin2upu['Z'] = 0                                                                                                                          
                message2print = "{(Step 34B/70) set" + "; origin2upu-set-by-write2image4upu to: " + str(origin2upu)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print) 
                # 
                code2ack  = code2ack + '34B-' 
            except:
                message2print = "{(Step 34Z/70) failed" + "; origin2upu-failed-by-write2image4upu: " + str(origin2upu) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)             
                code2nack  = code2nack + '34Z-'
        
            task4imagewrite2upu ['ORIGIN'] = origin2upu
                
                 
        if 'REGION' in keys2task:
            pass
        else:            
            # ---------------------------------------------------------------------------------------------
            # (35) REGION - REQUIRED                         
            # ---------------------------------------------------------------------------------------------             
            region2upu = {}
            
            message2print = "{(Step 35A/70) trying" + "; region2upu-trying-by-write2image4upu: " + str(region2upu)  + "; time: " + time4xi() + "}" 
            print4xi(class2print, method2print, message2print) 
                        
            try:                                                                                    
                region2upu['X'] = 160                                                                               # WIDTH  IN PIXELS TO WRITE 
                region2upu['Y'] = 90                                                                                # HEIGHT IN PIXELS TO WRITE
                region2upu['Z'] = 0                                                                                 # DEPTH  IN PIXELS TO WRITE                                                                                                                          
                message2print = "{(Step 35B/70) set" + "; region2upu-set-by-write2image4upu to: " + str(region2upu)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print) 
                # 
                code2ack  = code2ack + '35B-' 
            except:
                message2print = "{(Step 35Z/70) failed" + "; region2upu-failed-by-write2image4upu: " + str(region2upu) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)             
                code2nack  = code2nack + '35Z-'
        
            task4imagewrite2upu ['REGION'] = region2upu
            
                
                 
        if 'ROWPITCH' in keys2task:
            pass
        else:               
            # ---------------------------------------------------------------------------------------------
            # (36) ROWPITCH (STRIDE) - REQUIRED                              
            # ---------------------------------------------------------------------------------------------             
            # One can assume that memory representing an image or screen will have data laid out as one row of pixels followed directly by the next row of pixels. 
            # However many APIs (e.g. Direct3D) that return a pointer to image data add an extra block of data at the end of each row. 
            # int offset = x * bytesPerPixel + y * pitch;
            # When a video image is stored in memory, the memory buffer might contain extra padding bytes after each row of pixels. 
            # The padding bytes affect how the image is stored in memory, but do not affect how the image is displayed.
            # The stride is the number of bytes from one row of pixels in memory to the next row of pixels in memory. 
            # Stride is also called pitch. 
            # If padding bytes are present, the stride is wider than the width of the image, as shown in the following illustration.
     
            rowpitch2upu = 0
            size2element = 1                                                                                        # VOXEL (X,Y,Z) = SHOT IN PIXELS, R=1 PIXELS
            voxelquantity2layer = 1                                                                                 # MAX NUMBER OF VOXELS IN EACH 2D LAYER 
            quantity2direction = 1                                                                                  # ONE DIRECTIONS IN 2D
            
            message2print = "{(Step 36A/70) trying" + "; rowpitch2upu-trying-by-write2image4upu: " + str(rowpitch2upu)  + "; time: " + time4xi() + "}" 
            print4xi(class2print, method2print, message2print) 
            # 
                        
            try:                                                                                                    # X-AXEL IS THE WORST CASE TO SCAN WITH Y=0, Z=0
                rowpitch2upu = int(width2upu * size2element * voxelquantity2layer / quantity2direction)             # (800 * 8 * 10) / 8 = 8K PIXELS MAX TO SCAN 
                                                                                                                    # SPHERE IS SPARSE HAVING FEW SHOTS PER SCAN LINE 
                                                                                                                                         
                message2print = "{(Step 36B/70) set" + "; rowpitch2upu-set-by-write2image4upu to: " + str(rowpitch2upu)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print) 
                #                                                                                             
                 
                code2ack  = code2ack + '36B-' 
            except:
                message2print = "{(Step 36Z/70) failed" + "; rowpitch2upu-failed-by-write2image4upu: " + str(rowpitch2upu) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)             
                # 
                
                code2nack  = code2nack + '36Z-'
        
            task4imagewrite2upu ['ROWPITCH'] = rowpitch2upu     
                 
        if 'POINTER' in keys2task:
            pass
        else:                
            # ----------------------------------------------------------------------------------------------------
            # (37) POINTER - REQUIRED - THE POINTER TO A BUFFER IN HOST MEMORY WHERE IMAGE DATA IS TO BE READ FROM                          
            # ----------------------------------------------------------------------------------------------------             
            pointer2upu = None
            size2upu = int(width2upu * height2upu)                                                                  # CALCULATED MEMORY
            if size2upu == 0:
                size2upu = 4400                                                                                     # MINIMUM MEMORY
            
            message2print = "{(Step 37A/70) trying" + "; pointer2upu-trying-by-write2image4upu: " + str(pointer2upu)  + "; time: " + time4xi() + "}" 
            print4xi(class2print, method2print, message2print) 
            # 
                        
            try:
                arena2upu = Arena4Xi(size2upu)                                                                      # SYSTEM RESERVER MEMORY FOR IMAGE                             
                                                                                                                                         
                message2print = "{(Step 37B/70) got" + "; arena2upu-got-by-write2image4upu: " + str(arena2upu)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print) 
                # 
                
                size2arena = arena2upu.size                                                                         # 
                              
                message2print = "{(Step 37C/70) got" + "; size2arena-got-by-write2buffer4upu: " + str(size2arena)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)
                # 
                
                buffer2arena = arena2upu.buffer
                
                message2print = "{(Step 37D/70) got" + "; buffer2arena-got-by-write2buffer4upu: " + str(buffer2arena)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)                
                # 
                
                code2ack  = code2ack + '37D-' 
                
                if size2arena < size2upu:
                    size2upu = size2arena                                                                           # AVAILABLE MEMORY FOR IMAGE
                    message2print = "{(Step 37E/70) set" + "; size2upu-set-by-write2buffer4upu to: " + str(size2upu)  + "; time: " + time4xi() + "}"
                    print4xi(class2print, method2print, message2print)
                    # 
                    
                    code2ack  = code2ack + '37E-'
                else:
                    pass
                    
                pointer2upu = buffer2arena
                message2print = "{(Step 37F/70) set" + "; pointer2upu-set-by-write2buffer4upu to: " + str(pointer2upu)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)
                # 
                
                code2ack  = code2ack + '37F-'       
            except:
                message2print = "{(Step 37Z/70) failed" + "; pointer2upu-failed-by-write2image4upu: " + str(pointer2upu) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)             
                # 
                
                code2nack  = code2nack + '37Z-'
        
            task4imagewrite2upu ['POINTER'] = pointer2upu                            
            task4imagewrite2upu ['SIZE']    = size2upu                                                               # AUTOMATIC BY UPUXI
        
             
        if 'SLICEPITCH' in keys2task:
            pass
        else:               
            # --------------------------------------------------------------------------------------------------------
            # (38) SLICEPITCH - REQUIRED - THE SIZE IN BYTES OF THE 2D SLICE OF THE 3D REGION OF A 3D IMAGE BEING READ                         
            # --------------------------------------------------------------------------------------------------------             
            slicepitch2upu = None                                                                                   # SLICEPITCH IS ZERO FOR 2D IMAGES                           
           
            message2print = "{(Step 38A/70) trying" + "; slicepitch2upu-trying-by-write2image4upu: " + str(slicepitch2upu)  + "; time: " + time4xi() + "}" 
            print4xi(class2print, method2print, message2print) 
                        
            try:
                slicepitch2upu = 0                                                          
                message2print = "{(Step 38B/70) set" + "; slicepitch2upu-set-by-write2image4upu to: " + str(slicepitch2upu)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print) 
                # 
                code2ack  = code2ack + '38B-'                
                 
            except:
                message2print = "{(Step 38Z/70) failed" + "; slicepitch2upu-failed-by-write2image4upu: " + str(slicepitch2upu) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)             
                code2nack  = code2nack + '38Z-'
            
            task4imagewrite2upu ['SLICEPITCH'] = slicepitch2upu                  

        # ---------------------------------------------------------------------------------------------
        # EOF PARAMETER CHECKING AND PROVIDING FOR MISSING ONES
        # ---------------------------------------------------------------------------------------------
        keys2task = task4imagewrite2upu.keys()
        message2print = "{(Step 39/70) set" + "; keys2task-set-by-write2image4upu to: " + str(sorted(keys2task))  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)            
        # ['BLOCKING',
        #  'BUFFER',
        #  'COMMANDQUEUE',
        #  'CONTEXT',
        #  'DATA2CALLBACK',
        #  'DEVICE',
        #  'EVENT',
        #  'EVENTWAITLIST',
        #  'EVENTWAITLISTQUANTITY',
        #  'FUNCTION2CALLBACK',
        #  'IMAGE',
        #  'ORIGIN',
        #  'PIPE',
        #  'PLATFORM',
        #  'POINTER',
        #  'QUEUE',
        #  'REGION',
        #  'ROWPITCH',
        #  'SIZE',
        #  'SLICEPITCH']; time: 02:45:31 10/05/11 Pacific Daylight Time}
             
    except: 
        message2print = "{(Step 39Z/70) yielded" + "; task4imagewrite2upu-yielded-by-write2image4upu: " + str(task4imagewrite2upu) + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)  
        # 
        
        code2nack  = code2nack + '39Z-' 

    # ---------------------------------------------------------------------------------------------
    # CONTROL AND COMMAND
    # ---------------------------------------------------------------------------------------------
    control2writeimage4upu = {}       
                                
    # ---------------------------------------------------------------------------------------------
    # CONSUMER AND PUBLISHER CHANNEL FOR CONTROL AND COMMAND
    # ---------------------------------------------------------------------------------------------
    ack4channels2upu = False
    channels2upu = ()
    message2print = "{(Step 40A/70) trying" + "channels2upu-trying-by-write2image4upu: " + str(channels2upu)  + "; time: " + time4xi() + "}" 
    print4xi(class2print, method2print, message2print)     
    try:
        # 
        consumerchannel2upu, publisherchannel2upu = task4imagewrite2upu ['PIPE']

        message2print = "{(Step 40B/70) got" + "; consumerchannel2upu-got-by-write2image4upu: " + str(consumerchannel2upu)  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)            
        # <read-write PipeConnection, handle 568>

        message2print = "{(Step 40C/70) got" + "; publisherchannel2upu-got-by-write2image4upu: " + str(publisherchannel2upu)  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)            
        # <read-write PipeConnection, handle 572>                     
        ack4channels2upu = True
        
        control2writeimage4upu['CONSUMERCHANNEL']  = consumerchannel2upu
        
        
        control2writeimage4upu['PUBLISHERCHANNEL'] = publisherchannel2upu
        
        code2ack  = code2ack + '40D-' 
        
    except:
        ack4channels2upu = False
        code2nack  = code2nack + '40Z-'      
                        
    # ---------------------------------------------------------------------------------------------
    # QUEUE FOR CONTROL AND COMMAND
    # ---------------------------------------------------------------------------------------------
    ack4queue2upu = False
    queue2upu = None
    message2print = "{(Step 41A/70) trying" + "queue2upu-trying-by-write2image4upu: " + str(queue2upu)  + "; time: " + time4xi() + "}" 
    print4xi(class2print, method2print, message2print)     
    try:
        queue2upu = task4imagewrite2upu ['QUEUE']

        message2print = "{(Step 41B/70) got" + "; queue2upu-got-by-write2image4upu: " + str(queue2upu)  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)            
        # <queue4xi.Queue4Xi object at 0x0000000003C83A90> 
              
        ack4queue2upu = True
        control2writeimage4upu['QUEUE']  = queue2upu        
        code2ack  = code2ack + '41C-'
    except:
        ack4queue2upu = False  
        code2nack  = code2nack + '41Z-'
    
    
    # -----------------------------------------------------------------------------------------    
    # DUMMY PROCESS OBJECT = THREAD CREATION
    # -----------------------------------------------------------------------------------------           
    # dummy2image4upu = None
    # message2print = "{(Step 69A/70) trying" + "dummy2image4upu-trying-by-write2image4upu: " + str(dummy2image4upu)  + "; time: " + time4xi() + "}"   
    # print4xi(class2print, method2print, message2print)     
    # try:                            
        # dummy2image4upu = DummyProcess4Xi(target=manoeuvre2readimage4upu,                                               # in this module to handle queue and pipe channels
        #                                   args =(queue2upu,                                                         # caller puts and gets - does NOT call any functions
        #                                          consumerchannel2upu, publisherchannel2upu))                        # caller sends and receives - does NOT call any functions
        # message2print = "{(Step 69B/70) created" + "dummy2image4upu-created-by-write2image4upu: " + str(dummy2image4upu)  + "; time: " + time4xi() + "}" 
        # print4xi(class2print, method2print, message2print)  
        # code2ack  = code2ack + '69B-'
        
    # except:
        # code2nack  = code2nack + '69Z-'
        
    # ---------------------------------------------------------------------------------------------
    # RETURN CONTROL AND COMMAND DICTIONARY
    # ---------------------------------------------------------------------------------------------          
    message2print = "{(Step 70A/70) set" + "; code2ack-set-by-write2image4upu: " + str(code2ack) + "; time: " + time4xi() + "}"  
    print4xi(class2print, method2print, message2print)                                                             
    # 1B1-1B2-1C-2B-3B-4B-5B-6B-7B-8B-9B-10B-11B-12B-13B-14B-15B-16B-17D-17F-18B-40D-41C-
     
    message2print = "{(Step 70B/70) set" + "; code2nack-set-by-write2image4upu: " + str(code2nack) + "; time: " + time4xi() + "}" 
    print4xi(class2print, method2print, message2print)                                                             
    #  
        
    message2print = "{(Step 70C/70) returning" + "; control2writeimage4upu-returning-by-write2image4upu: " + str(control2writeimage4upu) + "; time: " + time4xi() + "}" 
    print4xi(class2print, method2print, message2print)                                                             
    # {'QUEUE': <queue4xi.Queue4Xi object at 0x0000000003C83A90>,
    #  'PUBLISHERCHANNEL': <read-write PipeConnection, handle 572>,
    #  'CONSUMERCHANNEL': <read-write PipeConnection, handle 568>}; time: 02:45:31 10/05/11 Pacific Daylight Time}

    return control2writeimage4upu
                    
#</function4xi> 


 
# -------------------------------------------------------------------------------------------------
#<function4xi function = "release2image4upu" usage = "
#
#   task4release2image  = {} 
#   task4release2image  ['COUNT2REFERENCE'] = 1 
#   response4release2image = {}
#    
#   print('(Step 12A/33) OF UPU2IMAGE4XI5TEST - TRYING TO RELEASE IMAGE OF UPU DEVICE ', time4xi())   
#   try:                               
#       response4release2image = release2image4upu(task4release2image)                                                                                             
#       print('(Step 12B/33) OF UPU2IMAGE4XI5TEST - RESPONSE4RELEASE2IMAGE : ', response4release2image)                           
#   except:
#       print('(Step 12Z1/33) OF UPU2IMAGE4XI5TEST *** FAILED TO RELEASE IMAGE OF UPU DEVICE ', time4xi())  
#   
#                 
#">

# -------------------------------------------------------------------------------------------------   
def release2image4upu(task4release2upu=None):

    '''
    releases upu2image4xi
    '''
    global upu2image4xi  
    global code2ack
    global code2nack
    
    # -----------------------------------------------------------------------------------------
    # (Step 1/33) PARAMETERS           
    # -----------------------------------------------------------------------------------------          
    class2print = "release2image4upu"
    method2print = "release2image4upu" 

    message2print = "{(Step 1A/33) started" + "; release2image4upu-started: " + str(release2image4upu)  + "; time: " + time4xi() + "}" 
    print4xi(class2print, method2print, message2print)                                                                     #  
    # <function release2image4upu at 0x0000000003A9E8C8>
    
    message2print = "{(Step 1B/33) passed" + "; task4release2upu-to-release2image4upu: " + str(task4release2upu) + "; time: " + time4xi() + "}"
    print4xi(class2print, method2print, message2print)                                                              
    # {'COUNT2REFERENCE': 1} 

             
    code2ack  = ' '
    code2nack = ' '

    try:
        if upu2image4xi is None:
            upu2image4xi = create2image4upu()                                                                       # INITIALIZE IMAGE                 
    except:
        try:
            upu2image4xi = create2image4upu()
        except:
            upu2image4xi = {}
            message2print = "{(Step 2Z0/70) yielded" + "; upu2image4xi-yielded-by-release2image4upu: " + str(upu2image4xi) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)  
            code2nack  = code2nack + '2Z0-' 
                
    message2print = "{(Step 1A/70) try" + "; release-try-by-release2image4upu: None"  + "; time: " + time4xi() + "}"
    print4xi(class2print, method2print, message2print)
    
    if task4release2upu is None:
        task4release2upu = {}
        task4release2upu['COUNT2REFERENCE'] = 1
           
    try:
        # -----------------------------------------------------------------------------------------
        # RELEASE
        # -----------------------------------------------------------------------------------------               
        upu2image4xi['COUNT2REFERENCE'] = upu2image4xi['COUNT2REFERENCE'] - task4release2upu['COUNT2REFERENCE']
        
        message2print = "{(Step 1A/70) calculated" + "; count2reference-calculated-by-release2image4upu: " + str(upu2image4xi['COUNT2REFERENCE']) + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)
        # 
         
        if upu2image4xi['COUNT2REFERENCE'] < 0:
            upu2image4xi['COUNT2REFERENCE'] = 0
            
            message2print = "{(Step 1B/70) warning" + "; count2reference-warning-by-release2image4upu: " + " ** " + str(upu2image4xi['COUNT2REFERENCE'])  + " ** " + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)      
            # { release2image4upu : release2image4upu ; (Step 8Z1/70) warning; count2reference-warning-by-release2image4upu:  ** 0 ** ; time: 23:41:37 09/06/11 Pacific Daylight Time}    
            code2nack  = code2nack + '1B-'
        else:
            code2ack  = code2ack + '1B-'
    except:
        try:
            upu2image4xi['COUNT2REFERENCE'] = 0
            
            message2print = "{(Step 8Z1/70) warning" + "; count2reference-warning-by-release2image4upu: " + " ** " + str(upu2image4xi['COUNT2REFERENCE'])  + " ** " + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)   
            # 
            code2nack  = code2nack + '8Z1-'
            
        except:
            message2print = "{(Step 8Z2/70) yielded" + "; upu2image4xi-yielded-by-release2image4upu: " + str(upu2image4xi) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)  
            code2nack  = code2nack + '8Z2-'
                                                                                          
            
    # -----------------------------------------------------------------------------------------
    # (Step 70/70)           
    # -----------------------------------------------------------------------------------------
    upu2image4xi ['ACK2RETURN']  = code2ack
    upu2image4xi ['NACK2RETURN'] = code2nack  
    
    message2print = "{(Step 70/70) returning" + "; upu2image4xi-returning-by-release2image4upu: " + str(upu2image4xi) + "; time: " + time4xi() + "}" 
    print4xi(class2print, method2print, message2print)                                                             
    # {'CODE2ERROR': 0,
    #  'ACK2RETURN': ' ',
    #  'ROWPITCH': 800,
    #  'DATA2CALLBACK': {'MESSAGE': '{(Step 1Z1/33) failed; create2image4upu-failed ; time: 23:41:37 09/06/11 Pacific Daylight Time}',
    #  'CLASS': 'create2image4upu',
    #  'METHOD': 'create2image4upu'},
    #  'WIDTH': 800,
    #  'BUFFER': {'CODE2ERROR': 0, 
    #  'COUNT2REFERENCE': 0,
    
    #  'DATA2CALLBACK': {'MESSAGE': '{(Step 1Z1/33) failed; create2buffer4upu-failed ; time: 23:41:37 09/06/11 Pacific Daylight Time}', 'CLASS': 'create2buffer4upu', 'METHOD': 'create2buffer4upu'},
    #  'FUNCTION2CALLBACK': <function trace2buffer4upu at 0x0000000003A97348>,
    #  'POINTER2BUFFER': <mmap.mmap object at 0x0000000003CDE298>,
    #  'FLAGS': 0, 'CONTEXT': {'COUNT2REFERENCE': 0, ...
    
    
    return upu2image4xi

#</function4xi> 

# -------------------------------------------------------------------------------------------------
#<function4xi function = "retain2image4upu" usage = "
#   task4retain2image  = {}
#   task4retain2image  ['DIMENSION'] = 2 
#   task4retain2image  ['COUNT2REFERENCE'] = 1 
#   response4retain2Dimage = {}
#   print('(Step 13A/700) OF UPU2IMAGE4XI5TEST - TRYING TO RETAIN 2D IMAGE OF UPU DEVICE ', time4xi())  
#   try:                               
#       response4retain2Dimage = retain2image4upu(task4retain2image)                                                                                                 
#       print('(Step 13B/700) OF UPU2IMAGE4XI5TEST - RESPONSE4RETAIN2DIMAGE : ', response4retain2Dimage)                          
#       # 
#   except:
#       print('(Step 13Z1/700) OF UPU2IMAGE4XI5TEST *** FAILED TO RETAIN 2D IMAGE OF UPU DEVICE ', time4xi()) 
#
#">

# -------------------------------------------------------------------------------------------------   
def retain2image4upu(task4retain2upu=None):

    '''
    retains either upu2image4xi
    '''
    global upu2image4xi 
    
     
    global code2ack
    global code2nack
    
    # -----------------------------------------------------------------------------------------
    # (Step 1/70) - PARAMETERS           
    # -----------------------------------------------------------------------------------------          
    class2print = "retain2image4upu"
    method2print = "retain2image4upu"

    message2print = "{(Step 1A/33) started" + "; retain2image4upu-started: " + str(retain2image4upu)  + "; time: " + time4xi() + "}" 
    print4xi(class2print, method2print, message2print)                                                                     #  
    # <function retain2image4upu at 0x0000000003A9E948>   
    
    message2print = "{(Step 1B/33) passed" + "; task4retain2upu-to-retain2image4upu: " + str(task4retain2upu) + "; time: " + time4xi() + "}"
    print4xi(class2print, method2print, message2print)                                                              
    # {'COUNT2REFERENCE': 1, 'DIMENSION': 2}
    # {'COUNT2REFERENCE': 1, 'DIMENSION': 3}
    code2ack  = ' '
    code2nack = ' '
    
    # -----------------------------------------------------------------------------------------
    # (Step 2/70) - PROVIDE TASKS, IF NECESSARY AND CHECK GIVEN TASKS 
    # -----------------------------------------------------------------------------------------    
    if task4retain2upu is None:
        task4retain2upu = {}
        task4retain2upu['DIMENSION'] = 2
        task4retain2upu['COUNT2REFERENCE'] = 1
        message2print = "{(Step 2A/33) set" + "; task4retain2upu-set-by-retain2image4upu to: " + str(task4retain2upu)  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)
        #           
    else:
        pass
        
    if 'DIMENSION' in task4retain2upu.keys():
        if task4retain2upu['DIMENSION'] == 2:
            pass
        else:
            task4retain2upu['DIMENSION'] = 2 
            message2print = "{(Step 2B/33) set" + "; dimension-set-by-retain2image4upu to: " + str(task4retain2upu['DIMENSION'])  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)
            #                       
    else:
        task4retain2upu['DIMENSION'] = 2
    
    # -----------------------------------------------------------------------------------------
    # (Step 3/70) - CHECK COUNT2REFERENCE
    # -----------------------------------------------------------------------------------------         
    if 'COUNT2REFERENCE' in task4retain2upu.keys():
        pass
    else:
        task4retain2upu['COUNT2REFERENCE'] = 1
        message2print = "{(Step 3A/33) set" + "; count2reference-set-by-retain2image4upu to: " + str(task4retain2upu['COUNT2REFERENCE'])  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)
        # 
                                   
    if task4retain2upu['COUNT2REFERENCE'] < 1:
        task4retain2upu['COUNT2REFERENCE'] = 1
        message2print = "{(Step 3B/33) set" + "; count2reference-set-by-retain2image4upu to: " + str(task4retain2upu['COUNT2REFERENCE'])  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)
        #           
        
    # -----------------------------------------------------------------------------------------
    # (Step 4/70) - INITIALIZE 2D IMAGE IF NECESSARY
    # -----------------------------------------------------------------------------------------
    try:
        if upu2image4xi is None:
            upu2image4xi = create2image4upu()
            message2print = "{(Step 4A/70) created" + "; upu2image4xi-created-by-retain2image4upu: " + str(upu2image4xi) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)                                                                                                          
    except:
        try:
            upu2image4xi = create2image4upu()
            message2print = "{(Step 4B/70) created" + "; upu2image4xi-created-by-retain2image4upu: " + str(upu2image4xi) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)                 
        except:
            upu2image4xi = {}
            message2print = "{(Step 4Z/70) yielded" + "; upu2image4xi-yielded-by-retain2image4upu: " + str(upu2image4xi) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)  
            code2nack  = code2nack + '4Z-' 


    # -----------------------------------------------------------------------------------------
    # (Step 5/70) - RETAIN
    # -----------------------------------------------------------------------------------------             
    try:
        upu2image4xi['COUNT2REFERENCE'] = upu2image4xi['COUNT2REFERENCE'] + task4retain2upu['COUNT2REFERENCE']
        message2print = "{(Step 6A/70) calculated" + "; count2reference-calculated-by-retain2image4upu: " + str(upu2image4xi['COUNT2REFERENCE']) + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)
        #              
        if upu2image4xi['COUNT2REFERENCE'] <= 0:
            upu2image4xi['COUNT2REFERENCE'] = 1
            message2print = "{(Step 6Z1/70) warning" + "; count2reference-warning-by-retain2image4upu: " + " ** " + str(upu2image4xi['COUNT2REFERENCE'])  + " ** " + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)      
            code2nack  = code2nack + '6Z1-'
        else:
            code2ack  = code2ack + '6C-'

    except:
        try:
            upu2image4xi['COUNT2REFERENCE'] = 1
        
            message2print = "{(Step 7Z1/70) warning" + "; count2reference-warning-by-retain2image4upu: " + " ** " + str(upu2image4xi['COUNT2REFERENCE'])  + " ** " + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)   
            code2nack  = code2nack + '7Z1-'
        
        except:
            message2print = "{(Step 7Z2/70) yielded" + "; upu2image4xi-yielded-by-retain2image4upu: " + str(upu2image4xi) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)  
            code2nack  = code2nack + '7Z2-'
        
                                                                                          
            
    # -----------------------------------------------------------------------------------------
    # (Step 70/70) - RETURN           
    # -----------------------------------------------------------------------------------------
    
    upu2image4xi ['ACK2RETURN']  = code2ack
    upu2image4xi ['NACK2RETURN'] = code2nack     
    message2print = "{(Step 70A/70) returning" + "; upu2image4xi-returning-by-retain2image4upu: " + str(upu2image4xi) + "; time: " + time4xi() + "}" 
    print4xi(class2print, method2print, message2print)                                                             
    # {'CODE2ERROR': 0, 'ACK2RETURN': ' 6C-', 'ROWPITCH': 800, 'DATA2CALLBACK': {'MESSAGE': '{(Step 1Z1/33) failed; create2image4upu-failed ; time: 23:25:02 10/05/11 Pacific Daylight Time}', 'CLASS': 'create2image4upu', 'METHOD': 'create2image4upu'}, 'WIDTH': 800, 'BUFFER': {'CODE2ERROR': 0, 
    #                   'COUNT2REFERENCE': 0, ...
    # 'COUNT2REFERENCE': 1, 
    # 'FLAGS': 0, 'CONTEXT': ...
    
    return upu2image4xi

#</function4xi> 


# -------------------------------------------------------------------------------------------------
#<function4xi function = "get2imageinfo4upu" usage = "
#   #1
#   print('(Step 3A/33) OF UPU2IMAGE4XI5TEST - TRYING TO GET 2D IMAGE INFO FOR UPU', time4xi())  
#   try:
#       imageinfo2upu = get2imageinfo4upu()                                                                       
#       print('(Step 3B/33) OF UPU2IMAGE4XI5TEST - GOT 2D IMAGE INFO FOR UPU: ', imageinfo2upu) 
#   except:
#       print('(Step 3Z/33) OF UPU2IMAGE4XI5TEST *** FAILED TO GET 2D IMAGE INFO FOR UPU', time4xi())
#   #2
#   task4getinfo2image = {}
#   task4getinfo2image ['DIMENSION'] = 2
#   task4getinfo2image ['WIDTH'] = 0
#   task4getinfo2image ['HEIGHT'] = 0
#   response4getinfo2image = {}
#   print('(Step 12C/33) OF UPU2IMAGE4XI5TEST - TRYING TO GET SELECTED 3D IMAGE INFO OF UPU DEVICE ', time4xi())    
# 
#   try:                               
#       response4getinfo2image = get2imageinfo4upu (task4getinfo2image)                                                                                             
#       print('(Step 12D/33) OF UPU2IMAGE4XI5TEST - RESPONSE4RELEASE2IMAGE : ', response4getinfo2image)                           
#       #
#   except:
#       print('(Step 12Z2/33) OF UPU2IMAGE4XI5TEST *** FAILED TO GET SELECTED 3D IMAGE INFO OF UPU DEVICE ', time4xi()) 
#            
#">

# -------------------------------------------------------------------------------------------------   
def get2imageinfo4upu(task4getinfo2image=None):

    '''
    gets upu2image4xi
    '''
    
    global upu2image4xi
    
    
    # -----------------------------------------------------------------------------------------
    # (Step 1/33)           
    # -----------------------------------------------------------------------------------------          
    class2print = "get2imageinfo4upu"
    method2print = "get2imageinfo4upu"

    message2print = "{(Step 0A/33) started" + "; get2imageinfo4upu-started: " + str(get2imageinfo4upu)  + "; time: " + time4xi() + "}"
    print4xi(class2print, method2print, message2print)
    # <function get2imageinfo4upu at 0x0000000003A9E9C8>                                                              
    
    message2print = "{(Step 0A/33) passed" + "; task4getinfo2image-passed-to-get2imageinfo4upu: " + str(task4getinfo2image) + "; time: " + time4xi() + "}"
    print4xi(class2print, method2print, message2print)   
    # {'WIDTH': 0, 'DIMENSION': 2, 'HEIGHT': 0}
    # {'WIDTH': 0, 'DEPTH': 0, 'DIMENSION': 3, 'HEIGHT': 0}
    # {'DOMAIN': 'ALL', 'DIMENSION': 2}
    
    image2upu = {}
    try: 
        if upu2image4xi is None:
            # -------------------------------------------------------------------------------------
            # INITIALIZE 2D IMAGE FOR UPU           
            # -------------------------------------------------------------------------------------           
            image2upu = create2image4upu()
            message2print = "{(Step 1B2/33) initialized" + "; image2upu-initialized-by-get2imageinfo4upu to: " + str(image2upu)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)
            # 
        else:
            message2print = "{(Step 1B2/33) using" + "; upu2image4xi-using-by-get2imageinfo4upu: " + str(upu2image4xi)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print) 
            # {'CODE2ERROR': 0, 
            #  'ACK2RETURN': ' 1D-1E-2C-2B-3B-4B-5B-6B-7B-8B-9B-10B-11B-14B-15B-16B-17B-17D-18B-18C-21A1-22B1-22C1-23B-24B-25B-26B-27B-28B-29B-30B-31B-34H-35H-36B-37B-38B-', 
            #  'ROWPITCH': 800, 
            #  'DATA2CALLBACK': {'MESSAGE': '{(Step 1Z1/33) failed; create2image4upu-failed ; time: 09:05:31 10/05/11 Pacific Daylight Time}', 
            #  'CLASS': 'create2image4upu', 
            #  'METHOD': 'create2image4upu'}, 
            #  'WIDTH': 800, 
            #  'BUFFER': ...
            
            
        # -------------------------------------------------------------------------------------
        # RETURN THE WHOLE OR PART OF THE IMAGE INFO OF UPU          
        # -------------------------------------------------------------------------------------     
        dimension2xi = 0
        
        if task4getinfo2image is None:
            task4getinfo2image = {}
            task4getinfo2image ['DIMENSION'] = 2 
            task4getinfo2image ['DOMAIN'] = 'ALL'
            image2upu = upu2image4xi
            message2print = "{(Step 1C/33) using" + "; image2upu-got-by-get2imageinfo4upu: " + str(image2upu)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)
            # {'CODE2ERROR': 0, 
            #  'DOMAIN': 'ALL', 
            #  'CONTEXT': ...
            
        else:
            pass    

        keys2task4imageinfo = task4getinfo2image.keys()
        if 'DIMENSION' in keys2task4imageinfo:
            dimension2xi = task4getinfo2image['DIMENSION']
            if dimension2xi == 2:
                pass
            else:
                dimension2xi = 2    
        else:
            pass
            
        if 'DOMAIN' in keys2task4imageinfo:
            domain2xi = task4getinfo2image['DOMAIN']
            if domain2xi == 'ALL':
                pass
            else:
                domain2xi = 'ALL'

            keys2domain = upu2image4xi.keys()
            for key2domain in keys2domain:
                task4getinfo2image[key2domain] = upu2image4xi[key2domain]
                    
            message2print = "{(Step 1C/33) using" + "; task4getinfo2image-using-by-get2imageinfo4upu: " + str(task4getinfo2image)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)                                          
        else:
            pass            
            
                    

        keys2task = task4getinfo2image.keys()
        for key2task in keys2task:
            try:
                message2print = "{(Step 2D2/33) using" + "; key2task-using-by-get2image4upu: " + str(key2task)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)  
                # WIDTH; DIMENSION; HEIGHT  
                image2upu [key2task] = upu2image4xi[key2task]
                
                message2print = "{(Step 2D2/33) set" + "; info-of-image2upu-set-by-get2imageinfo4upu to: " + str(image2upu [key2task])  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)
                # 800; 800  
            except:
                message2print = "{(Step 2Z/70) yielded" + "; key2task-yielded-by-get2image4upu: " + str(key2task) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)                               
                # DIMENSION;
                                   
                message2print = "{(Step 2Z2/70) using" + "; task4getinfo2image-using-by-get2image4upu: " + str(task4getinfo2image)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)  
                # 
                
                message2print = "{(Step 2Z3/70) using" + "; upu2image4xi-using-by-get2image4upu: " + str(upu2image4xi)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)
                #                           
    except:
        try:
            # -------------------------------------------------------------------------------------
            # INITIALIZE           
            # -------------------------------------------------------------------------------------           
            image2upu = create2image4upu()
            message2print = "{(Step 1F/33) initialized" + "; image2upu-initialized-by-get2imageinfo4upu to: " + str(image2upu)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)
            #
            if task4getinfo2image is None:
                image2upu = upu2image4xi
                message2print = "{(Step 1G/33) got" + "; image2upu-got-by-get2imageinfo4upu: " + str(image2upu)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)
                # 
            else:
                keys2task = task4getinfo2image.keys()
                for key2task in keys2task:
                    try:
                        message2print = "{(Step HD2/33) using" + "; key2task-using-by-get2image4upu: " + str(key2task)  + "; time: " + time4xi() + "}"
                        print4xi(class2print, method2print, message2print)  
                        # 
                        image2upu [key2task] = upu2image4xi[key2task]
                    
                        message2print = "{(Step HD2/33) set" + "; info-of-image2upu-set-by-get2imageinfo4upu to: " + str(image2upu [key2task])  + "; time: " + time4xi() + "}"
                        print4xi(class2print, method2print, message2print)
                        #   
                    except:
                        message2print = "{(Step HZ/70) yielded" + "; key2task-yielded-by-get2image4upu: " + str(key2task) + "; time: " + time4xi() + "}"
                        print4xi(class2print, method2print, message2print)                            
        except:
            image2upu = {}
            message2print = "{(Step 1Z3/70) yielded" + "; image2upu-yielded-by-get2image4upu: " + str(image2upu) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)  
                        
    message2print = "{(Step 33/33) returning" + "; image2upu-returning-by-get2imageinfo4upu: " + str(image2upu) + "; time: " + time4xi() + "}"  
    print4xi(class2print, method2print, message2print)
    # 2D: {'WIDTH': 800, 'HEIGHT': 800}
    # 3D: {'WIDTH': 800, 'DEPTH': 800, 'HEIGHT': 800}
    # 2D ALL: {'CODE2ERROR': 0, 
    #  'ROWPITCH': 800, 
    #  'DATA2CALLBACK': {'MESSAGE': '{(Step 1Z1/33) failed; create2image4upu-failed ; time: 09:05:31 10/05/11 Pacific Daylight Time}', 
    #  'CLASS': 'create2image4upu', 
    #  'METHOD': 'create2image4upu'}, 
    #  'FORMAT': {'TYPE': 'CL_UNORM_INT8', 
    #  'ORDER': 'CL_RGBA'}, 
    #  'BUFFER': ...
    
    # 3D ALL: {'PIPE': (<read-write PipeConnection, handle 520>, <read-write PipeConnection, handle 524>), 
    #  'ROWPITCH': 800, 
    #  'DATA2CALLBACK': {'MESSAGE': '{(Step 1Z1/33) failed; create3image4upu-failed ; time: 09:05:31 10/05/11 Pacific Daylight Time}', 
    #  'CLASS': 'create3image4upu', 
    #  'METHOD': 'create3image4upu'}, 
    #  'FORMAT': {'TYPE': 'CL_UNORM_INT8', 
    #  'ORDER': 'CL_RGBA'}, 
    #  'BUFFER': ....
    #  'HEIGHT': 800, 
    #  'QUEUE': <queue4xi.Queue4Xi object at 0x0000000004504400>, 
    #  'WIDTH': 800, 
    #  'DEPTH': 800, 
    #  'FLAGS': 0, 
    #  'CONTEXT': ...
    
    return image2upu
            
#</function4xi> 
 

# -------------------------------------------------------------------------------------------------
#<function4xi function = "get2supportedimageformats4upu" usage = "
#
#   print('(Step 3A/33) OF UPU2IMAGE4XI5TEST - TRYING TO GET SUPPORTED IMAGE FORMATS FOR UPU', time4xi())  
#   try:
#       imageformats = get2supportedimageformats4upu()                                                                       
#       print('(Step 3B/33) OF UPU2IMAGE4XI5TEST - GOT SUPPORTED IMAGE FORMATS FOR UPU: ', imageformats) 
#   except:
#       print('(Step 3Z/33) OF UPU2IMAGE4XI5TEST *** FAILED TO GET SUPPORTED IMAGE FORMATS FOR UPU', time4xi())
#            
#">

# -------------------------------------------------------------------------------------------------   
def get2supportedimageformats4upu(task4supportedimageformats2upu=None):  

    '''
    gets supported image formats for upu
    '''
    
    global upu2image4xi

    
    # -----------------------------------------------------------------------------------------
    # (Step 1/33)           
    # -----------------------------------------------------------------------------------------          
    class2print = "get2supportedimageformats4upu"
    method2print = "get2supportedimageformats4upu"

    message2print = "{(Step 0A/33) started" + "; get2supportedimageformats4upu-started: " + str(get2supportedimageformats4upu)  + "; time: " + time4xi() + "}"
    print4xi(class2print, method2print, message2print) 
    # <function get2supportedimageformats4upu at 0x0000000003BBE8C8>                                                          
    
    message2print = "{(Step 0B/33) passed" + "; task4supportedimageformats2upu-passed-to-get2supportedimageformats4upu: " + str(task4supportedimageformats2upu) + "; time: " + time4xi() + "}"
    print4xi(class2print, method2print, message2print)   
    # None; {'DIMENSION': 3} 
    
    
    if task4supportedimageformats2upu is None:
        task4supportedimageformats2upu = {} 
        task4supportedimageformats2upu['DIMENSION'] = 2                                                             # 2D image is the default
    
    dimension2xi = 2
    try:
        keys2supportedimageformats = task4supportedimageformats2upu.keys()    
        if 'DIMENSION' in keys2supportedimageformats:
            dimension2xi = task4supportedimageformats2upu['DIMENSION']
            if ((dimension2xi <=0) or (dimension2xi > 2)): 
                dimension2xi = 2
        else:
            dimension2xi = 2
    except:
        pass        
    
    message2print = "{(Step 1A/33) using" + "; dimension2xi-using-by-dimension2xi: " + str(dimension2xi)  + "; time: " + time4xi() + "}"
    print4xi(class2print, method2print, message2print)    
    # 2; 3
                        
    supportedimageformats2upu = {} 
    try: 
        if upu2image4xi is None:
            # -------------------------------------------------------------------------------------
            # INITIALIZE           
            # -------------------------------------------------------------------------------------           
            image2upu = create2image4upu()                                                                          # create upu2image4xi
            message2print = "{(Step 1B/33) initialized" + "; image2upu-initialized-by-get2supportedimageformats4upu to: " + str(image2upu)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)
            #                
        else:
            pass

                        
        # -------------------------------------------------------------------------------------
        # SUPPORTED IMAGE FORMATS FOR UPU       
        # -------------------------------------------------------------------------------------     
        supportedimageformats2upu = upu2image4xi['FORMAT']
        message2print = "{(Step 1E/33) got" + "; supportedimageformats2upu-got-by-get2supportedimageformats4upu: " + str(supportedimageformats2upu)  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)           
        # {'TYPE': 'CL_UNORM_INT8', 'ORDER': 'CL_RGBA'} 
            
    except:
        try:
            # -------------------------------------------------------------------------------------
            # INITIALIZE           
            # -------------------------------------------------------------------------------------           
            image2upu = create2image4upu()
            message2print = "{(Step 1F1/33) initialized" + "; image2upu-initialized-by-get2supportedimageformats4upu to: " + str(image2upu)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)
            # 
     
            
            # -------------------------------------------------------------------------------------
            # SUPPORTED IMAGE FORMATS FOR UPU       
            # -------------------------------------------------------------------------------------     
            supportedimageformats2upu = upu2image4xi['FORMAT']
            message2print = "{(Step 1H/33) got" + "; supportedimageformats2upu-got-by-get2supportedimageformats4upu: " + str(supportedimageformats2upu)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)           
            #  
                                              
        except:
            message2print = "{(Step 1Z/70) yielded" + "; supportedimageformats2upu-yielded-by-get2image4upu: " + str(supportedimageformats2upu) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)  
                        
    message2print = "{(Step 33/33) returning" + "; supportedimageformats2upu-returning-by-get2supportedimageformats4upu: " + str(supportedimageformats2upu) + "; time: " + time4xi() + "}" 
    print4xi(class2print, method2print, message2print)
    # 2: {'TYPE': 'CL_UNORM_INT8', 'ORDER': 'CL_RGBA'}
    # 3: {'TYPE': 'CL_UNORM_INT8', 'ORDER': 'CL_RGBA'}
    
    return supportedimageformats2upu
            
#</function4xi> 

 



# -------------------------------------------------------------------------------------------------
#<function4xi function = "wait2image4upu" usage = "
#   print('=' *33)          
#   print('(Step 16A/700) OF UPU2IMAGE4XI5TEST - TRYING TO WAIT 2D IMAGE FOR UPU ', time4xi())   
#   print('=' *33)      
#   try:
#       response4wait2upu = wait2image4upu()
#       keys2response = response4wait2upu.keys()
#       print('(Step 16B/700) OF UPU2IMAGE4XI5TEST - WAITED 2D IMAGE FOR UPU ', time4xi()) 
#       for key2response in sorted(keys2response):
#           print(key2response, ' : ', response4wait2upu[key2response])
#           # ACK2RETURN  :   6C-
#           # BUFFER  :  {'CODE2ERROR': 0 ...
#           # CALLBACK  :  <function trace2image4upu at 0x0000000004E93B48>
#           # CODE2ERROR  :  0
#           # COMMANDQUEUE  :  {'CODE2ERROR': 0, ...
#           # CONTEXT  :  {'COUNT2REFERENCE': 0 ...
#           # COUNT2REFERENCE  :  1
#           # DATA2CALLBACK  :  {'MESSAGE': '{(Step 1Z1/33) failed; create2image4upu-failed ; time: 02:00:43 10/06/11 Pacific Daylight Time}', 'CLASS': 'create2image4upu', 'METHOD': 'create2image4upu'}
#           # DEPTH  :  0
#           # DEVICE  :  {'RESOLUTION2TIMER4PROFILING': 318, 'ID2PLATFORM': '0000000001509568', 'PROFILING2QUEUE': 'ACK',...
#           # EMPTY2QUEUE  :  True
#           # EVENT  :  {'CODE2ERROR': 0, 'DATA2CALLBACK':...
#           # FLAGS  :  0
#           # FORMAT  :  {'TYPE': 'CL_UNORM_INT8', 'ORDER': 'CL_RGBA'}
#           # FULL2QUEUE  :  False
#           # FUNCTION2CALLBACK  :  <function trace2image4upu at 0x0000000004E93B48>
#           # HEIGHT  :  800
#           # NACK2RETURN  :   
#           # PIPE  :  (<read-write PipeConnection, handle 556>, <read-write PipeConnection, handle 560>)
#           # PLATFORM  :  {'list2platforms': ['DragonXi'], 'ack2return': ' 2C-2E-2F-2G-', 'nack2return': ' ', 'available2platform': 1, 'max2entries': 1}
#           # POINTER  :  <mmap.mmap object at 0x0000000004351500>
#           # QUEUE  :  <queue4xi.Queue4Xi object at 0x00000000044E95C0>
#           # ROWPITCH  :  800
#           # SIZE2QUEUE  :  2147483646 
#   except:
#       print('(Step 16Z1/700) OF UPU2IMAGE4XI5TEST *** FAILED TO WAIT 2D IMAGE FOR UPU: ', wait2upu)            
#">

# -------------------------------------------------------------------------------------------------   
def wait2image4upu(task4wait2image=None):

    '''
    waits upu2image4upu
    '''
    
    global upu2image4xi
    
    # -----------------------------------------------------------------------------------------
    # (Step 1/33)           
    # -----------------------------------------------------------------------------------------          
    class2print = "wait2image4upu"
    method2print = "wait2image4upu"

    message2print = "{(Step 1A1/33) started" + "; wait2image4upu-started: " + str(wait2image4upu)  + "; time: " + time4xi() + "}"
    print4xi(class2print, method2print, message2print)
    #  
    
    message2print = "{(Step 1A2/33) passed" + "; task4wait2image-passed-by-wait2image4upu to: " + str(task4wait2image)  + "; time: " + time4xi() + "}"
    print4xi(class2print, method2print, message2print)
    # 
    
    try: 
        if upu2image4xi is None:
            # -------------------------------------------------------------------------------------
            # INITIALIZE - KEEP SYSTEM RUNNING           
            # -------------------------------------------------------------------------------------           
            upu2image4xi = create2image4upu()
            
            message2print = "{(Step 1B/33) created" + "; upu2image4xi-created-by-wait2image4upu: " + str(upu2image4xi)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)
            #
                                                
            # -------------------------------------------------------------------------------------
            # WAIT         
            # -------------------------------------------------------------------------------------             
            queue2image4upu = None
            try:
                queue2image4upu = upu2image4xi['QUEUE']
                message2print = "{(Step 1C/33) got" + "; queue2image4upu-got-by-wait2image4upu to: " + str(queue2image4upu)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)
                #
            except:     
                message2print = "{(Step 1Z1/70) yielded" + "; upu2xi-yielded-by-wait2image4upu: " + str(upu2xi) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)    
            
            size2image4upu = 0
            try:                    
                size2image4upu = queue2image4upu.qsize()
                message2print = "{(Step 1D/33) got" + "; size2image4upu-got-by-wait2image4upu to: " + str(size2image4upu)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)                
                #
                               
                
            except:     
                message2print = "{(Step 1Z2/70) yielded" + "; upu2xi-yielded-by-wait2image4upu: " + str(upu2xi) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)
             
            empty2image4upu = True       
            try:                    
                empty2image4upu= queue2image4upu.empty()
                message2print = "{(Step 1E/33) got" + "; empty2image4upu-got-by-wait2image4upu to: " + str(empty2image4upu)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)                
                #
            except:     
                message2print = "{(Step 1Z3/70) yielded" + "; empty2image4upu-yielded-by-wait2image4upu: " + str(empty2image4upu) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)    
             
            full2image4upu = False       
            try:                                
                full2image4upu = queue2image4upu.full()
                message2print = "{(Step 1F/33) got" + "; full2image4upu-got-by-wait2image4upu to: " + str(full2image4upu)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)                
                #                
            except:     
                message2print = "{(Step 1Z4/70) yielded" + "; full2image4upu-yielded-by-wait2image4upu: " + str(full2image4upu) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)   
                            
            message2print = "{(Step 1G/33) finished" + "; upu2image4xi-finished-by-wait2image4upu to: " + str(upu2image4xi)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)
            #
                            
        else:
            # -------------------------------------------------------------------------------------
            # USE THE EXISTING IMAGE FOR UPU          
            # -------------------------------------------------------------------------------------     
            
            message2print = "{(Step 1H/33) used" + "; upu2image4xi-used-by-wait2image4upu: " + str(upu2image4xi)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)
            # 
            
            # -------------------------------------------------------------------------------------
            # WAIT         
            # -------------------------------------------------------------------------------------             
            queue2image4upu = None
            try:
                queue2image4upu = upu2image4xi['QUEUE']
                message2print = "{(Step 1I/33) got" + "; queue2image4upu-got-by-wait2image4upu to: " + str(queue2image4upu)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)
                # 
            except:     
                message2print = "{(Step 1Z5/70) yielded" + "; upu2xi-yielded-by-wait2image4upu: " + str(upu2xi) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)
                
            if task4wait2image is None:
                pass
            else:
                # -------------------------------------------------------------------------------------
                # IMAGE GIVEN AS PARAMETER          
                # -------------------------------------------------------------------------------------               
                message2print = "{(Step 1J1/33) passed" + "; task4wait2image-passed-by-wait2image4upu to: " + str(task4wait2image)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)
                # 
                queue2task = {}    
                try:
                    queue2task = task4wait2image ['QUEUE']
                    message2print = "{(Step 1J2/33) set" + "; queue2task-set-by-wait2image4upu to: " + str(queue2task)  + "; time: " + time4xi() + "}"
                    print4xi(class2print, method2print, message2print)
                    # 
                except:     
                    message2print = "{(Step 1Z61/70) yielded" + "; queue2task-yielded-by-wait2image4upu: " + str(queue2task) + "; time: " + time4xi() + "}"
                    print4xi(class2print, method2print, message2print)
                
                try:                        
                    if queue2task == queue2image4upu:
                        # -------------------------------------------------------------------------------------
                        # ACCEPTED - GIVEN QUEUE IS THE SAME          
                        # -------------------------------------------------------------------------------------                     
                        message2print = "{(Step 1K/33) accepted" + "; queue2task-accepted-by-wait2image4upu: " + str(queue2task) + "; time: " + time4xi() + "}"
                        print4xi(class2print, method2print, message2print)
                        # 
                    else:
                        try:
                            message2print = "{(Step 1L/33) warned" + "; queue2task-warned-by-wait2image4upu: " + " ** " + str(queue2task)  + " ** " + "; time: " + time4xi() + "}"
                            print4xi(class2print, method2print, message2print) 
                            # 
                        
                            message2print = "{(Step 1M/33) warned" + "; queue2image4upu-warned-by-wait2image4upu: " + " ** " + str(queue2image4upu)  + " ** " + "; time: " + time4xi() + "}"
                            print4xi(class2print, method2print, message2print)    
                            # 
                        except:
                            pass
                                                     
                except:     
                    message2print = "{(Step 1Z62/70) yielded" + "; queue2task-yielded-by-wait2image4upu: " + str(queue2task) + "; time: " + time4xi() + "}"
                    print4xi(class2print, method2print, message2print)                   
            
            size2queue4upu = 0        
            try:                    
                size2queue4upu = queue2image4upu.qsize()
                message2print = "{(Step 1N1/33) got" + "; size2queue4upu-got-by-wait2image4upu to: " + str(size2queue4upu)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)                
                # 
                
                upu2image4xi ['SIZE2QUEUE'] = size2queue4upu
                message2print = "{(Step 1N2/33) set" + "; size2queue4image-got-by-wait2image4upu to: " + str(upu2image4xi ['SIZE2QUEUE'])  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)                  
                # 
            except:     
                message2print = "{(Step 1Z7/70) yielded" + "; size2queue4upu-yielded-by-wait2image4upu: " + str(size2queue4upu) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)    
            
            empty2queue4upu = False
            try:                    
                empty2queue4upu = queue2image4upu.empty()
                message2print = "{(Step 1O1/33) got" + "; empty2queue4upu-got-by-wait2image4upu to: " + str(empty2queue4upu)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)                
                # 
                
                upu2image4xi ['EMPTY2QUEUE'] = empty2queue4upu
                message2print = "{(Step 1O2/33) set" + "; empty2queue4image-got-by-wait2image4upu to: " + str(upu2image4xi ['EMPTY2QUEUE'])  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)
                #               
            except:     
                message2print = "{(Step 1Z8/70) yielded" + "; empty2queue4upu-yielded-by-wait2image4upu: " + str(empty2queue4upu) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)    
            
            full2queue4upu = False
            try:                                
                full2queue4upu = queue2image4upu.full()
                message2print = "{(Step 1P1/33) got" + "; full2queue4upu-got-by-wait2image4upu to: " + str(full2queue4upu)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)                
                # 
                 
                upu2image4xi ['FULL2QUEUE'] = full2queue4upu
                message2print = "{(Step 1P2/33) set" + "; full2queue4image-got-by-wait2image4upu to: " + str(upu2image4xi ['FULL2QUEUE'])  + "; time: " + time4xi() + "}" 
                print4xi(class2print, method2print, message2print)
                #                          
            except:     
                message2print = "{(Step 1Z9/70) yielded" + "; full2queue4upu-yielded-by-wait2image4upu: " + str(full2queue4upu) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)    
            
            state2queue4upu = None
            try:                                
                state2queue4upu = queue2image4upu.__getstate__()
                message2print = "{(Step 1P3/33) got" + "; state2queue4upu-got-by-wait2image4upu to: " + str(state2queue4upu)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)                
                # 
                 
                upu2image4xi ['STATE2QUEUE'] = state2queue4upu
                message2print = "{(Step 1P4/33) set" + "; state2queue4image-set-by-wait2image4upu to: " + str(upu2image4xi ['STATE2QUEUE'])  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)
                #           
            except:     
                message2print = "{(Step 1Z10/70) yielded" + "; state2queue4upu-yielded-by-wait2image4upu: " + str(state2queue4upu) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)               
                
                                                            
            message2print = "{(Step 1Q/33) finished" + "; upu2image4xi-finished-by-wait2image4upu: " + str(upu2image4xi)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)
            # 
    except:
        try:
            # -------------------------------------------------------------------------------------
            # INITIALIZE           
            # -------------------------------------------------------------------------------------           
            upu2image4upu = create2image4upu()
            message2print = "{(Step 1R/33) got" + "; upu2image4xi-got-by-wait2image4upu to: " + str(upu2image4xi)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)
            #  
            
            # -------------------------------------------------------------------------------------
            # WAIT         
            # -------------------------------------------------------------------------------------             
            queue2image4upu = None
            try:
                queue2image4upu = upu2image4xi['QUEUE']
                message2print = "{(Step 1S/33) got" + "; queue2image4upu-got-by-wait2image4upu to: " + str(queue2image4upu)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)
                #
            except:     
                message2print = "{(Step 1Z10/70) yielded" + "; queue2image4upu-yielded-by-wait2image4upu: " + str(queue2image4upu) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)
                
            if task4wait2image is None:
                pass
            else:
                # -------------------------------------------------------------------------------------
                # IMAGE QUEUE GIVEN AS PARAMETER          
                # -------------------------------------------------------------------------------------               
                queue2task = None    
                try:
                    queue2task = task4wait2image ['QUEUE']
                    message2print = "{(Step 1T/33) passed" + "; queue2task-passed-by-wait2image4upu to: " + str(queue2task)  + "; time: " + time4xi() + "}"
                    print4xi(class2print, method2print, message2print)
                    #
                    
                    if queue2task == queue2image4upu:
                        # -------------------------------------------------------------------------------------
                        # ACCEPTED - GIVEN QUEUE IS THE SAME          
                        # -------------------------------------------------------------------------------------                     
                        message2print = "{(Step 1U/33) accepted" + "; queue2task-accepted-by-wait2image4upu: " + str(queue2task) + "; time: " + time4xi() + "}"
                        print4xi(class2print, method2print, message2print)
                        # 
                    else:
                        message2print = "{(Step 1X/33) warned" + "; queue2task-warned-by-wait2image4upu: " + " ** " + str(queue2task)  + " ** " + "; time: " + time4xi() + "}"
                        print4xi(class2print, method2print, message2print) 
                        # 
                        
                        message2print = "{(Step 1Y1/33) warned" + "; queue2image4upu-warned-by-wait2image4upu: " + " ** " + str(queue2image4upu)  + " ** " + "; time: " + time4xi() + "}" 
                        print4xi(class2print, method2print, message2print)    
                        #                      
                except:     
                    message2print = "{(Step 1Z11/70) yielded" + "; queue2task-yielded-by-wait2image4upu: " + str(queue2task) + "; time: " + time4xi() + "}"
                    print4xi(class2print, method2print, message2print) 
            
            size2image4upu = 0                             
            try:                    
                size2image4upu = queue2image4upu.qsize()
                message2print = "{(Step 1Y2/33) got" + "; size2image4upu-got-by-wait2image4upu to: " + str(size2image4upu)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)                
                #
            except:     
                message2print = "{(Step 1Z12/70) yielded" + "; size2image4upu-yielded-by-wait2image4upu: " + str(size2image4upu) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print) 
             
            empty2image4upu = True          
            try:                    
                empty2image4upu = queue2image4upu.empty()
                message2print = "{(Step 1Y3/33) got" + "; empty2image4upu-got-by-wait2image4upu to: " + str(empty2image4upu)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)                
                #
            except:     
                message2print = "{(Step 1Z13/70) yielded" + "; upu2xi-yielded-by-wait2image4upu: " + str(upu2xi) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print) 
             
            full2image4upu = False          
            try:                                
                full2image4upu= queue2image4upu.full()
                message2print = "{(Step 1Y4/33) got" + "; full2image4upu-got-by-wait2image4upu to: " + str(full2image4upu)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)                
                #                
            except:     
                message2print = "{(Step 1Z14/70) yielded" + "; full2image4upu-yielded-by-wait2image4upu: " + str(full2image4upu) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)    
                                                        
            message2print = "{(Step 1Y5/33) finished" + "; upu2image4xi-finished-by-wait2image4upu: " + str(upu2image4xi)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)                                    
        except:
            upu2image4upu= {}
            message2print = "{(Step 1Z15/70) yielded" + "; upu2image4xi-yielded-by-wait2image4upu: " + str(upu2image4xi) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)  
                        
    message2print = "{(Step 33/33) returning" + "; upu2image4xi-returning-by-wait2image4upu: " + str(upu2image4xi) + "; time: " + time4xi() + "}" 
    print4xi(class2print, method2print, message2print)
    # {'SIZE2QUEUE': 2147483646, 
    #  'FORMAT': {'TYPE': 'CL_UNORM_INT8', 
    #  'ORDER': 'CL_RGBA'}, 
    #  'QUEUE': <queue4xi.Queue4Xi object at 0x0000000004420E10>, 
    #  'WIDTH': 800, 
    #  'CONTEXT': ...
    
    return upu2image4xi
    
   
#</function4xi> 
                                               
# =================================================================================================
#<class4xi class = "UPU2Image4Xi" base = "object" child = "None"
#   synopsis = "offers methods for UPU 2D images" usage = "2D Image4Xi of UPU"
#>

# =================================================================================================
class UPU2Image4Xi(object): 
    ''' 
    UPU2Image4Xi offers methods for UPU 2D images
    '''            

    #                                        
    # ---------------------------------------------------------------------------------------------
    # sharable class variables
    # ---------------------------------------------------------------------------------------------      
    #<share4xi share = " 
    classvariable2upu = cpu_corecount4xi()
    #"
    #/>
        
    # ---------------------------------------------------------------------------------------------
    # Public API of this class
    # ---------------------------------------------------------------------------------------------    
    #<expose4xi expose = "
    _exposed_ = (
            'create2image',                    # ACK 5.31   p76    
            'copy2image',                      # ACK 5.31   p88
            'copy2imagetobuffer',              # ACK 5.3.4  p90
            'copy2buffertoimage',              # ACK 5.3.4  p92
            'read2image',                      # ACK 5.3.3  p84
            'write2image',                     # ACK 5.3.3  p84            
            'map2image',                       # ACK 5.3.5  p94            
            'get2imageinfo',                   # ACK 5.3.6  p97
            'get2supportedimageformats',       # ACK 5.3.2  p82 nb: a separate method to get format whereas using get2imageinfo you can access any element by key such as 'FORMAT'   
            'retain2image',                    # ACK 5.4.1  p99 nb: for compability to OpenCL - you can get any element of dictionary by get2imageinfo
            'release2image',                   # ACK 5.4.1  p99
            'trace2image',                     # ACK modernized 
            'set2imagecallback',               # ACK modernized
            'set2image',                       # ACK modernized nb: with memorymap features you can process 2D images as files!                    
            'wait2image',                      # ACK modernized nb: checks Queue object and thereby generates delay   
            'sense',                           # ACK modernized  
            'reason',                          # ACK modernized  
            'ace')                             # ACK modernized           
    #"
    #/>
 
    # ---------------------------------------------------------------------------------------------
    #<methodize4xi method = "init-of-UPU2Image4Xi" usage = "
    #   print('(Step 5A/33) OF UPU2IMAGE4XI5TEST - CREATING IMAGE4UPU OBJECT ', time4xi())     
    #   try:
    #       imageobject2upu = UPU2Image4Xi()
    #       print('(Step 5B/33) OF UPU2IMAGE4XI5TEST - CREATED IMAGE4UPU OBJECT: ', imageobject2upu)                                  
    #   except:
    #       print('(Step 5Z/33) OF UPU2IMAGE4XI5TEST *** FAILED TO CREATE IMAGE4UPU OBJECT')
    #">
    
    # ---------------------------------------------------------------------------------------------   
    def __init__(self, task4imagecreate2upu = None):
        '''
        _init__ method initializes UPU2Image4Xi
        ''' 
        
        # -----------------------------------------------------------------------------------------
        # (Step 1/33)           
        # -----------------------------------------------------------------------------------------
                    
        class2print = "UPU2Image4Xi"
        method2print = "__init__"
                     
        message2print = "{(Step 1A/33) started" + "; init-of-UPU2Image4Xi-started: " + time4xi() + "}" 
        print4xi(class2print, method2print, message2print)                                                          
        # 23:41:37 09/06/11 Pacific Daylight Time 

        message2print = "{(Step 1B/33) borning" + "; upu2image2xi-of-UPU2Image4Xi-borning: None" + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)                                                          
        # None 
                
        message2print = "{(Step 1C/33) inheriting" + "; object-inheriting-by-upu2image2xi-of-UPU2Image4Xi: " + str(object)  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)                                                          
        # <class 'object'>
        
        message2print = "{(Step 1D/33) sharing" + "; classvariable2upu-sharing-by-upu2image2xi-of-UPU2Image4Xi: " + str(UPU2Image4Xi.classvariable2upu)  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)                                                          
        # 4             
                                
        message2print = "{(Step 1E/33) passed" + "; task4imagecreate2upu-passed-to-init-of-UPU2Image4Xi: " + str(task4imagecreate2upu) + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)                                                          
        # None                            
        
        # -----------------------------------------------------------------------------------------
        # (Step 2/33)             
        # -----------------------------------------------------------------------------------------
        
        message2print = "{(Step 2A/33) passing" + "; task4imagecreate2upu-passing-to-create2image4upu-by-init-of-UPU2Image4Xi: " + str(task4imagecreate2upu) + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)                                                          
        # None  
        
        upu2image2xi = create2image4upu(task4imagecreate2upu)
        
        message2print = "{(Step 2B/33) performed" + "; create2image4upu-performed-by-init-of-UPU2Image4Xi: " + str(create2image4upu) + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)                                                          
        # <function create2image4upu at 0x0000000003A9E7C8>
                        
        # -----------------------------------------------------------------------------------------
        # (Step 3/33)             
        # -----------------------------------------------------------------------------------------
        message2print = "{(Step 3A/33) activated" + "; upu2image2xi-of-UPU2Image4Xi-activated: " + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)                                                          
        # { UPU2Image4Xi : __init__ ; (Step 3A/33) activated; upu2image2xi-of-UPU2Image4Xi-activated: ; time: 00:14:05 09/07/11 Pacific Daylight Time} 
                
        message2print = "{(Step 3B/33) inherited" + "; object-inherited-by-upu2image2xi-of-UPU2Image4Xi: " + str(object)  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)                                                          
        # <class 'object'> 
        
        message2print = "{(Step 3C/33) shared" + "; classvariable2upu-shared-by-upu2image2xi-of-UPU2Image4Xi: " + str(UPU2Image4Xi.classvariable2upu)  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)                                                          
        # 4           
       
        message2print = "{(Step 3D/33) returning" + "; init-of-UPU2Image4Xi-returning: " + time4xi() + "}" 
        print4xi(class2print, method2print, message2print)                                                          
        # 00:14:05 09/07/11 Pacific Daylight Time
        

    #</methodize4xi>
    

    # ---------------------------------------------------------------------------------------------
    #<methodize4xi method = "create2image-of-UPU2Image4Xi" usage = "
    #   print('(Step 23A/33) OF UPU2IMAGE4XI5TEST - TRYING TO CREATE UPU IMAGE', time4xi()) 
    #   try:
    #       xi2upu = UPU2Image4Xi()                            
    #       image2upu = xi2upu.create2image()                                  
    #       print('(Step 23B/33) OF UPU2IMAGE4XI5TEST - CREATED UPU IMAGE: ', image2upu)
    #   except:
    #       print('(Step 23Z/33) OF UPU2IMAGE4XI5TEST *** FAILED TO CREATE UPU IMAGE ***', time4xi())                  
    #                     
    #">   
    
    # ---------------------------------------------------------------------------------------------   
    def create2image(self, task4image2create = None):
        '''
        create2image method creates 2D image objects of UPU2Image4Xi
        ''' 
        
        # -----------------------------------------------------------------------------------------
        # (Step 1/33)           
        # -----------------------------------------------------------------------------------------              
        class2print = "UPU2Image4Xi"
        method2print = "create2image"
                     
        message2print = "{(Step 1A/33) started" + "; create2image-of-UPU2Image4Xi-started: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)   
        # 00:14:05 09/07/11 Pacific Daylight Time 
            
        message2print = "{(Step 1B/33) passed" + "; task4image2create-passed-to-create-of-UPU2Image4Xi: " + str(task4image2create) + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)                                      
        # None 
          
        # -----------------------------------------------------------------------------------------
        # (Step 2/33)           
        # -----------------------------------------------------------------------------------------
        message2print = "{(Step 2A/33) passing" + "; task4image2create-passing-to-create2image4upu-by-create2image-of-UPU2Image4Xi: " + str(task4image2create) + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)            
        # None      
  
        upu2image2xi = create2image4upu(task4image2create)

        message2print = "{(Step 2B/33) performed" + "; create2image4upu-performed-by-create2image-of-UPU2Image4Xi: " + str(create2image4upu) + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)             
        # <function create2image4upu at 0x00000000039DF7C8>
                                
        # -----------------------------------------------------------------------------------------
        # (Step 3/33)           
        # ----------------------------------------------------------------------------------------- 
        message2print = "{(Step 3/33) returning" + "; upu2image2xi-returning-by-create2image-of-UPU2Image4Xi: " + str(upu2image2xi) + "; time: " + time4xi() + "}" 
        print4xi(class2print, method2print, message2print)                                                          # 
        # {'CODE2ERROR': 0,
        #  'ACK2RETURN': ' 1D-1E-2C-2B-3B-4B-5B-6B-7B-8B-9B-10B-11B-14B-15B-16B-17B-17D-18B-18C-21A1-22B1-22C1-23B-24B-25B-26B-27B-28B-29B-30B-31B-34H-35H-36B-37B-38B-',
        #  'ROWPITCH': 800,
        #  'DATA2CALLBACK': {'MESSAGE': '{(Step 1Z1/33) failed; create2image4upu-failed ; time: 00:53:55 10/05/11 Pacific Daylight Time}',
        #  'CLASS': 'create2image4upu',
        #  'METHOD': 'create2image4upu'},
        #  'WIDTH': 800 ...

        return upu2image2xi
                                
    #                        
    #</methodize4xi>
    

    # ---------------------------------------------------------------------------------------------
    #<methodize4xi method = "set2image-of-UPU2Image4Xi" usage = "
    #   print('(Step 1A/33) OF UPU2IMAGE4XI - TRYING TO SET UPU IMAGE')
    #   try:
    #       cc2xi = UPU2Image4Xi()                            
    #       image2upu = cc2xi.set2image()                                  
    #       print('(Step 1B/33) OF UPU4XI5TEST - SET UPU IMAGE: ', image2upu)                                                  
    #   except:
    #       print('(Step 1Z/33) OF UPU4XI5TEST *** FAILED TO SET UPU IMAGE ***')  
    #">
    
    # ---------------------------------------------------------------------------------------------   
    def set2image(self, task4set2image = None):
        '''
        sets 2D image UPU objects
        ''' 
        
        # -----------------------------------------------------------------------------------------
        # (Step 1/33)           
        # -----------------------------------------------------------------------------------------              
        class2print = "UPU2Image4Xi"
        method2print = "set2image"
                     
        message2print = "{(Step 1A/33) started" + "; set2image-of-UPU2Image4Xi-started: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)   
        # 01:02:22 09/07/11 Pacific Daylight Time 
            
        message2print = "{(Step 1B/33) passed" + "; task4set2image-passed-to-set2image-of-UPU2Image4Xi: " + str(task4set2image) + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)                                                          
        # {'CODE2ERROR': 0,
        #  'QUEUE': <queue4xi.Queue4Xi object at 0x0000000004191E10>,
        #  'CONTEXT': {'COUNT2REFERENCE': 0, ...
        
          
        # -----------------------------------------------------------------------------------------
        # (Step 2/33)           
        # -----------------------------------------------------------------------------------------
        message2print = "{(Step 2A/33) passing" + "; task4set2image-passing-to-set2image4upu-by-set2image-of-UPU2Image4Xi: " + str(task4set2image) + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)                                                               
        # {'CODE2ERROR': 0, 
        #  'QUEUE': <queue4xi.Queue4Xi object at 0x0000000004191E10>, 
        #  'CONTEXT': {'COUNT2REFERENCE': 0 ...

        upu2image2xi = set2image4upu(task4set2image)

        message2print = "{(Step 2B/33) performed" + "; set2image4upu-performed-by-set2image-of-UPU2Image4Xi: " + str(set2image4upu) + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)             
        # <function set2image4upu at 0x0000000003AB36C8> 
                                
        # -----------------------------------------------------------------------------------------
        # (Step 3/33)           
        # ----------------------------------------------------------------------------------------- 
       
        message2print = "{(Step 3A/33) returning" + "; upu2image2xi-returning-by-set2image-of-UPU2Image4Xi: " + str(upu2image2xi) + "; time: " + time4xi() + "}" 
        print4xi(class2print, method2print, message2print) 
        # {'CODE2ERROR': 0,
        #  'ACK2RETURN': ' 23B-24B-25B-26B-27B-28B-',
        #  'ROWPITCH': 800,
        #  'DATA2CALLBACK': {'MESSAGE': '{(Step 1Z1/33) failed; create2image4upu-failed ; time: 01:02:22 09/07/11 Pacific Daylight Time}', 'CLASS': 'create2image4upu', 'METHOD': 'create2image4upu'},
        #  'WIDTH': 800,
        #  'BUFFER': {'CODE2ERROR': 0, 'COUNT2REFERENCE': 0, 'DATA2CALLBACK': {'MESSAGE': '{(Step 1Z1/33) failed; create2buffer4upu-failed ; time: 01:02:22 09/07/11 Pacific Daylight Time}',
        #  'CLASS': 'create2buffer4upu',
        #  'METHOD': 'create2buffer4upu'},
        #  'FUNCTION2CALLBACK': <function trace2buffer4upu at 0x0000000003AA81C8>,
        #  'POINTER2BUFFER': <mmap.mmap object at 0x0000000003CC92F0>,
        #  'FLAGS': 0,
        #  'CONTEXT': {'COUNT2REFERENCE': 0 ...
                    
                
        return upu2image2xi
                                
                            
    #</methodize4xi>

    # ---------------------------------------------------------------------------------------------
    #<methodize4xi method = "get2imageinfo-of-UPU2Image4Xi" usage = "
    #   print('(Step 1A/33) OF UPU2IMAGE4XI - TRYING TO GET UPU 2D IMAGE')
    #   try:
    #       cc2xi = UPU2Image4Xi()                            
    #       image2upu = cc2xi.get2imageinfo()                                  
    #       print('(Step 1B/33) OF UPU4XI5TEST - GOT UPU IMAGE: ', image2upu)                                                  
    #   except:
    #       print('(Step 1Z/33) OF UPU4XI5TEST *** FAILED TO GET UPU 2D IMAGE ***')   
    # ">
    
    # ---------------------------------------------------------------------------------------------   
    def get2imageinfo(self, task4getinfo2image=None):
        '''
        get2imageinfo method gets 2D objects of UPU2Image4Xi
        ''' 
        
        # -----------------------------------------------------------------------------------------
        # (Step 1/33)           
        # -----------------------------------------------------------------------------------------              
        class2print = "UPU2Image4Xi"
        method2print = "get2imageinfo"
                     
        message2print = "{(Step 1A/33) started" + "; get2imageinfo-of-UPU2Image4Xi-started: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)   
        # 00:14:05 09/07/11 Pacific Daylight Time 

        # -----------------------------------------------------------------------------------------
        # (Step 2/33)           
        # ----------------------------------------------------------------------------------------- 
        upu2image2xi = get2imageinfo4upu(task4getinfo2image)
        
        message2print = "{(Step 2A/33) returning" + "; upu2image2xi-returning-by-get2imageinfo-of-UPU2Image4Xi: " + str(upu2image2xi) + "; time: " + time4xi() + "}" 
        print4xi(class2print, method2print, message2print) 
        # {'CODE2ERROR': 0,
        #  'ACK2RETURN': ' 1D-1E-2C-2B-3B-4B-5B-6B-7B-8B-9B-10B-11B-14B-15B-16B-17B-17D-18B-18C-21A1-22B1-22C1-23B-24B-25B-26B-27B-28B-29B-30B-31B-34H-35H-36B-37B-38B-',
        #  'ROWPITCH': 800,
        #  'DATA2CALLBACK': {'MESSAGE': '{(Step 1Z1/33) failed; create2image4upu-failed ; time: 00:14:05 09/07/11 Pacific Daylight Time}', 'CLASS': 'create2image4upu', 'METHOD': 'create2image4upu'},
        #  'WIDTH': 800,
        #  'BUFFER': {'CODE2ERROR': 0, 'COUNT2REFERENCE': 0, 'DATA2CALLBACK': {'MESSAGE': '{(Step 1Z1/33) failed; create2buffer4upu-failed ; time: 00:14:05 09/07/11 Pacific Daylight Time}',
        #  'CLASS': 'create2buffer4upu',
        #  'METHOD': 'create2buffer4upu'},
        #  'FUNCTION2CALLBACK': <function trace2buffer4upu at 0x00000000039D8348>,
        #  'POINTER2BUFFER': <mmap.mmap object at 0x00000000037D32F0>,
        #  'FLAGS': 0,
        #  'CONTEXT': {'COUNT2REFERENCE': 0, ...
        # or when only two elemets required
        # {'WIDTH': 800, 'HEIGHT': 800}
        
        return upu2image2xi
                            
    #</methodize4xi>

    # ---------------------------------------------------------------------------------------------
    #<methodize4xi method = "get2supportedimageformats-of-UPU2Image4Xi" usage = "
    #  print('(Step 6B1/33) OF UPU2IMAGE4XI5TEST - TRYING TO GET SUPPORTED IMAGE FORMATS FOR UPU')
    #  try:
    #      cc2xi = UPU2Image4Xi()                            
    #      supportedimageformats2upu = cc2xi.get2supportedimageformats()                                  
    #      print('(Step 6B2/33) OF UPU4XI5TEST - GOT SUPPORTED IMAGE FORMATS FOR UPU: ', supportedimageformats2upu)
    #                                                   
    #  except:
    #      print('(Step 2Z/33) OF UPU4XI5TEST *** FAILED TO GET SUPPORTED IMAGE FORMATS FOR UPU***')     
    # ">
    
    # ---------------------------------------------------------------------------------------------   
    def get2supportedimageformats(self):
        '''
        get2supportedimageformats method gets supported imageformats by objects of UPU2Image4Xi
        ''' 
        
        # -----------------------------------------------------------------------------------------
        # (Step 1/33)           
        # -----------------------------------------------------------------------------------------              
        class2print = "UPU2Image4Xi"
        method2print = "get2supportedimageformats"
                     
        message2print = "{(Step 1A/33) started" + "; get2supportedimageformats-of-UPU2Image4Xi-started: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)   
        # 01:43:00 09/07/11 Pacific Daylight Time

        # -----------------------------------------------------------------------------------------
        # (Step 2/33)           
        # ----------------------------------------------------------------------------------------- 
        supportedimageformats2upu = get2supportedimageformats4upu()
        
        message2print = "{(Step 2A/33) returning" + "; supportedimageformats2upu-returning-by-get2imageinfo-of-UPU2Image4Xi: " + str(supportedimageformats2upu) + "; time: " + time4xi() + "}" 
        print4xi(class2print, method2print, message2print) 
        # {'TYPE': 'CL_UNORM_INT8', 'ORDER': 'CL_RGBA'} 
        
        return supportedimageformats2upu
                            
    #</methodize4xi>    
    
    
    # ---------------------------------------------------------------------------------------------
    #<methodize4xi method = "read2image-of-UPU2Image4Xi" usage = "
    #   print('(Step 127A/700) OF UPU2IMAGE4XI5TEST - TRYING TO ESTABLISH CONTROL AND COMMAND WITH READ IMAGE OBJECT ', time4xi()) 
    #   response4read2upu = {}
    #   try:
    #       xi2upu = UPU2Image4Xi()    
    #       response4read2upu = xi2upu.read2image()
    #       print('(Step 127B/700) OF UPU2IMAGE4XI5TEST - ESTABLISHED CONTROL AND COMMAND WITH READ IMAGE OBJECT, RESPONSE4READ2UPU: ', response4read2upu) 
    #   except:
    #       print('(Step 127Z1/700) OF UPU2IMAGE4XI5TEST *** FAILED TO ESTABLISH CONTROL AND COMMAND WITH READ IMAGE OBJECT ', time4xi())
    #   ... continue as with read2image4upu function
    #">
    
    # ---------------------------------------------------------------------------------------------   
    def read2image(self, task4read2image = None):
        '''
        reads UPU2Image4Xi objects
        ''' 
        
        # -----------------------------------------------------------------------------------------
        # (Step 1/33)           
        # -----------------------------------------------------------------------------------------              
        class2print = "UPU2Image4Xi"
        method2print = "read2image"
                     
        message2print = "{(Step 1A/33) started" + "; read2image-of-UPU2Image4Xi-started: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)   
        # 00:20:02 10/05/11 Pacific Daylight Time 
            
        message2print = "{(Step 1B/33) passed" + "; task4read2image-passed-to-read2image-of-UPU2Image4Xi: " + str(task4read2image) + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)                                                          
        # None 
          
        # -----------------------------------------------------------------------------------------
        # (Step 2/33)           
        # -----------------------------------------------------------------------------------------
        message2print = "{(Step 2A/33) passing" + "; task4read2image-passing-to-read2image4upu-by-read2image-of-UPU2Image4Xi: " + str(task4read2image) + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)                                                               
        # None
  
        response4readimage2upu = read2image4upu(task4read2image)

        message2print = "{(Step 2B/33) performed" + "; read2image4upu-performed-by-read2image-of-UPU2Image4Xi: " + str(read2image4upu) + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)             
        # <function read2image4upu at 0x000000000385DA48> 
                                
        # -----------------------------------------------------------------------------------------
        # (Step 3/33)           
        # ----------------------------------------------------------------------------------------- 
       
        message2print = "{(Step 3A/33) returning" + "; response4readimage2upu-returning-by-read2image-of-UPU2Image4Xi: " + str(response4readimage2upu) + "; time: " + time4xi() + "}" 
        print4xi(class2print, method2print, message2print) 
        # {'QUEUE': <queue4xi.Queue4Xi object at 0x000000000417DD68>,
        #  'PUBLISHERCHANNEL': <read-write PipeConnection, handle 936>,
        #  'CONSUMERCHANNEL': <read-write PipeConnection, handle 932>}; time: 00:20:02 10/05/11 Pacific Daylight Time}
  
        return response4readimage2upu
                                                           
    #</methodize4xi>
    
    # ---------------------------------------------------------------------------------------------
    #<methodize4xi method = "map2image-of-UPU2Image4Xi" usage = "
    #   print('(Step 127A/700) OF UPU2IMAGE4XI5TEST - TRYING TO ESTABLISH CONTROL AND COMMAND WITH MAP IMAGE OBJECT ', time4xi()) 
    #   xi2upu = UPU2Image4Xi() 
    #   ...
    #   response4map2upu = {}
    #   try:
    #       response4map2upu = xi2upu.map2image()
    #       print('(Step 127B/700) OF UPU2IMAGE4XI5TEST - ESTABLISHED CONTROL AND COMMAND WITH MAP IMAGE OBJECT, RESPONSE4READ2UPU: ', response4map2upu) 
    #   except:
    #       print('(Step 127Z1/700) OF UPU2IMAGE4XI5TEST *** FAILED TO ESTABLISH CONTROL AND COMMAND WITH MAP IMAGE OBJECT ', time4xi())
    #   ... continue as with map2image4upu function
    #">
    
    # ---------------------------------------------------------------------------------------------   
    def map2image(self, task4map2image = None):
        '''
        maps UPU2Image4Xi objects
        ''' 
        
        # -----------------------------------------------------------------------------------------
        # (Step 1/33)           
        # -----------------------------------------------------------------------------------------              
        class2print = "UPU2Image4Xi"
        method2print = "map2image"
                     
        message2print = "{(Step 1A/33) started" + "; map2image-of-UPU2Image4Xi-started: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)   
        # 01:28:44 10/05/11 Pacific Daylight Time 
            
        message2print = "{(Step 1B/33) passed" + "; task4map2image-passed-to-map2image-of-UPU2Image4Xi: " + str(task4map2image) + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)                                                          
        # None
          
        # -----------------------------------------------------------------------------------------
        # (Step 2/33)           
        # -----------------------------------------------------------------------------------------
        message2print = "{(Step 2A/33) passing" + "; task4map2image-passing-to-map2image4upu-by-map2image-of-UPU2Image4Xi: " + str(task4map2image) + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)                                                               
        # None
  
        response4mapimage2upu = map2image4upu(task4map2image)

        message2print = "{(Step 2B/33) performed" + "; map2image4upu-performed-by-map2image-of-UPU2Image4Xi: " + str(map2image4upu) + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)             
        # <function map2image4upu at 0x000000000390BAC8> 
                                
        # -----------------------------------------------------------------------------------------
        # (Step 3/33)           
        # ----------------------------------------------------------------------------------------- 
       
        message2print = "{(Step 3A/33) returning" + "; response4mapimage2upu-returning-by-map2image-of-UPU2Image4Xi: " + str(response4mapimage2upu) + "; time: " + time4xi() + "}" 
        print4xi(class2print, method2print, message2print) 
        # {'QUEUE': <queue4xi.Queue4Xi object at 0x00000000050E9C18>,
        #  'PUBLISHERCHANNEL': <read-write PipeConnection, handle 1100>,
        #  'CONSUMERCHANNEL': <read-write PipeConnection, handle 1096>}; time: 01:28:44 10/05/11 Pacific Daylight Time}

        
        return response4mapimage2upu
                                                           
    #</methodize4xi>
            
    # ---------------------------------------------------------------------------------------------
    #<methodize4xi method = "release2image-of-UPU2Image4Xi" usage = "
    #
    # ---------------------------------------------------------------------------------------------
    # RELEASE 2D IMAGE BY DECREMENTING REFERENCE COUNT WITH 1             
    # ---------------------------------------------------------------------------------------------    
    #   task4release2image  = {} 
    #   task4release2image  ['COUNT2REFERENCE'] = 1 
    #   response4release2image = {}
    #   print('(Step 133A/700) OF UPU2IMAGE4XI5TEST - TRYING TO RELEASE 2D IMAGE OF UPU DEVICE ', time4xi())    
    # 
    #   try:                               
    #       response4release2Dimage = xi2upu.release2image(task4release2image)                                                                                             
    #       print('(Step 133B/700) OF UPU2IMAGE4XI5TEST - RELEASED 2D IMAGE OF UPU DEVICE ', time4xi())                           
    #       keys2response = response4release2Dimage.keys()
    #       for key2response in sorted(keys2response):
    #           print(key2response, ' : ', response4release2Dimage[key2response])
    #           #
    #   except:
    #       print('(Step 133Z1/700) OF UPU2IMAGE4XI5TEST *** FAILED TO RELEASE IMAGE OF UPU DEVICE ', time4xi())  
    #   
    #   task4getinfo2image = {}
    #   task4getinfo2image ['COUNT2REFERENCE'] = 0
    #   response4getinfo2image = {}
    #   print('(Step 133C/700) OF UPU2IMAGE4XI5TEST - TRYING TO GET SELECTED 2D IMAGE INFO OF UPU DEVICE ', time4xi())     
    #   # 
    #   try:                               
    #       response4getinfo2Dimage = xi2upu.get2imageinfo (task4getinfo2image)                                                                                             
    #       keys2response = response4getinfo2Dimage.keys()
    #       print('(Step 133D/700) OF UPU2IMAGE4XI5TEST - GOT SELECTED 2D IMAGE INFO OF UPU DEVICE ', time4xi())   
    #       for key2response in sorted(keys2response):
    #           print(key2response, ' : ', response4getinfo2Dimage[key2response])
    #           # 
    #   except:
    #       print('(Step 133Z2/700) OF UPU2IMAGE4XI5TEST *** FAILED TO GET SELECTED 2D IMAGE INFO OF UPU DEVICE ', time4xi())  
    #">
    
    # ---------------------------------------------------------------------------------------------   
    def release2image(self, task4release2image = None):
        '''
        releases UPU2Image4Xi objects
        ''' 
        
        # -----------------------------------------------------------------------------------------
        # (Step 1/33)           
        # -----------------------------------------------------------------------------------------              
        class2print = "UPU2Image4Xi"
        method2print = "release2image"
                     
        message2print = "{(Step 1A/33) started" + "; release2image-of-UPU2Image4Xi-started: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)   
        # 
            
        message2print = "{(Step 1B/33) passed" + "; task4release2image-passed-to-release2image-of-UPU2Image4Xi: " + str(task4release2image) + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)                                                          
        # 
          
        # -----------------------------------------------------------------------------------------
        # (Step 2/33)           
        # -----------------------------------------------------------------------------------------
        message2print = "{(Step 2A/33) passing" + "; task4release2image-passing-to-release2image4upu-by-release2image-of-UPU2Image4Xi: " + str(task4release2image) + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)                                                               
        # 
  
        upu2image2xi = release2image4upu(task4release2image)

        message2print = "{(Step 2B/33) performed" + "; release2image4upu-performed-by-release2image-of-UPU2Image4Xi: " + str(release2image4upu) + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)             
        # 
                                
        # -----------------------------------------------------------------------------------------
        # (Step 3/33)           
        # ----------------------------------------------------------------------------------------- 
       
        message2print = "{(Step 3A/33) returning" + "; upu2image2xi-returning-by-release2image-of-UPU2Image4Xi: " + str(upu2image2xi) + "; time: " + time4xi() + "}" 
        print4xi(class2print, method2print, message2print) 
        #                          
                
        return upu2image2xi
                                                           
    #</methodize4xi>
    
    # ---------------------------------------------------------------------------------------------
    #<methodize4xi method = "retain2image-of-UPU2Image4Xi" usage = "
    #
    # ---------------------------------------------------------------------------------------------
    # RETAIN 2D IMAGE BY DECREASING REFERENCE WITH 1             
    # ---------------------------------------------------------------------------------------------    
    #   task4retain2image  = {}
    #   task4retain2image  ['COUNT2REFERENCE'] = 1 
    #   response4retain2Dimage = {}
    #   print('(Step 134A/700) OF UPU2IMAGE4XI5TEST - TRYING TO RETAIN 2D IMAGE OF UPU DEVICE ', time4xi())   
    #   try:                               
    #       response4retain2Dimage = xi2upu.retain2image(task4retain2image)                                                                                                 
    #       print('(Step 134B/700) OF UPU2IMAGE4XI5TEST - RESPONSE4RETAIN2DIMAGE : ', time4xi())                         
    #       keys2response = response4retain2Dimage.keys()
    #       for key2response in sorted(keys2response):
    #           print(key2response, ' : ', response4retain2Dimage[key2response])
    #           # 
    #   except:
    #       print('(Step 134Z1/700) OF UPU2IMAGE4XI5TEST *** FAILED TO RETAIN 2D IMAGE OF UPU DEVICE ', time4xi())
    #   task4getinfo2image = {}
    #   task4getinfo2image ['COUNT2REFERENCE'] = 0
    #   response4getinfo2image = {}
    #   print('(Step 134C/700) OF UPU2IMAGE4XI5TEST - TRYING TO GET SELECTED 2D IMAGE INFO OF UPU DEVICE ', time4xi())     
    #   # 
    #   try:                               
    #       response4getinfo2Dimage = xi2upu.get2imageinfo (task4getinfo2image)                                                                                             
    #       keys2response = response4getinfo2Dimage.keys()
    #       print('(Step 134D/700) OF UPU2IMAGE4XI5TEST - GOT SELECTED 2D IMAGE INFO OF UPU DEVICE ', time4xi())   
    #       for key2response in sorted(keys2response):
    #           print(key2response, ' : ', response4getinfo2Dimage[key2response])
    #           # 
    #   except:
    #       print('(Step 134Z2/700) OF UPU2IMAGE4XI5TEST *** FAILED TO GET SELECTED 2D IMAGE INFO OF UPU DEVICE ', time4xi())  
    #">
    
    # ---------------------------------------------------------------------------------------------   
    def retain2image(self, task4retain2image = None):
        '''
        retains UPU2Image4Xi objects
        ''' 
        
        # -----------------------------------------------------------------------------------------
        # (Step 1/33)           
        # -----------------------------------------------------------------------------------------              
        class2print = "UPU2Image4Xi"
        method2print = "retain2image"
                     
        message2print = "{(Step 1A/33) started" + "; retain2image-of-UPU2Image4Xi-started: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)   
        # 00:14:05 09/07/11 Pacific Daylight Time 
            
        message2print = "{(Step 1B/33) passed" + "; task4retain-passed-to-retain2image-of-UPU2Image4Xi: " + str(task4retain2image) + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)                                                          
        # {'COUNT2REFERENCE': 1}
          
        # -----------------------------------------------------------------------------------------
        # (Step 2/33)           
        # -----------------------------------------------------------------------------------------
        message2print = "{(Step 2A/33) passing" + "; task4retain-passing-to-retain2image4upu-by-retain-of-UPU2Image4Xi: " + str(task4retain2image) + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)                                                               
        # {'COUNT2REFERENCE': 1}
  
        upu2image2xi = retain2image4upu(task4retain2image)

        message2print = "{(Step 2B/33) performed" + "; retain2image4upu-performed-by-retain2image-of-UPU2Image4Xi: " + str(retain2image4upu) + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)             
        # <function retain2image4upu at 0x00000000039DF948>
                                
        # -----------------------------------------------------------------------------------------
        # (Step 3/33)           
        # ----------------------------------------------------------------------------------------- 
       
        message2print = "{(Step 3A/33) returning" + "; upu2image2xi-returning-by-retain2image-of-UPU2Image4Xi: " + str(upu2image2xi) + "; time: " + time4xi() + "}" 
        print4xi(class2print, method2print, message2print) 
        # {'CODE2ERROR': 0, 
        #  'ACK2RETURN': ' 1D-', 
        #  'ROWPITCH': 800, 
        #  'DATA2CALLBACK': {'MESSAGE': '{(Step 1Z1/33) failed; create2image4upu-failed ; time: 00:14:05 09/07/11 Pacific Daylight Time}', 
        #  'CLASS': 'create2image4upu', 
        #  'METHOD': 'create2image4upu'}, 
        #  'WIDTH': 800, 
        #  'BUFFER': {'CODE2ERROR': 0, 
        #  ...
        #  'COUNT2REFERENCE': 1,
        #  'FLAGS': 0,
        #  'CONTEXT': {'COUNT2REFERENCE': 0 ...
                         
                
        return upu2image2xi
                                                           
    #</methodize4xi>   
    
    
    # ---------------------------------------------------------------------------------------------
    #<methodize4xi method = "wait2image-of-UPU2Image4Xi" usage = "
    #   print('=' *33)          
    #   print('(Step 126A/700) OF UPU2IMAGE4XI5TEST - TRYING TO WAIT 2D IMAGE OPERATION FOR UPU ', time4xi()) 
    #   print('=' *33)      
    #   wait2upu = {}
    #   try:
    #       response4wait2Dimage = xi2upu.wait2image()
    #       print('(Step 122B/700) OF UPU2IMAGE4XI5TEST - WAITED 2D IMAGE OPERATION FOR UPU ', time4xi()) 
    #       keys2response = response4wait2Dimage.keys()
    #       for key2response in sorted(keys2response):
    #           print(key2response, ' : ', response4wait2Dimage[key2response])
    #           # 
    #   except:
    #       print('(Step 126Z1/700) OF UPU2IMAGE4XI5TEST *** FAILED TO 2D IMAGE OPERATION FOR UPU ', time4xi())    
    #">
    
    # ---------------------------------------------------------------------------------------------   
    def wait2image(self, task4wait2image = None):
        '''
        waits UPU2Image4Xi objects
        ''' 
        
        # -----------------------------------------------------------------------------------------
        # (Step 1/33)           
        # -----------------------------------------------------------------------------------------              
        class2print = "UPU2Image4Xi"
        method2print = "wait2image"
                     
        message2print = "{(Step 1A/33) started" + "; wait2image-of-UPU2Image4Xi-started: " + time4xi() + "}" 
        print4xi(class2print, method2print, message2print)   
        # 00:14:05 09/07/11 Pacific Daylight Time 
            
        message2print = "{(Step 1B/33) passed" + "; task4wait2image-passed-to-wait2image-of-UPU2Image4Xi: " + str(task4wait2image) + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)                                                          
        # None 
          
        # -----------------------------------------------------------------------------------------
        # (Step 2/33)           
        # -----------------------------------------------------------------------------------------
        message2print = "{(Step 2A/33) passing" + "; task4wait-passing-to-wait2image4upu-by-wait2image-of-UPU2Image4Xi: " + str(task4wait2image) + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)                                                               
        # None
  
        upu2image2xi = wait2image4upu(task4wait2image)

        message2print = "{(Step 2B/33) performed" + "; wait2image4upu-performed-by-wait2image-of-UPU2Image4Xi: " + str(wait2image4upu) + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)             
        # semaphore and lock analysis by multiprocessing4xi
        # <function wait2image4upu at 0x00000000039DFA48> 
                                
        # -----------------------------------------------------------------------------------------
        # (Step 3/33)           
        # ----------------------------------------------------------------------------------------- 
       
        message2print = "{(Step 3A/33) returning" + "; upu2image2xi-returning-by-wait2image-of-UPU2Image4Xi: " + str(upu2image2xi) + "; time: " + time4xi() + "}" 
        print4xi(class2print, method2print, message2print) 
        # {'SIZE2QUEUE': 2147483646,
        #  'FORMAT': {'TYPE': 'CL_UNORM_INT8', 'ORDER': 'CL_RGBA'},
        #  'QUEUE': <queue4xi.Queue4Xi object at 0x000000000434DDA0>,
        #  'WIDTH': 800,
        #  'CONTEXT': ...
                
        return upu2image2xi
                                                           
    #</methodize4xi> 
    
    
    # ---------------------------------------------------------------------------------------------
    #<methodize4xi method = "write2image-of-UPU2Image4Xi" usage = "
    #   print('(Step 127A/700) OF UPU2IMAGE4XI5TEST - TRYING TO ESTABLISH CONTROL AND COMMAND WITH WRITE IMAGE OBJECT', time4xi()) 
    #   response4write2upu = {}
    #   try:
    #       response4write2upu = xi2upu.write2image()
    #       print('(Step 127B/700) OF UPU2IMAGE4XI5TEST - ESTABLISHED CONTROL AND COMMAND WITH WRITE IMAGE OBJECT, RESPONSE4WRITE2UPU: ', response4write2upu) 
    #   except:
    #       print('(Step 127Z1/700) OF UPU2IMAGE4XI5TEST *** FAILED TO ESTABLISH CONTROL AND COMMAND WITH WRITE IMAGE OBJECT', time4xi()) 
    #
    #   ... continue as with write2image4upu function
    #">
    
    # ---------------------------------------------------------------------------------------------   
    def write2image(self, task4write2image = None):
        '''
        writes UPU2Image4Xi objects
        ''' 
        
        # -----------------------------------------------------------------------------------------
        # (Step 1/33)           
        # -----------------------------------------------------------------------------------------              
        class2print = "UPU2Image4Xi"
        method2print = "write2image"
                     
        message2print = "{(Step 1A/33) started" + "; write2image-of-UPU2Image4Xi-started: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)   
        # 02:45:31 10/05/11 Pacific Daylight Time
            
        message2print = "{(Step 1B/33) passed" + "; task4write2image-passed-to-write2image-of-UPU2Image4Xi: " + str(task4write2image) + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)                                                          
        # None
          
        # -----------------------------------------------------------------------------------------
        # (Step 2/33)           
        # -----------------------------------------------------------------------------------------
        message2print = "{(Step 2A/33) passing" + "; task4write2image-passing-to-write2image4upu-by-write2image-of-UPU2Image4Xi: " + str(task4write2image) + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)                                                               
        # None
  
        response4writeimage2upu = write2image4upu(task4write2image)

        message2print = "{(Step 2B/33) performed" + "; write2image4upu-performed-by-write2image-of-UPU2Image4Xi: " + str(write2image4upu) + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)             
        # <function write2image4upu at 0x000000000380AEC8>
                                
        # -----------------------------------------------------------------------------------------
        # (Step 3/33)           
        # ----------------------------------------------------------------------------------------- 
       
        message2print = "{(Step 3A/33) returning" + "; response4writeimage2upu-returning-by-write2image-of-UPU2Image4Xi: " + str(response4writeimage2upu) + "; time: " + time4xi() + "}" 
        print4xi(class2print, method2print, message2print) 
        # {'QUEUE': <queue4xi.Queue4Xi object at 0x0000000004D10E10>,
        #  'PUBLISHERCHANNEL': <read-write PipeConnection, handle 968>,
        #  'CONSUMERCHANNEL': <read-write PipeConnection, handle 964>}; time: 02:45:31 10/05/11 Pacific Daylight Time}

                     
                
        return response4writeimage2upu
                                                           
    #</methodize4xi>
    
    # ---------------------------------------------------------------------------------------------
    #<methodize4xi method = "copy2image-of-UPU2Image4Xi" usage = "
    #   print('(Step 127A/700) OF UPU2IMAGE4XI5TEST - TRYING TO ESTABLISH CONTROL AND COMMAND WITH WRITE IMAGE OBJECT', time4xi()) 
    #   response4copy2upu = {}
    #   try:
    #       response4copy2upu = xi2upu.copy2image()
    #       print('(Step 127B/700) OF UPU2IMAGE4XI5TEST - ESTABLISHED CONTROL AND COMMAND WITH WRITE IMAGE OBJECT, RESPONSE4WRITE2UPU: ', response4copy2upu) 
    #   except:
    #       print('(Step 127Z1/700) OF UPU2IMAGE4XI5TEST *** FAILED TO ESTABLISH CONTROL AND COMMAND WITH WRITE IMAGE OBJECT', time4xi()) 
    #
    #   ... continue as with copy2image4upu function
    #">
    
    # ---------------------------------------------------------------------------------------------   
    def copy2image(self, task4copy2image = None):
        '''
        copies UPU2Image4Xi objects
        ''' 
        
        # -----------------------------------------------------------------------------------------
        # (Step 1/33)           
        # -----------------------------------------------------------------------------------------              
        class2print = "UPU2Image4Xi"
        method2print = "copy2image"
                     
        message2print = "{(Step 1A/33) started" + "; copy2image-of-UPU2Image4Xi-started: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)   
        # 01:28:44 10/05/11 Pacific Daylight Time 
            
        message2print = "{(Step 1B/33) passed" + "; task4copy2image-passed-to-copy2image-of-UPU2Image4Xi: " + str(task4copy2image) + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)                                                          
        # None 
          
        # -----------------------------------------------------------------------------------------
        # (Step 2/33)           
        # -----------------------------------------------------------------------------------------
        message2print = "{(Step 2A/33) passing" + "; task4copy2image-passing-to-copy2image4upu-by-copy2image-of-UPU2Image4Xi: " + str(task4copy2image) + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)                                                               
        # None 
  
        response4copyimage2upu = copy2image4upu(task4copy2image)

        message2print = "{(Step 2B/33) performed" + "; copy2image4upu-performed-by-copy2image-of-UPU2Image4Xi: " + str(copy2image4upu) + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)             
        # <function copy2image4upu at 0x000000000390B8C8>
                                
        # -----------------------------------------------------------------------------------------
        # (Step 3/33)           
        # ----------------------------------------------------------------------------------------- 
       
        message2print = "{(Step 3A/33) returning" + "; response4copyimage2upu-returning-by-copy2image-of-UPU2Image4Xi: " + str(response4copyimage2upu) + "; time: " + time4xi() + "}" 
        print4xi(class2print, method2print, message2print) 
        # {'QUEUE': <queue4xi.Queue4Xi object at 0x0000000004EC4128>,
        #  'PUBLISHERCHANNEL': <read-write PipeConnection, handle 1000>,
        #  'CONSUMERCHANNEL': <read-write PipeConnection, handle 996>}; time: 01:28:44 10/05/11 Pacific Daylight Time}
        
        return response4copyimage2upu
                                                           
    #</methodize4xi>
    # ---------------------------------------------------------------------------------------------
    #<methodize4xi method = "copy2imagetobuffer-of-UPU2Image4Xi" usage = "
    #   print('(Step 127A/700) OF UPU2IMAGE4XI5TEST - TRYING TO ESTABLISH CONTROL AND COMMAND WITH COPY IMAGE TO BUFFER OBJECT', time4xi()) 
    #   response4copyimagetobuffer = {}
    #   try:
    #       response4copyimagetobuffer = xi2upu.copy2imagetobuffer()
    #       print('(Step 127B/700) OF UPU2IMAGE4XI5TEST - ESTABLISHED CONTROL AND COMMAND WITH COPY IMAGE TO BUFFER  OBJECT, RESPONSE4WRITE2UPU: ', response4copyimagetobuffer) 
    #   except:
    #       print('(Step 127Z1/700) OF UPU2IMAGE4XI5TEST *** FAILED TO ESTABLISH CONTROL AND COMMAND WITH COPY IMAGE TO BUFFER ', time4xi()) 
    #
    #   ... continue as with copy2imagetobuffer4upu function
    #">
    
    # ---------------------------------------------------------------------------------------------   
    def copy2imagetobuffer(self, task4copy2imagetobuffer = None):
        '''
        copies 2D images to buffer
        ''' 
        
        # -----------------------------------------------------------------------------------------
        # (Step 1/33)           
        # -----------------------------------------------------------------------------------------              
        class2print = "UPU2Image4Xi"
        method2print = "copy2imagetobuffer"
                     
        message2print = "{(Step 1A/33) started" + "; copy2imagetobuffer-of-UPU2Image4Xi-started: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)   
        # 01:28:44 10/05/11 Pacific Daylight Time 
            
        message2print = "{(Step 1B/33) passed" + "; task4copy2imagetobuffer-passed-to-copy2imagetobuffer-of-UPU2Image4Xi: " + str(task4copy2imagetobuffer) + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)                                                          
        # None
          
        # -----------------------------------------------------------------------------------------
        # (Step 2/33)           
        # -----------------------------------------------------------------------------------------
        message2print = "{(Step 2A/33) passing" + "; task4copy2imagetobuffer-passing-to-copy2imagetobuffer4upu-by-copy2imagetobuffer-of-UPU2Image4Xi: " + str(task4copy2imagetobuffer) + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)                                                               
        # None
  
        response4copyimage2upu = copy2imagetobuffer4upu(task4copy2imagetobuffer)

        message2print = "{(Step 2B/33) performed" + "; copy2imagetobuffer4upu-performed-by-copy2imagetobuffer-of-UPU2Image4Xi: " + str(copy2imagetobuffer4upu) + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)             
        # <function copy2imagetobuffer4upu at 0x000000000390B948>
                                
        # -----------------------------------------------------------------------------------------
        # (Step 3/33)           
        # ----------------------------------------------------------------------------------------- 
       
        message2print = "{(Step 3A/33) returning" + "; response4copyimage2upu-returning-by-copy2imagetobuffer-of-UPU2Image4Xi: " + str(response4copyimage2upu) + "; time: " + time4xi() + "}" 
        print4xi(class2print, method2print, message2print) 
        # {'QUEUE': <queue4xi.Queue4Xi object at 0x0000000004FE5FD0>,
        #  'PUBLISHERCHANNEL': <read-write PipeConnection, handle 1036>,
        #  'CONSUMERCHANNEL': <read-write PipeConnection, handle 1032>}; time: 01:28:44 10/05/11 Pacific Daylight Time}
          
        return response4copyimage2upu
                                                           
    #</methodize4xi>
    
    # ---------------------------------------------------------------------------------------------
    #<methodize4xi method = "copy2buffertoimage-of-UPU2Image4Xi" usage = "
    #   print('(Step 127A/700) OF UPU2IMAGE4XI5TEST - TRYING TO ESTABLISH CONTROL AND COMMAND WITH COPY BUFFER TO IMAGE OBJECT', time4xi()) 
    #   response4copybuffertoimage = {}
    #   try:
    #       response4copybuffertoimage = xi2upu.copy2buffertoimage()
    #       print('(Step 127B/700) OF UPU2IMAGE4XI5TEST - ESTABLISHED CONTROL AND COMMAND WITH COPY BUFFER TO IMAGE  OBJECT, RESPONSE4WRITE2UPU: ', response4copybuffertoimage) 
    #   except:
    #       print('(Step 127Z1/700) OF UPU2IMAGE4XI5TEST *** FAILED TO ESTABLISH CONTROL AND COMMAND WITH COPY BUFFER TO IMAGE ', time4xi()) 
    #
    #   ... continue as with copy2buffertoimage4upu function
    #">
    
    # ---------------------------------------------------------------------------------------------   
    def copy2buffertoimage(self, task4copy2buffertoimage = None):
        '''
        copies 2D images to buffer
        ''' 
        
        # -----------------------------------------------------------------------------------------
        # (Step 1/33)           
        # -----------------------------------------------------------------------------------------              
        class2print = "UPU2Image4Xi"
        method2print = "copy2buffertoimage"
                     
        message2print = "{(Step 1A/33) started" + "; copy2buffertoimage-of-UPU2Image4Xi-started: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)   
        # 01:28:44 10/05/11 Pacific Daylight Time 
            
        message2print = "{(Step 1B/33) passed" + "; task4copy2buffertoimage-passed-to-copy2buffertoimage-of-UPU2Image4Xi: " + str(task4copy2buffertoimage) + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)                                                          
        # None
          
        # -----------------------------------------------------------------------------------------
        # (Step 2/33)           
        # -----------------------------------------------------------------------------------------
        message2print = "{(Step 2A/33) passing" + "; task4copy2buffertoimage-passing-to-copy2buffertoimage4upu-by-copy2buffertoimage-of-UPU2Image4Xi: " + str(task4copy2buffertoimage) + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)                                                               
        # None
  
        response4copyimage2upu = copy2buffertoimage4upu(task4copy2buffertoimage)

        message2print = "{(Step 2B/33) performed" + "; copy2buffertoimage4upu-performed-by-copy2buffertoimage-of-UPU2Image4Xi: " + str(copy2buffertoimage4upu) + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)             
        # <function copy2buffertoimage4upu at 0x000000000390B9C8> 
                                
        # -----------------------------------------------------------------------------------------
        # (Step 3/33)           
        # ----------------------------------------------------------------------------------------- 
       
        message2print = "{(Step 3A/33) returning" + "; response4copyimage2upu-returning-by-copy2buffertoimage-of-UPU2Image4Xi: " + str(response4copyimage2upu) + "; time: " + time4xi() + "}" 
        print4xi(class2print, method2print, message2print) 
        # {'QUEUE': <queue4xi.Queue4Xi object at 0x000000000505A6D8>,
        #  'PUBLISHERCHANNEL': <read-write PipeConnection, handle 1068>,
        #  'CONSUMERCHANNEL': <read-write PipeConnection, handle 1064>}; time: 01:28:44 10/05/11 Pacific Daylight Time}

                
        return response4copyimage2upu
                                                           
    #</methodize4xi>    
                    
    # ---------------------------------------------------------------------------------------------
    #<methodize4xi method = "trace2image-of-UPU2Image4Xi" usage = "
    #   print('=' *33)          
    #   print('(Step 15A/25) OF UPU2IMAGE4XI5TEST - TRYING TO TRACE UPU: ', time4xi())      
    #   print('=' *33)      
    #   trace2upu = None
    #   try:
    #       trace2upu = trace2image4upu() 
    #       print('(Step 15B/25) OF UPU2IMAGE4XI5TEST - TRACE2UPU: ', trace2upu)  
    #       # {'MESSAGE': '{(Step 2Z0/33) failed; trace2image4upu-failed {}; time: 01:39:15 10/06/11 Pacific Daylight Time}', 'CLASS': 'trace2image4upu', 'METHOD': 'trace2image4upu'}
    #    except:
    #       print('(Step 15Z1/25) OF UPU2IMAGE4XI5TEST *** FAILED TO TRACE UPU: ', trace2upu)     
    #">
    
    # ---------------------------------------------------------------------------------------------   
    def trace2image(self, task4trace2image = None):
        '''
        traces UPU2Image4Xi objects
        ''' 
        
        # -----------------------------------------------------------------------------------------
        # (Step 1/33)           
        # -----------------------------------------------------------------------------------------              
        class2print = "UPU2Image4Xi"
        method2print = "trace2image"
                     
        message2print = "{(Step 1A/33) started" + "; trace2image-of-UPU2Image4Xi-started: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)   
        # 00:14:05 09/07/11 Pacific Daylight Time 
            
        message2print = "{(Step 1B/33) passed" + "; task4trace2image-passed-to-trace-of-UPU2Image4Xi: " + str(task4trace2image) + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)                                                          
        # None   
          
        # -----------------------------------------------------------------------------------------
        # (Step 2/33)           
        # -----------------------------------------------------------------------------------------
        message2print = "{(Step 2A/33) passing" + "; task4trace2image-passing-to-trace2image4upu-by-trace2image-of-UPU2Image4Xi: " + str(task4trace2image) + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)                                                               
        # None  
  
        trace2xi = trace2image4upu(task4trace2image)

        message2print = "{(Step 2B/33) performed" + "; trace2image4upu-performed-by-trace2image-of-UPU2Image4Xi: " + str(trace2image4upu) + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)             
        # <function trace2image4upu at 0x00000000039DF348>
                                
        # -----------------------------------------------------------------------------------------
        # (Step 3/33)           
        # ----------------------------------------------------------------------------------------- 
       
        message2print = "{(Step 3A/33) returning" + "; trace2xi-returning-by-trace2image-of-UPU2Image4Xi: " + str(trace2xi) + "; time: " + time4xi() + "}" 
        print4xi(class2print, method2print, message2print) 
        # {'MESSAGE': '{(Step 2Z0/33) failed; trace2image4upu-failed {}; time: 01:05:20 09/08/11 Pacific Daylight Time}',
        #  'CLASS': 'trace2image4upu',
        #  'METHOD': 'trace2image4upu'}; time: 01:05:20 09/08/11 Pacific Daylight Time}
                 
                
        return trace2xi
                                                           
    #</methodize4xi>       


    
    # ---------------------------------------------------------------------------------------------
    #<methodize4xi method = "set2imagecallback-of-UPU2Image4Xi" usage = "
    #   print('(Step 1A/33) OF UPU2IMAGE4XI - TRYING TO SET UPU IMAGE')
    #   try:
    #       cc2xi = UPU2Image4Xi()                            
    #       image2upu = cc2xi.set2imagecallback()                                  
    #       print('(Step 1B/33) OF UPU4XI5TEST - SET UPU IMAGE: ', image2upu)                                                  
    #   except:
    #       print('(Step 1Z/33) OF UPU4XI5TEST *** FAILED TO SET UPU IMAGE ***')  
    #">
    
    # ---------------------------------------------------------------------------------------------   
    def set2imagecallback(self, task4set2imagecallback = None): 
        '''
        set2imagecallback of UPU2Image4Xi objects
        ''' 
        
        # -----------------------------------------------------------------------------------------
        # (Step 1/33)           
        # -----------------------------------------------------------------------------------------              
        class2print = "UPU2Image4Xi"
        method2print = "set2imagecallback"
                     
        message2print = "{(Step 1A/33) started" + "; set2imagecallback-of-UPU2Image4Xi-started: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)   
        # 00:14:05 09/07/11 Pacific Daylight Time
            
        message2print = "{(Step 1B/33) passed" + "; task4set2imagecallback-passed-to-set2imagecallback-of-UPU2Image4Xi: " + str(task4set2imagecallback) + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)                                                          
        # None    
          
        # -----------------------------------------------------------------------------------------
        # (Step 2/33)           
        # -----------------------------------------------------------------------------------------
        message2print = "{(Step 2A/33) passing" + "; task4set2imagecallback-passing-to-set2imagecallback -by-set2imagecallback-of-UPU2Image4Xi: " + str(task4set2imagecallback) + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)                                                               
        # None
        
        callback2upu4image = {} 
        try:
            callback2upu4image = set2imagecallback4upu (task4set2imagecallback)

            message2print = "{(Step 2B/33) callback2upu4image" + "; callback2upu4image-set-by-set2imagecallback-of-UPU2Image4Xi to: " + str(callback2upu4image) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)             
            # <function trace2image4upu at 0x00000000039DF348>
        except:
            message2print = "{(Step 4A/33) failed" + "; callback2upu4image-failed-by-set2imagecallback-of-UPU2Image4Xi: " + str(callback2upu4image) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print) 
                                            
        # -----------------------------------------------------------------------------------------
        # (Step 3/33)           
        # ----------------------------------------------------------------------------------------- 
       
        message2print = "{(Step 3A/33) returning" + "; callback2upu4image-returning-by-set2imagecallback-of-UPU2Image4Xi: " + str(callback2upu4image) + "; time: " + time4xi() + "}" 
        print4xi(class2print, method2print, message2print) 
        # <function trace2image4upu at 0x0000000003AB31C8> 
                                             
        return callback2upu4image
                                                           
    #</methodize4xi>     
    
                                                              
    # ---------------------------------------------------------------------------------------------
    #<methodize4xi method = "sense-of-UPU2Image4Xi" usage = "
    #   try:
    #       print(xi2upu.sense())
    #       print('(Step 24B/33) OF UPU2IMAGE4XI5TEST - SENSED ', time4xi()) 
    #   except:
    #       print('(Step 24Z/33) OF UPU2IMAGE4XI5TEST *** FAILED TO SENSE ', time4xi())  
    # ">
    
    # ---------------------------------------------------------------------------------------------
    def sense(self):
        '''
        method sense: senses UPU2Image4Xi class objects
        '''
        
        # -----------------------------------------------------------------------------------------
        #
        # ----------------------------------------------------------------------------------------- 
        class2print = 'UPU2Image4Xi'
        method2print = 'sense'
        
                     
        message2print = "{(Step 1A/33) started" + "; sense-of-UPU2Image4Xi-started: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)   
        # 01:02:22 09/07/11 Pacific Daylight Time 
        
        # -----------------------------------------------------------------------------------------
        #
        # -----------------------------------------------------------------------------------------  
        
        message2print = "{(Step 2A/33) sensing" + "; UPU2Image4Xi-sensing-by-sense-of-UPU2Image4Xi: " + str(UPU2Image4Xi)  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)         
        # <class 'upu2image4xi.UPU2Image4Xi'>         
        
        # -----------------------------------------------------------------------------------------
        # get sensed knowledge for this object
        # -----------------------------------------------------------------------------------------        
        
        kb2xi = sense_object4xi(self)
        
        message2print = "{(Step 3A/33) got" + "; kb2xi-got-by-sense-of-UPU2Image4Xi: " + str(kb2xi)  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)
        # {'object': <upu2image4xi.UPU2Image4Xi object at 0x0000000004188400>}   
        
        # -----------------------------------------------------------------------------------------
        # add knowledge specific to this object
        # format example: kb2xi['time2xi'] = time4xi() #adding time 
        # -----------------------------------------------------------------------------------------         
        try:
            kb2xi['classvariable2upu-of-UPU2Image4Xi'] = UPU2Image4Xi.classvariable2upu
        except:
            message2print = "{(Step 4A/33) yielded" + "; kb2xi-yielded-by-sense-of-UPU2Image4Xi: " + str(kb2xi)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print) 
            
      
        message2print = "{(Step 5A/33) sensed" + "; UPU2Image4Xi-sensed-by-sense-of-UPU2Image4Xi: " + str(UPU2Image4Xi)  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)    
        # <class 'upu2image4xi.UPU2Image4Xi'>
        
        # -----------------------------------------------------------------------------------------
        # 
        # -----------------------------------------------------------------------------------------         
        
        message2print = "{(Step 6A/33) returning" + "; kb2xi-returning-by-sense-of-UPU2Image4Xi: " + str(kb2xi) + "; time: " + time4xi() + "}" 
        print4xi(class2print, method2print, message2print)                                                          
        # {'object': <upu2image4xi.UPU2Image4Xi object at 0x0000000004188400>, 'classvariable2upu-of-UPU2Image4Xi': 4}             
          
        return kb2xi
        
        # -----------------------------------------------------------------------------------------
        #
        # -----------------------------------------------------------------------------------------        
    #</methodize4xi>
    
        
    # ---------------------------------------------------------------------------------------------
    #<methodize4xi method = "reason-of-UPU2Image4Xi" usage = "
    #   print('(Step 25A/33) OF UPU2IMAGE4XI5TEST - REASONING ', time4xi())
    #   try:
    #       print(xi2upu.reason())
    #       print('(Step 25B/33) OF UPU2IMAGE4XI5TEST - REASONED ', time4xi())
    #   except:
    #       print('(Step 25Z/33) OF UPU2IMAGE4XI5TEST *** FAILED TO REASON ', time4xi())   
    #">
    
    # ---------------------------------------------------------------------------------------------
    def reason(self):
        '''
        method reason: reasons UPU2Image4Xi class objects
        '''
        
        # -----------------------------------------------------------------------------------------
        #
        # ----------------------------------------------------------------------------------------- 
        class2print = 'UPU2Image4Xi'
        method2print = 'reason'

                     
        message2print = "{(Step 1A/33) started" + "; reason-of-UPU2Image4Xi-started: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)   
        # 01:02:22 09/07/11 Pacific Daylight Time 
        
        # -----------------------------------------------------------------------------------------
        #
        # -----------------------------------------------------------------------------------------
        
        message2print = "{(Step 2A/33) reasoning" + "; UPU2Image4Xi-reasoning-by-reason-of-UPU2Image4Xi: " + str(UPU2Image4Xi) + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print) 
        # <class 'upu2image4xi.UPU2Image4Xi'>  
        
        # -----------------------------------------------------------------------------------------
        # get reasoned knowledge for this object
        # -----------------------------------------------------------------------------------------         
        
        kb4xi = reason_object4xi(self)
        
        
        message2print = "{(Step 3A/33) got" + "; kb4xi-got-by-reason-of-UPU2Image4Xi: " + str(kb4xi)  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)
        # {'object-reason-object4xi': <upu2image4xi.UPU2Image4Xi object at 0x0000000004188400>}      
        
        # -----------------------------------------------------------------------------------------
        # add knowledge specific to this object
        # format example: kb4xi['time2xi'] = time4xi() #adding time 
        # -----------------------------------------------------------------------------------------          
        try:
            kb4xi['time-reason-of-UPU2Image4Xi'] = time4xi() #adding time
        except:
            message2print = "{(Step 4Z/33) yielded" + "; kb4xi-yielded-by-reason-of-UPU2Image4Xi: " + str(kb4xi)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print) 
        
          
        
        message2print = "{(Step 5A/33) reasoned" + "; UPU2Image4Xi-reasoned-by-reason-of-UPU2Image4Xi: " + str(UPU2Image4Xi) + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)        
        # <class 'upu2image4xi.UPU2Image4Xi'>
        
        # -----------------------------------------------------------------------------------------
        #
        # -----------------------------------------------------------------------------------------        
        
        message2print = "{(Step 6A/33) returning" + "; kb4xi-returning-by-reason-of-UPU2Image4Xi: " + str(kb4xi) + "; time: " + time4xi() + "}" 
        print4xi(class2print, method2print, message2print)                                                          
        # {'object-reason-object4xi': <upu2image4xi.UPU2Image4Xi object at 0x0000000004188400>, 'time-reason-of-UPU2Image4Xi': '01:02:22 09/07/11 Pacific Daylight Time'}                
          
        return kb4xi
        
        # -----------------------------------------------------------------------------------------
        #
        # -----------------------------------------------------------------------------------------            
    #</methodize4xi>
    
    
    # ---------------------------------------------------------------------------------------------
    #<methodize4xi method = "ace-of-UPU2Image4Xi" usage = "
    #   print('(Step 26A/33) OF UPU2IMAGE4XI5TEST - ACING ', time4xi())   
    #   try:
    #       print(xi2upu.ace())
    #       print('(Step 26B/33) OF UPU2IMAGE4XI5TEST - ACED ', time4xi())                                                 
    #   except:
    #       print('(Step 26Z/33) OF UPU2IMAGE4XI5TEST *** FAILED TO ACE ', time4xi())                                       
    #    
    #">
    
    # ---------------------------------------------------------------------------------------------
    def ace(self):
        '''
        method ace: aces UPU2Image4Xi class objects 
        '''
        
        # -----------------------------------------------------------------------------------------
        #
        # -----------------------------------------------------------------------------------------       
        
        
        class2print = 'UPU2Image4Xi'
        method2print = 'ace'
        
                     
        message2print = "{(Step 1A/33) started" + "; ace-of-UPU2Image4Xi-started: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)   
        # 01:02:22 09/07/11 Pacific Daylight Time   
        
        # -----------------------------------------------------------------------------------------
        #
        # -----------------------------------------------------------------------------------------
        
        message2print = "{(Step 2A/33) acing" + "; UPU2Image4Xi-acing-by-ace-of-UPU2Image4Xi: " + str(UPU2Image4Xi) + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)         
        # <class 'upu2image4xi.UPU2Image4Xi'>           
        
        # -----------------------------------------------------------------------------------------
        # get aced knowledge for this object
        # -----------------------------------------------------------------------------------------        
        
        kb8xi = ace_object4xi(self)  
        
        message2print = "{(Step 3A/33) got" + "; kb8xi-got-by-ace-of-UPU2Image4Xi: " + str(kb8xi)  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)
        # {'object8xi': <upu2image4xi.UPU2Image4Xi object at 0x0000000004188400>}   


        # -----------------------------------------------------------------------------------------
        # add knowledge specific to this object
        # format example: kb8xi['time2xi'] = time4xi() #adding time 
        # -----------------------------------------------------------------------------------------              
        try:
             kb8xi['cores-of-UPU2Image4Xi'] = UPU2Image4Xi.classvariable2upu
        except:
             message2print = "{(Step 4A/33) yielded" + "; kb8xi-yielded-by-ace-of-UPU2Image4Xi: " + str(kb8xi)  + "; time: " + time4xi() + "}"
             print4xi(class2print, method2print, message2print) 
  
        message2print = "{(Step 5A/33) aced" + "; UPU2Image4Xi-aced-by-ace-of-UPU2Image4Xi: " + str(UPU2Image4Xi) + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)         
        # <class 'upu2image4xi.UPU2Image4Xi'>  
        
        # -----------------------------------------------------------------------------------------
        #
        # -----------------------------------------------------------------------------------------        
        message2print = "{(Step 6A/33) returning" + "; kb8xi-returning-by-ace-of-UPU2Image4Xi: " + str(kb8xi) + "; time: " + time4xi() + "}" 
        print4xi(class2print, method2print, message2print)                                                          
        # {'cores-of-UPU2Image4Xi': 4, 'object8xi': <upu2image4xi.UPU2Image4Xi object at 0x0000000004188400>}
        
        return kb8xi
        
        # -----------------------------------------------------------------------------------------
        #
        # -----------------------------------------------------------------------------------------            
    #</methodize4xi>
#</class4xi>
#</program4xi>

# -------------------------------------------------------------------------------------------------
# Recorded History                             
# ------------------------------------------------------------------------------------------------- 
#<record4xi record = "
#   upu2image4xi5test.txt
#   sense-upu2image4xi5test.xml
#   reason-upu2image4xi5test.xml
#   ace-upu2image4xi5test.xml
#"
#>

#<acknowledge4xi acknowledgement = "33" tester = "Small & Smart"/>

# -------------------------------------------------------------------------------------------------
# to-be-done                              
# -------------------------------------------------------------------------------------------------    
#<do4xi do = "
# nothing
#"
#/>

#<note4xi note = "
# parse4xi.py - to parse this module
# "/>

# -------------------------------------------------------------------------------------------------
# Legal Notice                             
# ------------------------------------------------------------------------------------------------- 
#<protect4xi protection = "copyright"
#   owner = "Small & Smart"
#   rights = "All rights reserved"
#   state = "California"
#   country = "United States of America"
#   codes = "Codes established in California and applicable to corporations in California"
#   compliance = "The California Law"
#   court = "Supreme Court of California, 350 McAllister Street, San Francisco, CA 94102-4783"
#/>

#<end4xi end = "
# -------------------------------------------------------------------------------------------------
# end of upu2image4xi.py                              
# ------------------------------------------------------------------------------------------------- 
#"
#/>
